unit AddBPUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, TypeDefs;

type
  TAddBPForm = class(TForm)
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    Label1: TLabel;
    TrEdit: TEdit;
    TrAddBtn: TButton;
    TrAddCloseBtn: TButton;
    Label2: TLabel;
    PsEdit: TEdit;
    PsAddBtn: TButton;
    PsAddCloseBtn: TButton;
    Label3: TLabel;
    TriggerEdit: TEdit;
    Label4: TLabel;
    FxEdit: TEdit;
    FxAddBtn: TButton;
    FxAddCloseBtn: TButton;
    CloseBtn: TBitBtn;
    procedure EditBoxKeyPress(Sender: TObject; var Key: Char);
    procedure EditBoxEnter(Sender: TObject);
    procedure AddBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  AddBPForm: TAddBPForm;

implementation

{$R *.DFM}

uses BPUnit, Main, LookupUnit;

var
  LastEditText : string;

procedure TAddBPForm.EditBoxKeyPress(Sender: TObject; var Key: Char);
begin
{Always allow the backspace, return and tab keys}
  if (Key = ^H) or (Key = ^M) or (Key = ^I) then exit;
  if not (Key in ['$','0'..'9','A'..'F','a'..'f']) then Key := chr(0);
end; // procedure TAddBPForm.EditBoxKeyPress

procedure TAddBPForm.EditBoxEnter(Sender: TObject);
begin
  LastEditText := (Sender as tEdit).Text;
  TrAddCloseBtn.Default := false;
  FxAddCloseBtn.Default := false;
  PsAddCloseBtn.Default := false;
  CloseBtn.Default := false;
  if Sender = TrEdit then TrAddCloseBtn.Default := true;
  if Sender = FxEdit then FxAddCloseBtn.Default := true;
  if Sender = PsEdit then PsAddCloseBtn.Default := true;
end; // procedure TAddBPForm.EditBoxEnter

procedure TAddBPForm.AddBtnClick(Sender: TObject);
var
  BPType    : tBPType;
  Edit      : tEdit;
  Tptr      : tBPRecordPtr;
  BPRecord  : tBPRecord;
  Addr      : word;
  AlsoClose : boolean;
  AreasUsed : tMemAreaSet;
  Comment   : tCmdLineComment;
begin
  BPType := bpNone;
  if (Sender = TrAddBtn) or (Sender = TrAddCloseBtn) then BPType := bpTransient;
  if (Sender = FxAddBtn) or (Sender = FxAddCloseBtn) then BPType := bpFixed;
  if (Sender = PsAddBtn) or (Sender = PsAddCloseBtn) then BPType := bpPass;
  AlsoClose := (Sender = TrAddCloseBtn) or
               (Sender = FxAddCloseBtn) or
               (Sender = PsAddCloseBtn);
  case BPType of
    bpTransient : Edit := TrEdit;
    bpFixed     : Edit := FxEdit;
    bpPass      : Edit := PsEdit
    else Edit := nil;
  end;
  Addr := StrToInt('$0' + Edit.Text);
  if RetrieveBP(Addr,[bpTransient..bpPass],Tptr)
  then begin
    MessageDlg('Breakpoint already set at address',
               mtError,[mbOK],0);
    case BPType of
      bpTransient : TrEdit.SetFocus;
      bpFixed     : FxEdit.SetFocus;
      bpPass      : PsEdit.SetFocus;
    end;
    exit;
  end;
  if not CheckAreas(Addr,Addr,[maRam,maExtRam,maEeprom],MemMap,AreasUsed,Comment)
  then begin
    MessageDlg('Cannot set breakpoint in non-modifiable memory',
               mtError,[mbOK],0);
    case BPType of
      bpTransient : TrEdit.SetFocus;
      bpFixed     : FxEdit.SetFocus;
      bpPass      : PsEdit.SetFocus;
    end;
    exit;
  end;

  MakeBPRecord(Addr,BPType,BPRecord);
  if BPType = bpPass then BPRecord.Trigger := StrToInt('0' + TriggerEdit.Text);
  AddBPToList(BPRecord);
  UpdateBPWindow;
  if AlsoClose
  then AddBPForm.Close
  else begin
    case BPType of
      bpTransient : TrEdit.SetFocus;
      bpFixed     : FxEdit.SetFocus;
      bpPass      : PsEdit.SetFocus;
    end;
  end;
end; // procedure TAddBPForm.AddBtnClick

procedure TAddBPForm.FormShow(Sender: TObject);
begin
  Left := DesktopCentre.x - (width div 2);
  Top  := MainForm.Top + 50;
end; // procedure TAddBPForm.FormShow

procedure TAddBPForm.CloseBtnClick(Sender: TObject);
begin
  ModalResult := mrOK;
  MainForm.CmdEdit.SetFocus;
end; // procedure TAddBPForm.CloseBtnClick

end.
