unit AddToWatchUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, TypeDefs;

type
  TAddToWatchForm = class(TForm)
    UDGroupBox: TGroupBox;
    IXGroupBox: TGroupBox;
    ViewSymbolBtn: TButton;
    CloseBtn: TBitBtn;
    HelpBtn: TBitBtn;
    UDAddrEdit: TEdit;
    UDDescEdit: TEdit;
    IXComboBox: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    UDAddBtn: TButton;
    UDAddCloseBtn: TButton;
    IXAddBtn: TButton;
    IXAddCloseBtn: TButton;
    IYGroupBox: TGroupBox;
    IYComboBox: TComboBox;
    IYAddBtn: TButton;
    IYAddCloseBtn: TButton;
    Label7: TLabel;
    Label4: TLabel;
    Label3: TLabel;
    procedure RelKeyPress(Sender: TObject; var Key: Char);
    procedure UDDescEditKeyPress(Sender: TObject; var Key: Char);
    procedure CloseBtnClick(Sender: TObject);
    procedure EditBoxEnter(Sender: TObject);
    procedure UDAddrEditBoxExit(Sender: TObject);
    procedure ComboBoxEnter(Sender: TObject);
    procedure ComboBoxExit(Sender: TObject);
    procedure AddBtnClick(Sender: TObject);
    procedure GroupBoxEnter(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ViewSymbolBtnClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  AddToWatchForm: TAddToWatchForm;

procedure RefreshWatch;

implementation

{$R *.DFM}

uses
  Main, SymbolUnit, MemRW, RegRW, MiscUnit;

var
  LastEditText,WatchListText : string;

procedure RefreshWatch;
var
  i,j,p      : integer;
  Buf,S      : string;
  Loc,Offset : word;
  LocalFrame : tStackFrame;
  B          : byte;
begin
  if Prog.State in [psDuffData,psConnected] then exit;
  Loc        := 0;
  Offset     := 0;
  LocalFrame := Prog.StackFrame;
  with MainForm do
  begin
    for i := 0 to WatchListBox.Items.Count - 1 do
    begin
      Buf := WatchListBox.Items[i];
      p   := pos(',X',Buf);
      if p > 4
      then begin
        j := p - 1;
        S := '';
        while upcase(Buf[j]) in ['$','0'..'9','A'..'F'] do
        begin S := Buf[j] + S; j := j - 1; end;
        try Offset := StrToInt(S); except on EConvertError do continue; end;
        Loc := LocalFrame.IX + Offset;
        delete(Buf,1,4);
        insert(IntToHex(Loc,4),Buf,1);
        WatchListBox.Items[i] := Buf;
      end;
      p   := pos(',Y',Buf);
      if p > 4
      then begin
        j := p - 1;
        S := '';
        while upcase(Buf[j]) in ['$','0'..'9','A'..'F'] do
        begin S := Buf[j] + S; j := j - 1; end;
        try Offset := StrToInt(S); except on EConvertError do continue; end;
        Loc := LocalFrame.IY + Offset;
        delete(Buf,1,4);
        insert(IntToHex(Loc,4),Buf,1);
        WatchListBox.Items[i] := Buf;
      end;
    end; // for i := 0 to WatchListBox.Items.Count - 1 do
    for i := 0 to WatchListBox.Items.Count - 1 do
    begin
      Buf := WatchListBox.Items[i];
      if length(Buf) > 17 then setlength(Buf,17);
      try
        Loc := StrToInt('$' + copy(Buf,1,4));
      except on EConvertError do continue end;
      B := ReadByte(Loc);
      SpaceTo(Buf,17);
      Buf := Buf + ' ' + IntToHex(B,2);
      WatchListBox.Items[i] := Buf;
    end;
  end; // with Mainform do
end; // procedure TMainForm.RefreshWatch

procedure ClearBtnDefaults;
begin
  with AddToWatchForm do
  begin
    UDAddBtn.Default := false;
    IXAddBtn.Default := false;
    IYAddBtn.Default := false;
    CloseBtn.Default := false;
  end;
end; // procedure ClearBtnDefaults

procedure TAddToWatchForm.RelKeyPress(Sender: TObject; var Key: Char);
begin
{Always allow the backspace, return and tab keys}
  if (Key = ^H) or (Key = ^M) or (Key = ^I) then exit;
  if not (Key in ['$','0'..'9','A'..'F','a'..'f']) then Key := chr(0);
end;

procedure TAddToWatchForm.UDDescEditKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = ' ') then Key := chr(0);
end;

procedure TAddToWatchForm.CloseBtnClick(Sender: TObject);
begin
  ModalResult := mrOK;
  MainForm.CmdEdit.SetFocus;
end; // procedure TAddToWatchForm.CloseBtnClick

procedure TAddToWatchForm.EditBoxEnter(Sender: TObject);
begin
  LastEditText := (Sender as tEdit).Text;
  UDAddCloseBtn.Default := false;
  IXAddCloseBtn.Default := false;
  IYAddCloseBtn.Default := false;
  CloseBtn.Default := false;
  if Sender = UDAddrEdit then UDAddCloseBtn.Default := true;
  if Sender = IXComboBox then IXAddCloseBtn.Default := true;
  if Sender = IYComboBox then IYAddCloseBtn.Default := true;
end; // procedure TAddToWatchForm.EditBoxEnter

procedure TAddToWatchForm.UDAddrEditBoxExit(Sender: TObject);
var
  i   : integer;
  Buf : string;
begin
  Buf := (Sender as tEdit).Text;
  for i := length(Buf) to 3 do insert('0',Buf,1);
  (Sender as tEdit).Text := Buf
end; // procedure TAddToWatchForm.EditBoxExit

procedure TAddToWatchForm.ComboBoxEnter(Sender: TObject);
begin
  LastEditText := (Sender as tComboBox).Text;
end; // procedure TAddToWatchForm.ComboBoxEnter

procedure TAddToWatchForm.ComboBoxExit(Sender: TObject);
begin
  if length((Sender as tComboBox).Text) = 0
  then begin
    MessageDlg('Combo box must not be left blank',mtError,[mbOK],0);
    (Sender as tComboBox).Text := LastEditText;
    (Sender as tComboBox).SetFocus;
  end;
end;

procedure TAddToWatchForm.AddBtnClick(Sender: TObject);
var
  AddClose : boolean;
begin
  AddClose := false;
  with MainForm.WatchListBox do
  begin
    if (Sender = UDAddBtn) or (Sender = UDAddCloseBtn)
    then begin
      Items.Add(uppercase(UDAddrEdit.Text) + ' ' + UDDescEdit.Text);
      AddClose := (Sender = UDAddCloseBtn);
    end;

    if (Sender = IXAddBtn) or (Sender = IXAddCloseBtn)
    then begin
      Items.Add('     ' + IXComboBox.Text + ',X');
      AddClose := (Sender = IXAddCloseBtn);
    end;

    if (Sender = IYAddBtn) or (Sender = IYAddCloseBtn)
    then begin
      Items.Add('     ' + IYComboBox.Text + ',Y');
      AddClose := (Sender = IYAddCloseBtn);
    end;
  end; // with MainForm.WatchListBox do
  if AddClose
  then begin
    AddToWatchForm.Close;
    MainForm.CmdEdit.SetFocus;
  end
  else begin
    if Sender = UDAddBtn then UDAddrEdit.SetFocus;
    if Sender = IXAddBtn then IXComboBox.SetFocus;
    if Sender = IYAddBtn then IYComboBox.SetFocus;
  end;
end; // procedure TAddToWatchForm.AddBtnClick

procedure TAddToWatchForm.GroupBoxEnter(Sender: TObject);
begin
  UDAddBtn.Default := false;
  IXAddBtn.Default := false;
  IYAddBtn.Default := false;
  if Sender = UDGroupBox then UDAddBtn.Default := true;
  if Sender = IXGroupBox then IXAddBtn.Default := true;
  if Sender = IYGroupBox then IYAddBtn.Default := true;
end;

procedure TAddToWatchForm.FormShow(Sender: TObject);
begin
  WatchListText := MainForm.WatchListBox.Items.Text;
  Left := DesktopCentre.x - (width div 2);
  Top  := MainForm.Top + 82;
  UDAddrEdit.SetFocus;
end; // procedure TAddToWatchForm.FormShow

procedure TAddToWatchForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if WatchListText <> MainForm.WatchListBox.Items.Text
  then Prog.ProjectHasChanged := true;
end; // procedure TAddToWatchForm.FormClose

procedure TAddToWatchForm.ViewSymbolBtnClick(Sender: TObject);
begin
  ShowSymbols;
  ModalResult := mrYes;
end; // procedure TAddToWatchForm.ViewSymbolBtnClick

procedure TAddToWatchForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(IDH_ADD_TO_WATCH);
end; // procedure TAddToWatchForm.HelpBtnClick

end.
