unit BPUnit;

interface

uses Classes, SysUtils, Graphics, TypeDefs, Dialogs;

type
  tATRecord = record
                Addr    : word;
                BPType  : tBPType;
                Trigger : integer;
              end;
  tATArray = array of tATRecord;

procedure MakeBPRecord(    BPAddr    : word;
                           NewBPType : tBPType;
                       var BPRecord  : tBPRecord);
procedure ClearBPList;
function  AddBPtoList(BPRecord : tBPRecord) : boolean;
function  DeleteBPatAddr(DelAddr      : word;
                         AllowedTypes : tSetofBPTypes) : boolean;
function  DeleteBPbyType(DelTypes : tSetOfBPTypes) : boolean;
function  RetrieveBP(    RetAddr : word;
                         BPTypes : tSetOfBPTypes;
                     var BPPtr   : tBPRecordPtr) : boolean;
procedure UpdateBPWindow;
function  SetBreak(Addr : word) : boolean;
function  SetBreaksX(BPTypes      : tSetofBPTypes;
                     ExceptAtAddr : word) : boolean;
function  ClearBreak(Addr : word) : boolean;
function  ClearBreaksByType(BPTypes : tSetofBPTypes) : boolean;
procedure RemoveTransientSetByTrace;
procedure RemoveTraceRequests;
procedure ResetPassCount(Addr : word);
procedure ResetAllPassCounts;

function  ATAddrExists(Address : word; var ATArray : tATArray) : boolean;
procedure ClearATArray(var ATArray : tATArray);
function  BRCheck(const CList   : tStringList;
                  var   Comment : tCmdLineComment;
                  var   Command : tCommand) : boolean;
function  BRPCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
function NoBRCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
function  RPCCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
procedure AddBR;
procedure AddBRP;
procedure NoBR;
procedure RPC;

implementation

uses Main, CmdUnit, Comms1, CheckUnit, MemRW, UnAsmUnit, LookupUnit, RegRW;

//const
//  DotOn : boolean = true;

var
  OrderAddedIndex : integer = 0;
  ATArray         : tATArray;

procedure MakeBPRecord(    BPAddr    : word;
                           NewBPType : tBPType;
                       var BPRecord  : tBPRecord);
begin
{Read in 5 memory locations because this covers the longest instructions}
  ReadMCU256(BPAddr,BPAddr+4,false);
  DisAsm1(BPAddr,BPRecord);
  with BPRecord do
  begin
    BPType   := NewBPType;
  end;
end; // procedure MakeBPRecord

procedure ClearBPList;
var
  i : integer;
  TPtr : tBPRecordPtr;
begin
  for i := 0 to (BPList.Count - 1) do
  begin
    TPtr := BPList[i];
    if TPtr <> nil then Dispose(TPtr);
  end;
  BPList.Clear;
end; // procedure ClearBPList

{Function will only add a BP to the list if it has a unique adress, unless
 the BP type is bpStart}
function  AddBPtoList(BPRecord : tBPRecord) : boolean;
var
  Tptr : tBPRecordPtr;
  i    : integer;
begin
  Result := false;
  if BPRecord.BPType <> bpStart
  then begin
    i := 0;
    while i < BPList.Count do
    begin
      Tptr := BPList[i];
      if Tptr^.Addr = BPRecord.Addr then break;
      i := i + 1;
    end;
    if i < BPList.Count then exit;
  end;
  OrderAddedIndex := OrderAddedIndex + 1;
  BPRecord.OrderAdded := OrderAddedIndex;
  new(Tptr);
  Tptr^ := BPRecord;
  BPList.Add(Tptr);
  Result := true;
end; // function  AddBPtoList

{Function returns false if it was not possible to delete the bp from the list}
function  DeleteBPatAddr(DelAddr      : word;
                         AllowedTypes : tSetofBPTypes) : boolean;
var
  Tptr : tBPRecordPtr;
  i    : integer;
begin
  Result := false;
  Tptr   := nil;
  i      := 0;
  while i < BPList.Count do
  begin
    Tptr := BPList[i];
    if (Tptr^.Addr = DelAddr) and (Tptr^.BPType in AllowedTypes) then break;
    i := i + 1;
  end;
  if (i = BPList.Count) or (Tptr = nil) then exit;
  
  ClearBreak(Tptr^.Addr);
  Dispose(Tptr);
  BPList.Delete(i);
  Result := true;
end; // function DeleteBPatAddr

{Function returns false if it was not possible to delete the bp from the list}
function  DeleteBPbyType(DelTypes : tSetOfBPTypes) : boolean;
var
  Tptr : tBPRecordPtr;
  i    : integer;
begin
  Result := false;
  i      := 0;
  while i < BPList.Count do
  begin
    Tptr := BPList[i];
    if Tptr^.BPType in DelTypes
    then begin
      if not DeleteBPAtAddr(Tptr^.Addr,DelTypes) then exit;
    end
    else i := i + 1;
  end;
  Result := i < BPList.Count
end; // function DeleteBPbyType

function  RetrieveBP(    RetAddr : word;
                         BPTypes : tSetOfBPTypes;
                     var BPPtr   : tBPRecordPtr) : boolean;
var
  Tptr : tBPRecordPtr;
  i    : integer;
begin
  i    := 0;
  TPtr := nil;
  while i < BPList.Count do
  begin
    Tptr := BPList[i];
    if (Tptr^.Addr = RetAddr) and (Tptr^.BPType in BPTypes) then break;
    i := i + 1;
  end;
  Result := i < BPList.Count;
  if Result then BPPtr := Tptr else BPPtr := nil;
end; // function  RetrieveBP

function  BPTypeAscending(Item1,Item2 : pointer) : integer;
begin
  Result := ord(tBPRecordPtr(Item1)^.BPType) - ord(tBPRecordPtr(Item2)^.BPType);
  if Result = 0
  then Result := tBPRecordPtr(Item1)^.Addr - tBPRecordPtr(Item2)^.Addr;
end; // function  BPTypeAscending

function  BPAddrAscending(Item1,Item2 : pointer) : integer;
begin
  Result := tBPRecordPtr(Item1)^.Addr - tBPRecordPtr(Item2)^.Addr;
end; // function  BPAddrAscending

function  BPOrderAdded(Item1,Item2 : pointer) : integer;
begin
  Result := tBPRecordPtr(Item1)^.OrderAdded - tBPRecordPtr(Item2)^.OrderAdded;
end; // function BPOrderAdded

procedure UpdateBPWindow;
var
  Tptr : tBPRecordPtr;
  i    : integer;
  Buf  : string;
  L    : tStringList;
begin
  MainForm.BPListBox.Clear;
  case BPSortOrder of
    soType       : BPlist.Sort(BPTypeAscending);
    soAddress    : BPlist.Sort(BPAddrAscending);
    soOrderAdded : BPlist.Sort(BPOrderAdded);
  end; // case BPSortOrder of

{Put Start Record at top}
  Tptr := nil;
  i := 0;
  while i < BPList.Count do
  begin
    if tBPRecordPtr(BPList[i])^.BPType = bpStart
    then begin
      Tptr := BPList[i];
      break;
    end;
    i := i + 1;
  end;
  if Tptr <> nil
  then begin
    BPList.Delete(i);
    BPList.Insert(0,Tptr);
  end;

  L := tStringList.Create;
  try
    for i := 0 to BPList.Count - 1 do
    begin
      Tptr := BPList[i];
      with Tptr^ do
      begin
        case BPType of
          bpStart     : Buf := ' (';
          bpTemp      : Continue;
//          bpTemp      : Buf := 'O ';
          bpTransient : Buf := 'T ';
          bpFixed     : Buf := 'X ';
          bpPass      : Buf := 'P ';
        end; // case
        Buf := Buf + BPStr;
        if BPType = bpFixed
        then Buf := Buf + ' (' + IntToStr(PassCount) + ')';
        if BPType = bpPass
        then Buf := Buf + ' (' + IntToStr(PassCount) + '/' +
                                 IntToStr(Trigger) + ')';
{$IFDEF DotOn}
        if IsNextTraceBP and (length(Buf) >1) then Buf[2] := '.';
{$ENDIF}
        if BPType = bpStart then Buf := Buf + ')';
        L.Add(Buf);
        if (Addr = Prog.BRAddr) and not (BPType = bpStart)
        then begin
          Buf := '0>';
          if SeqAddr <> $FFFF
            then Buf := Buf + IntToHex(SeqAddr,4) else Buf := Buf + '    ';
          Buf := Buf + '/';
          if AltAddr <> $FFFF
            then Buf := Buf + IntToHex(AltAddr,4) else Buf := Buf + '    ';
          if SeqAddr = NextAddr then Buf[1] := '1';
          if AltAddr = NextAddr then Buf[1] := '2';
          L.Add(Buf);
        end; // if Addr = Prog.BRAddr
      end; // with Tptr^ do
    end; // for i := 0 to BPList.Count - 1 do
    BRAddrStr := IntToHex(Prog.BRAddr,4);
    MainForm.BPListBox.Items.AddStrings(L);
  finally
    L.Free;
  end;
end; // procedure UpdateBPWindow

{Function returns TRUE if it was possible to set the memory location to $3F}
function SetBreak(Addr : word) : boolean;
var
  MemArea  : tMemArea;
  BPPtr    : tBPRecordPtr;
begin
  Result  := false;
  if not RetrieveBP(Addr,[bpTemp..bpPass],BPPtr) then exit;
  MemArea := GetMemArea(Addr,MemMap);
  if not (MemArea in [maRam,maExtRam,maEeprom]) then exit;

  if not BPPtr^.IsSet
  then begin
    case MemArea of                // $3F is the software interrupt instruction
      maRam,maExtRam : if not WriteByte(Addr,$3F,false) then exit;
      maEeprom : if not WriteByteEEPROM(Addr,BPptr^.MemVal,$3F) then exit;
    end; // case MemArea of
  end; // if not BPRecord.IsSet

  BPPtr^.IsSet := true;
  Result := true;
end; // function SetBreak

function  SetBreaksX(BPTypes      : tSetofBPTypes;
                     ExceptAtAddr : word) : boolean;
var
  Tptr : tBPRecordPtr;
  i    : integer;
begin
  Result := false;
  for i := 0 to BPList.Count - 1 do
  begin
    Tptr := BPList[i];                        
    with Tptr^ do
    begin
      if BPType = bpStart then continue;
      if BPType in BPTypes
      then begin
        if Addr <> ExceptAtAddr
        then begin
          if not SetBreak(Addr) then exit;
        end
        else begin
          if ActingAsTempBP or IsNextTraceBP
          then if not SetBreak(Addr) then exit;
        end;
      end; // if BPType in BPTypes
    end; // with Tptr^ do
  end; // for i := 0 to BPList.Count - 1 do
  Result := true;
end; // function SetAllBreaks

{Function returns TRUE if it was possible to set the memory location to MemVal}
function ClearBreak(Addr : word) : boolean;
var
  BPPtr   : tBPRecordPtr;
  MemArea : tMemArea;
begin
  Result  := false;
  MemArea := GetMemArea(Addr,MemMap);
  if not (MemArea in [maRam,maExtRam,maEeprom]) then exit;
  if not RetrieveBP(Addr,[bpTemp..bpPass],BPPtr) then exit;

  if BPPtr^.IsSet
  then begin
    case MemArea of
      maRam,maExtRam : if not WriteByte(Addr,BPptr^.MemVal,false) then exit;
      maEeprom : if not WriteByteEEPROM(Addr,$3F,BPPtr^.MemVal) then exit;
    end; // case MemArea of
  end; // if BPRecord.IsSet

  BPPtr^.IsSet := false;
  Result := true;
end; // function ClearBreak

{Function returns FALSE if it was not possible to set any of the the memory
 locations to MemVal}
function ClearBreaksByType(BPTypes : tSetofBPTypes) : boolean;
var
  Tptr : tBPRecordPtr;
  i    : integer;
begin
  Result := false;
  for i := 0 to BPList.Count - 1 do
  begin
    Tptr := BPList[i];
    with Tptr^ do
    begin
      if (BPType in BPTypes) and IsSet
      then if not ClearBreak(Addr) then exit;
    end;
  end;
  Result := true;
end; // function ClearBreaksByType

procedure RemoveTraceRequests;
var i : integer;
begin
  for i := 0 to BPList.Count - 1 do
    with tBPRecordPtr(BPList[i])^ do
    begin
      IsNextTraceBP := false;
      ActingAsTempBP := false;
    end;
end; // procedure RemoveTraceRequests

procedure RemoveTransientSetByTrace;
var
  Tptr : tBPRecordPtr;
  i    : integer;
begin
  i := 0;
  while i < BPList.Count do
  begin
    Tptr := BPList[i];
    with Tptr^ do
    begin
      if IsNextTraceBP and (BPType = bpTransient)
      then begin
        if IsSet then ClearBreak(Addr);
        DeleteBPAtAddr(Addr,[bpTransient]);
      end else i := i + 1;
    end;
  end;
end; // procedure RemoveTransientSetByTrace;

procedure ResetPassCount(Addr : word);
var
  BPPtr : tBPRecordPtr;
begin
  if not RetrieveBP(Addr,[bpFixed..bpPass],BPPtr) then exit;
  BPptr^.PassCount := 0;
end; // procedure ResetPassCount

procedure ResetAllPassCounts;
var
  BPPtr : tBPRecordPtr;
  i     : integer;
begin
  for i := 0 to BPList.Count - 1 do
  begin
    BPPtr := BPList[i];
    with BPptr^ do
    begin
      if BPType in [bpFixed,bpPass] then PassCount := 0;
    end;
  end;
end; // procedure ResetAllPassCounts

{****************************************************************************}

function  ATAddrExists(Address : word; var ATArray : tATArray) : boolean;
var i : integer;
begin
  i := 0;
  while i < length(ATArray) do
  begin
    if Address = ATArray[i].Addr then break;
    i := i + 1;
  end;
  Result := i < length(ATArray);
end; // function  ATAddrExists

procedure ClearATArray(var ATArray : tATArray);
var i : integer;
begin
  for i := 0 to length(ATArray) - 1 do
  begin
    fillchar(ATArray[i],sizeof(ATArray[i]),0);
  end;
end; // procedure ClearATArray

function BRCheck(const CList   : tStringList;
                 var   Comment : tCmdLineComment;
                 var   Command : tCommand) : boolean;
var
  AreasUsed   : tMemAreaSet;
  ArgType     : tArgType;
  i,j         : integer;
  Val         : longword;
  BreakType   : tBPType;
  BPPtr       : tBPRecordPtr;
  BreakAddr,RegAddr : word;
begin
  Result      := false;
  Comment     := clcArgErr;
  BreakAddr   := 0;
  BreakType   := bpNone;

  if not NumArgsOK(CList,1,5,Comment) then exit;
  if not StateIsAllowed([psStopped,psRunStop,psTraceStop],Comment) then exit;

  SetLength(ATArray,CList.Count-1);
  ClearATArray(ATArray);
{Subsequent arguments must be breakpoints}
  for i := 1 to CList.Count - 1 do
  begin
    if FoundArgOK(CList,i,[atWord,atWordX,atLabel],ArgType,Val)
    then begin
      case ArgType of
        atWord,atLabel :
          begin
            BreakAddr := Val;
            BreakType := bpTransient;
          end;
        atWordX :
          begin
            BreakAddr := Val;
            BreakType := bpFixed;
          end;
      end; // case ArgType of
    end; // if FoundArgOK(CList,i,[atWord,atWordX,atLabel],ArgType,Val) then..

{Check proposed memory area for this breakpoint}
    if not CheckAreas(BreakAddr,BreakAddr,[maRam,maExtRam,maEeprom],
                      MemMap,AreasUsed,Comment) then exit;

{Check that BPROT is OK, if necessary}
    if (maEeprom in AreasUsed) and
       (LookupCtrlRegAddr('BPROT',RegAddr,j))and
       ProjData.WarnOnBprot
    then begin
      if (ReadByte(RegAddr) and $0F) > 0
      then begin
        Comment := clcBPROTMayDisallowWrite;
        exit;
      end;
    end;

{Check that this breakpoint is not already in the list}
    if RetrieveBP(BreakAddr,[bpTransient..bpPass],BPPtr)
    then begin
      Comment := clcDuplicateBP;
      exit;
    end;

{If OK add to the address transfer list}
    if ATAddrExists(BreakAddr,ATArray)
    then begin
      Comment := clcDuplicateBP;
      exit;
    end;
    ATArray[i-1].Addr   := BreakAddr;
    ATArray[i-1].BPType := BreakType;
  end; // for i := 1 to CList.Count - 1 do
  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  Command   := cmdAddBR;
end; // function BRCheck

procedure AddBR;
var
  i           : integer;
  BreakRecord : tBPRecord;
begin
  for i := 0 to length(ATArray) - 1 do
  begin
    MakeBPRecord(ATArray[i].Addr,ATArray[i].BPTYpe,BreakRecord);
    AddBPtoList(BreakRecord);
  end;
  UpdateBPWindow;
end; // procedure AddBR

function BRPCheck(const CList   : tStringList;
                  var   Comment : tCmdLineComment;
                  var   Command : tCommand) : boolean;
var
  AreasUsed   : tMemAreaSet;
  ArgType     : tArgType;
  Val         : longword;
  BPPtr       : tBPRecordPtr;
  BreakAddr,RegAddr   : word;
  Repeats,i   : integer;
  Buf         : string;
begin
  Result      := false;
  Comment     := clcArgErr;
  BreakAddr   := 0;
  setlength(ATArray,1);
  ClearATArray(ATArray);

  if not NumArgsOK(CList,1,2,Comment) then exit;
  if not StateIsAllowed([psStopped,psRunStop,psTraceStop],Comment) then exit;

  Buf := uppercase(CList[0]);
  if length(Buf) > 3
  then begin
    delete(Buf,1,3);
    try
      Repeats := StrToInt(Buf);
    except on EConvertError do
      begin
        Comment := clcCmdErr;
        exit;
      end;
    end; // try.. except on EConvertError do
  end else Repeats := 1;

  if FoundArgOK(CList,1,[atWord,atLabel],ArgType,Val)
  then begin
    BreakAddr := Val;
  end;

{Check proposed memory area for this breakpoint}
  if not CheckAreas(BreakAddr,BreakAddr,[maRam,maExtRam,maEeprom],
                    MemMap,AreasUsed,Comment) then exit;
                    
{Check that BPROT is OK, if necessary}
    if (maEeprom in AreasUsed) and
       (LookupCtrlRegAddr('BPROT',RegAddr,i))and
       ProjData.WarnOnBprot
    then begin
      if (ReadByte(RegAddr) and $0F) > 0
      then begin
        Comment := clcBPROTMayDisallowWrite;
        exit;
      end;
    end;

{Check that this breakpoint is not already in the list}
  if RetrieveBP(BreakAddr,[bpTransient..bpPass],BPPtr)
  then begin
    Comment := clcDuplicateBP;
    exit;
  end;

{Check for a trigger count}
  if FoundArgOK(CList,2,[atNotExist,atInteger],ArgType,Val)
  then begin
    case ArgType of
      atInteger  : ATArray[0].Trigger := Val;
      atNotExist : ATArray[0].Trigger := Repeats;
    end;
  end else exit;

{If OK add to the address transfer list}
  ATArray[0].Addr   := BreakAddr;

  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  Command   := cmdAddBRP;
end; // function BRPCheck

procedure AddBRP;
var
  BPRecord : tBPRecord;
begin
  if length(ATArray) = 1
  then begin
    MakeBPRecord(ATArray[0].Addr,bpPass,BPRecord);
    BPRecord.Trigger := ATArray[0].Trigger;
    AddBPtoList(BPRecord);
  end;
  UpdateBPWindow;
end; // procedure AddBRP

function NoBRCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
var
  ArgType     : tArgType;
  Val         : longword;
  BPPtr       : tBPRecordPtr;
  BreakAddr   : word;
  i           : integer;
begin
  Result      := false;
  Comment     := clcArgErr;
  BreakAddr   := 0;
  ATArray     := nil;

  if not NumArgsOK(CList,0,5,Comment) then exit;
  if not StateIsAllowed([psStopped],Comment) then exit;

{If 'NOBR' used by itself, then clear all breaks}
  if CList.Count > 1
  then begin
    setlength(ATArray,CList.Count-1);
    ClearATArray(ATArray);
  end
  else begin  
    if BPList.Count = 0
    then begin
      Comment := clcBPNotFound;
      exit;
    end;
  end;

{Any subsequent arguments must be breakpoints}
  for i := 1 to CList.Count - 1 do
  begin
    if FoundArgOK(CList,i,[atWord,atLabel],ArgType,Val)
    then begin
      BreakAddr := Val;
    end;
{Check that this breakpoint is in the list}
    if not RetrieveBP(BreakAddr,[bpTransient..bpPass],BPPtr)
    then begin
      Comment := clcBPNotFound;
      exit;
    end;
{If OK add to address transfer list}
    if ATAddrExists(BreakAddr,ATArray)
    then begin
      Comment := clcDuplicateBP;
      exit;
    end;
    ATArray[i-1].Addr := BreakAddr;
  end; // for i := 1 to CList.Count - 1 do

  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  Command   := cmdNoBR;
end; // function NoBRCheck

procedure NoBR;
var
  i : integer;
begin
  if length(ATArray) = 0            // delete all breakpoints
  then begin
    DeleteBPByType([bpTemp..bpPass]);
  end
  else begin
    for i := 0 to length(ATArray) - 1 do
    begin
      DeleteBPatAddr(ATArray[i].Addr,[bpTransient..bpPass]);
    end;
  end;
  UpdateBPWindow;
end; // procedure NoBR

function  RPCCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
var
  ArgType     : tArgType;
  Val         : longword;
  BPPtr       : tBPRecordPtr;
  BreakAddr   : word;
  i           : integer;
begin
  Result      := false;
  Comment     := clcArgErr;
  BreakAddr   := 0;
  ATArray     := nil;

  if not NumArgsOK(CList,0,5,Comment) then exit;
  if not StateIsAllowed([psStopped,psRunStop,psTraceStop],Comment) then exit;

{If 'RPC' used by itself, then reset all pass counts}
  if CList.Count = 1
  then begin
{Check that there are breakpoints are in the list}
    if BPList.Count = 0
    then begin
      Comment := clcBPNotFound;
      exit;
    end
  end
  else begin
    setlength(ATArray,CList.Count-1);
    ClearATArray(ATArray);
  end;

{Any subsequent arguments must be breakpoints}
  for i := 1 to CList.Count - 1 do
  begin
    if FoundArgOK(CList,i,[atWord,atLabel],ArgType,Val)
    then begin
      BreakAddr := Val;
    end;
{Check that this breakpoint is in the list}
    if not RetrieveBP(BreakAddr,[bpFixed..bpPass],BPPtr)
    then begin
      Comment := clcBPNotFound;
      exit;
    end;
{If OK add address to transfer array}
    if ATAddrExists(BreakAddr,ATArray)
    then begin
      Comment := clcDuplicateBP;
      exit;
    end;
    ATArray[i-1].Addr := BreakAddr;
  end; // for i := 1 to CList.Count - 1 do

  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  Command   := cmdRPC;
end; // function  RPCCheck

procedure RPC;
var
  i : integer;
begin
  if length(ATArray) = 0            // delete all breakpoints
  then begin
    ResetAllPassCounts;
  end
  else begin
    for i := 0 to length(ATArray) - 1 do
    begin
      ResetPassCount(ATArray[i].Addr);
    end;
  end;
  UpdateBPWindow;
end; // procedure RPC


end.

