unit CFGUnit;

interface

uses Forms, Classes, SysUtils, Registry, TypeDefs, CPort, Dialogs;

procedure GetCFGSectionNames(const CFGList,NameList : tStringList);

function  GetCFGSectionLimits(const CFGList : tStringList;
                                    SectionName : string) : boolean;

function  CFGValueExists(const CFGList   : tStringList;
                               ValueName : string;
                         var   Index     : integer) : boolean;
function  GetCFGValue(const CFGList   : tStringList;
                            ValueName : string;
                        var Index     : integer) : string;

function  ReadCFGBln(const CFGList   : tStringList;
                           ValueName : string;
                     var   Value     : boolean) : boolean;
function  ReadCFGStr(const CFGList   : tStringList;
                           ValueName : string;
                     var   Value     : string)  : boolean;
function ReadCFGTfn(const CFGList   : tStringList; // Tfn = Tokenized File Name
                          ValueName : string;
                    var   Value     : string) : boolean;
function  ReadCFGInt(const CFGList   : tStringList;
                           ValueName : string;
                     var   Value     : integer) : boolean;
procedure WriteCFGBln(const CFGList   : tStringList;
                            ValueName : string;
                            Value     : boolean);
procedure WriteCFGStr(const CFGList   : tStringList;
                            ValueName : string;
                            Value     : string);
procedure WriteCFGTfn(const CFGList   : tStringList;
                            ValueName : string;
                            Value     : string);
procedure WriteCFGInt(const CFGList   : tStringList;
                            ValueName : string;
                            Value     : integer);

procedure WriteProjectFile(var Data            : tProjDataRecord;
                               ProjectFileName : string);
procedure WriteProjectCFGList(var   Data : tProjDataRecord;
                              const CFGList : tStringList);
procedure ReadProjectFile(    ProjectFileName : string;
                          var NeedToSaveOnExit : boolean);
procedure ConvertCFGListForErrorReport(CFGList : tStringList);
procedure ReadMCUDataFile(const MCUData : tStringList;
                                MCU     : string;
                          var   Data    : tProjDataRecord);

implementation


uses
  Main, MiscUnit, FileUnit;

var
  LoLimit,HiLimit : integer;

procedure GetCFGSectionNames(const CFGList,NameList : tStringList);
var
  i,j,k : integer;
begin
  NameList.Clear;
  for i := 0 to CFGList.Count - 1 do
  begin
    j := pos('[',CFGList[i]);
    k := pos(']',CFGList[i]);
    if (j = 1) and (k > 0) then NameList.Add(copy(CFGList[i],j+1,k-j-1));
  end;
end; // procedure GetCFGSectionNames

function  GetCFGSectionLimits(const CFGList     : tStringList;
                                    SectionName : string) : boolean;
var
  i : integer;
begin
  Result  := false;
  LoLimit := -1;
  HiLimit := -1;
  i := CFGList.IndexOf('[' + SectionName + ']');
  if i = -1 then exit;
  i := i + 1;
  if i < CFGList.Count then LoLimit := i else exit;
  while (i < CFGList.Count) and (copy(CFGList[i],1,1) <> '[') do inc(i);
  if i = LoLimit
  then begin LoLimit := -1; exit; end
  else HiLimit := i - 1;
  Result := true;
end; // function  GetCFGSectionLimits

procedure SetCFGSectionName(const CFGList     : tStringList;
                                  SectionName : string);
begin
  CFGList.Add('');
  CFGList.Add('[' + SectionName + ']');
end; // procedure SetCFGSectionName

function  CFGValueExists(const CFGList   : tStringList;
                               ValueName : string;
                         var   Index     : integer) : boolean;
var
  i : integer;
begin
  Result := false;
  Index  := -1;
  if (HiLimit < LoLimit) or (LoLimit < 0) or
     (HiLimit > CFGList.Count - 1) or (HiLimit < 0)
  then exit;
  for i := LoLimit to HiLimit do
  begin
    if uppercase(copy(CFGList[i],1,length(ValueName))) = uppercase(ValueName)
    then begin
      if pos('=',CFGList[i]) = 0 then exit;
      Result := true;
      Index  := i;
      break;
    end;
  end;
end; // function  CFGValueExists

function  GetCFGValue(const CFGList   : tStringList;
                            ValueName : string;
                        var Index     : integer) : string;
var
  i,EqualPos : integer;
begin
  Result := '';
  Index  := -1;
  if (HiLimit < LoLimit) or (LoLimit < 0) or
     (HiLimit > CFGList.Count - 1) or (HiLimit < 0)
  then exit;
  for i := LoLimit to HiLimit do
  begin
    if uppercase(copy(CFGList[i],1,length(ValueName))) = uppercase(ValueName)
    then begin
      EqualPos := pos('=',CFGList[i]);
      if EqualPos = 0 then exit;
      Result := copy(CFGList[i],EqualPos+1,length(CFGList[i])-EqualPos);
      DelLeadSpaceTab(Result);
      Index  := i;
      break;
    end;
  end;
end; // function GetCFGValue

function  ReadCFGBln(const CFGList   : tStringList;
                           ValueName : string;
                     var   Value     : boolean) : boolean;
var
  Buf : string;
  i   : integer;
begin
  Buf    := GetCFGValue(CFGList,ValueName,i);
  Result := (length(Buf) > 0) and (i > -1);
  if Result
  then begin
    Buf   := uppercase(Buf);
    Value := (Buf = 'TRUE') or (Buf = 'YES');
  end;
end; // function  ReadCFGBln

function  ReadCFGStr(const CFGList   : tStringList;
                           ValueName : string;
                     var   Value     : string) : boolean;
var
  Buf : string;
  i   : integer;
begin
  Buf    := GetCFGValue(CFGList,ValueName,i);
  Result := i > -1;
  if Result
  then begin
    if copy(Buf,1,1) = string(chr(149)) then Buf[1] := ' ';
    Value := Buf;
  end;  
end; // function  ReadCFGList

{Reads a string datum from a .cfg file and returns the de-tokenized form;
 suitable for use with file name strings}
function  ReadCFGTfn(const CFGList   : tStringList;
                           ValueName : string;
                     var   Value     : string) : boolean;
var
  Buf : string;
  i   : integer;
begin
  Buf    := DeTokenizeFileName(GetCFGValue(CFGList,ValueName,i));
  Result := (i > -1);
  if Result then Value := Buf;
end; // function ReadCFGTfn

function  ReadCFGInt(const CFGList   : tStringList;
                           ValueName : string;
                     var   Value     : integer) : boolean;
var
  Buf : string;
  i,j : integer;
begin
  Buf    := GetCFGValue(CFGList,ValueName,i);
  Result := (length(Buf) > 0) and (i > -1);
  if Result
  then begin
    try
      j     := StrToInt(Buf);
      Value := j
    except on EConvertError do Result := False;
    end;
  end;
end; // function  ReadCFGInt

procedure WriteCFGBln(const CFGList   : tStringList;
                            ValueName : string;
                            Value     : boolean);
var
  Buf : string;
begin
  Buf := ValueName + ' = ';
  case Value of
    true  : Buf := Buf + 'TRUE';
    false : Buf := Buf + 'FALSE';
  end;
  CFGList.Add(Buf);
end; // procedure WriteCFGBool

procedure WriteCFGStr(const CFGList   : tStringList;
                            ValueName : string;
                            Value     : string);
var
  Buf : string;
begin
  if copy(Value,1,1) = ' ' then Value[1] := chr(149); // fixes auto deletion of leading spaces on read
  Buf := ValueName + ' = ' + Value;
  CFGList.Add(Buf);
end; // procedure WriteCFGStr

procedure WriteCFGTfn(const CFGList   : tStringList;
                            ValueName : string;
                            Value     : string);
begin
  CFGList.Add(ValueName + '=' + TokenizeFileName(Value));
end; // procedure WriteCFGTfn

procedure WriteCFGInt(const CFGList   : tStringList;
                            ValueName : string;
                            Value     : integer);
var
  Buf : string;
begin
  Buf := ValueName + ' = ' + IntToStr(Value);
  CFGList.Add(Buf);
end; // procedure WriteCFGInt

procedure WriteCFGHex(const CFGList   : tStringList;
                            ValueName : string;
                            Value     : integer);
var
  Buf : string;
begin
  Buf := ValueName + ' = $' + IntToHex(Value,8);
  CFGList.Add(Buf);
end; // procedure WriteCFGInteger

procedure ReadMCUDataFile(const MCUData : tStringList;
                                MCU     : string;
                          var   Data    : tProjDataRecord);
var
  Buf : string;
begin
  MCU := uppercase(MCU);
  with Data do
  begin
    GetCFGSectionLimits(MCUData,MCU);
    ReadCFGTfn(MCUData,VN_FN_RegInfo            ,FN_CtrlRegs);
    ReadCFGTfn(MCUData,VN_FN_OpcodeInfo         ,FN_Opcodes);
    ReadCFGTfn(MCUData,VN_FN_RamTalker          ,FN_RamTalker);
    ReadCFGTfn(MCUData,VN_FN_RamMap             ,FN_RamMap);
    ReadCFGTfn(MCUData,VN_FN_EEOverlay          ,FN_EEOverlay);
    ReadCFGTfn(MCUData,VN_FN_EOverlay           ,FN_EOverlay);
    ReadCFGStr(MCUData,VN_RamMem                ,RamMem);
    ReadCFGStr(MCUData,VN_CtrlRegsMem           ,CtrlRegsMem);
    ReadCFGStr(MCUData,VN_MaskRomMem            ,MaskRomMem);
    ReadCFGStr(MCUData,VN_EepromMem             ,EepromMem);
    if ReadCFGStr(MCUData,VN_EpromMem    ,Buf)
    then EpromMem := Bar2Return(Buf);
    ReadCFGBln(MCUData,VN_MaskRomVisible        ,MaskRomVisible);
    ReadCFGBln(MCUData,VN_EepromVisible         ,EepromVisible);
    ReadCFGBln(MCUData,VN_EpromVisible          ,EpromVisible);
  end; // with Data do
end; // procedure ReadMCUDataFile

procedure ReadProjectFile(    ProjectFileName : string;
                          var NeedToSaveOnExit : boolean);
var
  CFGList : tStringList;
  i       : integer;
  Buf     : string;
  Flag    : boolean;
begin
  CFGList  := tStringList.Create;
  ProjData := DefaultProjData;
  Buf      := '';
  NeedToSaveOnExit := false;
  try
    CFGList.LoadFromFile(Prog.FN_Project);
    with ProjData do
    begin
{Notes on 'Notes' Tab}
      Notes := '';
      for i := 1 to CFGList.Count - 1 do          // always skip first line
      begin
        Buf := CFGList[i];
        if copy(Buf,1,2) = '* '
        then begin
          delete(Buf,1,2);
          Notes := Notes + Buf + CRLF;
        end;
        if copy(Buf,1,1) = '[' then break;
      end;
{General}
      GetCFGSectionLimits(CFGList,'GENERAL');
      ReadCFGStr(CFGList,VN_MCUType               ,MCUType);
      ReadCFGInt(CFGList,VN_BytesToList           ,BytesToList);
      ReadCFGInt(CFGList,VN_LinesToUnassemble     ,LinesToUnassemble);
      ReadCFGBln(CFGList,VN_AutoConnectAtStartUp  ,AutoConnectAtStartUp);
      ReadCFGBln(CFGList,VN_AutoResetAfterConnect ,AutoResetAfterConnect);
      ReadCFGBln(CFGList,VN_WarnOnBprot           ,WarnOnBprot);
      ReadCFGBln(CFGList,VN_WarnOnVprog           ,WarnOnVprog);
      ReadCFGBln(CFGList,VN_NoBreakHandshake      ,NoBreakHandshake);
      ReadCFGBln(CFGList,VN_IgnoreEchoErrors      ,IgnoreEchoErrors);
      ReadCFGTfn(CFGList,VN_FN_RegInfo            ,FN_CtrlRegs);
      ReadCFGTfn(CFGList,VN_FN_OpcodeInfo         ,FN_Opcodes);
{COM Port}
      GetCFGSectionLimits(CFGList,'COM PORT');
      ReadCFGStr(CFGList,VN_PortToUse             ,PortToUse);
      ReadCFGStr(CFGList,VN_CrystalFrequency      ,CrystalFrequency);
      ReadCFGInt(CFGList,VN_UploadBaudRate        ,UploadBaudRate);
      ReadCFGInt(CFGList,VN_CommsBaudRate         ,CommsBaudRate);
      ReadCFGBln(CFGList,VN_RemoteResetAvailable  ,RemoteResetAvailable);
      ReadCFGBln(CFGList,VN_ResetUsesDTR          ,ResetUsesDTR);
      ReadCFGBln(CFGList,VN_ResetPulseGoesPos     ,ResetPulseGoesPos);
      ReadCFGBln(CFGList,VN_RemoteSwitchAvailable ,RemoteSwitchAvailable);
      ReadCFGBln(CFGList,VN_IsSelectedWhenPos     ,IsSelectedWhenPos);
{Macros}
      GetCFGSectionLimits(CFGList,'MACRO & SYMBOL');

      if ReadCFGBln(CFGList,VN_RunBootScript,Flag)
      then begin
        case Flag of
          False : RunBootScript := rbsNever;
          True  : RunBootScript := rbsBoot;
        end;
        NeedToSaveOnExit := true;
      end
      else begin
        if ReadCFGInt(CFGList,VN_RunScript,i)
        then RunBootScript := tRunBootScript(i)
        else RunBootScript := rbsNever;
      end;

      if ReadCFGStr(CFGList,VN_BootScript         ,Buf)
      then BootScript := Bar2Return(Buf);
      ReadCFGBln(CFGList,VN_HasAssociatedMacro    ,HasAssociatedMacro);
      ReadCFGBln(CFGList,VN_NoAutoStart           ,NoAutoStart);
      ReadCFGBln(CFGList,VN_AbortOnFormatError    ,AbortOnFormatError);
      ReadCFGBln(CFGList,VN_AbortOnCmdFail        ,AbortOnCmdFail);
{Debug}
      GetCFGSectionLimits(CFGList,'DEBUG');
      ReadCFGBln(CFGList,VN_AddBreakSpaces        ,AddBreakSpaces);
      ReadCFGBln(CFGList,VN_NoVectorSubstitution  ,NoVectorSubstitution);
      ReadCFGBln(CFGList,VN_NotInStepOver         ,NotInStepOver);
      ReadCFGBln(CFGList,VN_ShowStartInst         ,ShowStartInst);
      ReadCFGBln(CFGList,VN_ShowStartRegs         ,ShowStartRegs);
      ReadCFGBln(CFGList,VN_ShowOnceOnly          ,ShowOnceOnly);
      ReadCFGBln(CFGList,VN_ShowBreakInst         ,ShowBreakInst);
      ReadCFGBln(CFGList,VN_ShowBreakRegs         ,ShowBreakRegs);
      if ReadCFGInt(CFGList,VN_SymbolTableToUse   ,i)
      then SymbolTableToUse := tSymbolTableToUse(i);
      ReadCFGTfn(CFGList,VN_FN_SymbolFixed        ,FN_SymbolFixed);
{Talkers}
      GetCFGSectionLimits(CFGList,'TALKERS');
      if ReadCFGInt(CFGList,VN_TalkerType         ,i)
      then TalkerType := tTalkerType(i);
      ReadCFGBln(CFGList,VN_WaitForEcho           ,WaitForEcho);
      ReadCFGTfn(CFGList,VN_FN_RamTalker          ,FN_RamTalker);
      ReadCFGTfn(CFGList,VN_FN_RamMap             ,FN_RamMap);
      ReadCFGTfn(CFGList,VN_FN_EEOverlay          ,FN_EEOverlay);
      ReadCFGTfn(CFGList,VN_FN_EOverlay           ,FN_EOverlay);
      ReadCFGTfn(CFGList,VN_FN_ByteWriteOverlay   ,FN_ByteWriteOverlay);
      ReadCFGTfn(CFGList,VN_FN_PageWriteOverlay   ,FN_PageWriteOverlay);
      ReadCFGTfn(CFGList,VN_FN_EEMap              ,FN_EEMap);
      ReadCFGTfn(CFGList,VN_FN_ExtMap             ,FN_ExtMap);
      if ReadCFGInt(CFGList,VN_ExtTalkerRequires  ,i)
      then ExtTalkerRequires := tTalkerType(i);
{Overlays}
      GetCFGSectionLimits(CFGList,'OVERLAYS');
      ReadCFGTfn(CFGList,VN_FN_EEOverlay          ,FN_EEOverlay);
      ReadCFGTfn(CFGList,VN_FN_EOverlay           ,FN_EOverlay);
      ReadCFGTfn(CFGList,VN_FN_ByteWriteOverlay   ,FN_ByteWriteOverlay);
      ReadCFGTfn(CFGList,VN_FN_PageWriteOverlay   ,FN_PageWriteOverlay);
{Memory Map}
      GetCFGSectionLimits(CFGList,'MEMORY MAP');
      ReadCFGStr(CFGList,VN_RamMem                ,RamMem);
      ReadCFGStr(CFGList,VN_CtrlRegsMem           ,CtrlRegsMem);
      ReadCFGStr(CFGList,VN_MaskRomMem            ,MaskRomMem);
      ReadCFGStr(CFGList,VN_EepromMem             ,EepromMem);
      if ReadCFGStr(CFGList,VN_EpromMem,Buf)     then EpromMem := Bar2Return(Buf);
      if ReadCFGStr(CFGList,VN_ExtRamMem,Buf)    then ExtRamMem := Bar2Return(Buf);
      if ReadCFGStr(CFGList,VN_ByteWriteMem,Buf) then ByteWriteMem := Bar2Return(Buf);
      if ReadCFGStr(CFGList,VN_PageWriteMem,Buf) then PageWriteMem := Bar2Return(Buf);
      ReadCFGBln(CFGList,VN_MaskRomVisible        ,MaskRomVisible);
      ReadCFGBln(CFGList,VN_EepromVisible         ,EepromVisible);
      ReadCFGBln(CFGList,VN_EpromVisible          ,EpromVisible);
      ReadCFGBln(CFGList,VN_ExtRamVisible         ,ExtRamVisible);
      ReadCFGBln(CFGList,VN_ByteWriteVisible      ,ByteWriteVisible);
      ReadCFGBln(CFGList,VN_PageWriteVisible      ,PageWriteVisible);
      ReadCFGInt(CFGList,VN_DataPageSize          ,DataPageSize);
      ReadCFGBln(CFGList,VN_WriteWholePage        ,WriteWholePage);

{Indirect Memory Tab}
      GetCFGSectionLimits(CFGList,'INDIRECT MEMORY');
      ReadCFGTfn(CFGList,VN_FN_IndRegs,FN_IndRegs);
      if ReadCFGStr(CFGList,VN_IndMemory ,Buf) then IndMemory := Bar2Return(Buf);
      ReadCFGBln(CFGList,VN_IMVisible,IMVisible);
      ReadCFGInt(CFGList,VN_IMPageSize,IMPageSize);
      ReadCFGTfn(CFGList,VN_FN_IndMemOvly,FN_IndMemOvly);
      ReadCFGTfn(CFGList,VN_FN_IndRegsOvly,FN_IndRegsOvly);

{File Names}
      GetCFGSectionLimits(CFGList,'FILES');
      ReadCFGTfn(CFGList,VN_FN_LoadS19            ,FN_LoadS19);
      ReadCFGTfn(CFGList,VN_FN_LoadBinary         ,FN_LoadBinary);
      if ReadCFGTfn(CFGList,VN_FN_Macro           ,FN_Macro)
      then FN_Macro := FN_Macro else FN_Macro := '';

{Watch Window}
      GetCFGSectionLimits(CFGList,'WATCH WINDOW');
      if ReadCFGStr(CFGList,VN_WatchStr           ,Buf)
      then MainForm.WatchListBox.Items.Text := Bar2Return(Buf);

      // Temporary repeat of terminal variable read because of changed format
      ReadCFGInt(CFGList,VN_InterCharDelay        ,InterCharDelay);
      ReadCFGBln(CFGList,VN_ShowNonDispChars      ,ShowNonDispChars);
      ReadCFGBln(CFGList,VN_LocalEchoOn           ,LocalEchoOn);
      ReadCFGBln(CFGList,VN_SendOnEnter           ,SendOnEnter);
      ReadCFGStr(CFGList,VN_OnEnterSend           ,OnEnterSend);
      ReadCFGStr(CFGList,VN_NewlineOnReceive      ,NewlineOnReceive);
      ReadCFGStr(CFGList,VN_CloseOnReceive        ,CloseOnReceive);
      ReadCFGBln(CFGList,VN_LoggingActive         ,LoggingActive);
      ReadCFGTfn(CFGList,VN_FN_Log                ,FN_Log);
      ReadCFGTfn(CFGList,VN_FN_ImageFile          ,FN_ImageFile);

{Terminal variables}
      GetCFGSectionLimits(CFGList,'TERMINAL');
      ReadCFGInt(CFGList,VN_InterCharDelay        ,InterCharDelay);
      ReadCFGBln(CFGList,VN_ShowNonDispChars      ,ShowNonDispChars);
      ReadCFGBln(CFGList,VN_LocalEchoOn           ,LocalEchoOn);
      ReadCFGBln(CFGList,VN_SendOnEnter           ,SendOnEnter);
      ReadCFGStr(CFGList,VN_OnEnterSend           ,OnEnterSend);
      ReadCFGStr(CFGList,VN_NewlineOnReceive      ,NewlineOnReceive);
      ReadCFGStr(CFGList,VN_CloseOnReceive        ,CloseOnReceive);
      ReadCFGBln(CFGList,VN_LoggingActive         ,LoggingActive);
      ReadCFGTfn(CFGList,VN_FN_Log                ,FN_Log);
      ReadCFGTfn(CFGList,VN_FN_ImageFile          ,FN_ImageFile);
    end; // with ProjData do
  finally
    CFGList.Free;
  end; // try.. finally..
end; // procedure ReadProjectFile

procedure WriteProjectCFGList(var   Data : tProjDataRecord;
                              const CFGList : tStringList);
var
  TempList : tStringList;
  i        : integer;
  Buf      : string;
begin
  with Data do
  begin
{General}
    SetCFGSectionName(CFGList,'GENERAL');
    WriteCFGStr(CFGList,VN_MCUType               ,MCUType);
    WriteCFGInt(CFGList,VN_BytesToList           ,BytesToList);
    WriteCFGInt(CFGList,VN_LinesToUnassemble     ,LinesToUnassemble);
    WriteCFGBln(CFGList,VN_AutoConnectAtStartUp  ,AutoConnectAtStartUp);
    WriteCFGBln(CFGList,VN_AutoResetAfterConnect ,AutoResetAfterConnect);
    WriteCFGBln(CFGList,VN_WarnOnBprot           ,WarnOnBprot);
    WriteCFGBln(CFGList,VN_WarnOnVprog           ,WarnOnVprog);
    WriteCFGBln(CFGList,VN_NoBreakHandshake      ,NoBreakHandshake);
    WriteCFGBln(CFGList,VN_IgnoreEchoErrors      ,IgnoreEchoErrors);
    WriteCFGTfn(CFGList,VN_FN_RegInfo            ,FN_CtrlRegs);
    WriteCFGTfn(CFGList,VN_FN_OpcodeInfo         ,FN_Opcodes);
{COM Port}
    SetCFGSectionName(CFGList,'COM PORT');
    WriteCFGStr(CFGList,VN_PortToUse             ,PortToUse);
    WriteCFGStr(CFGList,VN_CrystalFrequency      ,CrystalFrequency);
    WriteCFGInt(CFGList,VN_UploadBaudRate        ,UploadBaudRate);
    WriteCFGInt(CFGList,VN_CommsBaudRate         ,CommsBaudRate);
    WriteCFGBln(CFGList,VN_RemoteResetAvailable  ,RemoteResetAvailable);
    WriteCFGBln(CFGList,VN_ResetUsesDTR          ,ResetUsesDTR);
    WriteCFGBln(CFGList,VN_ResetPulseGoesPos     ,ResetPulseGoesPos);
    WriteCFGBln(CFGList,VN_RemoteSwitchAvailable ,RemoteSwitchAvailable);
    WriteCFGBln(CFGList,VN_IsSelectedWhenPos     ,IsSelectedWhenPos);
{Macros}
    SetCFGSectionName(CFGList,'MACRO & SYMBOL');
    WriteCFGInt(CFGList,VN_RunScript             ,ord(RunBootScript));
    WriteCFGStr(CFGList,VN_BootScript            ,Return2Bar(BootScript));
    WriteCFGBln(CFGList,VN_HasAssociatedMacro    ,HasAssociatedMacro);
    WriteCFGBln(CFGList,VN_NoAutoStart           ,NoAutoStart);
    WriteCFGBln(CFGList,VN_AbortOnFormatError    ,AbortOnFormatError);
    WriteCFGBln(CFGList,VN_AbortOnCmdFail        ,AbortOnCmdFail);
{Debug}
    SetCFGSectionName(CFGList,'DEBUG');
    WriteCFGBln(CFGList,VN_AddBreakSpaces        ,AddBreakSpaces);
    WriteCFGBln(CFGList,VN_NoVectorSubstitution  ,NoVectorSubstitution);
    WriteCFGBln(CFGList,VN_NotInStepOver         ,NotInStepOver);
    WriteCFGBln(CFGList,VN_ShowStartInst         ,ShowStartInst);
    WriteCFGBln(CFGList,VN_ShowStartRegs         ,ShowStartRegs);
    WriteCFGBln(CFGList,VN_ShowOnceOnly          ,ShowOnceOnly);
    WriteCFGBln(CFGList,VN_ShowBreakInst         ,ShowBreakInst);
    WriteCFGBln(CFGList,VN_ShowBreakRegs         ,ShowBreakRegs);
    WriteCFGInt(CFGList,VN_SymbolTableToUse      ,ord(SymbolTableToUse));
    WriteCFGTfn(CFGList,VN_FN_SymbolFixed        ,FN_SymbolFixed);
{Talkers}
    SetCFGSectionName(CFGList,'TALKERS');
    WriteCFGInt(CFGList,VN_TalkerType            ,ord(TalkerType));
    WriteCFGBln(CFGList,VN_WaitForEcho           ,WaitForEcho);
    WriteCFGTfn(CFGList,VN_FN_RamTalker          ,FN_RamTalker);
    WriteCFGTfn(CFGList,VN_FN_RamMap             ,FN_RamMap);
    WriteCFGTfn(CFGList,VN_FN_EEMap              ,FN_EEMap);
    WriteCFGTfn(CFGList,VN_FN_ExtMap             ,FN_ExtMap);
    WriteCFGInt(CFGList,VN_ExtTalkerRequires     ,ord(ExtTalkerRequires));
{Overlays}
    SetCFGSectionName(CFGList,'OVERLAYS');
    WriteCFGTfn(CFGList,VN_FN_EEOverlay          ,FN_EEOverlay);
    WriteCFGTfn(CFGList,VN_FN_EOverlay           ,FN_EOverlay);
    WriteCFGTfn(CFGList,VN_FN_ByteWriteOverlay   ,FN_ByteWriteOverlay);
    WriteCFGTfn(CFGList,VN_FN_PageWriteOverlay   ,FN_PageWriteOverlay);
{Memory Map}
    SetCFGSectionName(CFGList,'MEMORY MAP');
    WriteCFGStr(CFGList,VN_RamMem                ,RamMem);
    WriteCFGStr(CFGList,VN_CtrlRegsMem           ,CtrlRegsMem);
    WriteCFGStr(CFGList,VN_MaskRomMem            ,MaskRomMem);
    WriteCFGStr(CFGList,VN_EepromMem             ,EepromMem);
    WriteCFGStr(CFGList,VN_EpromMem              ,Return2Bar(EpromMem));
    WriteCFGStr(CFGList,VN_ExtRamMem             ,Return2Bar(ExtRamMem));
    WriteCFGStr(CFGList,VN_ByteWriteMem          ,Return2Bar(ByteWriteMem));
    WriteCFGStr(CFGList,VN_PageWriteMem          ,Return2Bar(PageWriteMem));
    WriteCFGBln(CFGList,VN_MaskRomVisible        ,MaskRomVisible);
    WriteCFGBln(CFGList,VN_EepromVisible         ,EepromVisible);
    WriteCFGBln(CFGList,VN_EpromVisible          ,EpromVisible);
    WriteCFGBln(CFGList,VN_ExtRamVisible         ,ExtRamVisible);
    WriteCFGBln(CFGList,VN_ByteWriteVisible      ,ByteWriteVisible);
    WriteCFGBln(CFGList,VN_PageWriteVisible      ,PageWriteVisible);
    WriteCFGInt(CFGList,VN_DataPageSize          ,DataPageSize);
    WriteCFGBln(CFGList,VN_WriteWholePage        ,WriteWholePage);
{Indirect Memory Tab}
    SetCFGSectionName(CFGList,'INDIRECT MEMORY');
    WriteCFGTfn(CFGList,VN_FN_IndRegs,FN_IndRegs);
    WriteCFGStr(CFGList,VN_IndMemory ,Return2Bar(IndMemory));
    WriteCFGBln(CFGList,VN_IMVisible,IMVisible);
    WriteCFGInt(CFGList,VN_IMPageSize,IMPageSize);
    WriteCFGTfn(CFGList,VN_FN_IndMemOvly,FN_IndMemOvly);
    WriteCFGTfn(CFGList,VN_FN_IndRegsOvly,FN_IndRegsOvly);
{File Names}
    SetCFGSectionName(CFGList,'FILES');
    WriteCFGTfn(CFGList,VN_FN_LoadS19            ,FN_LoadS19);
    WriteCFGTfn(CFGList,VN_FN_LoadBinary         ,FN_LoadBinary);
    WriteCFGTfn(CFGList,VN_FN_Macro              ,FN_Macro);
{Watch Window}
    SetCFGSectionName(CFGList,'WATCH WINDOW');
    TempList := tStringList.Create;
    try
      TempList.Assign(MainForm.WatchListBox.Items);
      for i := 0 to TempList.Count - 1 do
      begin
        Buf := TempList[i];
        if (pos(',S',Buf) > 0) or (pos(',X',Buf) > 0)
        then begin delete(Buf,1,4); insert('    ',Buf,1); end;
        TempList[i] := Buf;
      end;
      WriteCFGStr(CFGList,VN_WatchStr,Return2Bar(TempList.Text));
    finally
      TempList.Free;
    end;
{Terminal variables}
    SetCFGSectionName(CFGList,'TERMINAL');
    WriteCFGInt(CFGList,VN_InterCharDelay    ,InterCharDelay);
    WriteCFGBln(CFGList,VN_ShowNonDispChars  ,ShowNonDispChars);
    WriteCFGBln(CFGList,VN_LocalEchoOn       ,LocalEchoOn);
    WriteCFGBln(CFGList,VN_SendOnEnter       ,SendOnEnter);
    WriteCFGStr(CFGList,VN_OnEnterSend       ,OnEnterSend);
    WriteCFGStr(CFGList,VN_NewlineOnReceive  ,NewlineOnReceive);
    WriteCFGStr(CFGList,VN_CloseOnReceive    ,CloseOnReceive);
    WriteCFGBln(CFGList,VN_LoggingActive     ,LoggingActive);
    WriteCFGTfn(CFGList,VN_FN_Log            ,FN_Log);
    WriteCFGTfn(CFGList,VN_FN_ImageFile      ,FN_ImageFile);
  end; // with ProjData do
end; // procedure WriteProjectCFGList

procedure WriteProjectFile(var Data            : tProjDataRecord;
                               ProjectFileName : string);
var
  CFGList,NotesList  : tStringList;
  i,Attrs            : integer;
begin
  CFGList   := tStringList.Create;
  NotesList := tStringList.Create;
  try
    CFGList.Add('* JBug11 Project File. Do NOT edit manually');
    CFGList.Add('');
{Add the 'Notes'}
    NotesList.Text := Data.Notes;
    for i := 0 to NotesList.Count - 1 do
    begin
      CFGList.Add('* ' + NotesList[i]);
    end;
{Add the rest of the project data}
    WriteProjectCFGList(Data,CFGList);
    if FileExists(ProjectFileName)
    then begin
      Attrs := FileGetAttr(ProjectFileName);
      if (Attrs and faReadOnly) <> 0
      then begin
        MessageDlg('Project file is read-only' + CRLF +
                   'Save under a different name',
                   mtWarning,[mbOK],0);
        exit;
      end;
    end;
    CFGList.SaveToFile(ProjectFileName);
  finally
    CFGList.Free;
    NotesList.Free;
  end; // try.. finally..
end; // procedure WriteProjectFile

procedure ConvertCFGListForErrorReport(CFGList : tStringList);
var
  i,EqualPos             : integer;
  Buf,NamePart,ValuePart : string;
begin
  for i := 0 to CFGList.Count - 1 do
  begin
    Buf := CFGList[i];
    EqualPos := pos('=',Buf);
    if EqualPos = 0 then continue;
    NamePart  := copy(Buf,1,EqualPos-1);
    ValuePart := copy(Buf,EqualPos+1,length(Buf)-EqualPos);
    SpaceTo(NamePart,37);
    Buf := NamePart + ValuePart;
    CFGList[i] := Buf;
  end;
end; // procedure ConvertCFGListForErrorReport

end.



