unit CRCUnit;

interface

uses Classes, SysUtils, TypeDefs, Dialogs;

function  CRCCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
procedure CRCMCUbyFile;
procedure CRCFileLocal;
procedure CRCMemory;
procedure CRCMemoryLocal;

implementation

uses Main, CmdUnit, Comms1, CheckUnit, MemRW, LoadUnit, FileUnit, MemUnit;

var
  StartAddr,EndAddr : word;
  FileName          : string;

function  CRCCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
var
  Ext        : string;
  IsLocal    : boolean;
  ArgType    : tArgType;
  Val,A1,A2  : longword;
begin
  Result  := false;
  Comment := clcArgErr;
  IsLocal := uppercase(CList[0]) = 'CRCL';

  if not NumArgsOK(CList,1,2,Comment) then exit;

  if not IsLocal and not StateIsAllowed([psStopped],Comment) then exit;

  if CList.Count = 2
  then begin
    FileName := CList[1];
    if FileName[1] = chr(149) then delete(FileName,1,1);
    FileName := DetokenizeFileName(FileName);
    if not FileExists(FileName) then begin Comment := clcFileNotFound; exit; end;
    Ext := uppercase(ExtractFileExt(FileName));
    if (Ext <> '.S19') and (Ext <> '.REC')
    then begin Comment := clcUnrecognizedFileType; exit; end;

    if not NumArgsOK(CList,1,1,Comment) then exit;

    if not OpenS19FileOK(FileName,S19Listing,SupErrMsg)
    then begin
//      MessageDlg(ErrMsg,mtError,[mbOK],0);
      Comment      := clcBadS19Format;
      SupErrMsgSet := true;
      exit;
    end;

    StartAddr := 0; EndAddr := 0;
    Result    := true;
    Comment   := clcOK;
    if IsLocal then Command := cmdCRCFileLocal else Command := cmdCRCMCUbyFile;
  end
  else begin
    A2 := 0;
    if FoundArgOK(CList,1,[atWord,atLabel],ArgType,Val)
    then A1 := Val else exit;

    if FoundArgOK(CList,2,[atWord,atLabel,atLength],
                  ArgType,Val)
    then begin
      case ArgType of
        atWord,atLabel : A2 := Val;
        atLength       : A2 := A1 + Val;
      end;
    end else exit;

{Check that: 1. A2 is larger than A1
             2. A2 <= $FFFF
             3. (No address range exclusions apply to the list command) }
    if (A2 < A1) or (A2 > $FFFF)
    then begin
      Comment := clcAddrRangeErr;     // Address range error
      exit;
    end;

    StartAddr := A1;
    EndAddr   := A2;
    Result    := true;                // Arguments are OK
    Comment   := clcOK;
    if IsLocal then Command := cmdCRCLocal else Command := cmdCRC;
  end; // if CList.Count = 2 then.. else..
end; // function CRCCheck

{Adapted from:
 Delphi - crc-16  from: http://www.ibrtses.com/delphi/dcrc.html
 Copyright (99,2000) Ing.Bro R.Tschaggelar}

{crc over one byte}
procedure ByteCrc(data:byte;var crc:word);
VAR i:BYTE;
BEGIN
 FOR i:=0 TO 7 DO
  BEGIN
   IF ((data and $01)XOR(crc AND $0001)<>0) THEN
    BEGIN
     crc:=crc shr 1;
     crc:= crc XOR $A001;
    END
   ELSE crc:=crc shr 1;
   data:=data shr 1; // this line is not ELSE and is executed anyway.
  END;
END;

{notes

This crc is calculated as X^16+X^15+X^2+1 and can
protect 2^16 bits or 8192 bytes against single bit
errors. For longer datablocks, the crc-32 is to be
used.

This the slow implementation requiring little RAM.
There also exists a fast implementation requiring
more RAM to hold a table. One byte is processed in
a few assembly instructions; see:

http://www.ibrtses.com/delphi/tcrc.html  }

procedure CRCMCUbyFile;
var
  ReadMap : tList;
  i,j     : integer;
  CRC     : word;
begin
  ReadMap := tList.Create;
  CRC     := 0;
  try
    BuildS19MapOK(S19Listing,ReadMap,MemMap);
    ReadMCUMap(ReadMap,true);
    for i := 0 to ReadMap.Count - 1 do
    begin
      with tMemRecordPtr(ReadMap[i])^ do
      begin
        for j := SA to EA do ByteCRC(Memory[j],CRC);
      end;
    end;
  finally
    ClearMemMap(ReadMap);
    ReadMap.Free;
  end;
  MainForm.OWEdit.Lines.Add(
    'CRC-16 for MCU memory, according to file:' + CRLF +
    '       ' + TokenizeFileName(FileName) + CRLF +
    '       is $' + IntToHex(CRC,4) + ' (#' + format('%d',[CRC]) + ')');
end; // procedure CRCMCUbyFile

procedure CRCFileLocal;
var
  ReadMap : tList;
  i,j     : integer;
  CRC     : word;
begin
  ReadMap := tList.Create;
  CRC     := 0;
  try
    BuildS19MapOK(S19Listing,ReadMap,MemMap);
    LoadS19Local1(S19Listing,mtIntrinsic);
    for i := 0 to ReadMap.Count - 1 do
    begin
      with tMemRecordPtr(ReadMap[i])^ do
      begin
        for j := SA to EA do ByteCRC(Memory[j],CRC);
      end;
    end;
  finally
    ClearMemMap(ReadMap);
    ReadMap.Free;
  end;
  MainForm.OWEdit.Lines.Add(
    'CRC-16 for S19 file:' + CRLF +
    '       ' + TokenizeFileName(FileName) + CRLF +
    '       is $' + IntToHex(CRC,4) + ' (#' + format('%d',[CRC]) + ')');
end; // procedure CRCFileLocal

procedure CRCMemory;
var
  w,CRC : word;
begin
  CRC := 0;
  ReadMCU64k(StartAddr,EndAddr,true);
  for w := StartAddr to EndAddr do ByteCRC(Memory[w],CRC);
  MainForm.OWEdit.Lines.Add(
    'CRC-16 for MCU memory from ' +
    IntToHex(StartAddr,4) + ' to ' + IntToHex(EndAddr,4) + CRLF +
    '       is $' + IntToHex(CRC,4) + ' (#' + format('%d',[CRC]) + ')');
end; // procedure CRCMemory

procedure CRCMemoryLocal;
var
  w,CRC : word;
begin
  CRC := 0;
  for w := StartAddr to EndAddr do ByteCRC(Memory[w],CRC);
  MainForm.OWEdit.Lines.Add(
    'CRC-16 for local memory from ' +
    IntToHex(StartAddr,4) + ' to ' + IntToHex(EndAddr,4) + CRLF +
    '       is $' + IntToHex(CRC,4) + ' (#' + format('%d',[CRC]) + ')');
end; // procedure CRCMemoryLocal

end.

