unit CalcUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Mask, Menus, TypeDefs;

type
  TCalcForm = class(TForm)
    EquBtn: TBitBtn;
    CloseBtn: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    HexEdit: TEdit;
    DecEdit: TEdit;
    BinEdit: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    AsciiBtn: TButton;
    AutoCompleteChkBox: TCheckBox;
    HelpBtn: TBitBtn;
    KOTChkBox: TCheckBox;
    procedure EquBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure HexEditKeyPress(Sender: TObject; var Key: Char);
    procedure DecEditKeyPress(Sender: TObject; var Key: Char);
    procedure BinEditKeyPress(Sender: TObject; var Key: Char);
    procedure NumEditEnter(Sender: TObject);
    procedure AsciiBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure KOTChkBoxClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  CalcForm: TCalcForm;

type
  tSourceFigure = (Hex,Dec,Bin);
var
  SourceFigure  : tSourceFigure = Hex;

implementation

uses Main, AsciiUnit, MiscUnit;

{$R *.DFM}

{Variables associated with the auto-complete calculator function}
var
  Answer        : integer;
  ValidCalc     : boolean;

procedure TCalcForm.FormCreate(Sender: TObject);
begin
// HelpContext :=
end; // procedure TCalcForm.FormCreate

procedure TCalcForm.FormActivate(Sender: TObject);
begin
  case SourceFigure of
    Hex : HexEdit.SetFocus;
    Dec : DecEdit.SetFocus;
    Bin : BinEdit.SetFocus;
  end;
end; // procedure TCalcForm.FormActivate

procedure TCalcForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  AsciiForm.Close;
end; // procedure TCalcForm.FormClose

procedure TCalcForm.EquBtnClick(Sender: TObject);
var
  Buf  : string;
  Edit : tEdit;
begin
  ValidCalc := false;
  Edit      := nil;
  try // try.. finally.. clause to beep on exit with ValidClac = false
    case SourceFigure of
      Hex : try
              Answer    := StrToInt('$' + HexEdit.Text);
              ValidCalc := true;
              Edit      := HexEdit;
            Except on EConvertError do exit;
            end;
      Dec : try
              Answer := StrToInt(DecEdit.Text);
              if Answer > $FFFF then exit;
              ValidCalc := true;
              Edit      := DecEdit;
            Except on EConvertError do exit;
            end;
      Bin : try
              Buf := '%' + BinEdit.Text;
              Answer := BinStrToWord(Buf);
              ValidCalc := true;
              Edit      := BinEdit;
            Except on EConvertError do exit;
            end;
    end; // case

    if ValidCalc
    then begin
      case SourceFigure of
        Hex : begin
                DecEdit.Text := IntToStr(Answer);
                BinEdit.Text := WordToBinStr(Answer,16);
              end;
        Dec : begin
                HexEdit.Text := IntToHex(Answer,2);
                BinEdit.Text := WordToBinStr(Answer,16);
              end;
        Bin : begin
                HexEdit.Text := IntToHex(Answer,2);
                DecEdit.Text := IntToStr(Answer);
              end;
      end; // case SourceFigure of

      if AutoCompleteChkBox.Checked
      then begin
        with MainForm.CmdEdit do
        begin
          Text := Text + HexEdit.Text;
          SelStart := length(Text);
          SetFocus;
          exit;
        end;
      end; // if AutoCompleteChkBox.Checked then..
    end; // if ValidCalc then..
    
    Edit.SetFocus;
    Edit.SelectAll;
    if AsciiForm.Visible
    then begin
      with AsciiForm.AsciiGrid do
      begin
        Row := ((Answer and $FF) mod 16) + 1;
        Col := ((Answer and $FF) div 16) + 1;
      end;  
    end;
  finally
    if not ValidCalc then beep;
  end;
end; // procedure TCalcForm.EquBtnClick

//  AsciiForm.Close;
//  CalcForm.Close;

procedure TCalcForm.HexEditKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = ^H) or (Key = ^M) then exit; // always allow the backspace
                                         // and return keys
  if not (Key in HexChars) then Key := chr(0);
end; // procedure TCalcForm.HexEditKeyPress

procedure TCalcForm.DecEditKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = ^H) or (Key = ^M) then exit;
  if not (Key in DecChars) then Key := chr(0);
end; // procedure TCalcForm.DecEditKeyPress

procedure TCalcForm.BinEditKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = ^H) or (Key = ^M) then exit;
  if not (Key in BinChars) then Key := chr(0);
end; // procedure TCalcForm.BinEditKeyPress

procedure TCalcForm.NumEditEnter(Sender: TObject);
begin
  if Sender = HexEdit then SourceFigure := Hex;
  if Sender = DecEdit then SourceFigure := Dec;
  if Sender = BinEdit then SourceFigure := Bin;
end; // procedure TCalcForm.NumEditEnter

procedure TCalcForm.AsciiBtnClick(Sender: TObject);
begin
  AsciiForm.Show;
  with AsciiForm.AsciiGrid do
  begin
    Row := ((Answer and $FF) mod 16) + 1;
    Col := ((Answer and $FF) div 16) + 1;
  end;  
  HexEdit.SetFocus;
end; // procedure TCalcForm.AsciiBtnClick

procedure TCalcForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(IDH_BASE_CONVERTER);
end; // procedure TCalcForm.BitBtn1Click

procedure TCalcForm.KOTChkBoxClick(Sender: TObject);
begin
  if KOTChkBox.Checked then CalcForm.FormStyle := fsStayOnTop
                       else CalcForm.FormStyle := fsNormal;
end; // procedure TCalcForm.KOTChkBoxClick

procedure TCalcForm.FormShow(Sender: TObject);
begin
  KOTChkBox.Checked := FormStyle = fsStayOnTop;
end; // procedure TCalcForm.FormShow

end.


