unit CheckUnit;

interface

uses Classes, Forms, Windows, Buttons, TypeDefs, SysUtils, CPort, Dialogs,
     Graphics;

function  MakeComPortOK(    Data   : tProjDataRecord;
                        var ErrMsg : string) : boolean;
function  ProjectDataOK(var   Data                    : tProjDataRecord;
                        var   Page1,Page2,Page3,Page4 : tOpcodeData;
                        var   CtrlRegs                : tCtrlRegArray;
                        var   SerialMemRegs           : tCtrlRegArray;
                        const MemMap,IndMemMap        : tList;
                        var   TalkerArray             : tTalkArray;
                        var   TalkerLength            : integer;
                        var   MapData                 : tMapRecord;
                        const ErrMsgs                 : tStringList) : boolean;
function  StateIsAllowed(    AllowedStates : tSetOfProgStates;
                         var Comment       : tCmdLineComment) : boolean;
function NumArgsOK(const CList        : tStringList;
                         Min,Max      : integer;
                   var   Comment      : tCmdLineComment) : boolean;
function FoundArgOK(const CList       : tStringList;
                          Idx         : integer;
                          AllowedArgs : tSetOfArgType;
                    var   ArgType     : tArgType;
                    var   ArgValue    : longword) : boolean;
function  AreasVersusEepromTalkerOK(    AreasUsed : tMemAreaSet;
                                    var Comment   : tCmdLineComment) : boolean;
function  OverlaysAvailableOK(    AreasUsed : tMemAreaSet;
                              var Comment   : tCmdLineComment) : boolean;

implementation

uses Main, FileUnit, MemUnit, CFGUnit, MiscUnit, LookupUnit, VIUnit;

{Function returns TRUE for a valid combination of specified and available ports}

function  MakeComPortOK(    Data   : tProjDataRecord;
                        var ErrMsg : string) : boolean;
var
  i   : integer;
  Msg : string;
begin
  Result := false;
  Msg    := 'COM port: ';
  ErrMsg := '';

  if Ports.Count = 0
  then begin ErrMsg := Msg + 'No ports found on system';  exit; end;

  if length(Data.PortToUse) = 0
  then begin ErrMsg := Msg + 'No port specified';        exit; end;

  if not Ports.Find(Data.PortToUse,i)
  then begin ErrMsg := Msg + 'Specified port not found'; exit; end;

  Result := true;
end; // function  MakeComPortOK

function  ProjectDataOK(var   Data                    : tProjDataRecord;
                        var   Page1,Page2,Page3,Page4 : tOpcodeData;
                        var   CtrlRegs                : tCtrlRegArray;
                        var   SerialMemRegs           : tCtrlRegArray;
                        const MemMap,IndMemMap        : tList;
                        var   TalkerArray             : tTalkArray;
                        var   TalkerLength            : integer;
                        var   MapData                 : tMapRecord;
                        const ErrMsgs                 : tStringList) : boolean;
var
  ErrMsg  : string;
  i       : integer;
  Flag1,Flag2,Flag3,Flag4,Flag5,Flag6 : boolean;
begin
  Flag4  := true;
  Result := false;
  ErrMsg := '';
  with Data do
  begin
    try
      Flag1 := MakeComPortOK(Data,ErrMsg);
      if not Flag1 then ErrMsgs.Add(ErrMsg);
      Flag2 := OpenOpcodeFileOK(FN_Opcodes,Page1,Page2,Page3,Page4,ErrMsg);
      if not Flag2 then ErrMsgs.Add(ErrMsg);
      Flag3 := OpenCtrlRegsFileOK(FN_CtrlRegs,CtrlRegs,ErrMsg);
      if not Flag3 then ErrMsgs.Add(ErrMsg);

      case TalkerType of
        ttRam :
          begin
            Flag4 := OpenTalkerFileOK(FN_RamTalker,TalkerArray,
                                      TalkerLength,ErrMsg);
            if not Flag4 then ErrMsgs.Add(ErrMsg);
            Flag5 := OpenMapFileOK(FN_RamMap,MapData,ErrMsg);
            if not Flag5 then ErrMsgs.Add(ErrMsg);
          end;
        ttEeprom :
          begin
            Flag5 := OpenMapFileOK(FN_EEMap,MapData,ErrMsg);
            if not Flag5 then ErrMsgs.Add(ErrMsg);
          end;
        ttExternal :
          begin
            Flag5 := OpenMapFileOK(FN_ExtMap,MapData,ErrMsg);
            if not Flag5 then ErrMsgs.Add(ErrMsg);
            if ExtTalkerRequires = ttRam
            then begin
              Flag4 := OpenTalkerFileOK(FN_RamTalker,TalkerArray,
                                        TalkerLength,ErrMsg);
              if not Flag4 then ErrMsgs.Add(ErrMsg);
            end;
          end
      else begin Flag4 := true; Flag5 := true; end;
      end; // case TalkerType of


      Flag6 := MakeMemMapOK(Data,CtrlRegs,MemMap,MemTypeArray,ErrMsg) and
               MakeIndMemMapOK(Data,IndMemMap,ErrMsg);
      if not Flag6 then ErrMsgs.Add(ErrMsg);

      Result := Flag1 and Flag2 and Flag3 and Flag4 and Flag5 and Flag6;

      if FileExists(FN_IndRegs)
      then OpenCtrlRegsFileOK(FN_IndRegs,IndRegs,ErrMsg);

    finally
      MainForm.SwitchBtn.Enabled := Data.RemoteSwitchAvailable;
      if not Result
      then begin
        i := 0;
        while i < ErrMsgs.Count do
        if length(ErrMsgs[i]) = 0 then ErrMsgs.Delete(i) else i := i + 1;
      end; // if Result then.. else..
    end; // try.. finally..
  end; // with Data do
end; // function  ProjectDataOK

function  StateIsAllowed(    AllowedStates : tSetOfProgStates;
                         var Comment       : tCmdLineComment) : boolean;
begin
  Result := Prog.State in AllowedStates;
  if Result then exit;
  case Prog.State of
    psDuffData     : Comment := clcProjectError;
    psDisconnected : Comment := clcNotWhenDisconnected;
    psConnected    : Comment := clcNotWhenConnected;
    psStopped      : Comment := clcNotWhenStopped;
    psRun          : Comment := clcNotInRun;
    psTrace        : Comment := clcNotInTrace;
    psRunStop      : Comment := clcNotAtBP;
    psTraceStop    : Comment := clcNotAtBP;
  end;
end; // function ValidStates

function NumArgsOK(const CList        : tStringList;
                         Min,Max      : integer;
                   var   Comment      : tCmdLineComment) : boolean;
begin
  Result := false;
  if CList.Count -1 > Max then begin Comment := clcTooManyArgs; exit; end;
  if CList.Count -1 < Min then begin Comment := clcTooFewArgs; exit; end;
  Result := true;
end; // function NumArgsOK

function FoundArgOK(const CList       : tStringList;
                          Idx         : integer;
                          AllowedArgs : tSetOfArgType;
                    var   ArgType     : tArgType;
                    var   ArgValue    : longword) : boolean;
var
  S,T,U     : string;
  AT        : tArgType;
  w         : word;
  Ambiguous : boolean;
  PosX      : integer;
begin
  Result   := true;
  ArgType  := atUnknown;
  ArgValue := 0;

  if Idx < CList.Count then T := CList[Idx];
{ tArgType      = (atUnknown,atNotExist,atByte,atWord,atWordX,atInteger,
                   atLabel,atAsterisk,atLength); }
//  for AT := atNotExist to atLength do
  for AT := atNotExist to atLength do
  begin
    S := T;  U := uppercase(S);
    if not (AT in AllowedArgs) then continue;
    ArgType := AT;
    case AT of
      atNotExist : if Idx > (CList.Count - 1) then exit;
      atByte,atWord,atWordX :
        begin
          if AT = atWordX
          then begin
            PosX := pos('X',U);
            if PosX > 0 then delete(S,PosX,1);
          end;
          try
            if copy(S,1,1) = '%'
            then begin
              ArgValue := BinStrToWord(S);
            end
            else begin
              if copy(S,1,1) = '#'
              then begin
                delete(S,1,1);
                ArgValue := StrToInt(S);
              end
              else begin
                if copy(S,1,1) <> '$' then insert('$',S,1);
                ArgValue := StrToInt(S);
              end;
            end;
            if (AT = atByte) and (ArgValue <= $FF) then exit;
            if (AT in [atWord,atWordX]) and (ArgValue <= $FFFF) then exit;
          except on EConvertError do begin end;
          end; // try.. except on EConvertError..
        end;
      atInteger :
        begin
          try
            if copy(S,1,1) = '#' then delete(S,1,1);
            ArgValue := StrToInt(S);
            exit;
          except on EConvertError do begin end;
          end; // try.. except on EConvertError..
        end;

      atLabel :
        begin
          if LookupLabelValue(S,w,Ambiguous) and not Ambiguous
          then begin ArgValue := w; exit; end;
        end;

      atAsterisk :
        begin
          if (length(S) = 1) and (copy(S,1,1) = '*') then exit;
        end;
      atLength :
        begin
          if (copy(S,1,1) = '+') and (length(S) > 1)
          then begin
            delete(S,1,1);
            if not (S[1] in ['$','#']) then insert('$',S,1);
            try
              ArgValue := StrToInt(S);
              if ArgValue <= $FFFF then exit;
            except on EConvertError do begin end;
            end; // try.. except on EConvertError..
          end; // if (copy(S,1,1) = '+') and (length(S) > 1) then..
        end;
    end; // case
  end; // for AT := atNotExist to atLength do
  Result := false;
end; // function FoundArgOK

{Check that there is no writing to EE/Eprom or External FLASH if an
 EEPROM-resident talker is running. Returns true if no problem}
function  AreasVersusEepromTalkerOK(    AreasUsed : tMemAreaSet;
                                    var Comment   : tCmdLineComment) : boolean;
begin
  Result := true;
  if ProjData.TalkerType <> ttEeprom then exit;
  Result := (AreasUsed*[maEeprom,maEprom,maByteWrite,maPageWrite]) = [];
  if not Result then Comment := clcNotEepromTalker;
end; // function  AreasVersusEepromTalkerOK

{Check that the necessary overlays are available if a RAM-resident talker is
 being used. It is assumed that overlays are not necessary with a talker
 in external memory. Returns true if no problem}
function OverlaysAvailableOK(    AreasUsed : tMemAreaSet;
                             var Comment   : tCmdLineComment) : boolean;
begin
  Result := false;
  try
    if ProjData.TalkerType = ttRam
    then begin
      if (maEeprom in AreasUsed) and
         not OpenS19FileOK(ProjData.FN_EEOverlay,EEOvlyListing,SupErrMsg)
      then begin Comment := clcEepromOverlayError; exit; end;
      if (maEprom in AreasUsed) and
         not OpenS19FileOK(ProjData.FN_EOverlay,EOvlyListing,SupErrMsg)
      then begin Comment := clcEpromOverlayError; exit; end;
      if (maByteWrite in AreasUsed) and
         not OpenS19FileOK(ProjData.FN_ByteWriteOverlay,ByteOvlyListing,SupErrMsg)
      then begin Comment := clcByteOverlayError; exit; end;
      if (maPageWrite in AreasUsed) and
         not OpenS19FileOK(ProjData.FN_PageWriteOverlay,PageOvlyListing,SupErrMsg)
      then begin Comment := clcPageOverlayError; exit; end;
      if (maIndRegs in AreasUsed) and
         not OpenS19FileOK(ProjData.FN_IndRegsOvly,IndRegsOvlyListing,SupErrMsg)
      then begin Comment := clcIndRegsOverlayError; exit; end;
      if (maIndMem in AreasUsed) and
         not OpenS19FileOK(ProjData.FN_IndMemOvly,IndMemOvlyListing,SupErrMsg)
      then begin Comment := clcIndMemOverlayError; exit; end;
    end;
    Result := true;
  finally
    if not Result then SupErrMsgSet := true;
  end; // try.. finally..
end; // function OverlaysAvailableOK

end.

