unit CmdUnit;

interface

uses Classes, Controls, SysUtils, Graphics, Dialogs, ComCtrls, Forms,
     TypeDefs;

function  ProcessCR : boolean;
function  CheckLine(const CList       : tStringList;
                          LastCommand : tCommand;
                    var   Command     : tCommand) : tCmdLineComment;
procedure DoCommand(const CList       : tStringList;
                    var   Command     : tCommand);
procedure AddToOutput(const Buf : string);

implementation

uses
  Main, Comms1, InitUnit, SettingsUnit, ListUnit, UnAsmUnit, MMUnit,
  RegUnit, LoadUnit, VerifyUnit, SaveUnit, EraseUnit, RunTraceUnit,
  MacroUnit, PauseUnit, ConfigUnit, BPUnit, FillUnit, DupUnit, CompUnit,
  CRCUnit, FindUnit, TermUnit;

{Procedure called when the user terminates a command line with the enter key,
 or a macro line is processed}
function ProcessCR : boolean;
var
  CList      : tStringList;
  Command    : tCommand;
  Cmd1,Cmd2  : string;
  ResultCode : tCmdLineComment;
  i          : integer;
begin
  Result     := true;
  CList      := tStringList.Create;
  try
    Command := cmdNone;
{Cmd1 keeps a copy of the command exactly as it appeared in the command edit
 box.  Cmd2 has modified copy which is passed to the syntax checking routine}
    Cmd1    := MainForm.CmdEdit.Text;
    Cmd2    := Cmd1;

{Discard blank lines}
    if length(Cmd1) = 0 then exit;

{Single quotes have to be changed to double quotes or the parsing of the command
 line by assigning it to CList.CommaText will not work correctly.  CommaText
 expects lines in System Data Format wherein strings are delimited by double
 quotes}
    for i := 1 to length(Cmd2) - 1 do
    begin
      if (copy(Cmd2,i,2) = #32#39) or (copy(Cmd2,i,2) = #44#39)
      then Cmd2[i+1] := #34;          // change < '> to < ">  and <,'> to <,">
      if (copy(Cmd2,i,2) = #39#32) or (copy(Cmd2,i,2) = #39#44)
      then Cmd2[i]   := #34;          // change <' > to <" >  and <',> to <",>
    end;

{The character #149 is inserted as a flag to Tokenize (see CmdUnit) that a
 character string has been encountered.
 Multiple instances of the bullet flag #149 have to avoided, as might happen
 when using replaceable parameters in macros - hence the second test in the
 'if' clause below (actually, I think I've fixed this)}
    i := 1;
    while i < length(Cmd2)-1 do
    begin
      if ((copy(Cmd2,i,2) = #32#34) or (copy(Cmd2,i,2) = #44#34)) and
         (Cmd2[i+2] <> #149)
      then insert(#149,Cmd2,i+2);
      i := i + 1;
    end;

{A hack to allow for the case during the memory modify command when a space
 character is used to preserve the existing memory value.  Because the
 assignment of a string to CommaText strips out spaces (indeed it uses them
 to delimit the various elememts of the string), this hack will replace the
 space by the character #149 (a bullet)}
    if (LastCommand = cmdModifying) and
       (copy(Cmd2,1,1) = ' ') and (copy(Cmd2,length(Cmd2),1) = ' ')
    then Cmd2[length(Cmd2)] := chr(149);

    if length(Cmd2) > 0
    then begin
      try
{Split the command line up into its parts}
        CList.CommaText := Cmd2;
{Pass the command line to an initial syntax check.  Cmd1 is passed only for use
 by AsmCheck - no other command needs it}
        ResultCode := CheckLine(CList,LastCommand,Command);
{Modify the displayed command line with a comment if appropriate.  Note that
 CmdLineComments[clOK] is a zero length string}
        if length(CommentArray[ResultCode]) > 0
        then Cmd1 := Cmd1 + ' <-- ' + CommentArray[ResultCode];
{Update the History memo box which records previous commands }
        with MainForm.CHEdit do
        begin
          if not (Command in [cmdEnd,cmdModifying])
          then begin
            Lines.Add(Cmd1);
            LastCmdIndex := Lines.Count;
            SelStart     := length(Text);
          end;
        end;
{Reset the entry editing box}
        MainForm.CmdEdit.Text := '';
{Add the command line to the output window}
        AddToOutput(Cmd1);
{Add a suplementary error message, if any}
        if SupErrMsgSet
        then begin
          with MainForm.OWEdit do
          begin
            SelAttributes.Color := clRed;
            Lines.Add(SupErrMsg);
            SelAttributes.Color := clBlack;
          end;  
          SupErrMsg := '';
          SupErrMsgSet := false;
        end;
{If macro is recording, add line to macro editor}
        if Prog.MacroState = msRecording then MEForm.MEEdit.Lines.Add(Cmd1);
{If the syntax check performed by 'CheckLine' was OK then carry out the command}
        if ResultCode = clcOK
        then begin
          DoCommand(CList,Command);
          LastCommand := Command;
{Following certain commands, the first element of the command is re-inserted
 on the command edit line}
          if (Command in [cmdList,cmdListLocal,cmdUnassemble,
                          cmdUnassembleLocal,cmdTrace,cmdStepOver,cmdGo])
          then with MainForm.CmdEdit do
          begin
            Text      := CList[0];
            SelStart  := 0;
            SelLength := length(Text);
          end;
//          else MainForm.CmdEdit.Clear;
        end
        else begin
          Beep;
          LastCommand := cmdNone;
          if ProjData.AbortOnFormatError and (Prog.MacroState = msPlaying)
          then begin
            Prog.MacroState := msStopped;
            MainForm.OWEdit.Lines.Add('Macro aborted on command line format error');
          end;
          Result               := false;
        end; // if ResultCode in [clOK,clTerminate] then.. else..
{If a talker communication error occurs during the carrying out of a command,
 then ask operator if he/she wishes to re-boot the MCU}
      except on E : EMCUComErr do
        begin
          Result      := false;
          Command     := cmdNone;
          LastCommand := cmdNone;
          HandleMCUCommError(E.Message);
        end;
      end;
    end; // if length(Command) > 0
  finally
    CList.Free;
    MainForm.ProgressBar1.Position := 0;
//    LoadedOverlay := 0;
  end; // try.. finally..
end; // function ProcessCR

{Perform substitutions for PCbug11 commands}
function PCbug11Substitutions(var   Cmd   : string;
                              const CList : tStringList) : boolean;
begin
  Result := true;
  {ASM not implemented}
  if Cmd = 'BF'
  then begin
    Cmd      := 'F';
    exit;
  end;
  if Cmd = 'DASM'
  then begin
    Cmd      := 'U';
    exit;
  end;
  if (Cmd = 'DB') or (Cmd = 'MD')
  then begin
    Cmd      := 'L';
    exit;
  end;
  if Cmd = 'LOADS'
  then begin
    Cmd      := 'LD';
    exit;
  end;
  if Cmd = 'LSTM'
  then begin
    Cmd      := 'LM';
    exit;
  end;
  if Cmd = 'MOVE'
  then begin
    Cmd      := 'D';
    exit;
  end;
  {MS not implemented}
  {QUIT not implemented}
  if Cmd = 'QUIT'
  then begin
    Cmd      := 'Q';
    exit;
  end;
  if Cmd = 'RD'
  then begin
    Cmd      := 'R';
    exit;
  end;
  {RM not implemented}
  {RS not implemented}
  if Cmd = 'VERF'
  then begin
    if (CList.Count > 1) and (uppercase(CList[1]) = 'ERASE')
    then begin
      Cmd      := 'VE';
      CList.Delete(1);
      exit;
    end
    else begin
      Cmd      := 'V';
      exit;
    end;
  end;
  Result := false;
end; // function PCbug11Substitutions

{Function to check the syntax of a command line.  It returns with the value
 clcOK if there are no faults of syntax, otherwise with a value indicating
 the type of fault.

 While checking the syntax, it is also convenient to generate certain
 addresses, etc. which will be required when it comes to carrying out the
 command}
function  CheckLine(const CList       : tStringList;
                          LastCommand : tCommand;
                    var   Command     : tCommand) : tCmdLineComment;
var
//  i   : integer;
  Cmd : string;
begin
  Result := clcCmdErr;
  if length(CList[0]) = 0 then exit;
  Cmd := uppercase(CList[0]);

{Lines are checked first to see whether they are the name of a currently
 available macro. Note that if a macro is present with the same name as
 a native command, then it will overide the native command}
  if MacroExists(Cmd)
  then begin
    MacroCheck(CList,Result,Command);
    exit;
  end;

{Perform substitutions for PCbug11 commands}
//  PCbug11Substitutions(Cmd,CList);

  if (Cmd = 'BR')                      // Set breakpoints
  then begin
    BRCheck(CList,Result,Command);
    exit;
  end;

  if copy(Cmd,1,3) = 'BRP'             // Set a pass point
  then begin
    BRPCheck(CList,Result,Command);
    exit;
  end;

  if Cmd = 'CLS'                       // Clear the output window
  then begin
    Result  := clcOK;
    Command := cmdClearOutputWindow;
    exit;
  end;

  if Cmd = 'CLM'                       // Clear local memory
  then begin
    Result  := clcOK;
    Command := cmdClearLocal;
    exit;
  end;

  if (Cmd = 'C') or (Cmd = 'CL') or (Cmd = 'CMP') or (Cmd = 'CMPL')
  then begin                           // Compare blocks of memory
    CompareCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'CRC') or (Cmd = 'CRCL')
  then begin
    CRCCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'CONFIG')
  then begin
    ConfigCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'CONNECT')
  then begin
    ConnectCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'D') or (Cmd = 'DL')
  then begin
    DuplicateCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'DISCONNECT')
  then begin
    DisconnectCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'EBULK')
  then begin
    BulkEraseCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'EBYTE')
  then begin
    ByteEraseCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'F') or (Cmd = 'FL')
  then begin
    FillCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'FIND') or (Cmd = 'FINDL')
  then begin
    FindCheck(CList,Result,Command);
    exit;
  end;

  if Cmd = 'G'                               // Run a program in MCU memory
  then begin
    GoCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'L') or  (Cmd = 'LL') or  (Cmd = 'LI')   // List memory
  then begin
    ListCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'LD') or (Cmd = 'LDL')                   // Load an S19 file
  then begin
    LoadCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'LDI')                                   // Load an S19 file
  then begin
    IndirectLoadCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd='LM')                                      // List Macros
  then begin
    if MacroNameList.Count > 0
    then begin
      Result  := clcOK;
      Command := cmdListMacro;
    end
    else begin
      Result  := clcNoLibraryMacros;
      Command := cmdNone;
    end;
    exit;
  end;

  if (Cmd='M') or (Cmd='MM')                         // Modify MCU memory
  then begin
    MM1Check(CList,Result,Command);
    exit;
  end;

  if LastCommand = cmdModifying
  then begin                         // Supplementary lines of 'memory modify'
    MM2Check(CList,Result,LastCommand,Command);
    exit;
  end;

  if (Cmd = 'NEXT')                    // Find next occurence of byte string
  then begin                           //   previously specified
    NextCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'NOBR')                    // clear breakpoints
  then begin
    NoBRCheck(CList,Result,Command);
    exit;
  end;

  if Cmd = 'PAUSE'                     // Display or modify registers
  then begin
    PauseCheck(CList,Result,Command);
    exit;
  end;

  if Cmd = 'R'                         // Display or modify registers
  then begin
    RegCheck(CList,Result,Command);
    exit;
  end;

  if Cmd = 'RI'                        // Display or modify indirect memory registers
  then begin
    IndirectRegCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'RESET')
  then begin
    ResetCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'RPC')
  then begin
    RPCCheck(CList,Result,Command);
    exit;
  end;

  if Cmd = 'S'                         // Stop a running program
  then begin
    StopCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'SV') or (Cmd = 'SVL')    // Save memory to a file
  then begin
    SaveCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'SVI')                    // Save indirect memory to a file
  then begin
    IndirectSaveCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'SWITCH')
  then begin
    SwitchCheck(CList,Result,Command);
    exit;
  end;

  if Cmd = 'TERM'                      // Opens the Terminal Window
  then begin
    TermCheck(CList,Result,Command);
    exit;
  end;

  if (copy(Cmd,1,1) = 'T') or (copy(Cmd,1,1) = 'O')
  then begin                          // Trace or step over a program
    TraceCheck(CList,Result,Command);
    exit;
  end;

  if (copy(Cmd,1,1) = 'U') or (copy(Cmd,1,2) = 'UL') // Unassemble a program
  then begin
    UnassembleCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'V') or (Cmd = 'VL')       // Verify a program in memory
  then begin
    VerifyCheck(CList,Result,Command);
    exit;
  end;

  if Cmd = 'VE'                        // Verify MCU memory is all $FF (erased)
  then begin
    VerifyEraseCheck(CList,Result,Command);
    exit;
  end;

  if (Cmd = 'ASM') or (Cmd = 'CLRM') or (Cmd = 'QUIT')
  then begin
    Result  := clcNotImplemented;
    Command := cmdNone;
    exit;
  end;

end; // function  CheckLine

procedure DoCommand(const CList       : tStringList;
                    var   Command     : tCommand);
var
  LocalCopyCommand : tCommand;
begin
  LocalCopyCommand := Command;
  with Prog do
  begin
    case LocalCopyCommand of
      cmdAddBR                : AddBR;
      cmdAddBRP               : AddBRP;
      cmdAlterConfig          : AlterConfig;
      cmdBulkEraseEeprom      : BulkErase;
      cmdByteEraseEeprom      : ByteErase;
      cmdClearOutputWindow    : with MainForm.OWEdit do
                                begin
                                  Clear;
                                  DefAttributes.Color := clBlack;
                                end;
      cmdClearLocal           : fillchar(Memory,sizeof(Memory),0);
      cmdCompare              : Compare;
      cmdCompareLocal         : CompareLocal;
      cmdConnect              : Connect;
      cmdCRCMCUbyFile         : CRCMCUbyFile;
      cmdCRCFileLocal         : CRCFileLocal;
      cmdCRC                  : CRCMemory;
      cmdCRCLocal             : CRCMemoryLocal;
      cmdDisconnect           : Disconnect;
      cmdDuplicate            : Duplicate;
      cmdDuplicateLocal       : DuplicateLocal;
      cmdFill                 : Fill;
      cmdFillLocal            : FillLocal;
      cmdFind                 : Find;
      cmdFindLocal            : FindLocal;
      cmdGo                   : Go;
      cmdList                 : List;
      cmdListLocal            : ListLocal(mtIntrinsic);
      cmdListIndirect         : ListIndirect;
      cmdListMacro            : ListMacros;
      cmdLoadS19Local         : LoadS19Local2(S19Listing);
      cmdLoadS19MCU           : LoadS19MCU2(S19Listing);
      cmdLoadBinLocal         : LoadBinaryFileToLocal(mtIntrinsic);
      cmdLoadBinMCU           : LoadBinaryFileToMCU;
      cmdLoadS19Indirect      : LoadS19FileToIndirect;
      cmdLoadBinIndirect      : LoadBinaryFileToIndirect;
      cmdNoBR                 : NoBR;
      cmdModify               : DoModMem1(Command);
      cmdModifying            : DoModMem2(Command);
      cmdReset                : Reset;
      cmdOpenTerminal         : OpenTerminal;
      cmdPause                : Pause;
      cmdPlayMacro            : PlayMacro;
      cmdRegInherentDisplay   : RegInherentDisplay;
      cmdRegInherentChange    : RegInherentChange;
      cmdRegSingleChange      : RegSingleChange(maCtrlRegs);
      cmdRegSingleDisplay     : RegSingleDisplay(maCtrlRegs);
      cmdRegDoubleChange      : RegDoubleChange;
      cmdRegDoubleDisplay     : RegDoubleDisplay;
      cmdRegIndirectDisplay   : RegSingleDisplay(maIndRegs);
      cmdRegIndirectChange    : RegSingleChange(maIndRegs);
      cmdRPC                  : RPC;                //
      cmdSaveLocalToS19       : SaveLocalToS19File;
      cmdSaveMCUToS19         : SaveMCUToS19File;
      cmdSaveIndirectToS19    : SaveIndirectToS19File;
      cmdSaveLocalToBin       : SaveLocalToBinFile(mtIntrinsic);
      cmdSaveMCUToBin         : SaveMCUToBinFile;
      cmdSaveIndirectToBin    : SaveIndirectToBinFile;
      cmdStop                 : Stop;
      cmdSwitchOff            : SwitchOff;
      cmdSwitchOn             : SwitchOn;
      cmdTrace,cmdStepOver    : Trace;
      cmdUnassemble           : Unassemble;
      cmdUnassembleLocal      : UnassembleLocal;
      cmdVerifyErase          : VerifyErase;
      cmdVerifyS19Local       : VerifyS19Local;
      cmdVerifyS19MCU         : VerifyS19MCU;
    end; // case Command of
  end; // with Prog do
end; // procedure DoCommand

{Procedure which adds the command line to the output window, in green }
procedure AddToOutput(const Buf : string);
begin
  with MainForm.OWEdit do
  begin
    SelAttributes.Color := clGreen;
    Lines.Add(Buf);
    SelAttributes.Color := clBlack;
  end;
end; // procedure AddToOutput

end.


