unit Comms1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CPort, TypeDefs, ExtCtrls, StdCtrls;

type
  TCommsForm = class(TForm)
    ComPort: TComPort;
    Timer1: TTimer;
    procedure FormDestroy(Sender: TObject);
    procedure ComPortBeforeOpen(Sender: TObject);
    procedure ComPortBreak(Sender: TObject);
    procedure ComPortRxChar1(Sender: TObject; Count: Integer);    // SWI's
    procedure ComPortRxChar2(Sender: TObject; Count: Integer);    // Terminal
    procedure ComPortRxCharTest(Sender: TObject; Count: Integer); // Test
    procedure FormCreate(Sender: TObject);
    procedure ComPortAfterClose(Sender: TObject);
    procedure ComPortAfterOpen(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

function  ConnectCheck(const CList   : tStringList;
                       var   Comment : tCmdLineComment;
                       var   Command : tCommand) : boolean;
function  DisconnectCheck(const CList   : tStringList;
                          var   Comment : tCmdLineComment;
                          var   Command : tCommand) : boolean;
function  ResetCheck(const CList   : tStringList;
                     var   Comment : tCmdLineComment;
                     var   Command : tCommand) : boolean;
function  SwitchCheck(const CList   : tStringList;
                      var   Comment : tCmdLineComment;
                      var   Command : tCommand) : boolean;
procedure Connect;
procedure Disconnect;
procedure Reset;
procedure SwitchOff;
procedure SwitchOn;
procedure InitializeTalker;

var
  CommsForm: TCommsForm;
  SwitchSignalActive : boolean = false;

implementation

uses Main, MiscUnit, CheckUnit, CmdUnit, LoadUnit, LookupUnit,
     MacroUnit, RunTraceUnit, ListUnit, RegRW, IndRW, TermUnit,
     BPUnit, FileUnit;

{$R *.DFM}

function  ConnectCheck(const CList   : tStringList;
                       var   Comment : tCmdLineComment;
                       var   Command : tCommand) : boolean;
begin
  Result := false;
  if not NumArgsOK(CList,0,0,Comment) then exit;
  if Prog.State = psDuffData then begin Comment := clcProjectError; exit; end;
  if CommsForm.ComPort.Connected
  then begin
    Comment := clcAlreadyConnected;
//    MainForm.ConnectBtn.Down := false;
    exit;
  end;
  Result  := true;
  Comment := clcOK;
  Command := cmdConnect;
end; // procedure ConnectCheck

function  DisconnectCheck(const CList   : tStringList;
                          var   Comment : tCmdLineComment;
                          var   Command : tCommand) : boolean;
begin
  Result := false;
  if not NumArgsOK(CList,0,0,Comment) then exit;
  if Prog.State = psDuffData then begin Comment := clcProjectError; exit; end;
  if not CommsForm.ComPort.Connected
  then begin Comment := clcAlreadyDisconnected; exit; end;
  Result  := true;
  Comment := clcOK;
  Command := cmdDisconnect;
end; // procedure DisconnectCheck

function  ResetCheck(const CList   : tStringList;
                     var   Comment : tCmdLineComment;
                     var   Command : tCommand) : boolean;
begin
  Result := false;
  if not NumArgsOK(CList,0,0,Comment) then exit;
  if not StateIsAllowed([psConnected..psTraceStop],Comment) then exit;
  if not ProjData.RemoteResetAvailable
  then begin Comment := clcNoRemoteReset; exit; end;
  Result  := true;
  Comment := clcOK;
  Command := cmdReset;
end; // procedure ResetCheck

{   clcNoRemoteSwitch,
   clcNoModeSwitch,
   clcModeAlreadyOn,
   clcModeAlreadyOff,
   clcNoVprogSwitch,
   clcVprogAlreadyOn,
   clcVprogAlreadyOff,       }

function  SwitchCheck(const CList   : tStringList;
                      var   Comment : tCmdLineComment;
                      var   Command : tCommand) : boolean;
begin
  Result  := false;
  if not NumArgsOK(CList,1,1,Comment) then exit;
  if not StateIsAllowed([psConnected..psTraceStop],Comment) then exit;
  if not ProjData.RemoteSwitchAvailable
  then begin Comment := clcNoSwitch; exit; end;

  if uppercase(CList[1]) = 'OFF'
  then begin
    if not SwitchSignalActive
      then begin Comment := clcSwitchAlreadyOff; exit end;
    Result  := true;
    Comment := clcOK;
    Command := cmdSwitchOff;
  end
  else begin
    if uppercase(CList[1]) = 'ON'
    then begin
      if SwitchSignalActive
      then begin Comment := clcSwitchAlreadyOn; exit end;
      Result  := true;
      Comment := clcOK;
      Command := cmdSwitchOn;
    end
    else begin Comment := clcArgErr; end;
  end;
end; // function  SwitchCheck

procedure Connect;
begin
  try
    Prog.State := psConnected;

    if (ProjData.TalkerType = ttExternal) and  // does what 'InitializeTalker'
       (ProjData.ExtTalkerRequires = ttNone)   // would otherwise do
    then begin
      Prog.State := psStopped;
      ClearBPList;
      MainForm.BPListBox.Clear;
    end;

    if ProjData.AutoResetAfterConnect
    then begin
      MainForm.CmdEdit.Text := 'RESET';
      ProcessCR;
    end;

    if ProjData.RunBootScript = rbsConnect then PlayBootScript;

  except on EComPort do
    begin
      MessageDlg('Cannot open COM port. Port may not exist' + CRLF +
                 'or may be in use by another program',mtError,[mbOK],0);
      MainForm.ConnectBtn.Down := false;
    end;
  end;
end; // procedure Connect

procedure Disconnect;
begin
  Prog.State := psDisconnected;
end; // procedure Disconnect

procedure Reset;
begin
  Prog.BRAddr := $FFFF;
  with CommsForm.ComPort do
  begin
    case ProjData.ResetUsesDTR of
      true  : case ProjData.ResetPulseGoesPos of
                true  : begin
                          SetDTR(true);
                          Delay(100);
                          SetDTR(false);
                        end;
                false : begin
                          SetDTR(false);
                          Delay(100);
                          SetDTR(true);
                        end;
              end; // case ProjData.ResetPulseGoesPos of
      false : case ProjData.ResetPulseGoesPos of
                true  : begin
                          SetRTS(true);
                          Delay(100);
                          SetRTS(false);
                        end;
                false : begin
                          SetRTS(false);
                          Delay(100);
                          SetRTS(true);
                        end;
              end; // case ProjData.ResetPulseGoesPos of
    end; // case ProjData.ResetUsesDTR of
  end; // with CommsForm.ComPort1 do
//  Prog.State := psConnected;
end; // procedure Reset

procedure SwitchOff;
begin
  with ProjData,CommsForm.ComPort do
  begin
    case ResetUsesDTR of
      true:   begin           // therefore mode switch uses RTS
                case IsSelectedWhenPos of
                  true:  SetRTS(false);
                  false: SetRTS(true);
                end; // case
              end;
      false:  begin           // therefore mode switch uses DTR
                case IsSelectedWhenPos of
                  true:  SetDTR(false);
                  false: SetDTR(true);
                end; // case
              end;
    end; // case ResetUsesDTR of
  end; // with ProjData,CommsForm.ComPort do
  SwitchSignalActive := false;
end; // procedure SwitchOff

procedure SwitchOn;
begin
  with ProjData,CommsForm.ComPort do
  begin
    case ResetUsesDTR of
      true:   begin           // therefore mode switch uses RTS
                case IsSelectedWhenPos of
                  true:  SetRTS(true);
                  false: SetRTS(false);
                end; // case
              end;
      false:  begin           // therefore mode switch uses DTR
                case IsSelectedWhenPos of
                  true:  SetDTR(true);
                  false: SetDTR(false);
                end; // case
              end;
    end; // case ResetUsesDTR of
  end; // with ProjData,CommsForm.ComPort do
  SwitchSignalActive := true;
end; // procedure SwitchOn

{****************************************************************************}

procedure TCommsForm.FormCreate(Sender: TObject);
begin
//  ComPort.OnRxChar := nil;
end;

procedure TCommsForm.FormDestroy(Sender: TObject);
begin
  ComPort.Close;
end;

procedure TCommsForm.ComPortBeforeOpen(Sender: TObject);
begin
  ComPort.Port := ProjData.PortToUse;
  with ComPort.FlowControl,ProjData do
  begin
    if RemoteResetAvailable
    then begin
      if ResetUsesDTR
      then begin
        if ResetPulseGoesPos
        then ControlDTR := DTRDisable else ControlDTR := DTREnable;
      end
      else begin          // reset uses RTS
        if ResetPulseGoesPos
        then ControlRTS := RTSDisable else ControlRTS := RTSEnable;
      end;
    end;

    if RemoteSwitchAvailable
    then begin
      if ResetUsesDTR    // i.e. remote switching uses RTS
      then begin
        if IsSelectedWhenPos
        then ControlRTS := RTSDisable else ControlRTS := RTSEnable;
      end
      else begin         // remote switching uses DTR
        if IsSelectedWhenPos
        then ControlDTR := DTRDisable else ControlDTR := DTREnable;
      end;
    end;
  end; // with ComPort.FlowControl,ProjData do

{Set the baud rate to the default communication rate. This was previously
 done in BootLoadTalker & InitializeTalker. Moved on 13/03/07}
  ComPort.CustomBaudRate := ProjData.CommsBaudRate;  
end; // procedure TCommsForm.ComPort1BeforeOpen

procedure TCommsForm.ComPortAfterOpen(Sender: TObject);
begin
  with MainForm do
  begin
    OWEdit.Lines.Add('Connected at ' + TimeToStr(Time));
    Connect.Hint     := 'Disconnect (close COM port)';
    ConnectBtn.Hint  := Connect.Hint;
    Connect1.Caption := '&Disconnect';
    ConnectBtn.Down  := true; // ComPort.Connected;
  end; // with MainForm do
end; // procedure TCommsForm.ComPortAfterOpen

procedure TCommsForm.ComPortAfterClose(Sender: TObject);
begin
  with MainForm do
  begin
    OWEdit.Lines.Add('Disconnected at ' + TimeToStr(Time));
    Connect.Hint      := 'Connect (open COM port)';
    ConnectBtn.Hint   := Connect.Hint;
    Connect1.Caption  := '&Connect';
    ConnectBtn.Down   := false;
  end; // with MainForm do
end; // procedure TCommsForm.ComPortAfterClose

procedure TCommsForm.ComPortBreak(Sender: TObject);
begin
  InitializeTalker;
end; // procedure TCommsForm.ComPort1Break

procedure TCommsForm.ComPortRxChar1(Sender: TObject; Count: Integer);
var
  RxByte : byte;
begin
{Respond to the breakpoint-found signal}
  if Prog.State in [psRun,psTrace]
  then begin
    ComPort.Read(RxByte,1);
    if (RxByte = $4B) and (Prog.State = psRun)
    then begin
      ComPort.OnRxChar := nil;
      PauseEndSignalled := true;
    end;
    if RxByte = $4A
    then begin
      ComPort.OnRxChar := nil;
      PauseEndSignalled := false;
      HandleSWI;
    end;
  end;
end; // procedure TCommsForm.ComPortRxChar1
{$O-}
procedure TCommsForm.ComPortRxChar2(Sender: TObject; Count: Integer); // Terminal
var
  RxBuf      : string;
  NumBytes,i : integer;
begin
{Respond to characters received in terminal mode}
  NumBytes := ComPort.ReadStr(RxBuf,Count);
  for i := 1 to NumBytes do HandleRxByte(ord(RxBuf[i]));
end; // procedure TCommsForm.ComPortRxChar2
{$O+}
procedure TCommsForm.ComPortRxCharTest(Sender: TObject; Count: Integer);
{$IFDEF Debug}
var
  Buf1,Buf2 : string;
  i         : integer;
{$ENDIF}
begin
{$IFDEF Debug}
  ComPort.ReadStr(Buf1,Count);
  Buf2 := 'Bytes received:';
  for i := 1 to Count do Buf2 := Buf2 + ' ' + IntToHex(ord(Buf1[i]),2);
  ShowMessage(Buf2);
{$ENDIF}
end; // procedure TCommsForm.ComPortRxCharTest

{****************************************************************************}

procedure BootLoadTalker;                // private procedure
var
  i              : integer;
  EchoStr,ErrMsg : string;
  RxByte         : byte;
  ASyncPtr       : PAsync;
begin
  EchoStr := '';
  with MainForm.ProgressBar1 do
  begin
    Max      := TalkerLen;
    Step     := 16;
    Position := 0;
  end;
  with CommsForm.ComPort,ProjData do
  begin
    ClearBuffer(true,true);
    CustomBaudRate := UploadBaudRate;
    fillchar(Memory,sizeof(Memory),0);

    if not OpenTalkerFileOK(ProjData.FN_RamTalker,TalkArray,TalkerLen,ErrMsg)
    then begin
      MessageDlg(ErrMsg,mtError,[mbOK],0);
      exit;
    end;
    
{Mircea Agapie corresponded by email around the middle of July 2005. This
 resulted in the following code which separates the $FF byte from the rest
 of the talker, and appears to give more reliable reception of the echoed
 bytes - but I still don't really understand what is going on.

 Code again modified 8 Jun 2008 because of concerns raised by Amit Jain}

    InitAsync(AsyncPtr);
//    Write(TalkArray[0],1);             // write only the $FF byte // Commented out 8-Jun-2008
    try
      WriteAsync(TalkArray[0],1,AsyncPtr);   // Added 8-Jun-2008
      WaitForAsync(AsyncPtr);                // Added 8-Jun-2008
      ReadAsync(RxByte,1,AsyncPtr);    // mops up any $00 byte received
      for i := 1 to TalkerLen - 1 do
      begin
        Write(TalkArray[i],1);
//        WriteAsync(TalkArray[i],1,AsyncPtr);  // this doesn't appear to work
//        WaitForAsync(AsyncPtr);
        if i mod 16 = 15 then MainForm.ProgressBar1.StepIt;
      end;
    finally
      DoneAsync(AsyncPtr);
    end; // try.. finally..  
    Delay(20);     // to ensure the talker time-out in the MCU bootloader works
    ReadStr(EchoStr,TalkerLen - 1);
    ClearBuffer(true,true);
    CustomBaudRate := CommsBaudRate;      // return to communication baud rate
    MainForm.ProgressBar1.Position := 0;

//if CommsForm.ComPort.TriggersOnRxChar then ShowMessage('Yes') else ShowMessage('No');

    if (length(EchoStr) > 0) and (TalkerType = ttRam)
    then begin
      for i := 0 to length(EchoStr) - 1 do
        Memory[i + MapData.talker_start] := ord(EchoStr[i+1]);
      MainForm.OWEdit.Lines.Add('Talker bytes echoed:');
      ListTalker(MapData.talker_start,MapData.talker_start + length(EchoStr) - 1);
    end;
  end; // with CommsForm.ComPort,ProjData do
  with MainForm do
  begin
    if SwitchBtn.Down
    then begin
      CmdEdit.Text := 'SWITCH OFF';
      ProcessCR;
      SwitchBtn.Down := false;
    end;
  end;
  IndOverlay := oyNone;  
end; // procedure BootLoadTalker

procedure InitializeTalker;
var
  Save_Cursor  : TCursor;
begin
  Save_Cursor      := Screen.Cursor;
  Screen.Cursor    := crHourglass;    { Show hourglass cursor }
  try
    with CommsForm.ComPort,ProjData do
    begin
      case TalkerType of
        ttRam :
          begin
            MainForm.OWEdit.Lines.Add('Break received, now booting the talker');
            BootLoadTalker;    // Upload a boot-loaded talker
            MainForm.OWEdit.Lines.Add(
              'Using RAM based talker on ' + Port);
          end;
        ttEeprom :
          begin
            MainForm.OWEdit.Lines.Add('Break received, sending break to activate talker in EEPROM');
            SetBreak(TRUE);   // Send the 'break' character to activate an
            Delay(10);        // EEPROM-resident talker
            SetBreak(FALSE);
            Delay(100);       // allow EEPROM-resident talker time to initialise
                              // the jump vector table
            MainForm.OWEdit.Lines.Add(
              'Using EEPROM-resident talker on ' + Port);
          end;
        ttExternal :
          begin
            MainForm.OWEdit.Lines.Add('Break received, now booting activation talker');
            case ExtTalkerRequires of
              ttRam :
                begin
                  BootLoadTalker; // Upload a boot-loaded activating talker
                end;
              ttEeprom :
                begin
                  SetBreak(TRUE);  // start EEPROM activating talker
                  Delay(10);
                  SetBreak(FALSE);
                  Delay(20);
                end;
            end; // case ExtTalkerRequires of
            MainForm.OWEdit.Lines.Add('Using external talker on ' + Port);
          end;
      end; // case TalkerType of

//    CustomBaudRate  := CommsBaudRate;  // moved to OnBeforeOpen 13/03/07
      ClearFrameDisplay;
      ClearBPList;
      MainForm.BPListBox.Clear;
      Prog.State      := psStopped;
      InitialReadRegs := false;
      BprotOK         := false;
{Run Boot Script if appropriate}
      if ProjData.RunBootScript = rbsBoot then PlayBootScript;
{Run any 'AutoStart' macro}
      if (not ProjData.NoAutoStart) and MacroExists('AUTOSTART')
      then begin
        MainForm.CmdEdit.Text := 'AUTOSTART';
        ProcessCR;
      end;
    end; // with CommsForm.ComPort,ProjData do
  finally
    Screen.Cursor := Save_Cursor;
  end; // try.. finally..
end; // procedure InitializeTalker

end.

