unit CompUnit;

interface

uses Classes, SysUtils, Graphics, TypeDefs;

function CompareCheck(const CList   : tStringList;
                      var   Comment : tCmdLineComment;
                      var   Command : tCommand) : boolean;
procedure CompareLocal;
procedure Compare;

implementation

uses Main, CmdUnit, MemRW, LookupUnit, MiscUnit, CheckUnit, MemUnit;

var
  StartAddr,EndAddr,CompStartAddr,CompEndAddr : word;

function CompareCheck(const CList   : tStringList;
                      var   Comment : tCmdLineComment;
                      var   Command : tCommand) : boolean;
var
  ArgType      : tArgType;
  A1,A2,Val    : longword;
  Buf          : string;
  IsLocal      : boolean;
begin
  Result             := false;
  Comment            := clcArgErr;
  StartAddr          := 0;
  Buf                := '';
  A2 := 0;

  IsLocal := (uppercase(CList[0]) = 'CL') or (uppercase(CList[0]) = 'CMPL');

  if not NumArgsOK(CList,3,3,Comment) then exit;

  if not IsLocal
  then if not StateIsAllowed([psStopped],Comment) then exit;

  if FoundArgOK(CList,1,[atWord,atLabel],ArgType,Val)
  then A1 := Val else exit;

  if FoundArgOK(CList,2,[atWord,atLabel,atLength],ArgType,Val)
  then begin
    case ArgType of
      atWord,atLabel : A2 := Val;
      atLength       : A2 := A1 + Val;
    end;
  end else exit;

  if FoundArgOK(CList,3,[atWord,atLabel],ArgType,Val)
  then CompStartAddr := Val else exit;

{Check that: 1. A2 is larger than A1
             2. A2 <= $FFFF}
  if (A2 < A1) or (A2 > $FFFF)
  then begin
    Comment := clcAddrRangeErr; // Address range error
    exit;
  end;

  StartAddr := A1;
  EndAddr   := A2;
  CompEndAddr := CompStartAddr + A2 - A1;

{Check that the two address ranges do not overlap}
  if ((CompStartAddr < StartAddr) and (CompEndAddr >= StartAddr))
     or
     ((CompEndAddr > EndAddr) and (CompStartAddr <= EndAddr))
  then begin
    Comment := clcNoCompOverlap;
    exit;
  end;

  Result    := true;          // Arguments are OK
  case IsLocal of
    false : Command := cmdCompare;
    true  : Command := cmdCompareLocal;
  end; // case
  Comment := clcOK;
end; // function FCheck

procedure CompareLocal;
var
  i : word;
begin
  with MainForm.OWEdit do
  begin
    for i := 0 to EndAddr - StartAddr do
    begin
      if Memory[StartAddr + i] <> Memory[CompStartAddr + i]
      then begin
        SelAttributes.Color := clRed;
        Lines.Add('Compare fails at  ' +
                           format('%4.4x',[StartAddr + i]) +
                  ' = ' +  format('%2.2x',[Memory[StartAddr+i]]) +
                  '    ' + format('%4.4x',[CompStartAddr + i]) +
                  ' = ' +  format('%2.2x',[Memory[CompStartAddr+i]]));
        SelAttributes.Color := clBlack;
        if ProjData.AbortOnCmdFail and (Prog.MacroState = msPlaying)
        then begin
          Prog.MacroState := msStopped;
          Lines.Add('Macro aborted on comparison failure');
        end;
        exit;
      end;
    end;
    Lines.Add('Blocks are identical');
  end; // with MainForm.OWEdit do
end; // procedure CompareLocal

procedure Compare;
begin
  ReadMCU64K(StartAddr,EndAddr,true);
  ReadMCU64K(CompStartAddr,CompEndAddr,true);
  CompareLocal;
end; // procedure Compare

end.

