unit ConfigUnit;

interface

uses Classes, SysUtils, Graphics, TypeDefs, Dialogs;

function ConfigCheck(const CList   : tStringList;
                     var   Comment : tCmdLineComment;
                     var   Command : tCommand) : boolean;
procedure AlterConfig;

implementation

uses Main, CmdUnit, Comms1, CheckUnit, MemRW, LoadUnit, LookupUnit, MiscUnit,
     FileUnit;

var NewConfigByte : byte;

function ConfigCheck(const CList   : tStringList;
                     var   Comment : tCmdLineComment;
                     var   Command : tCommand) : boolean;
var
  Val       : longword;
  ArgType   : tArgType;
  RegAddr   : word;
  RxByte    : byte;
  i         : integer;
begin
  Result  := false;
  Comment := clcArgErr;

  if not NumArgsOK(CList,1,1,Comment) then exit;

  if FoundArgOK(CList,1,[atByte],ArgType,Val)
  then NewConfigByte := Val else exit;

  if not AreasVersusEepromTalkerOK([maEeprom],Comment) then exit;
  if not OverlaysAvailableOK([maEeprom],Comment) then exit;

  if LookupCtrlRegAddr('BPROT',RegAddr,i)
  then begin
    RxByte := ReadByte(RegAddr);
    if (RxByte and $10) > 0
    then begin
      Comment := clcPTCONset;
      exit;
    end;
  end;

  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  Command   := cmdAlterConfig;
end; // function ConfigCheck

procedure AlterConfig;

var
  i          : integer;
  ConfigAddr : word;
begin
  if not LookupCtrlRegAddr('CONFIG',ConfigAddr,i) then exit;

{The CONFIG eeprom location has to be separately erased because the erase
 routine in the overlay is fooled when reads of CONFIG read the latched value}
  ByteEraseEeprom(ConfigAddr);

  if Projdata.TalkerType = ttRam then LoadOverlay(oyEeprom);
  WriteByteEeprom2(ConfigAddr,NewConfigByte,false);
  if Projdata.TalkerType = ttRam then UnloadOverlay;
  MessageDlg('The CONFIG EEPROM location has been rewritten' + CRLF +
             'Reset MCU to activate the new value',
             mtInformation,[mbOK],0);
end; // procedure AlterConfig

end.

