unit DupUnit;

interface

uses Classes, SysUtils, Graphics, TypeDefs;

function DuplicateCheck(const CList   : tStringList;
                        var   Comment : tCmdLineComment;
                        var   Command : tCommand) : boolean;
procedure DuplicateLocal;
procedure Duplicate;

implementation

uses Main, CmdUnit, MemRW, LookupUnit, MiscUnit, CheckUnit, MemUnit;

var
  StartAddr,EndAddr,ToStartAddr,ToEndAddr : word;

function DuplicateCheck(const CList   : tStringList;
                        var   Comment : tCmdLineComment;
                        var   Command : tCommand) : boolean;
var
  AreasUsed    : tMemAreaSet;
  ArgType      : tArgType;
  A1,A2,Val    : longword;
  Buf          : string;
  IsLocal      : boolean;
  MemArea      : tMemArea;
begin
  Result             := false;
  Comment            := clcArgErr;
  StartAddr          := 0;
  Buf                := '';
  A2 := 0;

  IsLocal := uppercase(CList[0]) = 'DL';

  if not NumArgsOK(CList,3,3,Comment) then exit;

  if not IsLocal
  then if not StateIsAllowed([psStopped],Comment) then exit;

  if FoundArgOK(CList,1,[atWord,atLabel],ArgType,Val)
  then A1 := Val else exit;

  if FoundArgOK(CList,2,[atWord,atLabel,atLength],ArgType,Val)
  then begin
    case ArgType of
      atWord,atLabel : A2 := Val;
      atLength       : A2 := A1 + Val;
    end;
  end else exit;

  if FoundArgOK(CList,3,[atWord,atLabel],ArgType,Val)
  then ToStartAddr := Val else exit;

{Check that: 1. A2 is larger than A1
             2. A2 <= $FFFF}
  if (A2 < A1) or (A2 > $FFFF)
  then begin
    Comment := clcAddrRangeErr; // Address range error
    exit;
  end;

  StartAddr := A1;
  EndAddr   := A2;
  ToEndAddr := ToStartAddr + A2 - A1;

{Check for address range exclusions when writing to MCU controlled memory}
  if not IsLocal
  then begin
    if not CheckAreas(ToStartAddr,ToEndAddr,[maRam,maEeprom,maExtRam,maByteWrite,maPageWrite],
                      MemMap,AreasUsed,Comment)
    then exit;
    if not AreasVersusEepromTalkerOK(AreasUsed,Comment) then exit;
    if not OverlaysAvailableOK(AreasUsed,Comment) then exit;
  end; // if not IsLocal

{Check that the write area is one contiguous block of the same kind of memory}
  if (not IsLocal)
  then begin
    if (not IsOneBlock(StartAddr,EndAddr,MemArea,MemMap))
    then begin Comment := clcSpanNotAllowed; exit; end;
{Check that the necessary overlays are available if a RAM-resident talker is
 being used. It is assumed that overlays are not necessary with a talker
 in external memory}
    if not OverlaysAvailableOK([MemArea],Comment) then exit;
  end;

  Result    := true;          // Arguments are OK
  case IsLocal of
    false : Command := cmdDuplicate;
    true  : Command := cmdDuplicateLocal;
  end; // case
  Comment := clcOK;
end; // function FCheck

procedure DuplicateLocal;
var
  i : word;
begin
  if ToStartAddr > StartAddr
  then
    for i := EndAddr - StartAddr downto 0 do
      Memory[ToStartAddr + i] := Memory[StartAddr + i]
  else
    for i := 0 to EndAddr - StartAddr do
      Memory[ToStartAddr + i] := Memory[StartAddr + i]
end; // procedure DuplicateLocal

procedure Duplicate;
begin
  with Prog do
  begin
    DuplicateLocal;
    WriteOneBlock(ToStartAddr,ToEndAddr,true);
  end;
end; // procedure Duplicate

end.
