unit EraseUnit;

interface

uses Classes, Dialogs, Controls, Forms, TypeDefs, SysUtils;

function ByteEraseCheck(const CList   : tStringList;
                        var   Comment : tCmdLineComment;
                        var   Command : tCommand) : boolean;
function BulkEraseCheck(const CList   : tStringList;
                        var   Comment : tCmdLineComment;
                        var   Command : tCommand) : boolean;
procedure ByteErase;
procedure BulkErase;

implementation

uses Main, Comms1, CheckUnit, LookupUnit, CmdUnit, MiscUnit, MemRW;

var
  StartAddr : word;

function ByteEraseCheck(const CList   : tStringList;
                        var   Comment : tCmdLineComment;
                        var   Command : tCommand) : boolean;
var
  i         : integer;
  RegAddr   : word;
  RxByte    : byte;
  A1        : longword;
  ArgType   : tArgType;
  AreasUsed : tMemAreaSet;
begin
  Result  := false;
  Comment := clcArgErr;

  if not NumArgsOK(CList,1,1,Comment) then exit;

  if not StateIsAllowed([psStopped],Comment)
  then exit;

  if not FoundArgOK(CList,1,[atWord,atLabel],ArgType,A1) then exit;
  StartAddr := A1;

  if LookupCtrlRegAddr('CONFIG',RegAddr,i)
  then begin
    if RegAddr = StartAddr
    then begin end
    else begin
      if not CheckAreas(StartAddr,StartAddr,[maEeprom],MemMap,AreasUsed,Comment)
      then exit;
    end;
  end;    

  if LookupCtrlRegAddr('BPROT',RegAddr,i)
  then begin
    RxByte := ReadByte(RegAddr);
    if (RxByte and $0F) > 0
    then begin
      Comment := clcBPROTDisallowsErase;
      exit;
    end;
  end;

  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  Command := cmdByteEraseEeprom
end; // function ByteEraseCheck

function BulkEraseCheck(const CList   : tStringList;
                        var   Comment : tCmdLineComment;
                        var   Command : tCommand) : boolean;
var
  TPtr    : tMemRecordPtr;
  i       : integer;
  RegAddr : word;
  RxByte  : byte;
begin
  Result  := false;
  Comment := clcArgErr;

  if not NumArgsOK(CList,0,0,Comment) then exit;

  if not StateIsAllowed([psStopped],Comment)
  then exit;

  i := 0;
  while i < MemMap.Count do
  begin
    TPtr := MemMap[i];
    with TPtr^ do
      if Area = maEeprom then begin StartAddr := SA; break; end;
    i := i + 1;
  end;
  if i = MemMap.Count
  then begin
    Comment := clcNoEepromDefined;
    exit;
  end;

  if LookupCtrlRegAddr('BPROT',RegAddr,i)
  then begin
    RxByte := ReadByte(RegAddr);
    if (RxByte and $0F) > 0
    then begin
      Comment := clcBPROTDisallowsErase;
      exit;
    end;
  end;

  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  Command := cmdBulkEraseEeprom
end; // function BulkEraseCheck

procedure ByteErase;
begin
  ByteEraseEeprom(StartAddr);
end; // procedure ByteErase

procedure BulkErase;
begin
  if MessageDlg('Confirm erase EEPROM',mtConfirmation,[mbOK,mbCancel],0) =
     mrOK then BulkEraseEeprom(StartAddr);
end; // procedure BulkErase

end.

