unit FileUnit;

interface

uses
  Classes, Windows, StdCtrls, SysUtils, Registry, Dialogs, Controls, TypeDefs,
  Forms, ShellApi, ShlObj, ActiveX;

function OpenDialogOK(var FName : string; FileType : tFileType) : boolean;
function SaveDialogOK(var FName : string; FileType : tFileType) : boolean;
function SaveAsDialogOK(var FName : string; FileType : tFileType) : boolean;
function SaveAsStartupDialogOK(var FName : string; FileType : tFileType) : boolean;
function PromptToSaveProject(TypeOfSave : tSaveType) : tModalResult;
function PromptToSaveMacro(TypeOfSave : tSaveType) : tModalResult;
function GetSpecialFolder(CSIDLvalue : integer): string;

function OpenTalkerFileOK(    FN_Talker      : string;
                          var TalkerMemArray : tTalkArray;
                          var TalkerLength   : integer;
                          var ErrMsg         : string) : boolean;
function OpenMapFileOK(    FN_Map  : string;
                       var MapData : tMapRecord;
                       var ErrMsg  : string) : boolean;
function OpenOpcodeFileOK(    FN_Opcodes              : string;
                          var Page1,Page2,Page3,Page4 : tOpcodeData;
                          var ErrMsg                  : string) : boolean;
function OpenCtrlRegsFileOK(    FN_CtrlRegs : string;
                            var CtrlRegs    : tCtrlRegArray;
                            var ErrMsg    : string) : boolean;
function OpenS19FileOK(      FN_S19     : string;
                       const S19Listing : tStringList;
                       var   ErrMsg     : string) : boolean;
function  OpenProjectFileOK(    ProjectFileName  : string;
                            var NeedToSaveOnExit : boolean) : boolean;
function TokenizeFileName(FName : string) : string;
function DeTokenizeFileName(FName : string) : string;
procedure DetokenizeDefaultData;
function OpenSymbolFile : boolean;
procedure ReopenMacroAndSymbol;

var      // needed by GetSpecialFolder - see below
  SHGetFolderPath:  function (hwndOwner : HWND;
                              nFolder   : integer;
                              hToken    : THandle;
                              dwFlags   : DWORD;
                              lpszPath  : PChar): HRESULT; StDCall = NIL;

implementation

Uses
  Main, CFGUnit, MiscUnit, CheckUnit, SymbolUnit, MacroUnit, LoadUnit,
  MemUnit;

function OpenDialogOK(var FName : string; FileType : tFileType) : boolean;
begin
  with MainForm.OpenDialog1 do
  begin
    Filter     := tdFilter[FileType];
    InitialDir := ExtractFileDir(FName);
    Title      := 'Open ' + tdCaption[FileType];
    if FileExists(FName) then FileName := ExtractFileName(FName) else FileName := '';
    Result     := Execute;
    if Result then FName := FileName;
  end;
end; // function OpenDialogOK

function SaveDialogOK(var FName : string; FileType : tFileType) : boolean;
begin
  with MainForm.SaveDialog1 do
  begin
    DefaultExt := tdDefExt[FileType];
    Filter     := tdFilter[FileType];
    InitialDir := ExtractFileDir(FName);
    FileName   := ExtractFileName(FName);
    Title      := 'Save ' + tdCaption[FileType];
    Result     := Execute;
    if Result then FName := FileName;
  end;
end; // function OpenDialogOK

function SaveAsDialogOK(var FName : string; FileType : tFileType) : boolean;
begin
  with MainForm.SaveDialog1 do
  begin
    DefaultExt := tdDefExt[FileType];
    Filter     := tdFilter[FileType];
    InitialDir := ExtractFileDir(FName);
    Title      := 'Save ' + tdCaption[FileType] + ' As';
    FileName   := ExtractFileName(FName);
    Result     := Execute;
    if Result then FName := FileName;
  end;
end; // function OpenDialogOK

function SaveAsStartupDialogOK(var FName : string; FileType : tFileType) : boolean;
begin
  with MainForm.SaveDialog1 do
  begin
    DefaultExt := tdDefExt[FileType];
    Filter     := tdFilter[FileType];
    InitialDir := ExtractFileDir(FName);
    Title      := 'Save as Startup ' + tdCaption[FileType];
    if FileExists(FName)
    then FileName := ExtractFileName(FName) else FileName := '';
    Result     := Execute;
    if Result then FName := FileName;
  end;
end; // function OpenDialogOK

{Function to retrieve the actual path to My Documents. Information mainly from
 Sanjay Kanade's tips, with a few adaptations:
 Kanade's Delphi Stuff -- Utility functions and sample code, code item #6 }
Procedure FreePidl( pidl: PItemIDList );
var
  alloc: IMalloc;
begin
  If Succeeded(SHGetMalloc(alloc)) then alloc.Free(pidl);
end;

function GetSpecialFolder(CSIDLvalue : integer): string;
var
  pidl    : pItemIDList;
  nameBuf : Array [0..MAX_PATH] of Char;
begin
  Result := '';
  if SUCCEEDED(SHGetSpecialFolderLocation(
		  Application.handle, CSIDLValue, pidl)) then
  begin
    If pidl <> Nil Then
    Begin
      If SHGetPathFromIDList(pidl, namebuf) Then
        Result := StrPas(namebuf);
      FreePidl( pidl );
    End;
  end
  else
  //Fix:
  if assigned(SHGetFolderPath) and
    ((csidlvalue=CSIDL_APPDATA) or (csidlvalue=CSIDL_PERSONAL))
  then
  begin
    if SUCCEEDED(SHGetFolderPath(
		  Application.handle, csidlValue, 0, 0, namebuf)) then
       Result := StrPas(namebuf);
  end;
end; // function GetSpecialFolder

function PromptToSaveProject(TypeOfSave : tSaveType) : tModalResult;
var
  Buf1,Buf2 : string;
begin
  Buf1 := ExtractFileName(Prog.FN_Project);
  case TypeOfSave of
    stOpen  : Buf2 := 'opening another project?';
    stClose : Buf2 := 'closing?';
    stNew   : Buf2 := 'starting new project?';
  end; // case
  if length(Buf1) = 0 then Buf1 := '(unnamed)';
  Result := MessageDlg('The Settings recorded in project: ' + Buf1 + ' have changed' + CRLF +
                       'Save before ' + Buf2,
                       mtWarning,[mbYes,mbNo,mbCancel,mbHelp],
                       IDH_PROJECT_FILES);
end; // function PromptToSaveProject

function PromptToSaveMacro(TypeOfSave : tSaveType) : tModalResult;
var
  Buf2 : string;
begin
  case TypeOfSave of
    stOpen       : Buf2 := 'opening another macro?';
    stClose      : Buf2 := 'closing?';
    stNew        : Buf2 := 'starting a new macro?';
    stNewProject : Buf2 := 'opening a new project?';
  end; // case
  Result := MessageDlg('The currently open macro has changed' + CRLF +
                       'Save before ' + Buf2,
                       mtWarning,[mbYes,mbNo,mbCancel,mbHelp],
                       IDH_MACROS);
end; // function PromptToSaveMacro

{Function to open and read into the local memory array, 'TalkArray' the small
 program which is uploaded to the MCU to allow the monitor to communicate with
 the MCU.  If the file cannot be found, or has the wrong length, then the
 function returns FALSE.
 This function has been extended to load also the bytes needed for the EPROM
 programming subroutine which over-writes part of the talker.}

function OpenTalkerFileOK(    FN_Talker      : string;
                          var TalkerMemArray : tTalkArray;
                          var TalkerLength   : integer;
                          var ErrMsg         : string) : boolean;
var
  Talker  : tFileStream;
begin
  Result := false;
  if FileExists(FN_Talker)
  then Talker := tFileStream.Create(FN_Talker,fmOpenRead or fmShareDenyWrite)
  else begin
    ErrMsg := ' Talker File: ' + ExtractFileName(FN_Talker) + ' not found';
    exit;
  end;
{Read talker file into TalkArray}
  try
    TalkerLength := Talker.Read(TalkerMemArray,513);
    if TalkerLength = 0
    then begin
      ErrMsg := 'Talker File: ' + ExtractFileName(FN_Talker) + ' is empty';
      exit;
    end;
    Result := true;
  finally
    Talker.Free;
  end;
end; // function OpenTalkerFileOK

{Function to read in the talker address mapping information}
function OpenMapFileOK(    FN_Map  : string;
                       var MapData : tMapRecord;
                       var ErrMsg  : string) : boolean;
var
  SList : tStringList;
  i     : integer;
  Msg   : string;
begin
  Result := false;
  ErrMsg := '';
  SList  := tStringList.Create;

{Now convert the file lines into a form which can be read in by the CFG utility}
  try
    if not FileExists(FN_Map)
    then begin
      Msg := 'File ' + ExtractFileName(FN_Map) + ' not found';
      exit;
    end;

    SList.LoadFromFile(FN_Map);
    StripComments(SList,[cdDoubleSlash,cdSemiColon,cdAsterisk]);
    for i := 0 to SList.Count - 1 do
    begin
      SList[i] := StringReplace(SList[i],'$','= $',[]);
    end;
{Insert a Section Name}
    SList.Insert(0,'[MAP]');
{Get the section limits}
    GetCFGSectionLimits(SList,'MAP');
{Set map to default values}
    MapData := DefaultMap;
{Read in the map data}
    with MapData do
    begin
      if ReadCFGInt(SList,VN_talker_start,i) then talker_start := i;
      if ReadCFGInt(SList,VN_talker_idle ,i) then talker_idle  := i;
      if ReadCFGInt(SList,VN_swi_srv     ,i) then swi_srv      := i;
      if ReadCFGInt(SList,VN_swi_jmp     ,i) then swi_jmp      := i;
      if ReadCFGInt(SList,VN_illop_jmp   ,i) then illop_jmp    := i;
    end;
    Result := true;
  finally
    SList.Free;
    if not Result then ErrMsg := 'Map data:' + CRLF + '    ' + Msg;
  end; // try.. finally..
end; // function OpenMapFileOK

{Function to read in the information which is used to (dis)assemble MCU
 instructions.  As noted in the TypeDefs unit, this information is stored
 on the heap; pointed to by pointers held in one of two arrays depending
 on which opcode page it belongs to.
 Function returns TRUE if the file is found and is correctly formatted,
 otherwise fills in ErrMsgs with the problem}

function OpenOpcodeFileOK(    FN_Opcodes              : string;
                          var Page1,Page2,Page3,Page4 : tOpcodeData;
                          var ErrMsg                  : string) : boolean;
var
  i           : integer;
  LList,SList : tStringList;
  TPtr        : tOpcodeDatumPtr;
  Msg         : string;
begin
  Result := false;
  ErrMsg := '';
  LList  := tStringList.Create;
  SList  := tStringList.Create;

  try
{Empty the four arrays of opcode data, and dispose of the associated pointers}
    for i := 0 to 255 do
    begin
      if Page1[i] <> nil then dispose(Page1[i]);
      if Page2[i] <> nil then dispose(Page2[i]);
      if Page3[i] <> nil then dispose(Page3[i]);
      if Page4[i] <> nil then dispose(Page4[i]);
    end;

    if not FileExists(FN_Opcodes)
    then begin
      Msg := 'File ' + ExtractFileName(FN_Opcodes) + ' not found';
      exit;
    end;

    SList.LoadFromFile(FN_Opcodes);
    StripComments(SList,[cdDoubleSlash,cdAsterisk]);
    FixCSV(SList);

//  fillchar(AsmPA,sizeof(AsmPA),0);

    for i := 0 to SList.Count - 1 do
    begin
{Use the assignation to CommaText to split up the line into its components}
      LList.CommaText := SList[i];
      New(TPtr);
      with TPtr^ do
      begin
        try
          Mnemonic  := LList[0];
          Desc      := LList[1];
          Operation := LList[2];

          AddrMode := amDirective;
          while (AddrMode > amNone) and (AddrModeArray[AddrMode] <> LList[3])
          do AddrMode := pred(AddrMode);

          Opcode1   := StrToInt('$' + LList[4]);
          if length(LList[5]) > 0
          then Opcode2 := StrToInt('$' + LList[5])
          else Opcode2 := 0;

          Opd1Type := otRR;
          while (Opd1Type > otNone) and (OperandTypeArray[Opd1Type] <> LList[6])
          do Opd1Type := pred(Opd1Type);

          Opd2Type := otRR;
          while (Opd2Type > otNone) and (OperandTypeArray[Opd2Type] <> LList[7])
          do Opd2Type := pred(Opd2Type);

          Opd3Type := otRR;
          while (Opd3Type > otNone) and (OperandTypeArray[Opd3Type] <> LList[8])
          do Opd3Type := pred(Opd3Type);

          Cycles    := StrToInt(LList[9]);
          PCoffset  := StrToInt(LList[10]);
        except
{If an EConvertError occurs because a StrToInt operation has failed, or an
 EStringListError occurs because there are too few fields in the record }
          Msg := 'Error at line no. ' + IntToStr(i);
          exit;
        end; // try.. except..
        case Opcode1 of
          $18 : Page2[Opcode2] := TPtr;
          $1A : Page3[Opcode2] := TPtr;
          $CD : Page4[Opcode2] := TPtr
        else Page1[Opcode1] := TPtr;
        end;
      end; // with TPtr^ do

{Build the array for use with the ASM command}
//    MakeAsmPA(i+1,SList[i]);

    end; // for i := 0 to SList.Count - 1 do
(*
{Add the directives to the ASM array}
  for i := 1 to NumDirectives do
  begin
    MakeAsmPA(310 + i,Directives[i]);
  end;
*)
  Result := true;
  finally
    LList.Free;
    SList.Free;
    if not Result
    then ErrMsg := 'Opcode information:' + CRLF + '    ' + Msg;
  end;
end; // function OpenOpcodeFileOK

function OpenCtrlRegsFileOK(    FN_CtrlRegs : string;
                            var CtrlRegs    : tCtrlRegArray;
                            var ErrMsg      : string) : boolean;
var
  i,j         : integer;
  LList,SList : tStringList;
  Msg,Buf     : string;
begin
  Result   := false;
  ErrMsg   := '';
  CtrlRegs := nil;
  LList    := tStringList.Create;
  SList    := tStringList.Create;

  try
    if not FileExists(FN_CtrlRegs)
    then begin
      Msg := 'File ' + ExtractFileName(FN_CtrlRegs) + ' not found';
      exit;
    end;

    SList.LoadFromFile(FN_CtrlRegs);
    StripComments(SList,[cdDoubleSlash,cdAsterisk]);
    FixCSV(SList);
    SetLength(CtrlRegs,SList.Count);

    i := 0;
    while i < SList.Count do
    begin
      try
{Use the assignation to CommaText to split up the line into its components}
        LList.CommaText := SList[i];
        with CtrlRegs[i] do
        begin
          Buf  := LList[0];
          system.delete(Buf,1,2);
          Addr := StrToInt('$' + Buf);
          Name := LList[9];
          for j := 0 to 7 do Bits[j] := LList[8-j];
          Desc  := LList[10];
          case LList[11][1] of
            'S' : RegType := ioSingle;
            'H' : RegType := ioHigh;
            'L' : RegType := ioLow
          else RegType := ioNone;
          end; // case
        end; // with CtrlRegs[i] do
      except
        begin
{If an EConvertError occurs because a StrToInt operation has failed, or an
 EStringListError occurs because there are too few fields in the record }
          Msg := 'Format error in line ' + IntToStr(i+1);
          exit;
        end;
      end; // try.. except..
      inc(i);
    end; // while i < SList.Count do
    Result := true;
  finally
    LList.Free;
    SList.Free;
    if not Result
    then ErrMsg := 'Control Register data:' + CRLF + '    ' + Msg;
  end; // try.. finally..
end; // function OpenCtrlRegsFileOK

function OpenS19FileOK(      FN_S19     : string;
                       const S19Listing : tStringList;
                       var   ErrMsg     : string) : boolean;
var
  i,j,CheckSum,LineLength : integer;
  Buf                     : string;
  Msg                     : string;
  S19Map                  : tList;
begin
  Result    := false;
  ErrMsg    := '';
  S19Map    := tList.Create;
  try
    if not FileExists(FN_S19)
    then begin
      Msg := 'File not found';
      exit;
    end;
    S19Listing.LoadFromFile(FN_S19);

{Check the S19Listing, line by line, for:
 1. the correct Checksum
 2. that the Length Byte agrees with the length of the line, and
 3. the initial characters are S0, S1 or S9}

    i := 0;
    while i < S19Listing.Count do
    begin
      try  // try.. except on conversion error
        Buf := S19Listing[i];

{If line is of zero length, or begins with 'S0' (it is a comment), it
 should be ignored}
        if (length (Buf) = 0) or (copy(buf,1,2) = 'S0')
        then begin
          i := i + 1;
          continue;
        end;

{Check that the check sum is correct}
        CheckSum := 0;
        j := 3;
        while j < length(Buf)-2 do
        begin
          CheckSum := CheckSum + StrToInt('$' + copy(Buf,j,2));
          j := j + 2;
        end;
        CheckSum := Lo(CheckSum);   // Checksum is only 8 bits wide
        CheckSum := $FF - Checksum; // Checksum is one's complement
        if CheckSum <> StrToInt('$' + copy(Buf,j,2))
        then begin
          Msg := 'Checksum error in line ' + IntToStr(i);
          exit;
        end; // if CheckSum <> StrToInt('$' + copy(Buf,j,2)) then..

{Check the length byte}
        LineLength := (length(S19Listing[i]) shr 1) - 2;
        if StrToInt('$' + copy(Buf,3,2)) <> LineLength
        then begin
          Msg := 'Length error in line ' + IntToStr(i);
          exit;
        end;

{Check that the initial characters are S1 or S9}
        if not ((copy(Buf,1,2) = 'S1') or
                (copy(Buf,1,2) = 'S9') or
                (copy(Buf,1,2) = 'S0'))
        then begin
          Msg := 'Unknown record type in line ' + IntToStr(i);
          exit;
        end;

      except on EConvertError do
        begin
          Msg := 'Invalid character in line ' + IntToStr(i);
          exit;
        end;
      end; // try.. except on EConvertError do..
      i := i + 1;
    end; // while i < S19Listing.Count do

{Now check for overlapping data - i.e. any lines that re-write the same address}
    if not BuildS19MapOK(S19Listing,S19Map,MemMap)
    then begin
      Msg := 'File has overlapping addresses';
      exit;
    end;

    Result := true;
  finally
    if not Result
    then // ErrMsg := '[S19 file:' + ' ' + Msg + ']'; // replaced on 3 Mar 2009
         ErrMsg := '!! ' + ExtractFileName(FN_S19) + ' - ' + Msg;
    ClearMemMap(S19Map);
    S19Map.Free;
  end; // try.. finally..
end; // function  OpenS19FileOK

function  OpenProjectFileOK(    ProjectFileName  : string;
                            var NeedToSaveOnExit : boolean) : boolean;
var
  ErrMsgs : tStringList;
  i       : integer;
begin
  Result     := false;
  ErrMsgs    := tStringList.Create;
  try
    if length(ExtractFileName(ProjectFileName)) = 0
    then begin
      MessageDlg('No project file specified' + CRLF + CRLF +
                 chr(149) + ' Open another project' + CRLF +
                 'or' + CRLF +
                 chr(149) + ' Start a new project' + CRLF + CRLF +
                 '(First-time users should consult' + CRLF +
                 'Help>Getting Started)',mtInformation,[mbOK],0);
//      Prog.FN_Project := '';
      Prog.FN_Project := DetokenizeFileName('[JBug11]\Projects\');
      exit;
    end;
    if not FileExists(ProjectFileName)
    then begin
      MessageDlg('Project ' + ExtractFileName(ProjectFileName) + ' not found' + CRLF + CRLF +
                 chr(149) + ' Open another project' + CRLF +
                 'or' + CRLF +
                 chr(149) + ' Start a new project' + CRLF + CRLF +
                 '(File may have been moved)',mtInformation,[mbOK],0);
//      Prog.FN_Project := '';
      Prog.FN_Project := DetokenizeFileName('[JBug11]\Projects\');
      i := 0;
      while i < PFList.Count do
      begin
        if uppercase(PFList[i]) = uppercase(ProjectFileName)
        then PFList.Delete(i)
        else i := i + 1;
      end;
      exit;
    end;
    Prog.FN_Project := ProjectFileName;
    ReadProjectFile(Prog.FN_Project,NeedToSaveOnExit);      // in CFGUnit
    MainForm.OWEdit.Lines.Add('Opened project: ' + ExtractFileName(Prog.FN_Project));

    Result := ProjectDataOK(ProjData,
                            OpcodePage1,OpcodePage2,OpcodePage3,OpcodePage4,
                            CtrlRegs,IndRegs,MemMap,IndMemMap,TalkArray,TalkerLen,MapData,ErrMsgs);
    if Result
    then begin
      ClearFrameDisplay;
    end
    else begin
      MessageDlg('Project ' + ExtractFileName(Prog.FN_Project) + ' has errors' + CRLF + CRLF +
                 'Open and adjust Settings',mtWarning,[mbOK],
                 0);
    end
  finally
    ErrMsgs.Free;
  end; // try.. finally..
end; // function OpenProjectFileOK

function TokenizeFileName(FName : string) : string;
begin
  Result := StringReplace(FName,LaunchDirFull,LaunchDirToken,[rfIgnoreCase]);
  Result := StringReplace(Result,MyDocsDirFull,MyDocsDirToken,[rfIgnoreCase]);
{Additional line only for development purposes}
//  Result := StringReplace(Result,'D:\Program Files\JBug11',LaunchDirToken,[rfIgnoreCase]);
end; // function TokenizeFileName

function DeTokenizeFileName(FName : string) : string;
begin
  Result := StringReplace(FName,LaunchDirToken,LaunchDirFull,[rfIgnoreCase]);
  Result := StringReplace(Result,MyDocsDirToken,MyDocsDirFull,[rfIgnoreCase]);
end; // function DeTokenizeFileName

{Procedure to detokenize all the file names in DefaultProjData}
procedure DetokenizeDefaultData;
begin
  with DefaultProjData do
  begin
    FN_CtrlRegs    := DetokenizeFileName(FN_CtrlRegs);
    FN_Opcodes     := DetokenizeFileName(FN_Opcodes);
    FN_SymbolFixed := DetokenizeFileName(FN_SymbolFixed);
    FN_RamTalker   := DetokenizeFileName(FN_RamTalker);
    FN_RamMap      := DetokenizeFileName(FN_RamMap);
    FN_EEOverlay   := DetokenizeFileName(FN_EEOverlay);
    FN_EOverlay    := DetokenizeFileName(FN_EOverlay);
    FN_ByteWriteOverlay  := DetokenizeFileName(FN_ByteWriteOverlay);
    FN_PageWriteOverlay  := DetokenizeFileName(FN_PageWriteOverlay);
    FN_EEMap       := DetokenizeFileName(FN_EEMap);
    FN_ExtMap      := DetokenizeFileName(FN_ExtMap);
    FN_LoadS19     := DetokenizeFileName(FN_LoadS19);
    FN_LoadBinary  := DetokenizeFileName(FN_LoadBinary);
    FN_Macro       := DetokenizeFileName(FN_Macro);
  end;
end; // procedure DetokenizeDefaultData

{Procedure to make a '.sym' file from the '.LST' file produced by Pass11}
procedure MakeSymbolFileFromListFile;
var
  FName               : string;
  UpperDelimiterFound : boolean;
  LList,SList         : tStringList;
  i                   : integer;
begin
  if FileExists(ProjData.FN_LoadS19)
  then Fname := ChangeFileExt(ProjData.FN_LoadS19,'.lst')
  else Fname := ChangeFileExt(ProjData.FN_LoadS19,'.lst');
  if not FileExists(FName) then exit;
  LList := tStringList.Create;
  SList := tStringList.Create;
  UpperDelimiterFound := false;
  try
    LList.LoadFromFile(FName);
    i := 0;
    while i < LList.Count do
    begin
      if UpperDelimiterFound
      then SList.Add(LList[i])
      else begin
        if pos('** SYMBOL TABLE',Llist[i]) > 0
        then begin UpperDelimiterFound := true; i := i + 1; end;
      end;
      i := i + 1;
    end;
    FName := ChangeFileExt(ProjData.FN_LoadS19,'.sym');
    SList.SaveToFile(FName);
  finally
    LList.Free;
    SList.Free;
  end;
end; // procedure MakeSymbolFromList

{Function returns true if it manages to build a symbol table of more than
 zero length. If the Symbol form is visible, then it is updated}
function  OpenSymbolFile : boolean;
var
  SList,LList   : tStringList;
  Buf,FN_Symbol : string;
  i,j           : integer;
  A             : longword;
begin
  Result      := false;
  SymbolTable := nil;
  SList       := tStringList.Create;
  LList       := tStringList.Create;
  A           := $10000;

  case ProjData.SymbolTableToUse of
    stNone     :
      begin
        SymbolTable := nil;
        if SymbolForm.Visible and SymbolForm.SymbolsBtn.Down
        then ShowSymbols;
        exit;
      end;

    stAsS19 :
      begin

        if SymFromList
        then MakeSymbolFileFromListFile;  // make a symbol file out of the listing file

        if FileExists(ProjData.FN_LoadS19)
        then begin
          FN_Symbol := ChangeFileExt(ProjData.FN_LoadS19,'.sym');
        end
        else exit;
      end;

    stFromFile : FN_Symbol := ProjData.FN_SymbolFixed;
  end; // case ProjData.SymbolTableToUse of

  if not FileExists(FN_Symbol) then exit;

  SList.LoadFromFile(FN_Symbol);
  StripComments(SList,[cdDoubleSlash,cdSemiColon,cdAsterisk]);
{Now convert the file into the form in which it is held internally - as a
 dynamic array of records - see TypeDefs for the tSymbolTable type}
  try
    for i := 0 to SList.Count - 1 do
    begin
      Buf := SList[i];
      if length(Buf) = 0 then continue;
{Replace tabs and commas with spaces}
      for j := 1 to length(Buf) do if (Buf[j] = ',') or (Buf[j] = ^I)
                                   then Buf[j] := ' ';
{Delete control characters}
      j := 1;
      while j <= length(Buf) do
      if Buf[j] < ' ' then delete(Buf,j,1) else j := j + 1;
{Use the assignation to CommaText to split up the line into its components}
      LList.CommaText := Buf;
      if LList.Count < 2 then continue;
{Try interpreting the second item as a hexadecimal number}
      Buf := LList[1];
      if (length(Buf) > 0) and (Buf[1] <> '$') then Buf := '$' + Buf;
      try
        A := StrToInt(Buf);
      except on EConvertError do continue;
      end;
      if A > $FFFF then continue;
{Assign the items to SymbolTable}
      setlength(SymbolTable,length(SymbolTable)+1);
      SymbolTable[high(SymbolTable)].Name := LList[0];
      SymbolTable[high(SymbolTable)].Val  := A;
    end; // for i := 0 to SList.Count - 1 do
    Result := length(SymbolTable) > 0;
{Update the 'Select Symbol' form if appropriate}
    if Result and SymbolForm.Visible and SymbolForm.SymbolsBtn.Down
    then ShowSymbols;
  finally
    LList.Free;
    SList.Free;
  end; // try.. finally..
end; // function OpenSymbolFile

procedure ReopenMacroAndSymbol;
begin
  if ProjData.HasAssociatedMacro and FileExists(ProjData.FN_Macro)
  then begin
    MEForm.MEEdit.Lines.LoadFromFile(ProjData.FN_Macro);
    MainForm.PlayBtn.Hint := 'Play Macro (none selected)'
  end
  else begin
    ProjData.FN_Macro := '';
    MEForm.MEEdit.Clear;
  end;
  BuildMacroNameList;
  PlayMacroName := '';
  Prog.MacroHasChanged := false; // also force repaint of macro edit form caption
  OpenSymbolFile;
end; // procedure ReopenMacroAndSymbol

end.

