unit FillUnit;

interface

uses Classes, SysUtils, Graphics, TypeDefs;

function FillCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
procedure FillLocal;
procedure Fill;

implementation

uses Main, CmdUnit, MemRW, LookupUnit, MiscUnit, CheckUnit, MemUnit;

var
  StartAddr,EndAddr : word;
  ByteArray         : array of byte;

function FillCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
var
  AreasUsed   : tMemAreaSet;
  ArgType     : tArgType;
  i           : integer;
  A1,A2,Val   : longword;
  Buf         : string;
  IsLocal     : boolean;
  MemArea     : tMemArea;
begin
  Result      := false;
  Comment     := clcArgErr;
  StartAddr   := 0;
  Buf         := '';
  A2          := 0;
  ByteArray   := nil;

  IsLocal := uppercase(CList[0]) = 'FL';

  if not NumArgsOK(CList,3,3,Comment) then exit;

  if not IsLocal
  then if not StateIsAllowed([psStopped],Comment) then exit;

  if FoundArgOK(CList,1,[atWord,atLabel],ArgType,Val)
  then A1 := Val else exit;

  if FoundArgOK(CList,2,[atWord,atLabel,atLength],ArgType,Val)
  then begin
    case ArgType of
      atWord,atLabel : A2 := Val;
      atLength       : A2 := A1 + Val;
    end;
  end else exit;

{Last argument must be either a string enclosed in quotation marks or a string
 of bytes in hex format }
  Buf := CList[3];
  if Buf[1] = chr(149)
  then begin
    Delete(Buf,1,1);
    SetLength(ByteArray,length(Buf));
    for i := 1 to length(Buf) do ByteArray[i-1] := ord(Buf[i]);
  end
  else begin
    i := 1;
    SetLength(ByteArray,length(Buf) div 2);
    try
      while i < length(Buf) do
      begin
        ByteArray[i div 2] := StrToInt('$' + copy(Buf,i,2));
        i := i + 2;
      end;
    except on EConvertError do exit;
    end; //
  end; // if Buf[1] = chr(149) then.. else..

  if (length(ByteArray) < 1) or (length(ByteArray) > 32)
  then begin
    Comment := clcStringTooShortLong;
    exit;
  end;

{If A2 = A1 then the Fill command is arranged so that one copy of the byte
 string will be written completely to memory, subject to a restriction to
 a maximum of 32 characters.
 To do this the end address, A2, is suitably modified }
  if A2 = A1
  then begin
    A2 := A1 + word(Length(ByteArray) - 1);
  end;

{Check that: 1. A2 is larger than A1
             2. A2 <= $FFFF}
  if (A2 < A1) or (A2 > $FFFF)
  then begin
    Comment := clcAddrRangeErr; // Address range error
    exit;
  end;

  StartAddr := A1;
  EndAddr   := A2;

{Check for address range exclusions when writing to MCU controlled memory}
  if not IsLocal
  then begin
    if not CheckAreas(StartAddr,EndAddr,[maRam,maEeprom,maExtRam,maByteWrite,maPageWrite],
                      MemMap,AreasUsed,Comment)
    then exit;
    if not AreasVersusEepromTalkerOK(AreasUsed,Comment) then exit;
    if not OverlaysAvailableOK(AreasUsed,Comment) then exit;
  end; // if not IsLocal

{Check that the write area is one contiguous block of the same kind of memory}
  if (not IsLocal)
  then begin
    if (not IsOneBlock(StartAddr,EndAddr,MemArea,MemMap))
    then begin Comment := clcSpanNotAllowed; exit; end;
{Check that the necessary overlays are available if a RAM-resident talker is
 being used. It is assumed that overlays are not necessary with a talker
 in external memory}
    if not OverlaysAvailableOK([MemArea],Comment) then exit;
  end;

  Result    := true;          // Arguments are OK
  case IsLocal of
    false : Command := cmdFill;
    true  : Command := cmdFillLocal;
  end; // case
  Comment := clcOK;
end; // function FCheck

procedure FillLocal;
var
  i,j,ArrayLength : integer;
begin
  Arraylength := length(ByteArray);
  j := 0;
  for i := StartAddr to EndAddr do
  begin
    Memory[i] := ByteArray[j mod ArrayLength];
    j := j + 1;
  end;
end; // procedure FillLocal

procedure Fill;
begin
  with Prog do
  begin
    FillLocal;
    WriteOneBlock(StartAddr,EndAddr,true);
  end;
end; // procedure Fill

end.

