unit FindUnit;

interface

uses Classes, SysUtils, Graphics, TypeDefs;

function FindCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
function NextCheck(const CList       : tStringList;
                   var   Comment     : tCmdLineComment;
                   var   Command     : tCommand) : boolean;
procedure FindLocal;
procedure Find;

implementation

uses Main, CmdUnit, MemRW, LookupUnit, MiscUnit, CheckUnit;

var
  StartAddr,EndAddr,FoundAddr : word;
  ByteArray                   : array of byte;
  NotFound                    : boolean;

function FindCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
var
  ArgType     : tArgType;
  i           : integer;
  A1,A2,Val   : longword;
  Buf         : string;
  IsLocal     : boolean;
begin
  Result             := false;
  Comment            := clcArgErr;
  StartAddr          := 0;
  Buf                := '';
  A2 := 0;
  ByteArray := nil;

  NotFound := false;
  IsLocal  := uppercase(CList[0]) = 'FINDL';

  if not NumArgsOK(CList,3,3,Comment) then exit;

  if not IsLocal and not StateIsAllowed([psStopped],Comment) then exit;

  if FoundArgOK(CList,1,[atWord,atLabel],ArgType,Val)
  then A1 := Val else exit;

  if FoundArgOK(CList,2,[atWord,atLabel,atLength],ArgType,Val)
  then begin
    case ArgType of
      atWord,atLabel : A2 := Val;
      atLength       : A2 := A1 + Val;
    end;
  end else exit;

{Last argument must be either a string enclosed in quotation marks or a string
 of bytes in hex format }
  Buf := CList[3];
  if Buf[1] = chr(149)
  then begin
    Delete(Buf,1,1);
    SetLength(ByteArray,length(Buf));
    for i := 1 to length(Buf) do ByteArray[i-1] := ord(Buf[i]);
  end
  else begin
    i := 1;
    SetLength(ByteArray,length(Buf) div 2);
    try
      while i < length(Buf) do
      begin
        ByteArray[i div 2] := StrToInt('$' + copy(Buf,i,2));
        i := i + 2;
      end;
    except on EConvertError do exit;
    end; //
  end; // if Buf[1] = chr(149) then.. else..

  if (length(ByteArray) < 1) or (length(ByteArray) > 16)
  then begin
    Comment := clcStringTooShortLong;
    exit;
  end;

{Check that: 1. A2 is larger than A1
             2. A2 <= $FFFF}
  if (A2 < A1) or (A2 > $FFFF)
  then begin
    Comment := clcAddrRangeErr; // Address range error
    exit;
  end;

  StartAddr := A1;
  EndAddr   := A2;

  Result    := true;            // Arguments are OK
  case IsLocal of
    false : Command := cmdFind;
    true  : Command := cmdFindLocal;
  end; // case
  Comment := clcOK;
end; // function FindCheck

function NextCheck(const CList       : tStringList;
                   var   Comment     : tCmdLineComment;
                   var   Command     : tCommand) : boolean;
begin
  Result := false;
  if (LastCommand <> cmdFind) and (LastCommand <> cmdFindLocal)
  then begin
    Comment := clcToFollowFIND;
    Command := cmdNone;
    exit;
  end;
  if NotFound
  then begin
    Comment := clcFoundFailed;
    Command := cmdNone;
    exit;
  end;
  Comment   := clcOK;
  StartAddr := FoundAddr + 1;
  Command   := cmdFindLocal;
  Result    := true;
end; // function NextCheck

procedure FindLocal;
var
  i,j : integer;
begin
  with MainForm.OWEdit do
  begin
    i := StartAddr;
    while i <= (EndAddr - Length(ByteArray) + 1) do
    begin
      if Memory[i] = ByteArray[0]
      then begin
        j := 0;
        while (j < Length(ByteArray)) do
        begin
          if Memory[i+j] <> ByteArray[j] then break;
          inc(j);
        end;
        if j = Length(ByteArray)
        then begin
          FoundAddr := i;
          Lines.Add('String found at: $' + IntToHex(FoundAddr,4));
          exit;
        end;
      end; // if MemArray[FoundAddr + i] = ByteArray[i] then..
      inc(i);
    end; // while i < (EndAddr - Length(ByteArray)) do
    if i >= (EndAddr - Length(ByteArray))
    then begin
      SelAttributes.Color := clRed;
      Lines.Add('String not found');
      SelAttributes.Color := clBlack;
      NotFound := true;
      if ProjData.AbortOnCmdFail and (Prog.MacroState = msPlaying)
      then begin
        Prog.MacroState := msStopped;
        Lines.Add('Macro aborted on find failure');
      end;
      exit;
    end; // if i = (EndAddr - Length(ByteArray)) then..
  end; // with MainForm.OWEdit do
end; // procedure Find

procedure Find;
begin
  ReadMCU64k(StartAddr,EndAddr,true);
  FindLocal;
end;

end.

