unit ImportUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ExtCtrls, StdCtrls, Registry, TypeDefs;

type
  TImportForm = class(TForm)
    CloseBtn: TBitBtn;
    HelpBtn: TBitBtn;
    ConfigListBox: TListBox;
    Step1Label: TLabel;
    PreviewBtn: TButton;
    Bevel1: TBevel;
    Step2Label: TLabel;
    MCUComboBox: TComboBox;
    ProjFileEdit: TEdit;
    SaveBtn: TButton;
    Bevel2: TBevel;
    Step3Label: TLabel;
    Bevel3: TBevel;
    SaveAsBtn: TButton;
    procedure FormShow(Sender: TObject);
    procedure ConfigListBoxClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure PreviewBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MCUComboBoxChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure SaveAsBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

function  GetRootNames(var V4xxRoot,MostRecent,UserRoot : string) : boolean;

var
  ImportForm: TImportForm;

implementation

{$R *.DFM}

uses Main, CFGUnit, FileUnit, PreviewUnit, MiscUnit, InitUnit;

const
  V4RegRoot            = '\Software\BeattSoft\Jbug11\';
  V4FolderLocationKey  = 'Folder Locations';
  V4FileNameKey        = 'File Names';
  V4GeneralSettingsKey = 'General Settings';
  V4TerminalKey        = 'Terminal Settings';
  V4LayoutKey          = 'Layout';
  V4UserKey            = 'User Configs\';

var
  V4xxRoot,UserRoot,MostRecent : string;

const
  V4N_PortToUse        = 'Coms port to use';
  V4N_ResetPulse       = 'Reset pulse kind';
  V4N_UploadBaudRate   = 'Upload Baud rate';
  V4N_CommsBaudRate    = 'Communication Baud rate';

  V4N_LocalOnly        = 'Run in local mode only';
  V4N_BootETalker      = 'Boot to EEPROM-resident talker';
  V4N_WriteDiagnostics = 'Write diagnostics to O.W.?';
  V4N_HPRIOCheck       = 'Alter HPRIO on boot?';
  V4N_ByteToSend       = 'Byte to send HPRIO';
  V4N_NoBreakBoot      = 'Re-boot without Break';
  V4N_UseSymbols       = 'Use symbols when un-assembling';
  V4N_IgnoreEchoErrors = 'Ignore echo errors on writing';
  V4N_IgnoreRunTimeErr = 'Ignore run-time errors in macros';

  V4N_ConfigAllowWrite = 'Allow writing to CONFIG';
  V4N_EepromAllowWrite = 'Allow writing to EEPROM';
  V4N_EpromAllowWrite  = 'Allow writing to EPROM';
  V4N_FlashAllowWrite  = 'Allow writing to Flash';

  V4N_BootS19Check     = 'Load S19 file on boot?';
  V4N_BootS19Name      = 'S19 file to load on boot';
  V4N_BootMacroCheck   = 'Load macro on boot?';
  V4N_BootMacroName    = 'Macro to load on boot';

  V4N_RamStr           = 'RAM data';
  V4N_RomStr           = 'ROM data';
  V4N_RomVisible       = 'ROM visible';
  V4N_ConfigStr        = 'CONFIG data';
  V4N_EepromStr        = 'EEPROM data';
  V4N_EpromStr         = 'EPROM data';
  V4N_CtrlRegsStr      = 'Control registers';
  V4N_WriteExclusions  = 'Write Exclusions';
  V4N_FlashStr         = 'Flash EPROM data';
  V4N_DataPageSize     = 'Flash data page size';

  V4N_ShowBprotWarning = 'Show BPROT warning dialog';
  V4N_ShowBurnWarnings = 'Show burn warning dialogs';
  V4N_Version          = 'Version';
  V4N_S19DefaultExt    = 'S19 File default extension';
  V4N_WatchStr         = 'Watch window addresses';

{Terminal constants}
  V4N_InterCharDelay   = 'Inter-character delay time';
  V4N_TermWordWrap     = 'Wrap text in TW';
  V4N_ShowNonDispChars = 'Show non-displayable chars';
  V4N_LocalEchoOn      = 'Local echo on';
  V4N_SendOnEnter      = 'Send bytes on <Enter>';
  V4N_OnEnterSend      = 'Bytes to send on <Enter>';
  V4N_NewlineOnReceive = 'Bytes to trigger newline';
  V4N_CloseOnReceive   = 'Bytes to trigger TW closure';

{Files}
  V4N_Tk               =  'RAM talker';
  V4N_Tm               =  'RAM talker map file';
  V4N_Rm               =  'EEPROM-resident map file';
  V4N_Ee               =  'EEPROM overlay talker';
  V4N_Eo               =  'EPROM overlay talker';
  V4N_Fs               =  'Flash EPROM overlay';
  V4N_Oc               =  '68HC11 OpCodes';
  V4N_Rg               =  '68HC11 Registers';
  V4N_Cd               =  'Chip information';
  V4N_St               =  'Symbol table';
  V4N_S9               =  'S19 Files';
  V4N_Bn               =  'Binary Files';
  V4N_Mo               =  'Macros';
  V4N_Os               =  'Output window save';
  V4N_Cs               =  'Command window save';
  V4N_Lg               =  'Terminal log file';
  V4N_Ti               =  'Terminal send image file';

var
  V4PortToUse             : string   ;
  V4ResetPulse            : integer  ;
  V4UploadBaud            : integer  ;
  V4CommsBaud             : integer  ;

  V4LocalOnly             : boolean  ;
  V4BootETalkerFlag       : boolean  ;
  V4WriteDiagnostics      : boolean  ;
  V4HPRIOFlag             : boolean  ;
  V4ByteToSend            : integer  ;
  V4NoBreakBoot           : boolean  ;
  V4UseSymbols            : boolean  ;
  V4IgnoreEchoErrors      : boolean  ;
  V4IgnoreRunTimeErr      : boolean  ;

  V4ConfigAllowWrite      : boolean  ;
  V4EepromAllowWrite      : boolean  ;
  V4EpromAllowWrite       : boolean  ;
  V4FlashAllowWrite       : boolean  ;

  V4BootS19Flag           : boolean  ;
  V4BootS19Name           : string   ;
  V4BootMacroFlag         : boolean  ;
  V4BootMacroName         : string   ;

  V4RAMStr                : string   ;
  V4ROMStr                : string   ;
  V4RomVisible            : boolean  ;
  V4ConfigStr             : string   ;
  V4EepromStr             : string   ;
  V4EpromStr              : string   ;
  V4CtrlRegsStr           : string   ;

  V4WriteExclStr          : string   ;
  V4FlashStr              : string   ;
  V4DataPageSize          : integer  ;

  V4ShowBprotWarning      : boolean  ;
  V4ShowBurnWarnings      : boolean  ;
  V4S19DefaultExt         : string   ;
  V4WatchStr              : string   ;
  V4VersionStr            : string   ;


{Terminal variables}
  V4InterCharDelay        : integer  ;
  V4TermWordWrap          : boolean  ;
  V4ShowNonDispChars      : boolean  ;
  V4LocalEchoOn           : boolean  ;
  V4SendOnEnter           : boolean  ;
  V4OnEnterSend           : string   ;
  V4NewLineonReceive      : string   ;
  V4CloseOnReceive        : string   ;

{File variables}
  V4_FN_Tk                : string   ;
  V4_FN_Tm                : string   ;
  V4_FN_Rm                : string   ;
  V4_FN_Ee                : string   ;
  V4_FN_Eo                : string   ;
  V4_FN_Fs                : string   ;
  V4_FN_Oc                : string   ;
  V4_FN_Rg                : string   ;
  V4_FN_Cd                : string   ;
  V4_FN_St                : string   ;
  V4_FN_S9                : string   ;
  V4_FN_Bn                : string   ;
  V4_FN_Mo                : string   ;
  V4_FN_Os                : string   ;
  V4_FN_Cs                : string   ;
  V4_FN_Lg                : string   ;
  V4_FN_Ti                : string   ;

{Folder variables}
  V4_DN_Tk                : string   ;
  V4_DN_Tm                : string   ;
  V4_DN_Rm                : string   ;
  V4_DN_Ee                : string   ;
  V4_DN_Eo                : string   ;
  V4_DN_Fs                : string   ;
  V4_DN_Oc                : string   ;
  V4_DN_Rg                : string   ;
  V4_DN_Cd                : string   ;
  V4_DN_St                : string   ;
  V4_DN_S9                : string   ;
  V4_DN_Bn                : string   ;
  V4_DN_Mo                : string   ;
  V4_DN_Os                : string   ;
  V4_DN_Cs                : string   ;
  V4_DN_Lg                : string   ;
  V4_DN_Ti                : string   ;

var
  MCUData,MCUList : tStringList;

{**************************************************************************}

Procedure InitializeVars;
var
  DefaultFolder : string;
begin
  DefaultFolder := ExtractFilePath(Application.ExeName);

  V4PortToUse             := 'COM1';
  V4ResetPulse            := 0;
  V4UploadBaud            := 1200;
  V4CommsBaud             := 9600;

  V4LocalOnly             := true;
  V4BootETalkerFlag       := false;
  V4WriteDiagnostics      := true;
  V4HPRIOFlag             := false;
  V4ByteToSend            := $E5;
  V4NoBreakBoot           := false;
  V4UseSymbols            := false;
  V4IgnoreEchoErrors      := false;
  V4IgnoreRunTimeErr      := false;

  V4ConfigAllowWrite      := false;
  V4EepromAllowWrite      := false;
  V4EpromAllowWrite       := false;
  V4FlashAllowWrite       := false;

  V4BootS19Flag           := false;
  V4BootS19Name           := '';
  V4BootMacroFlag         := false;
  V4BootMacroName         := '';

  V4RAMStr                := 'On-chip RAM=0000..00FF';
  V4ROMStr                := 'Boot ROM=BF00..BFFF';
  V4RomVisible            := true;
  V4ConfigStr             := '103F';
  V4EepromStr             := '';
  V4EpromStr              := '';
  V4CtrlRegsStr           := '1000..103F';

  V4WriteExclStr          := '';
  V4FlashStr              := '';
  V4DataPageSize          := 64;

  V4ShowBprotWarning      := true;
  V4ShowBurnWarnings      := true;
  V4S19DefaultExt         := '.rec';
  V4WatchStr              := '';
  V4VersionStr            := '';


{Terminal variables}
  V4InterCharDelay        := 0;
  V4TermWordWrap          := true;
  V4ShowNonDispChars      := true;
  V4LocalEchoOn           := true;
  V4SendOnEnter           := true;
  V4OnEnterSend           := '';
  V4NewLineonReceive      := '';
  V4CloseOnReceive        := '';

{File variables}
  V4_FN_Tk                :=  'JBug_Talk_A.BOO';
  V4_FN_Tm                :=  'JBug_Talk_A.MAP';
  V4_FN_Rm                :=  'JBug11_Talk_EE.MAP';
  V4_FN_Ee                :=  'EE_Ovly.rec';
  V4_FN_Eo                :=  'E_Ovly.rec';
  V4_FN_Fs                :=  'E_Flash_Ovly.rec';
  V4_FN_Oc                :=  'HC11_Opcodes.csv';
  V4_FN_Rg                :=  'HC11_Registers.csv';
  V4_FN_Cd                :=  'HC11_Chips.csv';
  V4_FN_St                :=  '';
  V4_FN_S9                :=  '';
  V4_FN_Bn                :=  '';
  V4_FN_Mo                :=  '';
  V4_FN_Os                :=  '';
  V4_FN_Cs                :=  '';
  V4_FN_Lg                :=  '';
  V4_FN_Ti                :=  '';

{Folder variables}
  V4_DN_Tk                :=  DefaultFolder + 'Talkers\';
  V4_DN_Tm                :=  DefaultFolder + 'Talkers\';
  V4_DN_Rm                :=  DefaultFolder + 'Talkers\';
  V4_DN_Ee                :=  DefaultFolder + 'Talkers\';
  V4_DN_Eo                :=  DefaultFolder + 'Talkers\';
  V4_DN_Fs                :=  DefaultFolder + 'Talkers\';
  V4_DN_Oc                :=  DefaultFolder + 'Info\';
  V4_DN_Rg                :=  DefaultFolder + 'Info\';
  V4_DN_Cd                :=  DefaultFolder + 'Info\';
  V4_DN_St                :=  DefaultFolder + 'User\';
  V4_DN_S9                :=  DefaultFolder + 'User\';
  V4_DN_Bn                :=  DefaultFolder + 'User\';
  V4_DN_Mo                :=  DefaultFolder + 'User\';
  V4_DN_Os                :=  DefaultFolder + 'User\';
  V4_DN_Cs                :=  DefaultFolder + 'User\';
  V4_DN_Lg                :=  DefaultFolder + 'User\';
  V4_DN_Ti                :=  DefaultFolder + 'User\';
end; // procedure InitializeVars

function  GetRootNames(var V4xxRoot,MostRecent,UserRoot : string) : boolean;
var
  RegInfo    : tRegistry;
  SubKeys    : tStringList;
  i          : integer;
begin
  RegInfo    := tRegistry.Create;
  SubKeys    := tStringList.Create;
  V4xxRoot   := '';
  UserRoot   := '';
  MostRecent := '';
  try
    with RegInfo do
    begin
{Build a list of all the previous version keys}
      if OpenKeyReadOnly(V4RegRoot)
      then begin
        GetKeyNames(SubKeys);
        CloseKey;
      end;
{Find the most recent version 4xx key}
      if SubKeys.Count > 0
      then begin
        SubKeys.Sort;
        i := 0;
        while i < SubKeys.Count do
        begin
          if SubKeys[i][1] = '4'
          then begin
            MostRecent := SubKeys[i];
            i := i + 1;
          end else SubKeys.Delete(i);
        end;
      end;
      if copy(MostRecent,1,1) = '4'
      then V4xxRoot := v4RegRoot + MostRecent + '\';
{Check that the 'User' root is present}
      if KeyExists(V4RegRoot + V4UserKey)
      then UserRoot := V4RegRoot + V4UserKey;
    end; // with RegInfo do
{Function returns false if neither root name is found}
    Result := not ((length(V4xxRoot) = 0) and (length(UserRoot) = 0));
  finally
    RegInfo.Free;
    SubKeys.Free;
  end; // try.. finally..
end; // function  GetRootNames

procedure ReadVer4Registry(Root : string);
var
  RegInfo       : tRegistry;
begin
  RegInfo := tRegistry.Create;
  try
    with RegInfo do
    begin
{Read in the various items on Settings>General ------------------------------}
      OpenKey(Root + V4GeneralSettingsKey,false);

      ReadRegStr(RegInfo,V4N_PortToUse,        V4PortToUse);
      ReadRegInt(RegInfo,V4N_ResetPulse,       V4ResetPulse);
      ReadRegInt(RegInfo,V4N_UploadBaudRate,   V4UploadBaud);
      ReadRegInt(RegInfo,V4N_CommsBaudRate,    V4CommsBaud);

      ReadRegBln(RegInfo,V4N_LocalOnly,        V4LocalOnly);
      ReadRegBln(RegInfo,V4N_BootETalker,      V4BootETalkerFlag);
      ReadRegBln(RegInfo,V4N_WriteDiagnostics, V4WriteDiagnostics);
      ReadRegBln(RegInfo,V4N_HPRIOCheck,       V4HPRIOFlag);
      ReadRegInt(RegInfo,V4N_ByteToSend,       V4ByteToSend);
      ReadRegBln(RegInfo,V4N_NoBreakBoot,      V4NoBreakBoot);
      ReadRegBln(RegInfo,V4N_UseSymbols,       V4UseSymbols);
      ReadRegBln(RegInfo,V4N_IgnoreEchoErrors, V4IgnoreEchoErrors);
      ReadRegBln(RegInfo,V4N_IgnoreRunTimeErr, V4IgnoreRunTimeErr);

      ReadRegBln(RegInfo,V4N_ConfigAllowWrite, V4ConfigAllowWrite);
      ReadRegBln(RegInfo,V4N_EepromAllowWrite, V4EepromAllowWrite);
      ReadRegBln(RegInfo,V4N_EpromAllowWrite,  V4EpromAllowWrite);
      ReadRegBln(RegInfo,V4N_FlashAllowWrite,  V4FlashAllowWrite);

      ReadRegBln(RegInfo,V4N_BootS19Check,     V4BootS19Flag);
      ReadRegBln(RegInfo,V4N_BootMacroCheck,   V4BootMacroFlag);
      ReadRegStr(RegInfo,V4N_BootS19Name,      V4BootS19Name);
      ReadRegStr(RegInfo,V4N_BootMacroName,    V4BootMacroName);

      ReadRegStr(RegInfo,V4N_RamStr,           V4RamStr);
      ReadRegStr(RegInfo,V4N_RomStr,           V4RomStr);
      ReadRegBln(RegInfo,V4N_RomVisible,       V4RomVisible);
      ReadRegStr(RegInfo,V4N_ConfigStr,        V4ConfigStr);
      ReadRegStr(RegInfo,V4N_EepromStr,        V4EepromStr);
      ReadRegStr(RegInfo,V4N_EpromStr,         V4EpromStr);
      ReadRegStr(RegInfo,V4N_CtrlRegsStr,      V4CtrlRegsStr);
      ReadRegStr(RegInfo,V4N_WriteExclusions,  V4WriteExclStr);
      ReadRegStr(RegInfo,V4N_FlashStr,         V4FlashStr);
      ReadRegInt(RegInfo,V4N_DataPageSize,     V4DataPageSize);

{Read in other items, e.g. menu settings ------------------------------------}
      ReadRegBln(RegInfo,V4N_ShowBurnWarnings, V4ShowBurnWarnings);
      ReadRegBln(RegInfo,V4N_ShowBprotWarning, V4ShowBprotWarning);
      ReadRegStr(RegInfo,V4N_WatchStr,         V4WatchStr);
      CloseKey;

{Read in folder locations --------------------------------------------------}
      OpenKey(Root + V4FolderLocationKey,false);
      ReadRegStr(RegInfo,V4N_Tk,    V4_DN_Tk);
      ReadRegStr(RegInfo,V4N_Tm,    V4_DN_Tm);
      ReadRegStr(RegInfo,V4N_Rm,    V4_DN_Rm);
      ReadRegStr(RegInfo,V4N_Ee,    V4_DN_Ee);
      ReadRegStr(RegInfo,V4N_Eo,    V4_DN_Eo);
      ReadRegStr(RegInfo,V4N_Fs,    V4_DN_Fs);
      ReadRegStr(RegInfo,V4N_Oc,    V4_DN_Oc);
      ReadRegStr(RegInfo,V4N_Rg,    V4_DN_Rg);
      ReadRegStr(RegInfo,V4N_Cd,    V4_DN_Cd);
      ReadRegStr(RegInfo,V4N_St,    V4_DN_St);
      ReadRegStr(RegInfo,V4N_S9,    V4_DN_S9);
      ReadRegStr(RegInfo,V4N_Bn,    V4_DN_Bn);
      ReadRegStr(RegInfo,V4N_Mo,    V4_DN_Mo);
      ReadRegStr(RegInfo,V4N_Os,    V4_DN_Os);
      ReadRegStr(RegInfo,V4N_Cs,    V4_DN_Cs);
      ReadRegStr(RegInfo,V4N_Lg,    V4_DN_Lg);
      ReadRegStr(RegInfo,V4N_Ti,    V4_DN_Ti);
      CloseKey;

{Read in file names ---------------------------------------------------------}
      OpenKey(Root + V4FileNameKey    ,false);
      ReadRegStr(RegInfo,V4N_Tk,    V4_FN_Tk);
      ReadRegStr(RegInfo,V4N_Tm,    V4_FN_Tm);
      ReadRegStr(RegInfo,V4N_Rm,    V4_FN_Rm);
      ReadRegStr(RegInfo,V4N_Ee,    V4_FN_Ee);
      ReadRegStr(RegInfo,V4N_Eo,    V4_FN_Eo);
      ReadRegStr(RegInfo,V4N_Fs,    V4_FN_Fs);
      ReadRegStr(RegInfo,V4N_Oc,    V4_FN_Oc);
      ReadRegStr(RegInfo,V4N_Rg,    V4_FN_Rg);
      ReadRegStr(RegInfo,V4N_Cd,    V4_FN_Cd);
      ReadRegStr(RegInfo,V4N_St,    V4_FN_St);
      ReadRegStr(RegInfo,V4N_S9,    V4_FN_S9);
      ReadRegStr(RegInfo,V4N_Bn,    V4_FN_Bn);
      ReadRegStr(RegInfo,V4N_Mo,    V4_FN_Mo);
      ReadRegStr(RegInfo,V4N_Os,    V4_FN_Os);
      ReadRegStr(RegInfo,V4N_Cs,    V4_FN_Cs);
      ReadRegStr(RegInfo,V4N_Lg,    V4_FN_Lg);
      ReadRegStr(RegInfo,V4N_Ti,    V4_FN_Ti);
      ReadRegStr(RegInfo,V4N_S19DefaultExt,V4S19DefaultExt);
      CloseKey;

{Read in settings affecting the Terminal Window -----------------------------}
      OpenKey(Root + V4TerminalKey,false);
      ReadRegInt(RegInfo,  V4N_InterCharDelay  ,  V4InterCharDelay   );
      ReadRegBln(RegInfo,  V4N_TermWordWrap    ,  V4TermWordWrap     );
      ReadRegBln(RegInfo,  V4N_ShowNonDispChars,  V4ShowNonDispChars );
      ReadRegBln(RegInfo,  V4N_LocalEchoOn     ,  V4LocalEchoOn      );
      ReadRegBln(RegInfo,  V4N_SendOnEnter     ,  V4SendOnEnter      );
      ReadRegStr(RegInfo,  V4N_OnEnterSend     ,  V4OnEnterSend      );
      ReadRegStr(RegInfo,  V4N_NewlineOnReceive,  V4NewlineonReceive );
      ReadRegStr(RegInfo,  V4N_CloseOnReceive  ,  V4CloseOnReceive   );
      CloseKey;
    end; // with RegInfo do
  finally
    RegInfo.Free;
  end;
end; // procedure ReadVer4Registry

procedure AddLineToPreview(const PList : tStringList;S1,S2 : string);
var
  i : integer;
begin
{The following line determines how many tab characters have to be inserted
 to align the second column of data.  In variable-width fonts it appears
 to be the case that a tab width is twice something called a 'dialog base unit',
 and is, in practice, 48 pixels}
  for i := (PreviewForm.Canvas.TextWidth(S1) div 48) to 3 do S1 := S1 + ^I;
  PList.Add(S1 + S2);
end; // procedure AddLineToPreview

procedure AddToPreviewStr(const PList     : tStringList;
                          ValueName,Value : string);
begin
  AddLineToPreview(PList,ValueName,Value);
end; // procedure AddToPreviewStr

procedure AddToPreviewBln(const PList     : tStringList;
                                ValueName : string;
                                Value     : boolean);
var
  Buf : string;
begin
  case Value of
    true  : Buf := 'TRUE';
    false : Buf := 'FALSE';
  end;
  AddLineToPreview(PList,ValueName,Buf);
end; // procedure AddToPreviewBln

procedure AddToPreviewInt(const PList     : tStringList;
                                ValueName : string;
                                Value     : integer);
begin
  AddLineToPreview(PList,ValueName,IntToStr(Value));
end; // procedure AddToPreviewInt

procedure AddToPreviewHex(const PList     : tStringList;
                                ValueName : string;
                                Value     : integer);
begin
  AddLineToPreview(PList,ValueName,IntToHex(Value,4));
end; // procedure AddToPreviewHex

procedure FillPreview;
var
  PList : tStringList;
begin
  PList := tStringList.Create;
  try
    with PList do
    begin
{General Settings>General ------------------------------}
      Add(uppercase(V4GeneralSettingsKey));
      AddToPreviewStr(PList,V4N_PortToUse,        V4PortToUse);
      AddToPreviewInt(PList,V4N_ResetPulse,       V4ResetPulse);
      AddToPreviewInt(PList,V4N_UploadBaudRate,   V4UploadBaud);
      AddToPreviewInt(PList,V4N_CommsBaudRate,    V4CommsBaud);

      AddToPreviewBln(PList,V4N_LocalOnly,        V4LocalOnly);
      AddToPreviewBln(PList,V4N_BootETalker,      V4BootETalkerFlag);
      AddToPreviewBln(PList,V4N_WriteDiagnostics, V4WriteDiagnostics);
      AddToPreviewBln(PList,V4N_HPRIOCheck,       V4HPRIOFlag);
      AddToPreviewInt(PList,V4N_ByteToSend,       V4ByteToSend);
      AddToPreviewBln(PList,V4N_NoBreakBoot,      V4NoBreakBoot);
      AddToPreviewBln(PList,V4N_UseSymbols,       V4UseSymbols);
      AddToPreviewBln(PList,V4N_IgnoreEchoErrors, V4IgnoreEchoErrors);
      AddToPreviewBln(PList,V4N_IgnoreRunTimeErr, V4IgnoreRunTimeErr);

      AddToPreviewBln(PList,V4N_ConfigAllowWrite, V4ConfigAllowWrite);
      AddToPreviewBln(PList,V4N_EepromAllowWrite, V4EepromAllowWrite);
      AddToPreviewBln(PList,V4N_EpromAllowWrite,  V4EpromAllowWrite);
      AddToPreviewBln(PList,V4N_FlashAllowWrite,  V4FlashAllowWrite);

      AddToPreviewBln(PList,V4N_BootS19Check,     V4BootS19Flag);
      AddToPreviewBln(PList,V4N_BootMacroCheck,   V4BootMacroFlag);
      AddToPreviewStr(PList,V4N_BootS19Name,      V4BootS19Name);
      AddToPreviewStr(PList,V4N_BootMacroName,    V4BootMacroName);

      AddToPreviewStr(PList,V4N_RamStr,           V4RamStr);
      AddToPreviewStr(PList,V4N_RomStr,           V4RomStr);
      AddToPreviewBln(PList,V4N_RomVisible,       V4RomVisible);
      AddToPreviewStr(PList,V4N_ConfigStr,        V4ConfigStr);
      AddToPreviewStr(PList,V4N_EepromStr,        V4EepromStr);
      AddToPreviewStr(PList,V4N_EpromStr,         V4EpromStr);
      AddToPreviewStr(PList,V4N_CtrlRegsStr,      V4CtrlRegsStr);
      AddToPreviewStr(PList,V4N_WriteExclusions,  V4WriteExclStr);
      AddToPreviewStr(PList,V4N_FlashStr,         V4FlashStr);
      AddToPreviewInt(PList,V4N_DataPageSize,     V4DataPageSize);

      AddToPreviewBln(PList,V4N_ShowBurnWarnings, V4ShowBurnWarnings);
      AddToPreviewBln(PList,V4N_ShowBprotWarning, V4ShowBprotWarning);
      AddToPreviewStr(PList,V4N_WatchStr,         V4WatchStr);

{Folder locations --------------------------------------------------}
      Add('');
      Add(uppercase(V4FolderLocationKey));
      AddToPreviewStr(PList,V4N_Tk,    V4_DN_Tk);
      AddToPreviewStr(PList,V4N_Tm,    V4_DN_Tm);
      AddToPreviewStr(PList,V4N_Rm,    V4_DN_Rm);
      AddToPreviewStr(PList,V4N_Ee,    V4_DN_Ee);
      AddToPreviewStr(PList,V4N_Eo,    V4_DN_Eo);
      AddToPreviewStr(PList,V4N_Fs,    V4_DN_Fs);
      AddToPreviewStr(PList,V4N_Oc,    V4_DN_Oc);
      AddToPreviewStr(PList,V4N_Rg,    V4_DN_Rg);
      AddToPreviewStr(PList,V4N_Cd,    V4_DN_Cd);
      AddToPreviewStr(PList,V4N_St,    V4_DN_St);
      AddToPreviewStr(PList,V4N_S9,    V4_DN_S9);
      AddToPreviewStr(PList,V4N_Bn,    V4_DN_Bn);
      AddToPreviewStr(PList,V4N_Mo,    V4_DN_Mo);
      AddToPreviewStr(PList,V4N_Os,    V4_DN_Os);
      AddToPreviewStr(PList,V4N_Cs,    V4_DN_Cs);
      AddToPreviewStr(PList,V4N_Lg,    V4_DN_Lg);
      AddToPreviewStr(PList,V4N_Ti,    V4_DN_Ti);

{File names ---------------------------------------------------------}
      Add('');
      Add(uppercase(V4FileNameKey));
      AddToPreviewStr(PList,V4N_Tk,    V4_FN_Tk);
      AddToPreviewStr(PList,V4N_Tm,    V4_FN_Tm);
      AddToPreviewStr(PList,V4N_Rm,    V4_FN_Rm);
      AddToPreviewStr(PList,V4N_Ee,    V4_FN_Ee);
      AddToPreviewStr(PList,V4N_Eo,    V4_FN_Eo);
      AddToPreviewStr(PList,V4N_Fs,    V4_FN_Fs);
      AddToPreviewStr(PList,V4N_Oc,    V4_FN_Oc);
      AddToPreviewStr(PList,V4N_Rg,    V4_FN_Rg);
      AddToPreviewStr(PList,V4N_Cd,    V4_FN_Cd);
      AddToPreviewStr(PList,V4N_St,    V4_FN_St);
      AddToPreviewStr(PList,V4N_S9,    V4_FN_S9);
      AddToPreviewStr(PList,V4N_Bn,    V4_FN_Bn);
      AddToPreviewStr(PList,V4N_Mo,    V4_FN_Mo);
      AddToPreviewStr(PList,V4N_Os,    V4_FN_Os);
      AddToPreviewStr(PList,V4N_Cs,    V4_FN_Cs);
      AddToPreviewStr(PList,V4N_Lg,    V4_FN_Lg);
      AddToPreviewStr(PList,V4N_Ti,    V4_FN_Ti);
      AddToPreviewStr(PList,V4N_S19DefaultExt,V4S19DefaultExt);

{Terminal Window -----------------------------}
      Add('');
      Add(uppercase(V4TerminalKey));
      AddToPreviewInt(PList,V4N_InterCharDelay  ,  V4InterCharDelay   );
      AddToPreviewBln(PList,V4N_TermWordWrap    ,  V4TermWordWrap     );
      AddToPreviewBln(PList,V4N_ShowNonDispChars,  V4ShowNonDispChars );
      AddToPreviewBln(PList,V4N_LocalEchoOn     ,  V4LocalEchoOn      );
      AddToPreviewBln(PList,V4N_SendOnEnter     ,  V4SendOnEnter      );
      AddToPreviewStr(PList,V4N_OnEnterSend     ,  V4OnEnterSend      );
      AddToPreviewStr(PList,V4N_NewlineOnReceive,  V4NewlineonReceive      );
      AddToPreviewStr(PList,V4N_CloseOnReceive  ,  V4CloseOnReceive   );
    end; // with PList do
    PreviewForm.Memo1.Clear;
    PreviewForm.Memo1.Lines.Assign(PList);
    with ImportForm.ConfigListBox do
    begin
      PreviewForm.Caption := 'Previewing Configuration - ' +
                              Items[ItemIndex];
    end;
  finally
    PList.Free;
  end;
end; // procedure FillPreview

procedure ReadInV4Data(ConfigName : string);
begin
  if copy(ConfigName,1,17) = 'LAST_USED_IN_VER_'
  then ReadVer4Registry(V4xxRoot)
  else ReadVer4Registry(UserRoot + ConfigName +'\');
  FillPreview;
end; // procedure ReadInV4Data

procedure SaveProject(FN : string);
var
  LocalProj             : tProjDataRecord;
  Buf                   : string;
  BootMacroList,MemList : tStringList;
  EqualPos,i            : integer;
begin
  BootMacroList := tStringList.Create;
  MemList       := tStringList.Create;
  LocalProj     := DefaultProjData;
  ReadMCUDataFile(MCUData,ImportForm.MCUComboBox.Text,LocalProj);
  try
    with LocalProj,ImportForm do
    begin
      MCUType               := MCUComboBox.Text;
//      BytesToList           := 16;
//      LinesToUnassemble     := 5;
      AutoConnectAtStartUp  := false;
      AutoResetAfterConnect := false;
      WarnOnBprot           := V4ShowBPROTWarning;
      WarnOnVprog           := V4ShowBurnWarnings;
      NoBreakHandshake      := V4NoBreakBoot;
//      FN_CtrlRegs           := '';
//      FN_Opcodes            := '';

{COM Port Tab}
      PortToUse             := V4PortToUse;
      UploadBaudRate        := V4UploadBaud;
      CommsBaudRate         := V4CommsBaud;
      RemoteResetAvailable  := not (V4ResetPulse = 0);
      ResetUsesDTR          := (V4ResetPulse = 1) or (V4ResetPulse = 2);
      ResetPulseGoesPos     := (V4ResetPulse = 2) or (V4ResetPulse = 4);

{Macros Tab}
      RunBootScript         := rbsBoot;

      if V4HPRIOFlag
      then BootMacroList.Add('R HPRIO=' + IntToHex(V4ByteToSend,2));
      if V4BootS19Flag and (length(V4BootS19Name) > 0)
      then BootMacroList.Add('LD "' + TokenizeFileName(V4BootS19Name) + '"');
      BootScript            := BootMacroList.Text;

      HasAssociatedMacro    := V4BootMacroFlag;
      FN_Macro              := V4BootMacroName;
      AbortOnFormatError    := not V4IgnoreRunTimeErr;
      AbortOnCmdFail        := not V4IgnoreRunTimeErr;

{Debug Tab}
      if V4UseSymbols
      then SymbolTableToUse := stAsS19 else SymbolTableToUse := stNone;

      FN_SymbolFixed        := V4_DN_St + V4_FN_St;

{Talkers Tab}
      if V4BootETalkerFlag then TalkerType := ttEeprom else TalkerType := ttRam;
//      FN_RamTalker          := V4_DN_Tk + V4_FN_Tk;  // Taken from MCU data
//      FN_RamMap             := V4_DN_Tm + V4_FN_Tm;  // Taken from MCU data
//      FN_EEOverlay          := V4_DN_Ee + V4_FN_Ee;  // Taken from MCU data
//      FN_EOverlay           := V4_DN_Eo + V4_FN_Eo;  // Taken from MCU data
      FN_EEMap              := V4_DN_Rm + V4_FN_Rm;
      FN_ExtMap             := '';
      ExtTalkerRequires     := ttNone;

{Memory Map Tab}
//      RamMem                :=       // Taken from MCU data
//      CtrlRegsMem           :=       // Taken from MCU data
//      MaskRomMem            :=       // Taken from MCU data
//      EepromMem             :=       // Taken from MCU data
//      EpromMem              :=       // Taken from MCU data
      MemList.Text := Bar2Return(V4RamStr);
      for i := 0 to MemList.Count - 1 do
      begin
        Buf := MemList[i];
        EqualPos := pos('=',Buf);
        if EqualPos > 0
        then begin
          delete(Buf,1,EqualPos);
          DelAllSpaceTab(Buf);
        end;
        MemList[i] := Buf;
      end;
      if (MemList.Count > 0) and (copy(MemList[0],1,1) = '0')
      then MemList.Delete(0);          // delete what was on-chip RAM
      ExtRamMem := MemList.Text;

      PageWriteMem          := Bar2Return(V4FlashStr);
      MaskRomVisible        := V4RomVisible;
//      EepromVisible         :=       // Taken from MCU data
//      EpromVisible          :=       // Taken from MCU data
      ExtRamVisible         := true;
      PageWriteVisible      := length(V4FlashStr) > 0;
      DataPageSize          := V4DataPageSize;

{Notes Tab}
      with ImportForm.ConfigListBox do
      begin
        Buf := Items[ItemIndex];
      end;
      Notes := '(Note automatically generated by JBug11 on import)' + CRLF +
               'Imported from JBug11 version ' + MostRecent + CRLF +
               'From configuration: ' + Buf;
{File Names}
      FN_LoadS19            := V4_DN_S9 + V4_FN_S9;
      FN_LoadBinary         := V4_DN_Bn + V4_FN_Bn;
      
{Terminal variables}
      InterCharDelay        := V4InterCharDelay;
      ShowNonDispChars      := V4ShowNonDispChars;
      LocalEchoOn           := V4LocalEchoOn;
      SendOnEnter           := V4SendOnEnter;
      OnEnterSend           := V4OnEnterSend;
      NewlineOnReceive      := V4NewlineOnReceive;
      CloseOnReceive        := V4CloseOnReceive;
    end; // with LocalProj,ImportForm do
    WriteProjectFile(LocalProj,FN);
  finally
    BootMacroList.Free;
    MemList.Free;
  end; // try.. finally..
end; // procedure SaveProject

procedure FillConfigListBox;
var
  RegInfo    : tRegistry;
  SubKeys    : tStringList;
begin
  ImportForm.ConfigListBox.Clear;
  RegInfo := tRegistry.Create;
  SubKeys := tStringList.Create;
  try
    if length(UserRoot) > 0
    then begin
      RegInfo.OpenKey(UserRoot,false);
      RegInfo.GetKeyNames(SubKeys);
      RegInfo.CloseKey;
      ImportForm.ConfigListBox.Items.Assign(SubKeys);
    end;
//    else ImportForm.ConfigListBox.Items.Add(CRLF +
//                                            'No saved user' + CRLF +
//                                            'configurations found');
    if length(V4xxRoot) > 0
    then ImportForm.ConfigListBox.Items.Insert(0,'LAST_USED_IN_VER_' + MostRecent);
  finally
    RegInfo.Free;
    SubKeys.Free;
  end;
end; // procedure FillConfigListBox

procedure TImportForm.FormShow(Sender: TObject);
var
  Buf : string;
begin
  GetRootNames(V4xxRoot,MostRecent,UserRoot);
  FillConfigListBox;
{Read in the MCU data}
  MCUData := tStringList.Create;
  MCUList := tStringList.Create;
  Buf := DetokenizeFileName(FN_MCUData);
  if FileExists(Buf)
  then begin
    MCUData.LoadFromFile(Buf);
    GetCFGSectionNames(MCUData,MCUList);
    MCUComboBox.Items.Assign(MCUList);
  end
  else begin
    MCUData.Clear;
    MCUList.Clear;
    MCUComboBox.Items.Clear;
  end;
end; // procedure TImportForm.FormShow

procedure TImportForm.FormHide(Sender: TObject);
begin
  MCUData.Free;
  MCUList.Free;
end; // procedure TImportForm.FormHide

procedure TImportForm.ConfigListBoxClick(Sender: TObject);
var Buf : string;
begin
  Buf := ConfigListBox.Items[ConfigListBox.ItemIndex];
  ReadInV4Data(Buf);
  ProjFileEdit.Text    := '[JBug11]\Projects\' + Buf + ProjExt;
  Step2Label.Enabled   := true;
  MCUComboBox.Text     := '';
  MCUComboBox.Enabled  := true;
  Step3Label.Enabled   := false;
  ProjFileEdit.Enabled := false;
  SaveBtn.Enabled      := false;
  SaveAsBtn.Enabled    := false;
end;

procedure TImportForm.SaveBtnClick(Sender: TObject);
var
  FN : string;
begin
  FN := DetokenizeFileName(ImportForm.ProjFileEdit.Text);
  if FileExists(FN)
  then begin
    if  MessageDlg(FN + ' already exists.' + CRLF +
                   'Do you want to replace it?',
                   mtWarning,[mbYes,mbNo],
                   0) = mrNo
    then exit;
  end;
  SaveProject(FN);
  ConfigListBoxClick(nil);
end; // procedure TImportForm.SaveBtnClick

procedure TImportForm.PreviewBtnClick(Sender: TObject);
begin
  PreviewForm.Show;
end;

procedure TImportForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  PreviewForm.Close;
end;

procedure TImportForm.MCUComboBoxChange(Sender: TObject);
var Flag : boolean;
begin
  Flag := length(MCUComboBox.Text) <> 0;
  Step3Label.Enabled   := Flag;
  ProjFileEdit.Enabled := Flag;
  SaveBtn.Enabled      := Flag;
  SaveAsBtn.Enabled    := Flag;
end;

procedure TImportForm.FormCreate(Sender: TObject);
begin
  Left := 0;
  Top  := 0;
end;

procedure TImportForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(IDH_IMPORT);
end; // procedure TImportForm.HelpBtnClick

procedure TImportForm.SaveAsBtnClick(Sender: TObject);
var Buf : string;
begin
  Buf := DeTokenizeFileName(ProjFileEdit.Text);
  if SaveAsDialogOK(Buf,ftProject)
  then begin
    ProjFileEdit.Text := TokenizeFileName(Buf);
    SaveProject(Buf);
    ConfigListBoxClick(nil);
  end;  
end; // procedure TImportForm.BrowseBtnClick

end.


