unit IndRW;

interface

uses Classes,SysUtils,TypeDefs;

type
  tIndRWAction = (imNone,imReadRegs,imWriteRegs,imReadMem,imWriteMem);

procedure RWIndMemory(StartAddr,EndAddr            : word;
                      RWAction                     : tIndRWAction;
                      UpdateProgress,HandleOverlay : boolean);
procedure RWIndRegister(RegAddr         : word;        // read one register byte
                        RWAction        : tIndRWAction;
                        LeaveOvlyLoaded : boolean);
procedure ReadIndirectMap(const S19Map : tList);
procedure WriteIndirectMap(const WriteMap       : tList;
                                 HandleProgress : boolean);

var
  IndOverlay : tOverlayType = oyNone;

implementation

uses Comms1, LoadUnit, MiscUnit, Main, MemUnit;

procedure RWIndMemory(StartAddr,EndAddr            : word;
                      RWAction                     : tIndRWAction;
                      UpdateProgress,HandleOverlay : boolean);
var
  TxByte,RxByte,CmdByte   : byte;
  CmdOK                   : boolean;
  EMsg                    : string;
  PageTopAddr,PageBotAddr : longword;
  w,x,PageSize            : word;
  ProgressCount           : integer;
begin
  CmdOK         := true;
  ProgressCount := 0;
  w             := StartAddr;
  case RWAction of
    imReadMem  : begin
                   CmdByte := $33;
                   EMsg := 'Comms error reading indirect memory';
                   PageSize := 256;  // limited by talker max value of numbytes
                 end;
    imWriteMem : begin
                   CmdByte := $34;
                   EMsg := 'Comms error writing indirect memory';
                   PageSize := ProjData.IMPageSize;
                 end
  else exit
  end;

  try
    with CommsForm.ComPort do
    begin
      ClearBuffer(true,true);

{Overlay need only actually be loaded if the talker is a RAM-resident one.
 A talker in external memory is assumed to provide all the necessary routines
 without the need for overlays, as is an EEPROM-resident one}
      if HandleOverlay and (ProjData.TalkerType = ttRam)
      then LoadOverlay(oyIndMem);

      PageBotAddr := StartAddr;
      PageTopAddr := ((StartAddr div PageSize) + 1)*PageSize - 1;
      if EndAddr < PageTopAddr then PageTopAddr := EndAddr;

      while w <= EndAddr do
      begin
        TxByte := CmdByte;
        Write(TxByte,1);                               // Command
        Read(RxByte,1);
        CmdOK := RxByte = ($FF - TxByte);
        if not CmdOK then exit;

        TxByte := byte(PageTopAddr - PageBotAddr + 1); // NumBytes
        Write(TxByte,1);
        TxByte := hi(w);                               // Address
        Write(TxByte,1);
        TxByte := lo(w);
        Write(TxByte,1);

        for x := PageBotAddr to PageTopAddr do
        begin
          TxByte := IndMemory[x];
          Write(TxByte,1);
          Read(RxByte,1);
          IndMemory[x] := RxByte;

          ProgressCount := ProgressCount + 1;
          if UpdateProgress and (ProgressCount > 15)
          then begin
            MainForm.ProgressBar1.StepBy(16);
            ProgressCount := 0;
          end;
        end;

        PageBotAddr := PageTopAddr + 1;
        if PageBotAddr > $FFFF then break;
        w := PageBotAddr;
        PageTopAddr := ((PageBotAddr div PageSize) + 1)*PageSize - 1;
        if PageTopAddr > EndAddr then PageTopAddr := EndAddr;
      end; // while w <= EndAddr do
    end; // with ProjData,CommsForm.ComPort do
    if HandleOverlay then UnloadOverlay;
  finally
    if not CmdOK
    then begin
      UpdateDiagnostics(w,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
  end; // try.. finally..
end; // procedure RWIndMemory

procedure  RWIndRegister(RegAddr         : word;
                         RWAction        : tIndRWAction;
                         LeaveOvlyLoaded : boolean);
var
  TxByte,RxByte : byte;
  CmdOK         : boolean;
  EMsg          : string;
begin
  CmdOK         := true;
  case RWAction of
    imReadRegs  : begin
                    TxByte := $31;
                    EMsg := 'Comms error reading indirect register';
                  end;
    imWriteRegs : begin
                    TxByte := $32;
                    EMsg := 'Comms error writing indirect register';
                  end
  else exit
  end;
  try
    with CommsForm.ComPort do
    begin
      ClearBuffer(true,true);

{Overlay need only actually be loaded if the talker is a RAM-resident one.
 A talker in external memory is assumed to provide all the necessary routines
 without the need for overlays, as is an EEPROM-resident one}
      if ProjData.TalkerType = ttRam
      then begin
        if IndOverlay <> oyIndRegs
        then begin
          LoadOverlay(oyIndRegs);
          IndOverlay := oyIndRegs;
        end;
      end;

      Write(TxByte,1);                               // command byte
      Read(RxByte,1);
      CmdOK := RxByte = ($FF - TxByte);
      if not CmdOK then exit;

      TxByte := 1;                                   // NumBytes
      Write(TxByte,1);
      TxByte := hi(RegAddr);                         // Address
      Write(TxByte,1);
      TxByte := lo(RegAddr);
      Write(TxByte,1);

      TxByte := IndMemory[RegAddr];
      Write(TxByte,1);
      Read(RxByte,1);
      IndMemory[RegAddr] := RxByte;

      if IndOverlay = oyIndRegs
      then begin
        if not LeaveOvlyLoaded
        then begin
          UnloadOverlay;
          IndOverlay := oyNone;
        end;
      end;
    end; // with CommsForm.ComPort do
  finally
    if not CmdOK
    then begin
      UpdateDiagnostics(RegAddr,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
  end; // try.. finally..
end; // procedure RWIndRegister

{Procedure to read indirect memory according to a supplied map.}
procedure ReadIndirectMap(const S19Map : tList);
var
  i : integer;
begin
  try
    with MainForm.ProgressBar1 do
    begin
      Max      := TotalBytesInMap(S19Map);
      Position := 0;
    end; // with MainForm.ProgressBar1 do
    for i := 0 to S19Map.Count - 1 do
    begin
      with tMemRecordPtr(S19Map[i])^ do
        RWIndMemory(SA,EA,imReadMem,true,true);
    end;
  finally
    MainForm.ProgressBar1.Position := 0;
  end;
end; // procedure ReadMCUMap

{Procedure to write to indirect memory according to a supplied map.
 Handles overlays as necessary. Returns true if no error encountered}
procedure WriteIndirectMap(const WriteMap       : tList;
                                 HandleProgress : boolean);
var
  i    : integer;
  Tptr : tMemRecordPtr;
  Buf  : string;
  OverlayNeeded,OverlayLoaded : tOverlayType;
begin
  OverlayLoaded      := oyNone;
  if HandleProgress
  then begin
    with MainForm.ProgressBar1 do
    begin
      Max      := TotalBytesInMap(WriteMap);
      Position := 0;
    end; // with MainForm.ProgressBar1 do
  end;

  WriteMap.Sort(SortByAreas1);
  try
    for i := 0 to WriteMap.Count - 1 do
    begin
      Tptr := WriteMap[i];
      with Tptr^,ProjData do
      begin
        if Area <> maIndMem then exit;
{Overlays need only actually be loaded if the talker is a RAM-resident one.
 A talker in eeprom or external memory is assumed to provide the necessary
 routine for writing to indirect memory without the need for overlays}
        if TalkerType = ttRam
        then OverlayNeeded := oyIndMem
        else OverlayNeeded := oyNone;

{Load overlays as necessary}
        if OverlayNeeded <> OverlayLoaded
        then begin
          if OverlayLoaded <> oyNone
          then begin
            UnloadOverlay;
            OverlayLoaded := oyNone;
          end; // if OverlayLoaded <> oyNone

          if OverlayNeeded <> oyNone
          then begin
            LoadOverlay(OverlayNeeded);
            OverlayLoaded := OverlayNeeded;
          end; // if OverlayNeeded <> oyNone
        end; // if OverlayNeeded <> OverlayLoaded

        if HandleProgress
        then begin
          Buf := 'Writing ' + MemoryVNArray[Area];
          SpaceTo(Buf,24);
          Buf := Buf + IntToHex(SA,4) + '..' + IntToHex(EA,4);
          MainForm.OWEdit.Lines.Add(Buf);
        end;

        RWIndMemory(SA,EA,imWriteMem,true,false);

      end; // with Tptr^,ProjData do
    end; // for i := 0 to WriteMap.Count - 1 do

    if OverlayLoaded <> oyNone then UnloadOverlay;
  finally
    MainForm.ProgressBar1.Position := 0;
  end; // try.. finally..
end; // function WriteIndirectMap

end.
