unit InitUnit;

interface

uses
  Forms, Controls, Classes, SysUtils, Registry, Dialogs, TypeDefs, Windows,
  ShlObj, Graphics;
{
uses Classes, Forms, SysUtils, Registry, CPort, TypeDefs, Dialogs, Controls,
     ShlObj, Windows;
 }


function  ReadRegBln(const RegInfo   : tRegistry;
                           ValueName : string;
                     var   Value     : boolean) : boolean;
function  ReadRegStr(const RegInfo   : tRegistry;
                           ValueName : string;
                     var   Value     : string) : boolean;
function  ReadRegInt(const RegInfo   : tRegistry;
                           ValueName : string;
                     var   Value     : integer) : boolean;
procedure ReadLayout(LocalRoot : string);
procedure ReadGeneral(    LocalRoot       : string;
                      var ProjectFileName : string);
procedure WriteRegistry(LocalRoot : string);
function  RegEntryExists : boolean;
procedure SetupFileAssociation;
procedure UpdateRegInfo;
Procedure SetOWTabs;

var
  ShowMEForm     : boolean = false;
  ShowSymbolForm : boolean = false;
  ShowCalcForm   : boolean = false;
  ShowMapForm    : boolean = false;

implementation

uses Main, AddToWatchUnit, CalcUnit, SymbolUnit, MiscUnit, FileUnit,
     MacroUnit, MemUnit, ImportUnit, AsciiUnit, TermUnit, SettingsUnit;

function  ReadRegBln(const RegInfo   : tRegistry;
                           ValueName : string;
                     var   Value     : boolean) : boolean;
begin
  with RegInfo do
  begin
    Result := ValueExists(ValueName);
    if Result then Value := ReadBool(ValueName);
  end;
end; // function  ReadRegBln

function  ReadRegStr(const RegInfo   : tRegistry;
                           ValueName : string;
                     var   Value     : string) : boolean;
begin
  with RegInfo do
  begin
    Result := ValueExists(ValueName);
    if Result then Value := ReadString(ValueName);
  end;
end; // function  ReadRegStr

function  ReadRegInt(const RegInfo   : tRegistry;
                           ValueName : string;
                     var   Value     : integer) : boolean;
begin
  with RegInfo do
  begin
    Result := ValueExists(ValueName);
    if Result then Value := ReadInteger(ValueName);
  end;
end; // function  ReadRegInt

procedure ReadLayout(LocalRoot : string);
var
  RegInfo : tRegistry;
  i       : integer;
  Flag    : boolean;
begin
  RegInfo := tRegistry.Create;
  try
    with RegInfo do
    begin
{Read in the various items concerned with the position of the main form}
      OpenKey(LocalRoot + LayoutKey,false);
      with MainForm do
      begin
        if ReadRegInt(RegInfo,  VN_MainLeft       ,i) then Left := i;
        if ReadRegInt(RegInfo,  VN_MainTop        ,i) then Top := i;
        if ReadRegInt(RegInfo,  VN_MainWidth      ,i) then Width := i;
        if ReadRegInt(RegInfo,  VN_MainHeight     ,i) then Height := i;
        if ReadRegInt(RegInfo,  VN_CHEditHeight   ,i) then CHEdit.Height := i;
        if ReadRegInt(RegInfo,  VN_SidebarWidth   ,i) then SidebarPanel.Width := i;
        if ReadRegInt(RegInfo,  VN_BPPanelHeight  ,i) then BPPanel.Height := i;

        if not ReadRegInt(RegInfo,  VN_OWFontSize     ,OWFontSize)
        then OWFontSize := 8;

        if not ReadRegInt(RegInfo,  VN_CHFontSize     ,CHFontSize)
        then CHFontSize := 10;

        OWEdit.WordWrap := ReadRegBln(RegInfo,VN_OWWordWrap,Flag) and Flag;

        if ReadRegInt(RegInfo,VN_BoldColor,i)
        then BoldColor := tColor(i)
        else BoldColor := clBlack;
      end; // with MainForm do

      with MEForm do
      begin
        if ReadRegInt(RegInfo,  VN_MacroLeft      ,i) then Left := i;
        if ReadRegInt(RegInfo,  VN_MacroTop       ,i) then Top := i;
        if ReadRegInt(RegInfo,  VN_MacroWidth     ,i) then Width := i;
        if ReadRegInt(RegInfo,  VN_MacroHeight    ,i) then Height := i;
        if ReadRegBln(RegInfo,  VN_MacroKOT       ,Flag) and Flag
        then FormStyle := fsStayOnTop else FormStyle := fsNormal;
        if ReadRegInt(RegInfo,  VN_MEFontSize     ,MEFontSize)
        then MEEdit.Font.Size := MEFontSize;
//        MEEdit.WordWrap := ReadRegBln(RegInfo,VN_MacroWrap,Flag) and Flag;
        ReadRegBln(RegInfo,     VN_ShowMEForm     ,ShowMEForm);
      end; // with MEForm do

      with SymbolForm do
      begin
        if ReadRegInt(RegInfo,  VN_SymbolLeft     ,i) then Left := i;
        if ReadRegInt(RegInfo,  VN_SymbolTop      ,i) then Top := i;
        if ReadRegInt(RegInfo,  VN_SymbolHeight   ,i) then Height := i;
        if ReadRegBln(RegInfo,  VN_SymbolKOT      ,Flag) and Flag
        then FormStyle := fsStayOnTop else FormStyle := fsNormal;
        if ReadRegBln(RegInfo,  VN_SortByAddress  ,Flag)
        then begin
          AddrRadBtn.Checked := Flag;
          NameRadBtn.Checked := not Flag;
        end;
        ReadRegBln(RegInfo,     VN_ShowSymbolForm ,ShowSymbolForm);
      end; // with SymbolForm do

      with CalcForm do
      begin
        if ReadRegInt(RegInfo,  VN_ConverterLeft  ,i) then Left := i;
        if ReadRegInt(RegInfo,  VN_ConverterTop   ,i) then Top := i;
        if ReadRegBln(RegInfo,  VN_ConverterKOT   ,Flag) and Flag
        then FormStyle := fsStayOnTop else FormStyle := fsNormal;
        ReadRegBln(RegInfo,     VN_ShowCalcForm   ,ShowCalcForm);
        ReadRegBln(RegInfo,     VN_ShowMapForm    ,ShowMapForm);
      end; // with ConverterForm do

      with AsciiForm do
      begin
        if ReadRegInt(RegInfo,  VN_AsciiLeft      ,i) then Left := i;
        if ReadRegInt(RegInfo,  VN_AsciiTop       ,i) then Top := i;
      end; // with AsciiForm do

      with MapForm do
      begin
        if ReadRegInt(RegInfo,  VN_MemMapLeft     ,i) then Left   := i;
        if ReadRegInt(RegInfo,  VN_MemMapTop      ,i) then Top    := i;
        if ReadRegInt(RegInfo,  VN_MemMapHeight   ,i) then Height := i;
        if ReadRegBln(RegInfo,  VN_MemMapKOT      ,Flag) and Flag
        then FormStyle := fsStayOnTop else FormStyle := fsNormal;
        ReadRegBln(RegInfo,     VN_ShowMapForm    ,ShowMapForm);
      end; // with MapForm do

      with TermForm do
      begin
        if ReadRegInt(RegInfo,VN_TWLeft     ,i) then Left   := i;
        if ReadRegInt(RegInfo,VN_TWTop      ,i) then Top    := i;
        if ReadRegInt(RegInfo,VN_TWWidth    ,i) then Width  := i;
        if ReadRegInt(RegInfo,VN_TWHeight   ,i) then Height := i;
        AsciiEdit.WordWrap := ReadRegBln(RegInfo,VN_TermWordWrap,Flag) and Flag;
        HexEdit.WordWrap   := AsciiEdit.WordWrap;
        if ReadRegBln(RegInfo,VN_TWOnTop    ,Flag) and Flag
        then FormStyle := fsStayOnTop else FormStyle := fsNormal;
        if ReadRegInt(RegInfo,VN_AsciiWidth ,i) then AsciiPanel.Width := i;
        if ReadRegInt(RegInfo,VN_TWFontSize ,i)
        then begin AsciiEdit.Font.Size := i; HexEdit.Font.Size := i; end;
      end; // with TermForm do
{
      with SettingsForm do
      begin
        if ReadRegBln(RegInfo,VN_IMTabVisible,Flag) and Flag
        then IMTab.Visible := true else IMTab.Visible := false;
      end; // with SettingsForm do
}
      CloseKey;
    end; // with RegInfo do
  finally
    RegInfo.Free;
  end;
end; // procedure ReadLayout

procedure ReadGeneral(    LocalRoot       : string;
                      var ProjectFileName : string);
var
  RegInfo : tRegistry;
  Buf     : string;
  i       : integer;
begin
  RegInfo := tRegistry.Create;
  try
    with RegInfo do
    begin
      OpenKey(LocalRoot + GeneralKey,false);

      if ReadRegStr(RegInfo,  VN_PFList            ,Buf)
      then begin
        PFList.Text := Bar2Return(Buf);
        for i := 0 to PFList.Count - 1 do
          PFList[i] := DetokenizeFileName(PFList[i]);
      end
      else PFList.Clear;

      if ReadRegStr(RegInfo,  VN_FN_ProjectFile    ,Buf)
      then ProjectFileName := DetokenizeFileName(Buf)
      else ProjectFileName := '';
      
      if ReadRegStr(RegInfo,  VN_FN_ErrorReport    ,Buf)
      then FN_ErrorReport := DetokenizeFileName(Buf);

      if ReadRegInt(RegInfo,VN_BPSortOrder,i) and (i >= 0) and (i <= 2)
      then BPSortOrder := tBPSortOrder(i) else BPSortOrder := soOrderAdded;

      ReadRegBln(RegInfo,VN_SymFromList,SymFromList);
      CloseKey;
    end; // with RegInfo do
  finally
    RegInfo.Free;
  end;
end; // procedure ReadGeneral

procedure WriteRegistry(LocalRoot : string);
var
  RegInfo  : tRegistry;
  i        : integer;
  Flag     : boolean;
begin
  RegInfo := tRegistry.Create;
  try
    with RegInfo do
    begin
      OpenKey(LocalRoot + LayoutKey,true);
      with MainForm do
      begin
        WriteInteger(VN_MainLeft          ,Left);
        WriteInteger(VN_MainTop           ,Top);
        WriteInteger(VN_MainWidth         ,Width);
        WriteInteger(VN_MainHeight        ,Height);
        WriteInteger(VN_CHEditHeight      ,CHEdit.Height);
        WriteInteger(VN_SidebarWidth      ,SidebarPanel.Width);
        WriteInteger(VN_BPPanelHeight     ,BPPanel.Height);
        WriteInteger(VN_OWFontSize        ,OWFontSize);
        WriteInteger(VN_CHFontSize        ,CHFontSize);
        WriteBool(VN_OWWordWrap           ,OWEdit.WordWrap);
        WriteInteger(VN_BoldColor,        BoldColor);
      end; // with MainForm do

      with MEForm do
      begin
        WriteInteger(VN_MacroLeft         ,Left);
        WriteInteger(VN_MacroTop          ,Top);
        WriteInteger(VN_MacroWidth        ,Width);
        WriteInteger(VN_MacroHeight       ,Height);
        Flag := FormStyle = fsStayOnTop;
        WriteBool(VN_MacroKOT             ,Flag);
//        WriteBool(VN_MacroWrap            ,MEEdit.WordWrap);
        WriteBool(VN_ShowMEForm           ,Visible);
      end; // with MEForm do

      with SymbolForm do
      begin
        WriteInteger(VN_SymbolLeft        ,Left);
        WriteInteger(VN_SymbolTop         ,Top);
        WriteInteger(VN_SymbolHeight      ,Height);
        Flag := FormStyle = fsStayOnTop;
        WriteBool(VN_SymbolKOT            ,Flag);
        WriteBool(VN_SortByAddress        ,AddrRadBtn.Checked);
        WriteBool(VN_ShowSymbolForm       ,Visible);
      end; // with SymbolForm do

      with CalcForm do
      begin
        WriteInteger(VN_ConverterLeft     ,Left);
        WriteInteger(VN_ConverterTop      ,Top);
        Flag := FormStyle = fsStayOnTop;
        WriteBool(VN_ConverterKOT         ,Flag);
        WriteBool(VN_ShowCalcForm         ,Visible);
      end; // with ConverterForm do

      with AsciiForm do
      begin
        WriteInteger(VN_AsciiLeft         ,Left);
        WriteInteger(VN_AsciiTop          ,Top);
      end; // with AsciiForm do

      with MapForm do
      begin
        WriteInteger(VN_MemMapLeft        ,Left);
        WriteInteger(VN_MemMapTop         ,Top);
        WriteInteger(VN_MemMapHeight      ,Height);
        Flag := FormStyle = fsStayOnTop;
        WriteBool(VN_MemMapKOT            ,Flag);
        WriteBool(VN_ShowMapForm          ,Visible);
      end; // with MapForm do

      with TermForm do
      begin
        WriteInteger(VN_TWLeft            ,Left);
        WriteInteger(VN_TWTop             ,Top);
        WriteInteger(VN_TWWidth           ,Width);
        WriteInteger(VN_TWHeight          ,Height);
        WriteBool(VN_TermWordWrap         ,AsciiEdit.WordWrap);
        Flag := FormStyle = fsStayOnTop;
        WriteBool(VN_TWOnTop              ,Flag);
        WriteInteger(VN_AsciiWidth        ,AsciiPanel.Width);
        WriteInteger(VN_TWFontSize        ,AsciiEdit.Font.Size);
      end; // with TermForm do
{
      with SettingsForm do
      begin
        WriteBool(VN_IMTabVisible         ,IMTab.Visible);
      end;
}
      CloseKey;

      OpenKey(LocalRoot + GeneralKey,true);

      for i := 0 to PFList.Count - 1 do
        PFList[i] := TokenizeFileName(PFList[i]);
      WriteString( VN_PFList           ,Return2Bar(PFList.Text));
      WriteString( VN_FN_ProjectFile   ,TokenizeFileName(Prog.FN_Project));
      WriteString( VN_FN_ErrorReport   ,TokenizeFileName(FN_ErrorReport));
      WriteInteger(VN_BPSortOrder      ,ord(BPSortOrder));
      CloseKey;
    end;
  finally
    RegInfo.Free;
  end;
end; // procedure WriteRegistry

function  RegEntryExists : boolean;
var
  RegInfo    : tRegistry;
begin
  RegInfo    := tRegistry.Create;
  try
    Result := RegInfo.KeyExists(RegRoot);
  finally
    RegInfo.Free;
  end;
end; // function  RegEntryExists

{Following procedure registers the project file extension with the Windows
 Registry, and associates the exe file with it. This procedure is based on
 FAQ: TI4646D from the Borland Developer Network}
procedure SetupFileAssociation;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
{Set the root key to HKEY_CLASSES_ROOT}
    Reg.RootKey := HKEY_CLASSES_ROOT;
{Open the key, with the possibility to create the key if it doesn't exist}
    Reg.OpenKey(ProjExt, True);
{Write the file type description to it.
 This adds HKEY_CLASSES_ROOT\.abc\(Default) = 'Project1.FileType' }
    Reg.WriteString('', ProjFileType);
    Reg.CloseKey;

{Now create an association for that file type}
    Reg.OpenKey(ProjFileType, True);
{This adds HKEY_CLASSES_ROOT\Project1.FileType\(Default) = 'Project1 File'
 This is what you see in the file type description in Explorer}
    Reg.WriteString('',ProjFileDesc);
    Reg.CloseKey;

{Now write the default icon for my file type
 This adds HKEY_CLASSES_ROOT\Project1.FileType\DefaultIcon\(Default) =
           'Application Dir\Project1.exe,0'  }
    Reg.OpenKey(ProjFileType + '\DefaultIcon', True);
    Reg.WriteString('', Application.ExeName + ',0');
    Reg.CloseKey;

{Now write the open action in Explorer }
    Reg.OpenKey(ProjFileType + '\Shell\Open', True);
    Reg.WriteString('', '&Open');
    Reg.CloseKey;

{Write what application to use to open it with. This adds
     HKEY_CLASSES_ROOT\Project1.FileType\Shell\Open\Command
     (Default) = '"Application Dir\Project1.exe" "%1"'
 Your application must scan the command line parameters to see what file
 was passed to it.  }
    Reg.OpenKey(ProjFileType + '\Shell\Open\Command', True);
    Reg.WriteString('', '"' + Application.ExeName + '" "%1"');
    Reg.CloseKey;

{Finally, we want Windows Explorer to realize we added our file type by
 using the SHChangeNotify API.  }
    SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, nil, nil);
  finally
    Reg.Free;
  end;
end;

procedure UpdateRegInfo;
var
  RegInfo    : tRegistry;
  SubKeys    : tStringList;
  i          : integer;
  MostRecent : string;
begin
  RegInfo    := tRegistry.Create;
  SubKeys    := tStringList.Create;
  try
    with RegInfo do
    begin
{Exit immediately if keys for the current version are defined}
      if KeyExists(WorkRoot) then exit;
{Build a list of all the previous version keys}
      if OpenKeyReadOnly(RegRoot)
      then begin
        GetKeyNames(SubKeys);
        CloseKey;
      end;
{Find the most recent version key}
      if SubKeys.Count > 0
      then begin
        SubKeys.Sort;
        i := 0;
        while i < SubKeys.Count do
        begin
          if SubKeys[i][1] in DecChars
          then begin
            MostRecent := SubKeys[i];
            i := i + 1;
          end else SubKeys.Delete(i);
        end;
      end;
{If most recent key is more recent than the new key then exit!}
      if MostRecent > ThisVersion then exit;
{Ask the user whether to transfer info}
      if KeyExists(RegRoot + MostRecent)
      then ReadInRoot := RegRoot + MostRecent + '\';
    end; // with RegInfo do
  finally
    RegInfo.CloseKey;
    RegInfo.Free;
    SubKeys.Free;
  end; // try.. finally..
end; // procedure UpdateRegInfo

{Set the tab positions in the Output Window}
Procedure SetOWTabs;
var
  Width8,i : integer; // width of eight characters, in points
//  PPI  : integer; // pixels per inch
begin
  with MainForm.OWEdit do
  begin
//    PPI := Font.PixelsPerInch;
    case byte(Font.Size) of
      8:  Width8 := 42;
      10: Width8 := 48;
      12: Width8 := 60
    else Width8 := 53;  // a really odd amount
    end;
    Paragraph.TabCount := 10;
    for i := 0 to Paragraph.TabCount - 1 do Paragraph.Tab[i] := (i+1)*Width8;
  end;
end; // Procedure SetOWTabs

end.

