unit ListUnit;

interface

uses Classes, SysUtils, Graphics, TypeDefs;

function ListCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
procedure ListLocal(MemType : tMemType);
procedure List;
procedure ListTalker(FirstAddr,LastAddr : word);
procedure ListIndirect;

implementation

uses Main, CmdUnit, Comms1, CheckUnit, LookupUnit, MemRW, IndRW;

var
  StartAddr,EndAddr : word;

function ListCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
var
  A1,A2,Val            : longword;
  ArgType              : tArgType;
  IsLocal,IsIndirect   : boolean;
  AreasUsed            : tMemAreaSet;
begin
  Result  := false;
  Comment := clcArgErr;
  A1 := 0; A2 := 0;
  IsLocal    := uppercase(CList[0]) = 'LL';
  IsIndirect := uppercase(CList[0]) = 'LI';

  if not NumArgsOK(CList,0,2,Comment) then exit;

  if not (IsLocal or IsIndirect) and
     not StateIsAllowed([psStopped,psRun,psTrace,psRunStop,psTraceStop],Comment)
  then exit;

  if IsIndirect and
     not StateIsAllowed([psStopped],Comment)
  then exit;

  if FoundArgOK(CList,1,[atNotExist,atWord,atLabel,atAsterisk],
                ArgType,Val)
  then begin
    case ArgType of
      atNotExist     : A1 := Prog.LUAddr;
      atAsterisk     : A1 := Prog.StackFrame.PC;
      atWord,atLabel : A1 := Val;
    end;
  end else exit;

  if FoundArgOK(CList,2,[atNotExist,atWord,atLabel,atLength],
                ArgType,Val)
  then begin
    case ArgType of
      atNotExist     : A2 := A1 + word(ProjData.BytesToList) - 1;
      atWord,atLabel : A2 := Val;
      atLength       : A2 := A1 + Val;
    end;
  end else exit;

{Check that the necessary overlay is available if this is an indirect memory
 list command, anda RAM-resident talker is being used. It is assumed that
 overlays are not necessary with a talker in external memory or Eeprom}
  if IsIndirect and (not OverlaysAvailableOK([maIndMem],Comment)) then exit;

{Check that: 1. A2 is larger than A1
             2. A2 <= $FFFF
             3. (No address range exclusions apply to the list command) }
  if (A2 < A1) or (A2 > $FFFF)
  then begin
    Comment := clcAddrRangeErr;     // Address range error
    exit;
  end;

{Check for address range exclusions}
  if IsIndirect
  then begin
    if not CheckAreas(A1,A2,[maIndMem],IndMemMap,AreasUsed,Comment)
    then exit;
  end; // if IsIndirect

  StartAddr := A1;
  EndAddr   := A2;
  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  if IsLocal then Command := cmdListLocal else Command := cmdList;
  if IsIndirect then Command := cmdListIndirect;
end; // function ListCheck

procedure ListLocal(MemType : tMemType);

var
  Buf     : string;
  i,j     : integer;

const
  Heading = '      +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F';

procedure AddAscii(var Buf : string);
var
  i,j,k  : integer;
  Meme   : string[2];
begin
  k := length(Buf);
  for j := k to 53 do Buf := Buf + ' ';
  j := 7;
  while j < k - 1 do
  begin
    meme := copy(Buf,j,2);
    if (Meme = '  ')
    then Buf := Buf + ' '
    else begin
      i := StrToInt('$' + Meme);
      if i in [$20..$7E]
      then Buf := Buf + chr(i)
      else Buf := Buf + '.';
    end;
    j := j + 3;
  end;
end; // procedure AddAscii

begin // procedure ListLocal
  with Prog,MainForm.OWEdit do
  begin
    if (LastCommand = cmdList) or (LastCommand = cmdListLocal)
    then begin end
    else begin
      SelAttributes.Color := clRed;
      Lines.Add(Heading);
      SelAttributes.Color := clBlack;
    end;

    i := StartAddr and $FFF0;
    Buf := format('%-6.4x',[i]);
    j := (EndAddr and $FFF0) + $0F + 1; // +1 to force output of final line
    while i <= j do
    begin
      if i < StartAddr
      then Buf := Buf + '   '
      else if i <= EndAddr
           then begin
             case MemType of
               mtIntrinsic : Buf := Buf + format('%-3.2x',[Memory[i]]);
               mtIndirect  : Buf := Buf + format('%-3.2x',[IndMemory[i]]);
             end
           end
           else Buf := Buf + '   ';
      inc(i);
      if i mod $10 = 0
      then begin
        AddAscii(Buf);
        Lines.Add(Buf);
        Buf := format('%-6.4x',[i]);
      end; // if i mod $10 = 0 then..
    end; // while i <= j do
    if EndAddr < $FFFF then Prog.LUAddr := EndAddr + 1;
  end; // with Prog,MainForm.OWEdit do
end; // procedure ListLocal

procedure List;
begin
  with MainForm.ProgressBar1 do
  begin
    Max      := EndAddr - StartAddr;
    Position := 0;
  end; // with MainForm.ProgressBar1 do
  ReadMCU64K(StartAddr,EndAddr,true);
  ListLocal(mtIntrinsic);
  MainForm.ProgressBar1.Position := 0;
end; // procedure List

procedure ListTalker(FirstAddr,LastAddr : word);
begin
  StartAddr := FirstAddr;
  EndAddr   := LastAddr;
  ListLocal(mtIntrinsic);
  Prog.LUAddr := 0;
end; // procedure ExtList

procedure ListIndirect;
begin
  with MainForm.ProgressBar1 do
  begin
    Max      := EndAddr - StartAddr + 1;
    Position := 0;
  end; // with MainForm.ProgressBar1 do
  RWIndMemory(StartAddr,EndAddr,imReadMem,true,true);
  ListLocal(mtIndirect);
  MainForm.ProgressBar1.Position := 0;
end; // procedure ListIndirect

end.

