unit LoadUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, TypeDefs;

type
  TLoadAddrForm = class(TForm)
    Label1: TLabel;
    LoadAddrEdit: TEdit;
    BitBtn1: TBitBtn;
    BitBtn3: TBitBtn;
    HelpBtn: TBitBtn;
    procedure LoadAddrEditKeyPress(Sender: TObject; var Key: Char);
    procedure HelpBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

function  S19StartAddr(const S19Listing : tStringList;
                       var   S9Addr     : word) : boolean;
function  BuildS19MapOK(const S19Listing : tStringList;
                        const S19Map     : tList;
                        const MemMap     : tList) : boolean;
function  LoadCheck(const CList   : tStringList;
                    var   Comment : tCmdLineComment;
                    var   Command : tCommand) : boolean;
function IndirectLoadCheck(const CList   : tStringList;
                           var   Comment : tCmdLineComment;
                           var   Command : tCommand) : boolean;
procedure LoadS19Local1(const S19Listing : tStringList;
                              MemType    : tMemType);
procedure LoadS19Local2(S19Listing : tStringList);
function  LoadS19MCU1(S19Listing : tStringList) : boolean;
procedure LoadS19MCU2(S19Listing : tStringList);
procedure LoadBinaryFileToLocal(MemType : tMemType);
procedure LoadBinaryFileToMCU;
procedure LoadS19FileToIndirect;
procedure LoadBinaryFileToIndirect;

procedure LoadOverlay(Overlay : tOverlayType);
procedure UnloadOverlay;

var
  LoadAddrForm: TLoadAddrForm;

implementation

{$R *.DFM}

uses Main, CmdUnit, MemRW, FileUnit, LookupUnit, CheckUnit, MemUnit,
     VerifyUnit, EraseUnit, SaveUnit, IndRW;

type
  tConditionTest = function(var w : word) : boolean;

const
  aaFlash    = $80;
  aaErase    = $40;
  aaUnderlap = $02;
  aaS19      = $01;

var
  FileName          : string;
  StartAddr,EndAddr : word;

{****************************************************************************}

procedure TLoadAddrForm.LoadAddrEditKeyPress(Sender: TObject;
  var Key: Char);
begin
{Always allow the backspace, return and tab keys}
  if (Key = ^H) or (Key = ^M) or (Key = ^I) then exit;
  if not (Key in HexChars) then Key := chr(0);
end; // procedure TLoadAddrForm.LoadAddrEditKeyPress

procedure TLoadAddrForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(IDH_LOAD_MEMORY);
end; // procedure TLoadAddrForm.HelpBtnClick

{****************************************************************************}

function  S19StartAddr(const S19Listing : tStringList;
                       var   S9Addr     : word) : boolean;
var
  i   : integer;
  Buf : string;
begin
  Result := false;
  for i := 0 to S19Listing.Count - 1 do
  begin
    Buf := S19Listing[i];
    if copy(Buf,1,2) = 'S9'
    then begin
      S9Addr := StrToInt('$' + copy(Buf,5,4));
      Result := true;
    end;
  end; // for i := 0 to S19Listing.Count - 1 do
end; // function  S19StartAddr

function BuildS19MapOK(const S19Listing : tStringList;
                       const S19Map     : tList;
                       const MemMap     : tList) : boolean;
var
  i,S19Line,NumChars,StartAddrIndex,EndAddrIndex : integer;
  Buf                                            : string;
  StartAddr,EndAddr                              : word;
  StartAddrArea,EndAddrArea                      : tMemArea;
  Tptr                                           : tMemRecordPtr;
begin
  ClearMemMap(S19Map);
  S19Map.Capacity := S19Listing.Count;
  for S19Line := 0 to S19Listing.Count - 1 do
  begin
    Buf := uppercase(S19Listing[S19Line]);
    if copy(Buf,1,2) <> 'S1' then continue;
{Get address of first byte to load}
    StartAddr := StrToInt('$' + copy(Buf,5,4));
{Get number of bytes in line}
    NumChars  := StrToInt('$' + copy(Buf,3,2)) - 3;
    if NumChars = 0 then showmessage(Buf);
{Calculate end address}
    EndAddr   := StartAddr + NumChars - 1;

    StartAddrArea  := GetMemArea(StartAddr,MemMap);
    StartAddrIndex := GetMemIndex(StartAddr,MemMap);
    EndAddrArea    := GetMemArea(EndAddr,MemMap);
    EndAddrIndex   := GetMemIndex(EndAddr,MemMap);

    if EndAddrIndex = StartAddrIndex
    then begin
      new(Tptr);
      with Tptr^ do
      begin
        Area := StartAddrArea;
        SA   := StartAddr;
        EA   := EndAddr;
        Desc := '';
      end;
      S19Map.Add(Tptr);
    end
    else begin
      new(Tptr);
      with Tptr^ do
      begin
        Area := StartAddrArea;
        SA   := StartAddr;
        EA   := tMemRecordPtr(MemMap[StartAddrIndex])^.EA;
        Desc := '';
      end;
      S19Map.Add(Tptr);
      i := StartAddrIndex + 1;
      while i < EndAddrIndex do
      begin
        new(Tptr);
        Tptr^ := tMemRecordPtr(MemMap[i])^;
        S19Map.Add(Tptr);
        i := i + 1;
      end;
      new(Tptr);
      with Tptr^ do
      begin
        Area := EndAddrArea;
        SA   := tMemRecordPtr(MemMap[EndAddrIndex])^.SA;
        EA   := EndAddr;
        Desc := '';
      end;
      S19Map.Add(Tptr);
    end; // if EndAddrIndex = StartAddrIndex then.. else..
  end; // for S19Line := 0 to S19Listing.Count - 1 do
  Result := CondenseMapOK(S19Map);
end; // function BuildS19MapOK

{Function checks a supplied S19 map against a list of allowed areas, and
 returns TRUE if the load addresses in the S19 file all fall within one or
 other of the allowed areas.
 Function fills in 'Areas' with all the types of memory area encountered
 in the file.}
function  S19AreasOK(const S19Map        : tList;
                           AllowedAreas  : tMemAreaSet;
                     var   AreasUsed     : tMemAreaSet;
                     var   CmdLineResult : tCmdLineComment) : boolean;
var
  i    : integer;
  Tptr : tMemRecordPtr;
begin
  Result    := false;
  AreasUsed := [];
  for i := 0 to S19Map.Count - 1 do
  begin
    Tptr := S19Map[i];
    if not (Tptr^.Area in AllowedAreas)
    then begin
      CmdLineResult :=
        tCmdLineComment(ord(clcUndefinedMem) + ord(Tptr^.Area));
      exit;
    end
    else AreasUsed := AreasUsed + [Tptr^.Area];
  end; // for i := 0 to S19Listing.Count - 1 do
  Result := true;
end; // function  S19AreasOK


{*****************************************************************************}

function LoadCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
var
  Extension           : string;
  MemArea             : tMemArea;
  AreasUsed           : tMemAreaSet;
  IsLocal,IsS19,IsBin : boolean;
  S19Map              : tList;
  ArgType             : tArgType;
  val,A2              : longword;
  BinFile             : tFileStream;
  NumBytes            : integer;
begin
  Result  := false;
  Comment := clcArgErr;
  IsLocal := uppercase(CList[0]) = 'LDL';

  if not NumArgsOK(CList,1,2,Comment) then exit;

  if not IsLocal and not StateIsAllowed([psStopped],Comment) then exit;

  FileName := CList[1];
  if FileName[1] = chr(149) then delete(FileName,1,1);
  FileName := DetokenizeFileName(FileName);

  if not FileExists(FileName) then begin Comment := clcFileNotFound; exit; end;

  Extension := uppercase(ExtractFileExt(FileName));

  IsS19 := (Extension = '.S19') or (Extension = '.REC');
  IsBin := (Extension = '.BIN') or (Extension = '.OBJ');

  if not (IsS19 or IsBin)
  then begin Comment := clcUnrecognizedFileType; exit; end;

  if IsS19
  then begin
    if not NumArgsOK(CList,1,1,Comment) then exit;

    if not OpenS19FileOK(FileName,S19Listing,SupErrMsg)
    then begin
      Comment      := clcBadS19Format;
      SupErrMsgSet := true;
      exit;
    end;

    if not IsLocal
    then begin
{Check that only modifiable memory is included}
      S19Map := tList.Create;
      try
        BuildS19MapOK(S19Listing,S19Map,MemMap);
        if not S19AreasOK(S19Map,[maCtrlRegs,maRam,maEeprom,maEprom,
                                  maExtRam,maByteWrite,maPageWrite],
                          AreasUsed,Comment) then exit;
      finally
        ClearMemMap(S19Map);
        S19Map.Free;
      end;


{Check that there is no writing to EE/Eprom or External FLASH if an
 EEPROM-resident talker is running}
      if not AreasVersusEepromTalkerOK(AreasUsed,Comment) then exit;

{Check that the necessary overlays are available if a RAM-resident talker is
 being used. It is assumed that overlays are not necessary with a talker
 in external memory}
      if not OverlaysAvailableOK(AreasUsed,Comment) then exit;
    end; // if not IsLocal

{Force reopening of 'AsS19' symbol table if new file loaded}
    if ProjData.SymbolTableToUse = stAsS19 then OpenSymbolFile;
  end; // if IsS19

  if IsBin
  then begin
    if not NumArgsOK(CList,2,2,Comment) then exit;
    if not FoundArgOK(CList,2,[atWord,atLabel],ArgType,val) then exit;

    StartAddr := Val;
    BinFile := tFileStream.Create(FileName,fmOpenRead or fmShareDenyWrite);
    try
      NumBytes := BinFile.Size;
    finally
      BinFile.Free;
    end;
    A2 := StartAddr + NumBytes - 1;
    if A2 > $FFFF
    then begin
      Comment := clcFileTooLarge;
      exit;
    end else EndAddr := A2;

    if not IsLocal
    then begin
      if not CheckAreas(StartAddr,EndAddr,
                        [maRam,maEeprom,maExtRam,maByteWrite,maPageWrite],
                        MemMap,AreasUsed,Comment)
      then exit;
{Check that the write area is one contiguous block of the same kind of memory}
      if not IsOneBlock(StartAddr,EndAddr,MemArea,MemMap)
      then begin
        Comment := clcSpanNotAllowed;
        exit;
      end;
{Check that the necessary overlays are available if a RAM-resident talker is
 being used. It is assumed that overlays are not necessary with a talker
 in external memory}
      if not OverlaysAvailableOK([MemArea],Comment) then exit;

    end; // if not IsLocal
  end; // if IsBin

  Result    := true;                // Arguments are OK
  Comment   := clcOK;

  if IsS19
  then begin
    if IsLocal then Command := cmdLoadS19Local else Command := cmdLoadS19MCU;
  end;
  if IsBin then begin
    if IsLocal then Command := cmdLoadBinLocal else Command := cmdLoadBinMCU;
  end;
end; // function  LoadCheck

function IndirectLoadCheck(const CList   : tStringList;
                           var   Comment : tCmdLineComment;
                           var   Command : tCommand) : boolean;
var
  Extension           : string;
  MemArea             : tMemArea;
  AreasUsed           : tMemAreaSet;
  IsS19,IsBin         : boolean;
  S19Map              : tList;
  ArgType             : tArgType;
  val,A2              : longword;
  BinFile             : tFileStream;
  NumBytes            : integer;
begin
  Result  := false;
  Comment := clcArgErr;

  if not NumArgsOK(CList,1,2,Comment) then exit;
  if not StateIsAllowed([psStopped],Comment) then exit;

  FileName := CList[1];
  if FileName[1] = chr(149) then delete(FileName,1,1);
  FileName := DetokenizeFileName(FileName);

  if not FileExists(FileName) then begin Comment := clcFileNotFound; exit; end;

  Extension := uppercase(ExtractFileExt(FileName));

  IsS19 := (Extension = '.S19') or (Extension = '.REC');
  IsBin := (Extension = '.BIN') or (Extension = '.OBJ');

  if not (IsS19 or IsBin)
  then begin Comment := clcUnrecognizedFileType; exit; end;

  if IsS19
  then begin
    if not NumArgsOK(CList,1,1,Comment) then exit;

    if not OpenS19FileOK(FileName,S19Listing,SupErrMsg)
    then begin
      Comment      := clcBadS19Format;
      SupErrMsgSet := true;
      exit;
    end;

{Check that only modifiable memory is included}
    S19Map := tList.Create;
    try
      BuildS19MapOK(S19Listing,S19Map,IndMemMap);
      if not S19AreasOK(S19Map,[maIndMem],AreasUsed,Comment) then exit;
    finally
      ClearMemMap(S19Map);
      S19Map.Free;
    end;
  end;

  if IsBin
  then begin
    if not NumArgsOK(CList,2,2,Comment) then exit;
    if not FoundArgOK(CList,2,[atWord,atLabel],ArgType,val) then exit;

    StartAddr := Val;
    BinFile := tFileStream.Create(FileName,fmOpenRead or fmShareDenyWrite);
    try
      NumBytes := BinFile.Size;
    finally
      BinFile.Free;
    end;
    A2 := StartAddr + NumBytes - 1;
    if A2 > $FFFF
    then begin
      Comment := clcFileTooLarge;
      exit;
    end else EndAddr := A2;

    if not CheckAreas(StartAddr,EndAddr,[maIndMem],IndMemMap,AreasUsed,Comment)
    then exit;
{Check that the write area is one contiguous block of the same kind of memory}
    if not IsOneBlock(StartAddr,EndAddr,MemArea,IndMemMap)
    then begin
      Comment := clcSpanNotAllowed;
      exit;
    end;
  end; // if IsBin

{Check that the necessary overlay is available if a RAM-resident talker is
 being used. It is assumed that overlays are not necessary with a talker
 in eeprom or external memory}
  if not OverlaysAvailableOK(AreasUsed,Comment) then exit;

  Result    := true;                // Arguments are OK
  Comment   := clcOK;

  if IsS19 then command := cmdLoadS19Indirect;
  if IsBin then command := cmdLoadBinIndirect;
end; // function  IndirectLoadCheck

{Procedure to convert the string list copy of the S19 file into bytes in
 local memory}
procedure LoadS19Local1(const S19Listing : tStringList;
                              MemType    : tMemType);
var
  i,j,k  : integer;
  MemLoc : word;
  Buf    : string;
begin
  for i := 0 to S19Listing.Count - 1 do
  begin
    Buf := S19Listing[i];
{Convert characters to code}
    if copy(Buf,1,2) = 'S1'
    then begin
      MemLoc := StrToInt('$' + copy(Buf,5,4));
      j := 9;
      k := 0;
      while j < length(Buf) - 2 do
      begin
        case MemType of
          mtIntrinsic : Memory[MemLoc + k] := StrToInt('$' + copy(Buf,j,2));
          mtIndirect  : IndMemory[MemLoc + k] := StrToInt('$' + copy(Buf,j,2));
        end;
        j := j + 2;
        k := k + 1;
      end;
    end; // if copy(Buf,1,2) = 'S1'
  end; // for i := 0 to S19Listing.Count - 1 do
end; // procedure LoadS19Local1

{A copy of the above procedure, with the addition of notification to the user
 that loading is finished}
procedure LoadS19Local2(S19Listing : tStringList);
begin
  LoadS19Local1(S19Listing,mtIntrinsic);
  MainForm.OWEdit.Lines.Add('Loading complete');
  ProjData.FN_LoadS19 := FileName;
  OpenSymbolFile;
end; // procedure LoadS19Local2

{Procedure to convert the string list copy of the S19 file into bytes in
 local memory, and then transfer them to the MCU}
function  LoadS19MCU1(S19Listing : tStringList) : boolean;
var
  S19Map : tList;
begin
  S19Map := tList.Create;
  try
    BuildS19MapOK(S19Listing,S19Map,MemMap);
    LoadS19Local1(S19Listing,mtIntrinsic);
    Result := WriteMCUMap(S19Map,true);
  finally
    ClearMemMap(S19Map);
    S19Map.Free;
  end;
end; // procedure LoadS19MCU1

{A copy of the above procedure, with the addition of notification to the user
 that loading is finished}
procedure LoadS19MCU2(S19Listing : tStringList);
var S9Addr : word;
begin
  if LoadS19MCU1(S19Listing)
  then begin
    if S19StartAddr(S19Listing,S9Addr) then Prog.TGAddr := S9Addr;
    MainForm.OWEdit.Lines.Add('Loading complete');
  end;
  ProjData.FN_LoadS19 := FileName;
  OpenSymbolFile;
end; // procedure LoadS19MCU2

procedure LoadBinaryFileToLocal(MemType : tMemType);
var
  BinFile    : tFileStream;
  i,NumBytes : integer;
  RxByte     : byte;
begin
  BinFile := tFileStream.Create(FileName,fmOpenRead or fmShareDenyWrite);
  try
    NumBytes := BinFile.size;
    if StartAddr + NumBytes - 1 <> EndAddr then exit;          // safety catch
    for i := 0 to NumBytes - 1 do
    begin
      BinFile.Read(RxByte,1);
      case MemType of
        mtIntrinsic : Memory[StartAddr + i] := RxByte;
        mtIndirect  : IndMemory[StartAddr + i] := RxByte;
      end;
    end;
  finally
    BinFile.Free;
  end;
end; // procedure LoadBinaryFileToLocal

procedure LoadBinaryFileToMCU;
begin
  LoadBinaryFileToLocal(mtIntrinsic);
  WriteOneBlock(StartAddr,EndAddr,true);
end; // procedure LoadBinaryFileToMCU

procedure LoadS19FileToIndirect;
var
  S19Map : tList;
begin
  S19Map := tList.Create;
  try
    BuildS19MapOK(S19Listing,S19Map,IndMemMap);
    LoadS19Local1(S19Listing,mtIndirect);
    WriteIndirectMap(S19Map,true);
    MainForm.OWEdit.Lines.Add('Loading complete');
    ProjData.FN_LoadS19 := FileName;
    OpenSymbolFile;
  finally
    ClearMemMap(S19Map);
    S19Map.Free;
  end;
end; // procedure LoadS19FileToIndirect

procedure LoadBinaryFileToIndirect;
var
  BinMap : tList;
  Tptr   : tMemRecordPtr;
begin
  LoadBinaryFileToLocal(mtIndirect);
  BinMap := tList.Create;
  new(Tptr);
  try
    with Tptr^ do
    begin
      SA   := StartAddr;
      EA   := EndAddr;
      Area := maIndMem;
    end;
    BinMap.Add(Tptr);
    WriteIndirectMap(BinMap,true);
  finally
    ClearMemMap(BinMap);
    BinMap.Free;
  end;
end; // procedure LoadBinaryFileToIndirect

{****************************************************************************}

procedure LoadOverlay(Overlay : tOverlayType);
var
  OverlayMap : tList;
begin
  TalkerOverlainListing.Clear;
  OverlayMap := tList.Create;
  try
    case Overlay of
      oyEEprom    : BuildS19MapOK(EEOvlyListing,OverlayMap,MemMap);
      oyEprom     : BuildS19MapOK(EOvlyListing,OverlayMap,MemMap);
      oyByteWrite : BuildS19MapOK(ByteOvlyListing,OverlayMap,MemMap);
      oyPageWrite : BuildS19MapOK(PageOvlyListing,OverlayMap,MemMap);
      oyIndRegs   : BuildS19MapOK(IndRegsOvlyListing,OverlayMap,MemMap);
      oyIndMem    : BuildS19MapOK(IndMemOvlyListing,OverlayMap,MemMap);
    end; // case OverlayType of

{Quit if certain conditions are not met}
    if OverlayMap.Count = 0 then exit;

{Read in the talker bytes which will be overlain, and store in
 TalkerOverlainListing}
    ReadMCUMap(OverlayMap,false);
    SaveMemToS19ListingUsingMap(OverlayMap,mtIntrinsic,TalkerOverlainListing);

{Notify the user of overlay loading}
    MainForm.OWEdit.Lines.Add('Loading overlay for ' + OvlyTypeArray[Overlay]);

{Open the overlay file into local memory}
    case Overlay of
      oyEEprom    : LoadS19Local1(EEOvlyListing,mtIntrinsic);
      oyEprom     : LoadS19Local1(EOvlyListing,mtIntrinsic);
      oyByteWrite : LoadS19Local1(ByteOvlyListing,mtIntrinsic);
      oyPageWrite : LoadS19Local1(PageOvlyListing,mtIntrinsic);
      oyIndRegs   : LoadS19Local1(IndRegsOvlyListing,mtIntrinsic);
      oyIndMem    : LoadS19Local1(IndMemOvlyListing,mtIntrinsic);
    end;

{Transfer the overlay file from local memory to the MCU}
    WriteMCUMap(OverlayMap,false);
  finally
    ClearMemMap(OverlayMap);
    OverlayMap.Free;
  end;
end; // procedure LoadOverlay

{Procedure to write back to the talker, the bytes that were replaced by the
 overlay}
procedure UnloadOverlay;
var
  OverlayMap : tList;
begin
  OverlayMap := tList.Create;
  try
{Notify the user of overlay unloading}
    MainForm.OWEdit.Lines.Add('Unloading overlay');

    BuildS19MapOK(TalkerOverlainListing,OverlayMap,MemMap);
    LoadS19Local1(TalkerOverlainListing,mtIntrinsic);
    WriteMCUMap(OverlayMap,false);
  finally
    ClearMemMap(OverlayMap);
    OverlayMap.Free;
  end;
end; // procedure UnloadOverlay

end.


