unit LookupUnit;

interface

uses Classes, SysUtils, TypeDefs;

//procedure OpenAs19;
function  LookupSymbolName(Key : word; var Name : string) : boolean;
function  LookupSymbolValue(      Key       : string;
                            const Table     : tSymbolTable;
                              var Value     : word;
                              var Ambiguous : boolean) : boolean;

function  LookupInherentReg(    Key : string;
                            var IR  : tInherentRegister) : boolean;
function  LookupCtrlRegAddr(    Key   : string;
                            var Value : word;
                            var Index : integer) : boolean;
function  LookupIndirectRegAddr(    Key   : string;
                                var Value : word;
                                var Index : integer) : boolean;
function  LookupDblRegAddr(    Key   : string;
                           var Value : word;
                           var Index : integer) : boolean;
function  LookupCtrlRegName(    Key   : word;
                            var Value : string;
                            var Index : integer) : boolean;
function  LookupIndirectRegName(    Key   : word;
                                var Value : string;
                                var Index : integer) : boolean;

function  LookupLabelName(Key : word; var Name : string) : boolean;
function  LookupLabelValue(    Key       : string;
                           var Value     : word;
                           var Ambiguous : boolean) : boolean;

function  GetMemArea(var   Addr   : word;
                     const MemMap : tList) : tMemArea;
function  GetMemIndex(var   Addr   : word;
                      const MemMap : tList) : integer;
function  GetMemAreaAddresses(      Area               : tMemArea;
                              const MemMap             : tList;
                              var   FirstAddr,LastAddr : word) : boolean;
function  CheckAreas(      Addr1,Addr2  : word;
                           AllowedAreas : tMemAreaSet;
                     const MemMap       : tList;
                     var   AreasUsed    : tMemAreaSet;
                     var   Comment      : tCmdLineComment) : boolean;
function  GetProgramRowStart(      Addr   : word;
                             const MemMap : tList) : word;
function  GetProgramRowEnd(      Addr   : word;
                           const MemMap : tList) : word;
function  AddressesOverlapMap(      StartAddr,EndAddr : word;
                              const Map               : tList) : boolean;

implementation

uses CmdUnit, MiscUnit, Comms1, FileUnit;

function  LookupSymbolName(Key : word; var Name : string) : boolean;
var
  i : integer;
begin
//  OpenAs19;
  i := 0;
  while (i < length(SymbolTable)) and
        (SymbolTable[i].Val <> Key) do inc(i);
  Result := i < length(SymbolTable);
  if Result then Name := SymbolTable[i].Name;
end; // function  LookupSymbolName

function  LookupSymbolValue(      Key       : string;
                            const Table     : tSymbolTable;
                              var Value     : word;
                              var Ambiguous : boolean) : boolean;
var
  i : integer;
begin
//  OpenAs19;
  i := 0;
  Ambiguous := false;
{First check the supplied key with case-sensitivity}
  while (i < length(Table)) and
        (Table[i].Name <> Key) do inc(i);
  Result := i < length(Table);
  if Result
  then begin
    Value := Table[i].Val;
    exit;
  end;
{Then check without case-sensitivity}
  i := 0;
  while (i < length(Table)) and
        (uppercase(Table[i].Name) <> uppercase(Key)) do inc(i);
  Result := i < length(Table);
  if Result then Value := Table[i].Val;
{Check further for a second occurence of a non-case-sensitive match. If found,
 then there is a possible ambiguity}
  i := i + 1;
  while (i < length(Table)) and
        (uppercase(Table[i].Name) <> uppercase(Key)) do inc(i);
  Ambiguous := i < length(Table);
end; // function  LookupSymbolValue

function  LookupInherentReg(    Key : string;
                            var IR  : tInherentRegister) : boolean;
begin
  Key    := uppercase(Key);
  Result := true;
  if Key = 'A'
  then IR := ihA
  else if Key = 'B'
       then IR := ihB
       else if Key = 'IX'
            then IR := ihIX
            else if Key = 'IY'
                 then IR := ihIY
                 else if Key = 'SP'
                      then IR := ihSP
                      else if Key = 'CCR'
                           then IR := ihCCR
                           else if Key = 'PC'
                                then IR := ihPC
                                else Result := false;
end; // function  LookupInherentRegEdit

function  LookupCtrlRegAddr(    Key   : string;
                            var Value : word;
                            var Index : integer) : boolean;
var
  i : integer;
  FirstAddr,LastAddr : word;
begin
  GetMemAreaAddresses(maCtrlRegs,MemMap,FirstAddr,LastAddr);
  i := 0;
  while (i < length(CtrlRegs)) and (uppercase(CtrlRegs[i].Name) <> uppercase(Key))
  do inc(i);
  Result := i < length(CtrlRegs);
  if Result
  then begin
    Value := CtrlRegs[i].Addr + FirstAddr;
    Index := i;
  end;
end; // function  LookupCtrlRegAddr

function  LookupIndirectRegAddr(    Key   : string;
                                var Value : word;
                                var Index : integer) : boolean;
var
  i : integer;
begin
  i := 0;
  while (i < length(IndRegs)) and (uppercase(IndRegs[i].Name) <> uppercase(Key))
  do inc(i);
  Result := i < length(IndRegs);
  if Result
  then begin
    Value := IndRegs[i].Addr;
    Index := i;
  end;
end; // function  LookupIndirectRegAddr

function  LookupDblRegAddr(    Key   : string;
                           var Value : word;
                           var Index : integer) : boolean;
var
  i : integer;
  FirstAddr,LastAddr : word;
begin
  GetMemAreaAddresses(maCtrlRegs,MemMap,FirstAddr,LastAddr);
  i := 0;
  while (i < length(CtrlRegs)) and (uppercase(CtrlRegs[i].Name) <> uppercase(Key + 'H'))
  do inc(i);
  Result := (i < length(CtrlRegs)) and (CtrlRegs[i].RegType = ioHigh);
  if Result
  then begin
    Value := CtrlRegs[i].Addr + FirstAddr;
    Index := i;
  end else Index := 0;
end; // function LookupDblRegAddr

{Function to look up the name of an I/O register, given its address}
function  LookupCtrlRegName(    Key   : word;
                            var Value : string;
                            var Index : integer) : boolean;
var
  i : integer;
  FirstAddr,LastAddr : word;
begin
  GetMemAreaAddresses(maCtrlRegs,MemMap,FirstAddr,LastAddr);
  i := 0;
  while (i < length(CtrlRegs)) and ((CtrlRegs[i].Addr + FirstAddr) <> Key) do inc(i);
  Result := i < length(CtrlRegs);
  if Result
  then begin
    Value := CtrlRegs[i].Name;
    Index := i;
  end else Index := 0;
end; // function  LookupCtrlRegName

function  LookupIndirectRegName(    Key   : word;
                                var Value : string;
                                var Index : integer) : boolean;
var
  i : integer;
begin
  i := 0;
  while (i < length(CtrlRegs)) and (IndRegs[i].Addr <> Key) do inc(i);
  Result := i < length(IndRegs);
  if Result
  then begin
    Value := IndRegs[i].Name;
    Index := i;
  end else Index := 0;
end; // function  LookupIndirectRegName

function  LookupLabelValue(    Key       : string;
                           var Value     : word;
                           var Ambiguous : boolean) : boolean;
var
  DummyIndex : integer;
begin
  Ambiguous := false;
  Result := LookupSymbolValue(Key,SymbolTable,Value,Ambiguous) or
            LookupCtrlRegAddr(Key,Value,DummyIndex);
end; // function  LookupLabelValue

function  LookupLabelName(Key : word; var Name : string) : boolean;
var
  Dummy : integer;
begin
  Result := LookupSymbolName(Key,Name) or
            LookupCtrlRegName(Key,Name,Dummy);
end; // function  LookupLabelName

{-----------------------------------------------------------------------------}

{Function to return the memory area corresponding to a supplied address}
function  GetMemArea(var   Addr   : word;
                     const MemMap : tList) : tMemArea;
var
  i : integer;
begin
  i      := 0;
  while (i < MemMap.Count) and (tMemRecordPtr(MemMap[i])^.EA < Addr) do inc(i);
  if i < MemMap.Count
  then Result := tMemRecordPtr(MemMap[i])^.Area
  else Result := maUndefined;
end; // function  GetMemArea

{Function to return the index of the entry in the memory map, corresponding
 to a supplied address}
function  GetMemIndex(var   Addr   : word;
                      const MemMap : tList) : integer;
var
  i : integer;
begin
  i := 0;
  while (i < MemMap.Count - 1) and not
        ((Addr >= tMemRecordPtr(MemMap[i])^.SA) and
         (Addr <= tMemRecordPtr(MemMap[i])^.EA)) do i := i + 1;
  Result := i;
end; // function  GetMemIndex

{Function to return the highest and lowest addresses for a particular type
 of memory}
function  GetMemAreaAddresses(      Area               : tMemArea;
                              const MemMap             : tList;
                              var   FirstAddr,LastAddr : word) : boolean;
var
  i    : integer;
  Tptr : tMemRecordPtr;
begin
  FirstAddr := $FFFF;
  LastAddr  := 0;
  for i := 0 to MemMap.Count - 1 do
  begin
    Tptr := MemMap[i];
    if Tptr^.Area <> Area then continue;
    if Tptr^.SA < FirstAddr then FirstAddr := Tptr^.SA;
    if Tptr^.EA > LastAddr  then LastAddr  := Tptr^.EA;
  end;
  Result := FirstAddr <= LastAddr;
end; // function  GetMemAreaAddresses

{The following function returns a TRUE result only if the whole range of
 addresses specified by the beginning and ending addresses all fall within
 one or another of the allowed areas. If a false result is generated, then
 the CmdLineResult variable is updated with the reason.
 The actual areas used are also noted. }
function  CheckAreas(      Addr1,Addr2  : word;
                           AllowedAreas : tMemAreaSet;
                     const MemMap       : tList;
                     var   AreasUsed    : tMemAreaSet;
                     var   Comment      : tCmdLineComment) : boolean;
var
  j       : word;
  MemArea : tMemArea;
begin
  Result    := false;
  AreasUsed := [];
  j         := Addr1;
  while j <= Addr2 do
  begin
    MemArea   := GetMemArea(j,MemMap);
    AreasUsed := AreasUsed + [MemArea];
    if not (MemArea in AllowedAreas)
    then begin
      Comment := tCmdLineComment(ord(MemArea) + ord(clcUndefinedMem));
      exit;
    end; // if MemArea in AllowedAreas
    if j = $FFFF
    then begin Result := true; exit; end
    else j := j + 1;
  end; // for j := Addr1 to Addr2 do
  Result := true;
end; // function  CheckAreas

function  GetProgramRowStart(      Addr   : word;
                             const MemMap : tList) : word;
begin;
  Result :=
    (Addr div ProjData.DataPageSize) * ProjData.DataPageSize;
{Are these in the same type of memory?}
  if GetMemArea(Result,MemMap) in [maPageWrite]
  then exit
{If not, set the start address to the SA address of the map area}
  else Result := tMemRecordPtr(MemMap[GetMemIndex(Addr,MemMap)])^.SA;
end; //

function  GetProgramRowEnd(      Addr   : word;
                           const MemMap : tList) : word;
begin
  Result :=
    ((Addr div ProjData.DataPageSize) + 1) * ProjData.DataPageSize - 1;
{Are these in the same type of memory?}
  if GetMemArea(Result,MemMap) in [maPageWrite]
  then exit
{If not, set the end address to the EA address of the map area}
  else Result := tMemRecordPtr(MemMap[GetMemIndex(Addr,MemMap)])^.EA;
end; //

function  AddressesOverlapMap(      StartAddr,EndAddr : word;
                              const Map               : tList) : boolean;
var
  i    : integer;
begin
  Result := false;
  for i := 0 to Map.Count - 1 do
  begin
    with tMemRecordPtr(Map[i])^ do
    begin
      if ((SA <= StartAddr) and (StartAddr <= EA)) or
         ((SA <= EndAddr)   and (EndAddr   <= EA)) or
         ((StartAddr < SA)  and (EndAddr > EA))
      then begin Result := true; exit; end;
    end;
  end;
end; // function  AddrOverlapMap

end.

