unit MMUnit;

interface

uses Classes, SysUtils, TypeDefs;

function MM1Check(const CList   : tStringList;
                  var   Comment : tCmdLineComment;
                  var   Command : tCommand) : boolean;
function MM2Check(const CList   : tStringList;
                  var   Comment : tCmdLineComment;
                        LastCommand : tCommand;
                  var   Command : tCommand) : boolean;
procedure DoModMem1(var Command : tCommand);
procedure DoModMem2(var Command : tCommand);

implementation

uses Main, CmdUnit, MemRW, LookupUnit, CheckUnit;

type
  tMMmove = (mmNone,mmAdvance,mmRetard,mmNew);
var
  StartAddr           : word;
  CurrentByte,NewByte : byte;
  MMmove              : tMMmove;

function MM1Check(const CList   : tStringList;
                  var   Comment : tCmdLineComment;
                  var   Command : tCommand) : boolean;
var
  A1,Val     : longword;
  ArgType    : tArgType;
  AreasUsed  : tMemAreaSet;
  RegAddr    : word;
  i          : integer;
  B          : byte;
begin
  Result  := false;
  Comment := clcArgErr;
  if not NumArgsOK(CList,1,1,Comment) then exit;
  if not StateIsAllowed([psStopped,psRunStop,psTraceStop],Comment) then exit;

  if FoundArgOK(CList,1,[atByte,atWord,atLabel],ArgType,Val)
  then A1 := Val else exit;

  if not CheckAreas(A1,A1,[maCtrlRegs,maRam,maEeprom,maExtRam],
                    MemMap,AreasUsed,Comment) then exit;

  if Prog.MacroState in [msPlaying,msRecording]
  then begin
    Comment := clcNotInMacros;
    exit;
  end;
    
  if (maEeprom in AreasUsed) and
     (LookupCtrlRegAddr('BPROT',RegAddr,i)) and
     ProjData.WarnOnBprot
  then begin
    B := ReadByte(RegAddr);
    if (B and $0F) > 0
    then begin
      Comment := clcBPROTMayDisallowWrite;
      exit;
    end;
  end;

  StartAddr := A1;
  Result    := true;
  Comment   := clcOK;
  Command   := cmdModify;
end; // function MM1Check

function MM2Check(const CList   : tStringList;
                  var   Comment : tCmdLineComment;
                        LastCommand : tCommand;
                  var   Command : tCommand) : boolean;
var
  Buf,NextValueStr,AddrBuf : string;
  DotPos,NewAddr           : integer;
  ArgType                  : tArgType;
  Val                      : longword;
begin
  Result  := false;
  Comment := clcArgErr;
  Command := cmdNone;
  MMmove  := mmNone;

{If previous command was not cmdModifying then exit}
  if not (LastCommand in [CmdModify,CmdModifying])
  then exit;

  Buf := CList[1];

{Typing Return at the dot causes the Memory Modify command to terminate}
  DotPos := pos('.',Buf);
  if (DotPos = 0) or (DotPos = length(Buf))
  then begin
    Comment := clcBlank;
    Command := cmdEnd;
    exit;
  end;

{Typing a '+' or '-' advances or retards the modification address}
  if pos('+',Buf) = 4 then MMmove := mmAdvance;
  if pos('-',Buf) = 4 then MMmove := mmRetard;
  if pos('=',Buf) = 4 then MMmove := mmNew;
  if MMmove in [mmAdvance,mmRetard,mmNew]
  then begin
    AddrBuf := copy(Buf,5,length(Buf)-4);
    if length(AddrBuf) = 0
    then begin
      case MMmove of
        mmAdvance,mmRetard : AddrBuf := '1';
        mmNew              : AddrBuf := IntToHex(StartAddr,4);
      end;
    end;

    CList.Add(AddrBuf);
    if not FoundArgOK(CList,2,[atWord,atLabel],ArgType,val)
    then begin
      Comment := clcBlank;
      Command := cmdEnd;
      exit;
    end;

    case MMmove of
      mmAdvance : NewAddr := StartAddr + integer(val) - 1;
      mmRetard  : NewAddr := StartAddr - integer(val) - 1;
      mmNew     : NewAddr := integer(val) - 1;
    else NewAddr := $10000;
    end;

    if (NewAddr >= 0) and (NewAddr <= $FFFF)
    then StartAddr := word(NewAddr)
    else begin
      Comment := clcBlank;
      Command := cmdEnd;
      exit;
    end; //  if MMmove in [mmAdvance,mmRetard,mmNew] then..

    Result  := true;
    Comment := clcOK;
    Command := cmdModifying;
    exit;
  end;  

{Typing a space at the dot preserves the previous memory value.  This
 space is converted to the character #149 by the procedure ProcessCR, see
 the unit CmdUnit}
  if Buf[length(Buf)] = chr(149)
  then begin end // omits the modification of MemArray[Talker.StartAddr]

{Any other typing is presumed to be a byte value}
  else begin
    NextValueStr := copy(Buf,DotPos+1,length(Buf)-DotPos);
    CList.Add(NextValueStr);
    if not FoundArgOK(CList,2,[atByte],ArgType,Val)
    then begin
      Comment := clcArgOutOfRange;
      exit;
    end;
    NewByte := Val;
  end; // if Buf[length(Buf)] = chr(149) then.. else..

  Result  := true;
  Comment := clcOK;
  Command := cmdModifying;
end; // function MM2Check

{The MM command has two halves to it.  In the first half, it displays the
 current address and the byte at that address followed by a period, on the
 command line }
procedure DoModMem1(var Command : tCommand);
var
  Buf : string;
begin
  with MainForm.CmdEdit do
  begin
    CurrentByte := ReadByte(StartAddr);
    Buf := format('%6.4x',[StartAddr]) +
           format('%4.2x',[CurrentByte]) + '.';
    Command  := cmdModifying;
    Text     := Buf;
    SelStart := length(Text);          // move cursor to end of line
  end; // with MainForm.CmdEdit,Prog do
end; // procedure DoModMem1

procedure DoModMem2(var Command : tCommand);
var
  A1            : word;
  DummyComment  : tCmdLineComment;
  AreasUsed     : tMemAreaSet;
begin
  if MMmove = mmNone
  then begin
    if GetMemArea(StartAddr,MemMap) <> maEeprom
    then begin
      if not WriteByte(StartAddr,NewByte,true)
      then begin
        Command := cmdNone;
        exit;
      end;
    end
    else begin
      if CurrentByte <> $FF then ByteEraseEeprom(StartAddr);
      if not WriteByteEeprom(StartAddr,CurrentByte,NewByte)
      then begin
        Command := cmdNone;
        exit;
      end;
    end;
  end;  

{Advance address by 1, and check that it is still within a valid memory area}
  A1 := StartAddr + 1;
  if not CheckAreas(A1,A1,[maCtrlRegs,maRam,maEeprom,maExtRam],
                    MemMap,AreasUsed,DummyComment)
  then begin
    Command := cmdNone;
    exit;
  end
  else StartAddr := A1;
  DoModMem1(Command);
end; // procedure DoModMem2

end.

