unit MacroUnit;

interface               

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, StdActns, ActnList, TypeDefs;

type
  TMEForm = class(TForm)
    MacroMenu: TMainMenu;
    MEEdit: TMemo;
    File1: TMenuItem;
    Edit1: TMenuItem;
    Options1: TMenuItem;
    Help1: TMenuItem;
    New1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    SaveAs1: TMenuItem;
    N1: TMenuItem;
    Close1: TMenuItem;
    Macro1: TMenuItem;
    Play1: TMenuItem;
    Record2: TMenuItem;
    Stop1: TMenuItem;
    MEActionList: TActionList;
    EditCut1: TEditCut;
    EditCopy1: TEditCopy;
    EditPaste1: TEditPaste;
    Copy1: TMenuItem;
    Cut1: TMenuItem;
    Paste1: TMenuItem;
    KeeponTop1: TMenuItem;
    FontSize1: TMenuItem;
    Index1: TMenuItem;
    Contents1: TMenuItem;
    HelponMacroEditor1: TMenuItem;
    PopupMenu1: TPopupMenu;
    Copy2: TMenuItem;
    Cut2: TMenuItem;
    Paste2: TMenuItem;
    SelectAll2: TMenuItem;
    N3: TMenuItem;
    N4: TMenuItem;
    ExecuteLine1: TMenuItem;
    N5: TMenuItem;
    SelectAll1: TMenuItem;
    N08pt1: TMenuItem;
    N10pt1: TMenuItem;
    N12pt1: TMenuItem;
    N2: TMenuItem;
    Undo1: TMenuItem;
    N14pt1: TMenuItem;
    SelectAllA: TAction;
    UndoA: TAction;
    ExecuteLineA: TAction;
    DeleteLineA: TAction;
    DeleteLine1: TMenuItem;
    N6: TMenuItem;
    Undo2: TMenuItem;
    DeleteLine2: TMenuItem;
    N7: TMenuItem;
    MakeMacro1: TMenuItem;
    CheckSyntax1: TMenuItem;
    MakeMacro: TAction;
    CheckSyntax: TAction;
    N8: TMenuItem;
    MakeintoMacro1: TMenuItem;
    CheckSyntax2: TMenuItem;
    MEFileOpen: TAction;
    MEFileSave: TAction;
    MEFileNew: TAction;
    MEFileSaveAs: TAction;
    procedure Options1Click(Sender: TObject);
    procedure MEEditChange(Sender: TObject);
    procedure EditCut1Execute(Sender: TObject);
    procedure EditCopy1Execute(Sender: TObject);
    procedure EditPaste1Execute(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure FontSizeClick(Sender: TObject);
    procedure KeeponTop1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure UndoAExecute(Sender: TObject);
    procedure SelectAllAExecute(Sender: TObject);
    procedure ExecuteLineAExecute(Sender: TObject);
    procedure DeleteLineAExecute(Sender: TObject);
    procedure Index1Click(Sender: TObject);
    procedure Contents1Click(Sender: TObject);
    procedure HelponMacroEditor1Click(Sender: TObject);
    procedure Edit1Click(Sender: TObject);
    procedure File1Click(Sender: TObject);
    procedure Macro1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MEEditExit(Sender: TObject);
    procedure CheckSyntaxExecute(Sender: TObject);
    procedure MakeMacroExecute(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure MEFileOpenExecute(Sender: TObject);
    procedure MEFileSaveExecute(Sender: TObject);
    procedure MEFileNewExecute(Sender: TObject);
    procedure MEFileSaveAsExecute(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MEForm: TMEForm;

function  MacroCheck(const CList   : tStringList;
                     var   Comment : tCmdLineComment;
                     var   Command : tCommand) : boolean;
procedure PlayMacro;
procedure PlayMacroLines(const LineList   : tStringList);
procedure PlayBootScript;
function  MacroExists(Name : string) : boolean;
procedure BuildMacroNameList;
procedure ListMacros;


implementation

{$R *.DFM}

uses Main, CmdUnit, MiscUnit, CheckUnit, TermSettingsUnit;

type
  tLineType  = (ltTOF,ltDEFM,ltBegin,ltCmd,ltEnd,ltBOF);
  tErrorCode = (ecNone,ecDefmExp,ecBeginExp,ecEndExp,
                ecDupDefm,ecDupBegin,ecDupEnd,
                ecDupParam,ecParamErr,ecBeginNoArgs,ecEndNoArgs,ecUndefParam,
                ecNoName,ecDupName);
  tErrorCodeArray = array[ltTOF..ltEnd,ltDEFM..ltBOF] of tErrorCode;

const
  ErrorCodeArray : tErrorCodeArray =                 // Previous line type:
  ((ecNone,ecDefmExp,ecDefmExp,ecDefmExp,ecDefmExp),            // TOF
   (ecDupDefm,ecNone,ecBeginExp,ecBeginExp,ecBeginExp),         // Defm
   (ecEndExp,ecDupBegin,ecNone,ecNone,ecEndExp),                // Begin
   (ecEndExp,ecEndExp,ecNone,ecNone,ecEndExp),                  // Cmd
   (ecNone,ecDefmExp,ecDefmExp,ecDupEnd,ecNone));               // End

  ErrorStrArray : array[tErrorCode] of string =
   ('',
    'Macro definition (DEFM) expected',
    'BEGIN expected',
    'END expected',
    'Duplicate DEFM',
    'Duplicate BEGIN',
    'Duplicate END',
    'Duplicate parameter',
    'Parameter error',
    'BEGIN cannot take arguments',
    'END cannot take arguments',
    'Undefined parameter',
    'No name specified',
    'Duplicate macro name');

var
  PlayText : string;

procedure TMEForm.FormShow(Sender: TObject);
begin
  MEEdit.Font.Size := MEFontSize;
end; // procedure TMEForm.FormShow

procedure TMEForm.FormDestroy(Sender: TObject);
begin
  FreeSubMenu(Play1);
end; // procedure TMEForm.FormDestroy

procedure TMEForm.File1Click(Sender: TObject);
begin
//  MainForm.SaveMacro.Enabled := Prog.MacroHasChanged;
  MEFileSave.Enabled := Prog.MacroHasChanged;
end; // procedure TMEForm.File1Click

procedure TMEForm.Edit1Click(Sender: TObject);
begin
  PopupMenu1Popup(nil);                          // enables/disables actions
end; // procedure TMEForm.Edit1Click

procedure TMEForm.Macro1Click(Sender: TObject);
var
  MItem : tMenuItem;
  i     : integer;
begin
{Build the 'Play Macro' sub-menu}
  FreeSubMenu(Play1);
  if NameListNeedsRebuilding then BuildMacroNameList;
  for i := 0 to (MacroNameList.Count - 1) do
  begin
    MItem := tMenuItem.Create(MacroMenu);
    MItem.Caption := MacroNameList[i];
    MItem.OnClick := MainForm.MacroListMenuClick;
    Play1.Add(MItem);
  end;
  Play1.Enabled := Play1.Count > 0;
end;

procedure TMEForm.Options1Click(Sender: TObject);
begin
  KeepOnTop1.Checked := (MEForm.FormStyle = fsStayOnTop);
//  WordWrap1.Checked  := MEEdit.WordWrap;
  N08pt1.Checked := (MEEdit.Font.Size =  8);
  N10pt1.Checked := (MEEdit.Font.Size = 10);
  N12pt1.Checked := (MEEdit.Font.Size = 12);
  N14pt1.Checked := (MEEdit.Font.Size = 14);
end; // procedure TMEForm.MacroMenuClick

procedure TMEForm.Close1Click(Sender: TObject);
begin
  MEForm.Close;
end; // procedure TMEForm.Close1Click

procedure TMEForm.UndoAExecute(Sender: TObject);
begin
  MEEdit.Undo;
end; // procedure TMEForm.UndoAExecute

procedure TMEForm.EditCopy1Execute(Sender: TObject);
begin
  MEEdit.CopyToClipboard;
end; // procedure TMEForm.EditCopy1Execute

procedure TMEForm.EditPaste1Execute(Sender: TObject);
begin
  MEEdit.PasteFromClipboard;
end; // procedure TMEForm.EditPaste1Execute

procedure TMEForm.EditCut1Execute(Sender: TObject);
begin
  MEEdit.CutToClipboard;
end; // procedure TMEForm.EditCut1Execute

procedure TMEForm.SelectAllAExecute(Sender: TObject);
begin
  MEEdit.SelectAll;
end; // procedure TMEForm.SelectAllAExecute

procedure TMEForm.DeleteLineAExecute(Sender: TObject);
var
  MEEditStartRow : integer;
begin
  with MEEdit do
  begin
    MEEditStartRow  := SendMessage(Handle,EM_LINEFROMCHAR,SelStart,0);
    MEEdit.Lines.Delete(MEEditStartRow);
  end;
end; // procedure TMEForm.DeleteLineAExecute

procedure TMEForm.KeeponTop1Click(Sender: TObject);
begin
  KeeponTop1.Checked := not KeeponTop1.Checked;
  if KeeponTop1.Checked
  then MEform.FormStyle := fsStayOnTop
  else MEform.FormStyle := fsNormal;
end; // procedure TMEForm.KeeponTop1Click

procedure TMEForm.FontSizeClick(Sender: TObject);
begin
  if Sender = N08pt1 then MEFontSize :=  8;
  if Sender = N10pt1 then MEFontSize := 10;
  if Sender = N12pt1 then MEFontSize := 12;
  if Sender = N14pt1 then MEFontSize := 14;
  MEEdit.Font.Size := MEFontSize;
end; // procedure TMEForm.FontSizeClick

procedure TMEForm.Index1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_TAB, -2);        // Open with 'Find' tab
end; // procedure TMEForm.Index1Click

procedure TMEForm.Contents1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_TAB,  0);        // Open with 'Contents' tab
end; // procedure TMEForm.Contents1Click

procedure TMEForm.HelponMacroEditor1Click(Sender: TObject);
begin
  Application.HelpContext(IDH_MACRO_EDITOR);
end; // procedure TMEForm.HelponMacroEditor1Click

procedure TMEForm.MEEditChange(Sender: TObject);
begin
  Prog.MacroHasChanged    := true;
  NameListNeedsRebuilding := true;
end; // procedure TMEForm.MEEditChange

procedure TMEForm.PopupMenu1Popup(Sender: TObject);
begin
  EditCut1.Enabled     := MEEdit.SelLength > 0;
  EditCopy1.Enabled    := MEEdit.SelLength > 0;
  SelectAllA.Enabled   := MEEdit.Lines.Count > 0;
  UndoA.Enabled        := MEEdit.CanUndo;
  ExecuteLineA.Enabled := MEEdit.Lines.Count > 0;
  DeleteLineA.Enabled  := MEEdit.Lines.Count > 0;
end; // procedure TMEForm.PopupMenu1Popup

function MECharIndexByPos(Memo: TMemo; X, Y: Integer): Integer;
var
  P: TPoint;
begin
  P := Point(X, Y);
  Result := SendMessage(Memo.Handle, EM_CHARFROMPOS, 0, Longint(@P));
end; // function MECharIndexByPos

procedure TMEForm.ExecuteLineAExecute(Sender: TObject);
var
  MEEditStartRow : integer;
  Buf            : string;
begin
  with MEEdit do
  begin
// EM_EXLINEFROMCHAR has the value WM_USER + 54 = 1024 + 54 = 1078
{Discover the line containing SelStart}
    MEEditStartRow  := SendMessage(Handle,EM_LINEFROMCHAR,SelStart,0);
    Buf := Lines[MEEditStartRow];
    if copy(Buf,1,1) = '*' then exit;
    DelComment(Buf,[cdSemiColon,cdDoubleSlash,cdCurlyBrace]);
    DelLeadSpaceTab(Buf);
    DelTrailSpaceTab(Buf);
    MainForm.CmdEdit.Text := Buf;
  end;
  ProcessCR;
  MainForm.CmdEdit.SetFocus;
end; // procedure TMEForm.ExecuteLineExecute

procedure TMEForm.MEEditExit(Sender: TObject);
begin
  if NameListNeedsRebuilding then BuildMacroNameList;
end; // procedure TMEForm.MEEditExit

{****************************************************************************}
{$O-}
function  MacroCheck(const CList   : tStringList;
                     var   Comment : tCmdLineComment;
                     var   Command : tCommand) : boolean;
var
  i : integer;  // index into the macro file
  j : integer;  // various uses
  c : char;     // used to index ParamArray
  Buf                       : string;
  LinesList,DefmList,MEList : tStringList;
{ParamArray holds the values to be substituted for @1, @2 etc.}
  ParamArray                : array['0'..'9'] of string;
begin
  Result  := false;
  Comment := clcArgErr;

  if not NumArgsOK(CList,0,10,Comment) then exit;

  fillchar(ParamArray,sizeof(ParamArray),0);
  LinesList       := tStringList.Create; // Lines as they will be played
  DefmList        := tStringList.Create; // StringList version of each 'DEFM' line
  MEList          := tStringList.Create; // The complete macro file

  Prog.MacroState := msPlaying;
  Result          := false;
  Comment         := clcCmdErr;
  LinesList.Clear;
  
  try
{Following line is a workaround because otherwise wrapped lines are treated as
 separate lines}
    MEList.Text := MEForm.MEEdit.Lines.Text;
{Find the line index for this macro name}
    i := 0;
    while i < MEList.Count do
    begin
      Buf := uppercase(MEList[i]);
      DelLeadSpaceTab(Buf);
      DelComment(Buf,[cdDoubleSlash,cdSemiColon,cdCurlyBrace]);
      DefmList.CommaText := Buf;

      if (DefmList.Count > 1) and (DefmList[0] = 'DEFM')
      then begin
        DefmList.Delete(0);
        if DefmList[0] = uppercase(CList[0]) // does the DEFM name match the CList name?
        then break;
      end;
      i := i + 1;
    end; // while i < MEList.Count do

{The CmdUnit has already checked whether or not the command has the same name
 as a defined macro, so we should never need this check}
    if i = MEList.Count
    then begin
      ShowMessage('Internal error - macro not found in second pass');
      exit;
    end;

{Does the macro have replaceable parameters? - if so, build an array of the
 constants which are to be substituted for @1, @2, etc.
 DefmList has already had 'DEFM' deleted, so if it now has a length of 2, then
 it must have 1 replaceable parameter, etc.}
    if not NumArgsOK(CList,DefmList.Count-1,DefmList.Count-1,Comment) then exit;
    j  := 1;
    while (j < DefmList.Count) do
    begin
      if (length(DefmList[j]) = 2) and
         ((DefmList[j][1] = '@') or (DefmList[j][1] = '%')) and
         (DefmList[j][2] in DecChars)
      then ParamArray[DefmList[j][2]] := CList[j];
      j := j + 1;
    end; // while (j < DefmList.Count) do

{Next line should be 'BEGIN'}
    i := i + 1;
    Buf := MEList[i];
    DelLeadSpaceTab(Buf);
    if uppercase(copy(Buf,1,5)) <> 'BEGIN'
    then begin
      Comment := clcBeginNotFound;
      exit;
    end;

{Make, in LinesList, a stringlist of the macro as it is to be played}
    i := i + 1;
    while i < MEList.Count do
    begin
      Buf := MEList[i];
      DelLeadSpaceTab(Buf);
      DelComment(Buf,[cdDoubleSlash,cdSemiColon,cdCurlyBrace]);
      if uppercase(Buf) = 'END' then break;
{Ignore asterisk and blank lines}
      if (length(Buf) = 0) or (copy(Buf,1,1) = '*')
      then begin i := i + 1; continue; end;
{Make parameter substitutions as necessary}
      for c := '0' to '9' do
      begin
        Buf := StringReplace(Buf,'@'+c,ParamArray[c],[rfReplaceAll]);
        Buf := StringReplace(Buf,'%'+c,ParamArray[c],[rfReplaceAll]);
      end;
{Add the substituted line to the output list}
      LinesList.Add(Buf);
      i := i + 1;
    end;
    PlayText := LinesList.Text;
  finally
    LinesList.Free;
    DefmList.Free;
    MEList.Free;
  end;
  Command  := cmdPlayMacro;
  Result   := true;
  Comment  := clcOK;
end; // function  MacroCheck
{$O+}
procedure PlayMacro;
var
  LinesList      : tStringList;
begin
  LinesList := tStringList.Create;
  MacroNestingCount := MacroNestingCount + 1;
  try
    LinesList.Text := PlayText;
    PlayMacroLines(LinesList);
  finally
    LinesList.Free;
    MacroNestingCount := MacroNestingCount - 1;
    if MacroNestingCount < 0 then MacroNestingCount := 0;
    if MacroNestingCount = 0 then Prog.MacroState := msStopped;
  end;
end; // procedure PlayMacro

procedure BuildMacroNameList;
var
  i,j : integer;
  Buf : string;
  MEList : tStringList;
begin
  MacroNameList.Clear;
  MEList := tStringList.Create;
  try
    MEList.Text := MEForm.MEEdit.Lines.Text;
    for i := 0 to MEList.Count - 1 do
    begin
      Buf := MEList[i];
      DelLeadSpaceTab(Buf);
      if (uppercase(copy(Buf,1,4)) = 'DEFM') and
         (length(Buf) > 4) and
         ((Buf[5] = ' ') or (Buf[5] = ^I))
      then begin
        system.delete(Buf,1,4);
        DelLeadSpaceTab(Buf);
        j := 1;
        while (j <= length(Buf)) and not (Buf[j] in [' ',^I,',']) do j := j + 1;
        system.delete(Buf,j,length(Buf)-j+1);
        if length(Buf) > 0 then MacroNameList.Add(Buf);
      end; // if uppercase(copy(Buf,1,4)) = 'DEFM' then..
    end; // for i := 0 to Lines.Count do
    NameListNeedsRebuilding := false;
  finally
    MEList.Free;
  end;
end; // procedure BuildMacroNameList

function MacroExists(Name : string) : boolean;
var i : integer;
begin
  i := 0;
  while i < MacroNameList.Count do
  begin
    if Uppercase(Name) = Uppercase(MacroNameList[i]) then break;
    i := i + 1;
  end;
  Result := i < MacroNameList.Count;
end; // function MacroExists

procedure PlayMacroLines(const LineList   : tStringList);
var
  i     : integer;
  Buf   : string;
begin
  MainForm.CmdEdit.SetFocus;
{Play the macro}
  for i := 0 to LineList.Count - 1 do
  begin
    if Prog.MacroState <> msPlaying then exit;
    Buf := LineList[i];
    DelLeadSpaceTab(Buf);
{Delete comments}
    DelComment(Buf,[cdAsterisk,cdDoubleSlash,cdSemiColon,cdCurlyBrace]);
    if length(Buf) = 0 then continue;
{Copy command to CmdEdit}
    MainForm.CmdEdit.Text := Buf;
    ProcessCR;
  end; // for i := 0 to MList.Count - 1 do
end; // procedure PlayMacroLines

procedure PlayBootScript;
var
  MList : tStringList;
begin
  if ProjData.RunBootScript = rbsNever then exit;   // safety net
  MacroNestingCount := MacroNestingCount + 1;
  Prog.MacroState   := msPlaying;
  MList := tStringList.Create;
  try
    MList.Text := ProjData.BootScript;
{Play the macro}
    PlayMacroLines(MList);
  finally
    MList.Free;
    MacroNestingCount := MacroNestingCount - 1;
    if MacroNestingCount < 0 then MacroNestingCount := 0;
    if MacroNestingCount = 0 then Prog.MacroState := msStopped;
  end;
end; // procedure PlayBootScript

procedure ListMacros;
var
  i               : integer;
  DefmList,MEList : tStringList;
  Buf             : string;
begin
  DefmList := tStringList.Create;
  MEList   := tStringList.Create;
{Find the line index for this macro name}
  i := 0;
  try
    MEList.Text := MEForm.MEEdit.Lines.Text;
    while i < MEList.Count do
    begin
      Buf := MEList[i];
      DelLeadSpaceTab(Buf);
      DefmList.CommaText := Buf;
      if (DefmList.Count > 1) and (uppercase(DefmList[0]) = 'DEFM')
      then begin
        System.Delete(Buf,1,4);
        MainForm.OWEdit.Lines.Add(Buf);
      end;
      i := i + 1;
    end; // while i < MEForm.MEEdit.Lines.Count do
  finally
    DefmList.Free;
    MEList.Free;
  end;

  for i := 0 to MacroNameList.Count - 1 do
  begin
    MainForm.CHEdit.Lines.Add(MacroNameList[i] + ' <--');
  end;
end; // procedure ListMacros

procedure SyntaxCheck;
var
  PrevLineType,CurrLineType  : tLineType;
  ErrorCode                  : tErrorCode;
  i,j,HelpCtx,CharIndex      : integer;
  Buf                        : string;
  LineList,NameList          : tStringList;
  ParamDefined               : array['0'..'9'] of boolean;
begin
  fillchar(ParamDefined,sizeof(ParamDefined),0);
  LineList     := tStringList.Create;
  NameList     := tStringList.Create;
  PrevLineType := ltTOF;
  CurrLineType := ltBOF;
  i            := 0;
  ErrorCode    := ecNone;
  MEForm.MEEdit.WordWrap := false;
  try
    while i <= MEForm.MEEdit.Lines.Count do
    begin
{Find the current line type}
      if i < MEForm.MEEdit.Lines.Count
      then begin
        Buf := uppercase(MEForm.MEEdit.Lines[i]);
        DelLeadSpaceTab(Buf);
        DelComment(Buf,[cdAsterisk,cdDoubleSlash,cdSemiColon,cdCurlyBrace]);
        if length(Buf) = 0 then begin i := i + 1; continue; end;
        LineList.CommaText := Buf;
        if LineList[0] = 'DEFM'
        then CurrLineType := ltDefm
        else
          if LineList[0] = 'BEGIN'
          then CurrLineType := ltBegin
          else
            if LineList[0] = 'END'
            then CurrLineType := ltEnd
            else CurrLineType := ltCmd;
      end else CurrLineType := ltBOF;

{Check that the correct types of line follow one another}
      ErrorCode := ErrorCodeArray[PrevLineType,CurrLineType];
      if ErrorCode <> ecNone then exit;

{Check for errors in each type of line}
      case CurrLineType of
        ltDefm :
          begin
{Check for the existence of a name}          
            if LineList.Count < 2
            then begin
              ErrorCode := ecNoName;
              exit;
            end;
{Check for a duplicate macro name}
            if NameList.IndexOf(uppercase(LineList[1])) >= 0
            then begin
              ErrorCode := ecDupName;
              exit;
            end else NameList.Add(uppercase(LineList[1]));
{Does the macro have replaceable parameters? - if so, build array of checks}
            fillchar(ParamDefined,sizeof(ParamDefined),0);
            for j := 2 to LineList.Count - 1 do
            begin
              Buf := LineList[j];
              if (length(Buf) = 2) and
                 ((Buf[1] = '@') or (Buf[1] = '%')) and
                 (Buf[2] in DecChars)
              then begin
                if ParamDefined[Buf[2]]
                then begin
                  ErrorCode := ecDupParam;
                  exit;
                end else ParamDefined[Buf[2]] := true;
              end
              else begin
                ErrorCode := ecParamErr;
                exit;
              end;
            end;
          end;

        ltBegin :
          begin
            if LineList.Count > 1
            then begin
              ErrorCode := ecBeginNoArgs;
              exit;
            end;
          end;

        ltEnd :
          begin
            if LineList.Count > 1
            then begin
              ErrorCode := ecEndNoArgs;
              exit;
            end;
          end;

        ltCmd :
          begin
{Check parameter substitutions in command lines}
            for j := 1 to LineList.Count - 1 do
            begin
              Buf := LineList[j];
              if (length(Buf) = 2) and
                 ((Buf[1] = '@') or (Buf[1] = '%')) and
                 (Buf[2] in DecChars)
              then begin
                if not ParamDefined[Buf[2]]
                then begin
                  ErrorCode := ecUndefParam;
                  exit;
                end;
              end;
            end;
          end;
      end; // case LineType of
      i := i + 1;
      PrevLineType := CurrLineType;
    end; // while i < MEForm.MEEdit.Lines.Count do
  finally
    LineList.Free;
    NameList.Free;
    if ErrorCode = ecNone then ShowMessage('Syntax checks OK')
    else begin
{  tErrorCode = (ecNone,ecDefmExp,ecBeginExp,ecEndExp,
                ecDupDefm,ecDupBegin,ecDupEnd,
                ecDupParam,ecParamErr,ecBeginNoArgs,ecEndNoArgs,ecUndefParam,
                ecNoName,ecDupName);
}
      if ErrorCode in [ecDupParam,ecParamErr,ecUndefParam]
      then HelpCtx := IDH_REPLACEABLE_PARAMETERS
      else HelpCtx := IDH_MACROS;
      MessageDlg('Syntax error encountered at line ' + IntToStr(i+1) + CRLF + CRLF +
                 ErrorStrArray[ErrorCode],mtError,[mbOK,mbHelp],HelpCtx);
      CharIndex := SendMessage(MEForm.MEEdit.Handle,EM_LINEINDEX,i,0);
      MEForm.MEEdit.SelStart := CharIndex;
    end;
  end;
end; // procedure SyntaxCheck

procedure TMEForm.CheckSyntaxExecute(Sender: TObject);
begin
  SyntaxCheck;
end;

procedure TMEForm.MakeMacroExecute(Sender: TObject);
var
  i,StartRow,EndRow,StartChar : integer;
  Buf                         : string;
begin
  with MEForm.MEEdit do
  begin
{Discover the line containing SelStart}
    StartRow  := SendMessage(Handle,EM_LINEFROMCHAR,SelStart,0);

{Discover the line containing SelStart+SelLength}
    EndRow    := SendMessage(Handle,EM_LINEFROMCHAR,SelStart+SelLength,0);

{Indent intermediate lines}
    i := StartRow;
    while i <= EndRow do
    begin
      Buf := Lines[i];
      DelLeadSpaceTab(Buf);
      System.Insert('  ',Buf,1);
      Lines[i] := Buf;
      i := i + 1;
    end;

{Insert DEFM, BEGIN and END}
    if EndRow < Lines.Count - 1 then Lines.Insert(EndRow+1,'END')
                                else Lines.Add('END');
    Lines.Insert(StartRow,'BEGIN');
    Lines.Insert(StartRow,'DEFM ');

{Position cursor to insert macro name}
    StartChar := SendMessage(Handle,EM_LINEINDEX,StartRow,0);
    SelStart  := StartChar + 5;
  end; // with MEForm.Memo1 do
end; // procedure TMEForm.MakeMacroExecute

procedure TMEForm.FormDeactivate(Sender: TObject);
begin
  if NameListNeedsRebuilding then BuildMacroNameList;
end; // procedure TMEForm.FormDeactivate

procedure TMEForm.MEFileNewExecute(Sender: TObject);
begin
  MainForm.NewMacroExecute(nil);
end; // procedure TMEForm.MEFileNewExecute

procedure TMEForm.MEFileOpenExecute(Sender: TObject);
begin
  MainForm.OpenMacroExecute(nil);
end; // procedure TMEForm.MEFileOpenExecute

procedure TMEForm.MEFileSaveExecute(Sender: TObject);
begin
  MainForm.SaveMacroExecute(nil);
end; // procedure TMEForm.MEFileSaveExecute

procedure TMEForm.MEFileSaveAsExecute(Sender: TObject);
begin
  MainForm.SaveMacroAsExecute(nil);
end; // procedure TMEForm.MEFileSaveAsExecute

end.

