unit MemRW;

interface

uses Windows, Forms, Dialogs, SysUtils, Controls, CPort, Classes, TypeDefs;

function  BprotWarningDlg : boolean;
function  ReadByte(Address : word) : byte;
function  ReadWord(Address : word) : word;
procedure ReadMCU256(StartAddr,EndAddr : word;     // read up to 256 bytes
                     UpdateProgress    : boolean);
procedure ReadMCU64K(StartAddr,EndAddr : word;     // read any number of bytes
                     HandleProgress : boolean);
procedure ReadMCUMap(const S19Map         : tList; // read according to map
                           HandleProgress : boolean);
function  WriteByte(Address        : word;
                    B              : byte;
                    ShowEchoErrors : boolean) : boolean;
function  WriteByteEeprom2(Address        : word;             // uses overlay
                           B              : byte;
                           ShowEchoErrors : boolean) : boolean;
function  WriteWord(Address        : word;
                    W              : word;
                    ShowEchoErrors : boolean) : boolean;
function  WriteMCUMap(const WriteMap       : tList; // write according to map
                            HandleProgress : boolean) : boolean;
function  WriteOneBlock(StartAddr,EndAddr : word;
                        HandleProgress    : boolean) : boolean;

function  WriteByteEEPROM(Address : word; OldValue,NewValue : byte) : boolean;
                                                   // write single EEPROM byte
procedure ByteEraseEEPROM(Address : word);         // erase single EEPROM byte
procedure BulkEraseEEPROM(Address : word);         // erase the who;e of EEPROM

implementation

uses Main,CmdUnit, LookupUnit, Comms1, LoadUnit, MemUnit, MiscUnit;

const
  RdMcuCmdErrMsg  : string = 'Comms error reading MCU memory';
  WrRamCmdErrMsg  : string = 'Comms error writing RAM';
  WrRamEchoErrMsg : string = 'Echo error writing RAM';

  RdSPCmdErrMsg   : string = 'Comms error reading Stack Pointer';
  SrvSWICmdErrMsg : string = 'Comms error servicing SWI';

{Returns false if BPROT value inhibits writing, and cancel is 'Cancel' clicked}
function  BprotWarningDlg : boolean;
var
  RxByte  : byte;
  RegAddr : word;
  i       : integer;
begin
  Result := true;
  if not LookupCtrlRegAddr('BPROT',RegAddr,i) then exit;
  RxByte := ReadByte(RegAddr);
  if (RxByte and $0F) > 0
  then begin
    if MessageDlg('Current value of BPROT ($' + IntToHex(RxByte,2) + ')' + CRLF +
                  'may not allow writing to EEPROM' + CRLF + CRLF +
                  'OK to continue anyway,' + CRLF +
                  'CANCEL to abort command' + CRLF + CRLF +
                  '(Disable this message in Settings>General)',
                  mtWarning,[mbCancel,mbOK,mbHelp],
                  IDH_MANAGING_THE_BPROT_REGISTER) = mrCancel
    then  begin Result := false; exit; end;
  end;
end; // function  BprotWarningDlg

function BurnWarningDlg : boolean;
begin
  Result :=  MessageDlg(
             'JBug11 is about to program EPROM' + CRLF +
             'Check that programming voltage is applied' + CRLF + CRLF +
             'OK when ready to continue,' + CRLF +
             'CANCEL to abort command'  + CRLF + CRLF +
             '(Disable this message in Settings>General)',
             mtWarning,[mbCancel,mbOK,mbHelp],
             IDH_WRITING_ON_CHIP_EPROM_AND_EEPROM) <> mrCancel;
end; // function BurnWarningDlg

procedure ShowBurnFinishDlg;
begin
  MessageDlg('JBug11 has finished programming EPROM' + CRLF +
             'Programming voltage may now be removed' + CRLF + CRLF +
             '(Disable this message in Settings>General)',
             mtInformation,[mbOK,mbHelp],IDH_WRITING_ON_CHIP_EPROM_AND_EEPROM);
end; // procedure ShowBurnFinishDlg

procedure ShowBurnFinishDlg2;
begin
  MessageDlg('JBug11 has finished programming EPROM' + CRLF +
             'Programming voltage may now be removed' + CRLF +
             '(Disable this message in Settings>General)' + CRLF + CRLF +
             'Reset target board to continue',
             mtInformation,[mbOK,mbHelp],IDH_WRITING_ON_CHIP_EPROM_AND_EEPROM);
end; // procedure ShowBurnFinishDlg

{Read Procedures ***********************************************************}

function  ReadByte(Address : word) : byte;
begin
  ReadMCU256(Address,Address,false);
  Result := Memory[Address];
end; // function  ReadByte

function  ReadWord(Address : word) : word;
begin
  ReadMCU256(Address,Address+1,false);
  Result := (Memory[Address] shl 8) + Memory[Address+1];
end; // function  ReadWord

{Procedure to read in up to 256 bytes of MCU-controlled memory.  This
 limitation is a function of the talker.
 Uses the local memory array}
procedure ReadMCU256(StartAddr,EndAddr : word;
                     UpdateProgress    : boolean);
var
  i,NumBytes,ProgressCount : integer;
  TxByte,RxByte   : byte;
  CmdOK           : boolean;
  EMsg,RxStr      : string;
begin
  ProgressCount := 0;
  CmdOK         := true;
  RxStr         := '';
  NumBytes      := EndAddr - StartAddr + 1;
  if (NumBytes > 256) or (NumBytes < 1) then exit;
  try
    with CommsForm.ComPort do
    begin
      ClearBuffer(true,true);
      TxByte := $01;
      Write(TxByte,1);
      Read(RxByte,1);
      CmdOK := RxByte = $FE;
      if not CmdOK then exit;

      TxByte := lo(NumBytes);      // 'lo' so that 0 sent for Numbytes = 256
      Write(TxByte,1);
      TxByte := hi(StartAddr);
      Write(TxByte,1);
      TxByte := lo(StartAddr);
      Write(TxByte,1);
{
      for i := StartAddr to StartAddr + NumBytes - 1 do
      begin
        Read(RxByte,1);
        Memory[i] := RxByte;
        Write(TxByte,1);
        ProgressCount := ProgressCount + 1;
        if UpdateProgress and (ProgressCount > 15)
        then begin
          MainForm.ProgressBar1.StepBy(16);
          ProgressCount := 0;
        end;
      end;
}
{Above 12 lines replaced by code below on 14 Jan 05. Huge increase in speed as
 a result. Appears to be due to the way Windows processes single read requests
 versus reading in a string from the com port}

      Read(RxByte,1);
      Memory[StartAddr] := RxByte;
      for i := 1 to NumBytes do
      begin
        Write(RxByte,1);                           // any old byte will do
        ProgressCount := ProgressCount + 1;
        if UpdateProgress and (ProgressCount > 15)
        then begin
          MainForm.ProgressBar1.StepBy(16);
          ProgressCount := 0;
        end;
      end;
      if NumBytes > 1
      then begin
        ReadStr(RxStr,NumBytes-1);
        for i := 1 to length(RxStr) do Memory[StartAddr + i] := ord(RxStr[i]);
      end;

      if UpdateProgress then MainForm.ProgressBar1.StepBy(ProgressCount);
    end; // with CommsForm.ComPort do
  finally
    if not CmdOK
    then begin
      EMsg := 'Comms error reading MCU memory';
      UpdateDiagnostics(StartAddr,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
  end; // try.. finally..
end; // function  ReadMCU256

{Procedure to read any number of bytes from MCU-controlled memory.  Calls
 ReadMCU256 repeatedly to read the memory in max 256 byte chunks.
 Uses the local memory array}
procedure ReadMCU64K(StartAddr,EndAddr : word;
                     HandleProgress    : boolean);
var
  TempAddr : word;
begin
  try
    if HandleProgress
    then begin
      with MainForm.ProgressBar1 do
      begin
        Max      := EndAddr - StartAddr + 1;
        Step     := 16;
        Position := 0;
      end; // with MainForm.ProgressBar1 do
    end; // if HandleProgress then..
    TempAddr := StartAddr;
    while (EndAddr - TempAddr) > 255 do
    begin
      ReadMCU256(TempAddr,TempAddr+255,HandleProgress);
      TempAddr := TempAddr + 256;
    end;
    ReadMCU256(TempAddr,EndAddr,HandleProgress);
  finally
    if HandleProgress then MainForm.ProgressBar1.Position := 0;
  end;
end; // function  ReadMCUMem

{Procedure to read MCU memory to local memory according to a supplied map.}
procedure ReadMCUMap(const S19Map         : tList;
                           HandleProgress : boolean);
var
  i : integer;
begin
  try
    if HandleProgress
    then begin
      with MainForm.ProgressBar1 do
      begin
        Max      := TotalBytesInMap(S19Map);
        Position := 0;
      end; // with MainForm.ProgressBar1 do
    end;
    for i := 0 to S19Map.Count - 1 do
    begin
      with tMemRecordPtr(S19Map[i])^ do ReadMCU64K(SA,EA,HandleProgress);
    end;
  finally
    if HandleProgress then MainForm.ProgressBar1.Position := 0;
  end;
end; // procedure ReadMCUMap

{Write Procedures ***********************************************************}

{General function  for writing 1-256 bytes via the talker.
 'MemArea' selects the control byte to send. 'NumBytes' and 'StartAddr' are
 always sent as-is to the talker. 'WaitForEcho' determines whether or not
 the routine waits for the talker to echo each byte before it sends the next
 one. Note that MemArea = maPageWrite expects that WaitForEcho = false, i.e.
 bytes are echoed immediately, not after the page has been written - this
 implies that a subsequent verification is always a good idea.

 If they are needed, overlays must be in position before calling this routine.}

function  WriteKernel256(MemArea        : tMemArea;
                         NumBytes       : integer;
                         StartAddr      : word;
                         WaitForEcho    : boolean;
                         ShowEchoErrors : boolean;
                         UpdateProgress : boolean) : boolean;
var
  i,ProgressCount        : integer;
  TxByte,RxByte,CtrlByte : byte;
  CmdOK,EchoOK           : boolean;
  EMsg,RxStr             : string;
begin
  i             := 0;
  ProgressCount := 0;
  EchoOK        := true;
  CmdOK         := true;
  Result        := false;

  if (NumBytes > 256) or (NumBytes < 1) then exit;

  CtrlByte := CtrlByteArray[MemArea];
  EMsg     := 'error writing ' + MemoryVNArray[MemArea] + ' memory';

  try
    with CommsForm.ComPort do
    begin
      ClearBuffer(true,true);
{Send the command byte}
      TxByte := CtrlByte;
      Write(TxByte,1);
      Read(RxByte,1);
      CmdOK := RxByte = ($FF - CtrlByte);
      if not CmdOK then exit;
{Send the number of bytes}
      TxByte := lo(NumBytes);      // 'lo' so that 0 sent for Numbytes = 256
      Write(TxByte,1);
{Send the starting address}
      TxByte := hi(StartAddr);
      Write(TxByte,1);
      TxByte := lo(StartAddr);
      Write(TxByte,1);
{Send the bytes. Use different methods depending upon 'WaitForEcho'}
      i := 0;
      while i < NumBytes do
      begin
        TxByte := Memory[i + StartAddr];
        Write(TxByte,1);
        if WaitForEcho
        then begin
          Read(RxByte,1);
          if ShowEchoErrors and
             (RxByte <> TxByte) and
             not ProjData.IgnoreEchoErrors
          then begin EchoOK := false; exit; end;
        end;
        ProgressCount := ProgressCount + 1;
        if UpdateProgress and (ProgressCount > 15)
        then begin
          MainForm.ProgressBar1.StepBy(16);
          ProgressCount := 0;
        end;
        i := i + 1;
      end;
      if UpdateProgress then MainForm.ProgressBar1.StepBy(ProgressCount);

      if not WaitForEcho
      then begin
        if MemArea = maPageWrite then Delay(20); // to allow onboard FLASH routine
        ReadStr(RxStr,NumBytes);
        if ShowEchoErrors and not ProjData.IgnoreEchoErrors
        then begin
          i := 0;
          EchoOK := length(RxStr) = NumBytes;
          if not EchoOK then exit;
          while i < NumBytes do
          begin
            EchoOK := Memory[i + StartAddr] = ord(RxStr[i+1]);
            if not EchoOK then exit;
            i := i + 1;
          end;
        end; // if ShowEchoErrors
      end;
    end; // with MainForm.ComPort1 do
  finally
    if not CmdOK
    then begin
      EMsg := 'Comms ' + EMsg;
      UpdateDiagnostics(StartAddr,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
    if not EchoOK
    then begin
      EMsg := 'Echo ' + EMsg;
      UpdateDiagnostics(StartAddr+i,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
    Result := CmdOK and EchoOK;
  end; // try.. finally..
end; // function  WriteKernel256

function  WriteByte(Address        : word;
                    B              : byte;
                    ShowEchoErrors : boolean) : boolean;
begin
  Memory[Address] := B;
  Result := WriteKernel256(maRam,1,Address,true,ShowEchoErrors,false);
end; // function WriteByte

function  WriteByteEeprom2(Address        : word;
                           B              : byte;
                           ShowEchoErrors : boolean) : boolean;
begin
  Memory[Address] := B;
  Result := WriteKernel256(maEeprom,1,Address,true,ShowEchoErrors,false);
end; // function WriteByte

function  WriteWord(Address        : word;
                    W              : word;
                    ShowEchoErrors : boolean) : boolean;
begin
  Memory[Address]   := hi(W);
  Memory[Address+1] := lo(W);
  Result := WriteKernel256(maRam,2,Address,true,ShowEchoErrors,false);
end; // function WriteWord

{Procedure to write any number of bytes to MCU-controlled memory.  Calls
 WriteMCU256 repeatedly to write the memory in chunks. Overlays must be in
 position before calling this routine.
 This is not a published procedure}
function  WriteMCU64K(    StartAddr,EndAddr : word;
                          MemArea           : tMemArea;
                          UpdateProgress    : boolean;
                          ShowEchoErrors    : boolean) : boolean;
var
  TempAddr  : word;
  EchoWait  : boolean;
  NumBytes  : integer;
begin
  Result   := false;

{Certain overlays must use the WaitForEcho type of interaction because of the
 delay inherent in programming EEPROM and EPROM. Note that ProjData.WaitForEcho
 is only taken account of by this routine when it comes to writes to RAM}
  EchoWait := false;
  case MemArea of
    maUndefined,maRom            : exit;
    maRam,maCtrlRegs,maExtRam    : EchoWait := ProjData.WaitForEcho;
    maPageWrite                  : EchoWait := false; // bytes are echoed immediately
                                                      // after they are written to the on-board buffer
    maEeprom,maEprom,maByteWrite : EchoWait := true;
  end;

  TempAddr := StartAddr;
  while (EndAddr - TempAddr) > 255 do
  begin
    if not WriteKernel256(MemArea,256,TempAddr,EchoWait,
                          ShowEchoErrors,UpdateProgress) then exit;
    TempAddr := TempAddr + 256;
  end;

  NumBytes := integer(EndAddr) - integer(TempAddr) + 1;
  if NumBytes > 0
  then begin
    if not WriteKernel256(MemArea,NumBytes,TempAddr,EchoWait,
                          ShowEchoErrors,UpdateProgress) then exit;
  end;
  Result := true;
end; // function  WriteMCU64K

{Procedure to write to external memory that must be written in whole pages
 at a time. This is complicated by having to read in the bytes which are
 overlapped by the page erasing}
function  WriteMCUWholePage(StartAddr,EndAddr : word;
                            UpdateProgress,ShowEchoErrors : boolean) : boolean;
var
  FAddr1,FAddr2 : longword;
  TempAddr      : word;
begin
  Result := false;
{If the starting and ending addresses do not coincide with data page boundaries,
 then additional bytes have to be read in first}
  with ProjData do
  begin
    FAddr1 := (StartAddr div DataPageSize) * DataPageSize;
    if FAddr1 < StartAddr
    then ReadMCU256(FAddr1,StartAddr - 1,false);
    FAddr2 := (((EndAddr + 1) div DataPageSize) * DataPageSize) - 1;
    if FAddr2 < EndAddr
    then begin
      FAddr2 := FAddr2 + word(DataPageSize);
      ReadMCU256(EndAddr + 1,FAddr2,false);
    end;

    if (FAddr1 > $FFFF) or (FAddr2 > $FFFF) then exit;
    TempAddr := FAddr1;
    while (FAddr2 - TempAddr) > word(DataPageSize - 1) do
    begin
      if not WriteKernel256(maPageWrite,DataPageSize,TempAddr,false,
                            ShowEchoErrors,UpdateProgress) then exit;
      TempAddr := TempAddr + DataPageSize;
    end;
    if not WriteKernel256(maPageWrite,DataPageSize,TempAddr,false,
                          ShowEchoErrors,UpdateProgress) then exit;
  end; // with ProjData do
  Result := true;
end; // function  WriteMCUWholePage
// {$O-}

{Procedure to write to external memory where all the bytes to be written at
 any one time must lie within a page. External EEPROM chips are often like this.
 This routine is like WriteMcu64K but works in chunks of DataPageSize rather
 than 256 bytes, and adjusts the first write to be within a page}
function  WriteMCUWithinPage(StartAddr,EndAddr : word;
                             UpdateProgress,ShowEchoErrors : boolean) : boolean;
var
  FirstBlockEndAddr : longword;
  TempAddr          : longword;
  NumBytes          : integer;
begin
  Result := false;
  with ProjData do
  begin
{Write the first partial block}
    FirstBlockEndAddr := (((StartAddr div DataPageSize) + 1) * DataPageSize) - 1;

    if EndAddr < FirstBlockEndAddr
    then NumBytes := EndAddr - StartAddr + 1
    else NumBytes := FirstBlockEndAddr - StartAddr + 1;

    if not WriteKernel256(maPageWrite,NumBytes,StartAddr,false,
                          ShowEchoErrors,UpdateProgress) then exit;

{Write subsequent full blocks (pages), if any}
    TempAddr := FirstBlockEndAddr + 1;
    while (integer(TempAddr) + (DataPageSize-1)) < integer(EndAddr) do
    begin
      if not WriteKernel256(maPageWrite,DataPageSize,TempAddr,false,
                            ShowEchoErrors,UpdateProgress) then exit;
      TempAddr := TempAddr + word(DataPageSize);
    end;

{Write the last partial block, if any}
    NumBytes := integer(EndAddr) - integer(TempAddr) + 1;
    if NumBytes > 0
    then begin
      if not WriteKernel256(maPageWrite,NumBytes,TempAddr,false,
                            ShowEchoErrors,UpdateProgress) then exit;
    end;
  end; // with ProjData do
  Result := true;
end; // function  WriteMCUWithinPage

{Function which may be used to send any number of bytes to be programmed into
 OTP EPROM using the on-board, ROM-resident routine found in 711E9 and 711D3
 chips}
function WriteUsingRomRoutine(StartAddr,EndAddr : word; UpdateProgress,ShowEchoErrors : boolean) : boolean;
var
  i,ProgressCount                  : integer;
  TxByte,RxByte,CtrlByte,DelayByte : byte;
  CmdOK,EchoOK                     : boolean;
  EMsg                             : string;
  CrystalFreq                      : real;
  w                                : word;
begin
  i             := 0;
  ProgressCount := 0;
  EchoOK        := true;
  CmdOK         := true;
  Result        := false;

  CtrlByte := CtrlByteArray[maEprom];
  EMsg     := 'error writing ' + MemoryVNArray[maEprom] + ' memory using ROM-resident routine';

  try
    with CommsForm.ComPort do
    begin
      ClearBuffer(true,true);
{Send the command byte}
      TxByte := CtrlByte;
      Write(TxByte,1);
      Read(RxByte,1);
      CmdOK := RxByte = ($FF - CtrlByte);
      if not CmdOK then exit;
{Send a byte which will be used to determine the programming delay. The value
 to send is:
 Bus frequency x 0.004 [4 ms] / 256. This may well be a fraction, so use the
 next higher whole number. Now Bus frequency = Crystal frequency / 4, so
 Delay := CrystalFrequency / 4 x 0.004 / 256 =
          CrystalFrequency * 1000 / 256 when CrystalFrequency is in MHz  }
      val(ProjData.CrystalFrequency,CrystalFreq,i);
      DelayByte := lo(trunc(CrystalFreq * 1000 / 256) + 1);
      Write(DelayByte,1);
{Send the starting address}
      TxByte := hi(StartAddr);
      Write(TxByte,1);
      TxByte := lo(StartAddr);
      Write(TxByte,1);
      
{Send the bytes. The ROM-resident routine on the chip begins by sending an $FF
 byte. When this is received the first byte should be sent. We have then to
 wait for this byte to be echoed back before sending the next one. This could
 happen indefinitely, in fact the only way to stop is to reset the MCU  }
      TxByte := $FF;        // for the use of "UpdateDiagnostics" if necessary
      Read(RxByte,1);
      EchoOK := RxByte = $FF;
      if not EchoOK then exit;

      w := StartAddr;
      while (w <= EndAddr) do
      begin
        TxByte := Memory[w];
        Write(TxByte,1);
        Read(RxByte,1);
        if ShowEchoErrors and (RxByte <> TxByte) and (not ProjData.IgnoreEchoErrors)
        then begin EchoOK := false; exit; end;

        ProgressCount := ProgressCount + 1;
        if UpdateProgress and (ProgressCount > 15)
        then begin
          MainForm.ProgressBar1.StepBy(16);
          ProgressCount := 0;
        end;
        w := w + 1;
      end;
      if UpdateProgress then MainForm.ProgressBar1.StepBy(ProgressCount);
    end; // with MainForm.ComPort1 do
  finally
    if not CmdOK
    then begin
      EMsg := 'Comms ' + EMsg;
      UpdateDiagnostics(StartAddr,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
    if not EchoOK
    then begin
      EMsg := 'Echo ' + EMsg;
      UpdateDiagnostics(StartAddr+i,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
    Result := CmdOK and EchoOK;
  end; // try.. finally..
end; // function WriteUsingRomRoutine

// {$O+}
{Procedure to write local memory to the MCU according to a supplied map.
 Handles overlays as necessary. Returns true if no error encountered}
function  WriteMCUMap(const WriteMap       : tList;
                            HandleProgress : boolean) : boolean;
var
  i    : integer;
  Tptr : tMemRecordPtr;
  Buf  : string;
  OverlayNeeded,OverlayLoaded : tOverlayType;
  EepromWarningShown,EpromWarningShown,RomRoutineUsed : boolean;
begin
  Result             := false;
  OverlayLoaded      := oyNone;
  EepromWarningShown := false;
  EpromWarningShown  := false;
  RomRoutineUsed     := false;
  if HandleProgress
  then begin
    with MainForm.ProgressBar1 do
    begin
      Max      := TotalBytesInMap(WriteMap);
      Position := 0;
    end; // with MainForm.ProgressBar1 do
  end;

  WriteMap.Sort(SortByAreas1);
  try
    for i := 0 to WriteMap.Count - 1 do
    begin
      Tptr := WriteMap[i];
      with Tptr^,ProjData do
      begin
{Overlays need only actually be loaded if the talker is a RAM-resident one.
 A talker in external memory is assumed to provide all the necessary routines
 without the need for overlays, and an EEPROM-resident talker is assumed to
 be incapable of writing to EEprom, Eprom or external FLASH}
        if TalkerType = ttRam
        then begin
          case Area of
            maEEprom    : OverlayNeeded := oyEEprom;
            maEprom     : OverlayNeeded := oyEprom;
            maByteWrite : OverlayNeeded := oyByteWrite;
            maPageWrite : OverlayNeeded := oyPageWrite
          else OverlayNeeded := oyNone;
          end; // case
        end else OverlayNeeded := oyNone;

{Show BPROT warning message}
        if not EepromWarningShown and (Area = maEeprom) and WarnOnBprot
        then begin
          if not BprotWarningDlg then exit;
          EepromWarningShown := true;
        end; // if not EepromWarningShown and (Area = maEeprom) then..

{Show Vprog start-of-programming message}
        if not EpromWarningShown and (Area = maEprom)
        then begin
          if WarnOnVprog then if not BurnWarningDlg then exit;
          EpromWarningShown := true;
        end; // if not EpromWarningShown and (Area = maEprom) then..

{Show Vprog end-of-programming message}
        if EpromWarningShown and (Area <> maEprom)
        then begin
          if WarnOnVprog then ShowBurnFinishDlg;
          EpromWarningShown := false;
        end; // if EpromWarningShown and (Area <> maEprom) then..

{Load overlays as necessary}
        if OverlayNeeded <> OverlayLoaded
        then begin
          if (OverlayLoaded <> oyNone) and (not RomRoutineUsed)
          then begin
            UnloadOverlay;
            OverlayLoaded := oyNone;
          end; // if OverlayLoaded <> oyNone

          if OverlayNeeded <> oyNone
          then begin
            LoadOverlay(OverlayNeeded);
            OverlayLoaded := OverlayNeeded;
          end; // if OverlayNeeded <> oyNone
        end; // if OverlayNeeded <> OverlayLoaded

        if HandleProgress
        then begin
          Buf := 'Writing ' + MemoryDescArray[Area] + ': ';
//          SpaceTo(Buf,24);
          Buf := Buf + IntToHex(SA,4) + '..' + IntToHex(EA,4);
          MainForm.OWEdit.Lines.Add(Buf);
        end;

        if (Area = maPageWrite)
        then begin
          if WriteWholePage
          then WriteMCUWholePage(SA,EA,HandleProgress,true)
          else WriteMCUWithinPage(SA,EA,HandleProgress,true)
        end
        else WriteMCU64K(SA,EA,Area,HandleProgress,true);

      end; // with Tptr^,ProjData do
    end; // for i := 0 to WriteMap.Count - 1 do

{After using the overlay that, in turn, makes use of the ROM-resident routine
 on a 711E9 or 711D3, the only way out is to reset the MCU}
    if (OverlayLoaded <> oyNone) and (not RomRoutineUsed) then UnloadOverlay;

    if EpromWarningShown
    then begin
      if RomRoutineUsed
      then ShowBurnFinishDlg2
      else if ProjData.WarnOnVprog then ShowBurnFinishDlg;
    end; // if EpromWarningShown then..
  finally
    MainForm.ProgressBar1.Position := 0;
  end; // try.. finally..
  Result := true;
end; // function WriteMCUMap

{procedure to write a single block of memory from local memory to the MCU.
 The block must occupy a single defined memory area}
function  WriteOneBlock(StartAddr,EndAddr : word;
                        HandleProgress    : boolean) : boolean;
var
  TMap    : tList;
  Tptr    : tMemRecordPtr;
  MemArea : tMemArea;
begin
  Result := false;
  if not IsOneBlock(StartAddr,EndAddr,MemArea,MemMap) then exit;
  TMap := tList.Create;
  new(Tptr);
  try
    with Tptr^ do
    begin
      SA := StartAddr;
      EA := EndAddr;
      Area := MemArea;
    end;
    TMap.Add(Tptr);
    if not WriteMCUMap(TMap,HandleProgress) then exit;
  finally
    ClearMemMap(TMap);    // also disposes of Tptr
    TMap.Free;
  end;
  Result := true;
end; // procedure WriteOneBlock

{}
function  AlterEeprom(Address : word;
                      OpType  : char;
                      B       : byte;
                      IgnoreEchoErrors : boolean) : boolean;
var
  RxByte,P1,P2,P3 : byte;
  ProgCtrlReg     : word;
  EMsg            : string;
  i               : integer;
begin
  Result := false;
  if not LookupCtrlRegAddr('PPROG',ProgCtrlReg,i) then exit;
  case OpType of
    'W' : begin
            P1     := $02;
            EMsg   := 'writing byte to EEPROM';
          end;
    'B' : begin
            P1     := $06;
            EMsg   := 'bulk erasing EEPROM';
          end;
    'E' : begin
            P1     := $16;
            EMsg   := 'erasing EEPROM byte';
          end
    else begin
      P1     := 0;
      EMsg   := '';
    end;
  end; // case
  P2 := P1 + 1;
  P3 := 0;
  WriteByte(ProgCtrlReg,P1,true);      // set
  WriteByte(Address,B,false);
  WriteByte(ProgCtrlReg,P2,true);      // set PGM bit

  if ProjData.CommsBaudRate > 9600 then Delay(5);

  WriteByte(ProgCtrlReg,P3,true);      // clear PROG and set to read mode
  RxByte := ReadByte(Address);
  Result := RxByte = B;
  if (not Result) and (not IgnoreEchoErrors)
  then begin
    EMsg := 'Echo error ' + EMsg;
    UpdateDiagnostics(Address,B,RxByte,EMsg);
    Raise EMCUComErr.Create(EMsg);
  end; // if TxByte <> RxByte then..
end; // function  AlterEEPROM

procedure ByteEraseEEPROM(Address : word);
var
  IgnoreEchoErrors : boolean;
  RegAddr          : word;
  i                : integer;
begin
  LookupCtrlRegAddr('CONFIG',RegAddr,i);
  IgnoreEchoErrors := RegAddr = Address;
  AlterEeprom(Address,'E',$FF,IgnoreEchoErrors);
end; // function  ByteEraseEEPROM

function  WriteByteEEPROM(Address : word; OldValue,NewValue : byte) : boolean;
begin
  if OldValue = NewValue then begin Result := true; exit; end;
  if OldValue <> $FF then ByteEraseEeprom(Address);
  Result := AlterEeprom(Address,'W',NewValue,false);
end; // function  WriteByteEEPROM

procedure BulkEraseEEPROM(Address : word);
begin
  AlterEeprom(Address,'B',$FF,false);
end; // function  BulkEraseEEPROM

end.
