unit MemRWold;

interface

uses Windows, Forms, Dialogs, SysUtils, Controls, CPort, Classes, TypeDefs;

function  BprotWarningDlg : boolean;
function  ReadByte(Address : word) : byte;
function  ReadWord(Address : word) : word;
procedure ReadMCU256(StartAddr,EndAddr : word;     // read up to 256 bytes
                     UpdateProgress    : boolean);
procedure ReadMCU64K(StartAddr,EndAddr : word;     // read any number of bytes
                     HandleProgress : boolean);
procedure ReadMCUMap(const S19Map         : tList; // read according to map
                           HandleProgress : boolean);
procedure WriteByte(Address : word; B : byte; ShowEchoErrors : boolean);
procedure WriteWord(Address : word; W : word; ShowEchoErrors : boolean);
function  WriteRam256(StartAddr,EndAddr : word;    // write up to 256 bytes
                      MemArea           : tMemArea;
                      UpdateProgress    : boolean;
                      ShowEchoErrors    : boolean) : boolean;
function  WriteProm256(StartAddr,EndAddr : word;
                       MemArea           : tMemArea;
                       UpdateProgress    : boolean;
                       ShowEchoErrors    : boolean) : boolean;
procedure WriteMCUMap(const WriteMap       : tList; // write according to map
                            HandleProgress : boolean);
procedure WriteOneBlock(StartAddr,EndAddr : word;
                        HandleProgress    : boolean);

function  WriteByteEEPROM(Address : word): boolean;// write single EEPROM byte
procedure ByteEraseEEPROM(Address : word);         // erase single EEPROM byte
procedure BulkEraseEEPROM(Address : word);         // erase the who;e of EEPROM

implementation

uses Main,CmdUnit, LookupUnit, Comms1, LoadUnit, MemUnit, MiscUnit;

const
  RdMcuCmdErrMsg  : string = 'Command error reading MCU memory';
  WrRamCmdErrMsg  : string = 'Command error writing RAM';
  WrRamEchoErrMsg : string = 'Echo error writing RAM';

  RdSPCmdErrMsg   : string = 'Command error reading Stack Pointer';
  SrvSWICmdErrMsg : string = 'Command error servicing SWI';

function  BprotWarningDlg : boolean;
var
  RxByte  : byte;
  RegAddr : word;
  i       : integer;
begin
  Result := true;
  if not LookupCtrlRegAddr('BPROT',RegAddr,i) then exit;
  ReadMCU256(RegAddr,RegAddr,false);
  RxByte := Memory[RegAddr];
  if (RxByte and $0F) > 0
  then begin
    if MessageDlg('Current value of BPROT ($' + IntToHex(RxByte,2) + ')' + CRLF +
                  'may not allow writing to EEPROM' + CRLF + CRLF +
                  'OK to continue anyway,' + CRLF +
                  'CANCEL to abort command' + CRLF + CRLF +
                  '(Disable this message in Settings>General)',
                  mtWarning,[mbCancel,mbOK,mbHelp],
                  IDH_MANAGING_THE_BPROT_REGISTER) = mrCancel
    then  begin Result := false; exit; end;
  end;
end; // function  BprotWarningDlg

function BurnWarningDlg : boolean;
begin
  Result := (MessageDlg(
             'JBug11 is about to program EPROM' + CRLF +
             'Check that programming voltage is applied' + CRLF + CRLF +
             'OK when ready to continue,' + CRLF +
             'CANCEL to abort command'  + CRLF + CRLF +
             '(Disable this message in Settings>General)',
             mtWarning,[mbCancel,mbOK,mbHelp],IDH_PROGRAMMING_EPROM)<> mrCancel)
end; // function BurnWarningDlg

procedure ShowBurnFinishDlg;
begin
  MessageDlg('JBug11 has finished programming EPROM' + CRLF +
             'Programming voltage may now be removed' + CRLF + CRLF +
             '(Disable this message in Settings>General)',
             mtInformation,[mbOK,mbHelp],IDH_PROGRAMMING_EPROM);
end; // procedure ShowBurnFinishDlg

{Read Procedures ***********************************************************}

function  ReadByte(Address : word) : byte;
begin
  ReadMCU256(Address,Address,false);
  Result := Memory[Address+1];
end; // function  ReadByte

function  ReadWord(Address : word) : word;
begin
  ReadMCU256(Address,Address+1,false);
  Result := (Memory[Address] shl 8) + Memory[Address+1];
end; // function  ReadWord

{Procedure to read in up to 256 bytes of MCU-controlled memory.  This
 limitation is a function of the talker.
 Uses the local memory array}
procedure ReadMCU256(StartAddr,EndAddr : word;
                     UpdateProgress    : boolean);
var
  i,NumBytes,ProgressCount : integer;
  TxByte,RxByte   : byte;
  CmdOK           : boolean;
  EMsg,RxStr      : string;
begin
  ProgressCount := 0;
  CmdOK         := true;
  RxStr         := '';
  NumBytes      := EndAddr - StartAddr + 1;
  if (NumBytes > 256) or (NumBytes < 1) then exit;
  try
    with CommsForm.ComPort do
    begin
      ClearBuffer(true,true);
      TxByte := $01;
      Write(TxByte,1);
      Read(RxByte,1);
      CmdOK := RxByte = $FE;
      if not CmdOK then exit;

      TxByte := lo(NumBytes);      // 'lo' so that 0 sent for Numbytes = 256
      Write(TxByte,1);
      TxByte := hi(StartAddr);
      Write(TxByte,1);
      TxByte := lo(StartAddr);
      Write(TxByte,1);
{
      for i := Address to Address + NumBytes - 1 do
      begin
        Read(RxByte,1);
        Memory[i] := RxByte;
        Write(TxByte,1);
        ProgressCount := ProgressCount + 1;
        if UpdateProgress and (ProgressCount > 15)
        then begin
          MainForm.ProgressBar1.StepBy(16);
          ProgressCount := 0;
        end;
      end;
}
{Above 12 lines replaced by code below on 14 Jan 05. Huge increase in speed as
 a result. Appears to be due to the way Windows processes single read requests
 versus reading in a string from the com port}

      Read(RxByte,1);
      Memory[StartAddr] := RxByte;
      for i := 1 to NumBytes do
      begin
        Write(RxByte,1);                           // any old byte will do
        ProgressCount := ProgressCount + 1;
        if UpdateProgress and (ProgressCount > 15)
        then begin
          MainForm.ProgressBar1.StepBy(16);
          ProgressCount := 0;
        end;
      end;
      if NumBytes > 1
      then begin
        ReadStr(RxStr,NumBytes-1);
        for i := 1 to length(RxStr) do Memory[StartAddr + i] := ord(RxStr[i]);
      end;  

      if UpdateProgress then MainForm.ProgressBar1.StepBy(ProgressCount);
    end; // with CommsForm.ComPort do
  finally
    if not CmdOK
    then begin
      EMsg := 'Command error reading MCU memory';
      UpdateDiagnostics(0,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
  end; // try.. finally..
end; // function  ReadMCU256

{Procedure to read any number of bytes from MCU-controlled memory.  Calls
 ReadMCU256 repeatedly to read the memory in max 256 byte chunks.
 Uses the local memory array}
procedure ReadMCU64K(StartAddr,EndAddr : word;
                     HandleProgress    : boolean);
var
  TempAddr : word;
begin
  try
    if HandleProgress
    then begin
      with MainForm.ProgressBar1 do
      begin
        Max      := EndAddr - StartAddr + 1;
        Step     := 16;
        Position := 0;
      end; // with MainForm.ProgressBar1 do
    end; // if HandleProgress then..
    TempAddr := StartAddr;
    while (EndAddr - TempAddr) > 255 do
    begin
      ReadMCU256(TempAddr,TempAddr+255,HandleProgress);
      TempAddr := TempAddr + 256;
    end;
    ReadMCU256(TempAddr,EndAddr,HandleProgress);
  finally
    if HandleProgress then MainForm.ProgressBar1.Position := 0;
  end;
end; // function  ReadMCUMem

{Procedure to read MCU memory to local memory according to a supplied map.}
procedure ReadMCUMap(const S19Map         : tList;
                           HandleProgress : boolean);
var
  i : integer;
begin
  try
    with MainForm.ProgressBar1 do
    begin
      Max      := TotalBytesInMap(S19Map);
      Position := 0;
    end; // with MainForm.ProgressBar1 do
    for i := 0 to S19Map.Count - 1 do
    begin
      with tMemRecordPtr(S19Map[i])^ do ReadMCU64K(SA,EA,HandleProgress);
    end;
  finally
    MainForm.ProgressBar1.Position := 0;
  end;
end; // procedure ReadMCUMap

{Write Procedures ***********************************************************}

procedure WriteByte(Address : word; B : byte; ShowEchoErrors : boolean);
begin
  Memory[Address] := B;
  WriteRAM256(Address,Address,maRam,false,ShowEchoErrors);
end; // procedure WriteByte

procedure WriteWord(Address : word; W : word; ShowEchoErrors : boolean);
begin
  Memory[Address]   := hi(W);
  Memory[Address+1] := lo(W);
  WriteRAM256(Address,Address+1,maRam,false,ShowEchoErrors);
end; // procedure WriteWord

{General function  for writing 1-256 bytes via the talker.  The control
 byte sent may be varied depending upon whether the standard talker, or one
 of the overlay ones is running on the MCU. This routine is slow for writing
 to RAM, because of the way echoed bytes are read individually. Use the
 routine 'WriteRam256' for the fastest writing to ram type memory.
 If they are needed, overlays must be in position before calling this routine.}
// {$O-}
function  WriteProm256(StartAddr,EndAddr : word;
                       MemArea           : tMemArea;
                       UpdateProgress    : boolean;
                       ShowEchoErrors    : boolean) : boolean;
var
  i,ProgressCount,NumBytes        : integer;
  TxByte,RxByte,CtrlByte : byte;
  CmdOK,EchoOK           : boolean;
  EMsg                   : string;
begin
  i             := 0;
  ProgressCount := 0;
  EchoOK        := true;
  CmdOK         := true;
  NumBytes      := EndAddr - StartAddr + 1;
  Result        := false;

  if (NumBytes > 256) or (NumBytes < 1) then exit;

  case MemArea of
    maEeprom    : begin
                    CtrlByte := $02;                // 1'complement = $FD
                    EMsg     := 'writing EEPROM';
                  end;
    maEprom     : begin
                    CtrlByte := $20;                // 1'complement = $DF
                    EMsg     := 'programming EPROM';
                  end
  else exit;
  end; // case

  try
    with CommsForm.ComPort do
    begin
      ClearBuffer(true,true);
      TxByte := CtrlByte;
      Write(TxByte,1);
      Read(RxByte,1);
      CmdOK := RxByte = ($FF - CtrlByte);
      if not CmdOK then exit;
      TxByte := lo(NumBytes);      // 'lo' so that 0 sent for Numbytes = 256
      Write(TxByte,1);
      TxByte := hi(StartAddr);
      Write(TxByte,1);
      TxByte := lo(StartAddr);
      Write(TxByte,1);
      i := 0;
      while i < NumBytes do
      begin
        TxByte := Memory[i + StartAddr];
        Write(TxByte,1);
        Read(RxByte,1);
        if ShowEchoErrors and (RxByte <> TxByte)
          then begin EchoOK := false; exit; end;
        ProgressCount := ProgressCount + 1;
        if UpdateProgress and (ProgressCount > 15)
        then begin
          MainForm.ProgressBar1.StepBy(16);
          ProgressCount := 0;
        end;
        i := i + 1;
      end;
      if UpdateProgress then MainForm.ProgressBar1.StepBy(ProgressCount);
    end; // with MainForm.ComPort1 do
  finally
    if not CmdOK
    then begin
      EMsg := 'Command error ' + EMsg;
      UpdateDiagnostics(0,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
    if not EchoOK
    then begin
      EMsg := 'Echo error ' + EMsg;
      UpdateDiagnostics(i + StartAddr,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
    Result := CmdOK and EchoOK;
  end; // try.. finally..
end; // function  WriteMCU256
// {$O+}

{General function  for writing 1-256 bytes to Ram memory via the talker.
 Used also by the external FLASH writing routine because the transfer of
 bytes from the PC to the talker is only to a RAM buffer}
function  WriteRam256(    StartAddr,EndAddr : word;
                          MemArea           : tMemArea;
                          UpdateProgress    : boolean;
                          ShowEchoErrors    : boolean) : boolean;
var
  i,ProgressCount,NumBytes        : integer;
  TxByte,RxByte,CtrlByte : byte;
  CmdOK,EchoOK           : boolean;
  EMsg,RxStr             : string;
begin
  i             := 0;
  ProgressCount := 0;
  EchoOK        := true;
  CmdOK         := true;
  Result        := false;

  NumBytes      := EndAddr - StartAddr + 1;
  if (NumBytes > 256) or (NumBytes < 1) then exit;

  case MemArea of
    maRam,maCtrlRegs,maExtRam :
                  begin
                    CtrlByte := $41;                // 1'complement = $BE
                    EMsg     := 'writing RAM';
                  end;
    maExtFlash  : begin
                    CtrlByte := $22;                // 1'complement = $DD
                    EMsg     := 'programming external Flash memory';
                  end
  else exit;
  end; // case

  try
    with CommsForm.ComPort do
    begin
      ClearBuffer(true,true);
      TxByte := CtrlByte;
      Write(TxByte,1);
      Read(RxByte,1);
      CmdOK := RxByte = ($FF - CtrlByte);
      if not CmdOK then exit;

      TxByte := lo(NumBytes);      // 'lo' so that 0 sent for Numbytes = 256
      Write(TxByte,1);
      TxByte := hi(StartAddr);
      Write(TxByte,1);
      TxByte := lo(StartAddr);
      Write(TxByte,1);
      i := 0;
      while i < NumBytes do
      begin
        TxByte := Memory[i + StartAddr];
        Write(TxByte,1);
        ProgressCount := ProgressCount + 1;
        if UpdateProgress and (ProgressCount > 15)
        then begin
          MainForm.ProgressBar1.StepBy(16);
          ProgressCount := 0;
        end;
        i := i + 1;
      end;
      if UpdateProgress then MainForm.ProgressBar1.StepBy(ProgressCount);

      ReadStr(RxStr,NumBytes);
      if MemArea = maExtFlash then Delay(20); // to allow onboard FLASH routine

      if ShowEchoErrors
      then begin
        i := 0;
        EchoOK := length(RxStr) = NumBytes;
        if not EchoOK then exit;
        while i < NumBytes do
        begin
          EchoOK := Memory[i + StartAddr] = ord(RxStr[i+1]);
          if not EchoOK then exit;
          i := i + 1;
        end;
      end; // if ShowEchoErrors

    end; // with MainForm.ComPort1 do
  finally
    if not CmdOK
    then begin
      EMsg := 'Command error ' + EMsg;
      UpdateDiagnostics(0,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
    if not EchoOK
    then begin
      EMsg := 'Echo error ' + EMsg;
      UpdateDiagnostics(i + StartAddr,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
    Result := CmdOK and EchoOK;
  end; // try.. finally..
end; // function  WriteRam256

{Procedure to write any number of bytes to MCU-controlled memory.  Calls
 WriteMCU256 repeatedly to write the memory in chunks. Overlays must be in
 position before calling this routine.
 This is not a published procedure}
function  WriteMCU64K(    StartAddr,EndAddr : word;
                          MemArea           : tMemArea;
                          UpdateProgress    : boolean;
                          ShowEchoErrors    : boolean) : boolean;
type
  tWriteProc = function(StartAddr,EndAddr : word;
                        MemArea           : tMemArea;
                        UpdateProgress    : boolean;
                        ShowEchoErrors    : boolean) : boolean;
var
  TempAddr  : word;
  WriteProc : tWriteProc;
begin
  Result := false;
  case MemArea of
    maNone,maUnimplemented               : exit;
    maRam,maCtrlRegs,maExtRam,maExtFlash : WriteProc := WriteRam256;
    maEeprom,maEprom                     : WriteProc := WriteProm256
    else exit;
  end;
  TempAddr := StartAddr;
  while (EndAddr - TempAddr) > 255 do
  begin
    if not WriteProc(TempAddr,TempAddr+255,MemArea,UpdateProgress,ShowEchoErrors)
    then exit;
    TempAddr := TempAddr + 256;
  end;
  if not WriteProc(TempAddr,EndAddr,MemArea,UpdateProgress,ShowEchoErrors)
  then exit;
  Result := true;
end; // function  WriteMCU64K

{Procedure to write to external FLASH memory. This is complicated by having to
 read in bytes which are overlapped by the page erasing}
function  WriteMCUExtFlash(StartAddr,EndAddr : word;
                           UpdateProgress,ShowEchoErrors : boolean) : boolean;
var
  FAddr1,FAddr2,TempAddr : word;
begin
  Result := false;
{If the starting and ending addresses do not coincide with data page boundaries,
 then additional bytes have to be read in first}
  with ProjData do
  begin
    FAddr1 := (StartAddr div ExtFlashDataSize) * ExtFlashDataSize;
    if FAddr1 < StartAddr
    then ReadMCU256(FAddr1,StartAddr - 1,false);
    FAddr2 := (((EndAddr + 1) div ExtFlashDataSize) * ExtFlashDataSize) - 1;
    if FAddr2 < EndAddr
    then begin
      FAddr2 := FAddr2 + ExtFlashDataSize;
      ReadMCU256(EndAddr + 1,FAddr2,false);
    end;

    TempAddr := FAddr1;
    while (FAddr2 - TempAddr) > (ExtFlashDataSize - 1) do
    begin
      if not WriteRam256(TempAddr,TempAddr+ExtFlashDataSize-1,
                         maExtFlash,UpdateProgress,ShowEchoErrors) then exit;
      TempAddr := TempAddr + ExtFlashDataSize;
    end;
    if not WriteRam256(TempAddr,Faddr2,
                       maExtFlash,UpdateProgress,ShowEchoErrors) then exit;
  end; // with ProjData do
  Result := true;
end; // function  WriteMCUExtFlash

{Procedure to write local memory to the MCU according to a supplied map.
 Handles overlays as necessary}
procedure WriteMCUMap(const WriteMap       : tList;
                            HandleProgress : boolean);
var
  i    : integer;
  Tptr : tMemRecordPtr;
  Buf  : string;
  OverlayNeeded,OverlayLoaded : tOverlayType;
  EepromWarningShown,EpromWarningShown : boolean;
begin
  OverlayLoaded      := oyNone;
  EepromWarningShown := false;
  EpromWarningShown  := false;
  if HandleProgress
  then begin
    with MainForm.ProgressBar1 do
    begin
      Max      := TotalBytesInMap(WriteMap);
      Position := 0;
    end; // with MainForm.ProgressBar1 do
  end;

  WriteMap.Sort(SortByAreas1);
  try
    for i := 0 to WriteMap.Count - 1 do
    begin
      Tptr := WriteMap[i];
      with Tptr^,ProjData do
      begin
{Overlays need only actually be loaded if the talker is a RAM-resident one.
 A talker in external memory is assumed to provide all the necessary routines
 without the need for overlays, and an EEPROM-resident talker is assumed to
 be incapable of writing to EEprom, Eprom or external FLASH}
        if TalkerType = ttRam
        then begin
          case Area of
            maExtFlash : OverlayNeeded := oyExtFlash;
            maEprom    : OverlayNeeded := oyEprom;
            maEEprom   : OverlayNeeded := oyEEprom
          else OverlayNeeded := oyNone;
          end; // case
        end else OverlayNeeded := oyNone;

{Show BPROT warning message}
        if not EepromWarningShown and (Area = maEeprom)
        then begin
          if not BprotWarningDlg then exit;
          EepromWarningShown := true;
        end; // if not EepromWarningShown and (Area = maEeprom) then..

{Show Vprog start-of-programming message or auto activate}
        if not EpromWarningShown and (Area = maEprom)
        then begin
          if RemoteSwitchAvailable and not SignalSwitchesMode
          then begin
            MainForm.CmdEdit.Text := 'VPROG ON';
            ProcessCR;
          end
          else if WarnOnVprog then if not BurnWarningDlg then exit;
          EpromWarningShown := true;
        end; // if not EpromWarningShown and (Area = maEprom) then..

{Show Vprog end-of-programming message or auto de-activate}
        if EpromWarningShown and (Area <> maEprom)
        then begin
          if RemoteSwitchAvailable and not SignalSwitchesMode
          then begin
            MainForm.CmdEdit.Text := 'VPROG OFF';
            ProcessCR;
          end
          else if WarnOnVprog then ShowBurnFinishDlg;
          EpromWarningShown := false;
        end; // if EpromWarningShown and (Area <> maEprom) then..

{Load overlays as necessary}
        if OverlayNeeded <> OverlayLoaded
        then begin
          if OverlayLoaded <> oyNone
          then begin
            UnloadOverlay;
            OverlayLoaded := oyNone;
          end; // if OverlayLoaded <> oyNone

          if OverlayNeeded <> oyNone
          then begin
            LoadOverlay(OverlayNeeded);
            OverlayLoaded := OverlayNeeded;
          end; // if OverlayNeeded <> oyNone
        end; // if OverlayNeeded <> OverlayLoaded

        if HandleProgress
        then begin
          Buf := 'Writing ' + MemoryVNArray[Area];
          SpaceTo(Buf,24);
          Buf := Buf + IntToHex(SA,4) + '..' + IntToHex(EA,4);
          MainForm.OWEdit.Lines.Add(Buf);
        end;

        if Area = maExtFlash
        then WriteMCUExtFlash(SA,EA,HandleProgress,true)
        else WriteMCU64K(SA,EA,Area,HandleProgress,true);

      end; // with Tptr^,ProjData do
    end; // for i := 0 to WriteMap.Count - 1 do

    if OverlayLoaded <> oyNone then UnloadOverlay;

    if EpromWarningShown
    then begin
      if ProjData.RemoteSwitchAvailable and not ProjData.SignalSwitchesMode
      then begin
        MainForm.CmdEdit.Text := 'VPROG OFF';
        ProcessCR;
      end
      else if ProjData.WarnOnVprog then ShowBurnFinishDlg;
    end; // if EpromWarningShown then..
  finally
    MainForm.ProgressBar1.Position := 0;
  end; // try.. finally..
end; // procedure WriteMCUMap

{procedure to write a single block of memory from local memory to the MCU.
 The block must occupy a single defined memory area}
procedure WriteOneBlock(StartAddr,EndAddr : word;
                        HandleProgress    : boolean);
var
  TMap    : tList;
  Tptr    : tMemRecordPtr;
  MemArea : tMemArea;
begin
  if not IsOneBlock(StartAddr,EndAddr,MemArea) then exit;
  TMap := tList.Create;
  new(Tptr);
  try
    with Tptr^ do
    begin
      SA := StartAddr;
      EA := EndAddr;
      Area := MemArea;
    end;
    TMap.Add(Tptr);
    WriteMCUMap(TMap,HandleProgress);
  finally
    ClearMemMap(TMap);    // also disposes of Tptr
    TMap.Free;
  end;
end; // procedure WriteOneBlock

{}
function  AlterEeprom(Address : word; OpType : char) : boolean;
var
  TxByte,P1,P2,P3 : byte;
  ProgCtrlReg     : word;
  EMsg : string;
  i               : integer;
begin
  Result := false;
  if not LookupCtrlRegAddr('PPROG',ProgCtrlReg,i) then exit;
  case OpType of
    'W' : begin
            P1     := $02;
            EMsg   := 'writing byte to EEPROM';
            TxByte := Memory[Address];
          end;
    'B' : begin
            P1     := $06;
            EMsg   := 'bulk erasing EEPROM';
            TxByte := $FF;
          end;
    'E' : begin
            P1     := $16;
            EMsg   := 'erasing EEPROM byte';
            TxByte := $FF
          end
    else begin
      P1     := 0;
      EMsg   := '';
      TxByte := 0;
    end;
  end; // case
  P2 := P1 + 1;
  P3 := 0;
  try
    Memory[ProgCtrlReg] := P1;             // set latch bits
    WriteRAM256(ProgCtrlReg,ProgCtrlReg,maCtrlRegs,false,false);
    if Memory[ProgCtrlReg] <> P1 then exit;

    WriteRam256(Address,Address,maRam,false,false);

    Memory[ProgCtrlReg] := P2;             // set PGM bit
    WriteRAM256(ProgCtrlReg,ProgCtrlReg,maCtrlRegs,false,false);
    if Memory[ProgCtrlReg] <> P2 then exit;
    if ProjData.CommsBaudRate > 9600 then Delay(5);

    Memory[ProgCtrlReg] := P3;             // clear PROG and set to read mode
    WriteRAM256(ProgCtrlReg,ProgCtrlReg,maCtrlRegs,false,false);
    if Memory[ProgCtrlReg] <> P3 then exit;

    ReadMCU256(Address,Address,false);

  finally
    Result := TxByte = Memory[Address];
    if not Result
    then begin
      EMsg := 'Echo error ' + EMsg;
      UpdateDiagnostics(Address,TxByte,Memory[Address],EMsg);
      Raise EMCUComErr.Create(EMsg);
    end; // if TxByte <> RxByte then..
  end; // try.. finally..
end; // function  AlterEEPROM

function  WriteByteEEPROM(Address : word) : boolean;
begin
  Result := AlterEeprom(Address,'W');
end; // function  WriteByteEEPROM

procedure ByteEraseEEPROM(Address : word);
begin
  AlterEeprom(Address,'E');
end; // function  ByteEraseEEPROM

procedure BulkEraseEEPROM(Address : word);
begin
  AlterEeprom(Address,'B');
end; // function  BulkEraseEEPROM

end.
