unit MemUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, CPort, ExtCtrls, TypeDefs;

type
  TMapForm = class(TForm)
    Memo1: TMemo;
    CloseBtn: TBitBtn;
    HelpBtn: TBitBtn;
    FNBitBtn: TBitBtn;
    Image1: TImage;
    Image2: TImage;
    KOTChkBox: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure FNBitBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure KOTChkBoxClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MapForm: TMapForm;

function MakeMemMapOK(var   Data         : tProjDataRecord;
                      const CtrlRegArray : tCtrlRegArray;
                      const MemMap       : tList;
                      var   MemTypeArray : tMemTypeArray;
                      var   ErrMsg       : string) : boolean;
procedure ClearMemMap(const MemMap : tList);
function  CondenseMapOK(const MemMap : tList) : boolean;
procedure DisplayMap(const MemMap : tList; Up : boolean);
procedure AddMapToOW(const Map : tList; Title : string);
function  StartAddressesAscending(Item1,Item2 : pointer) : integer;
function  SortByAreas1(Item1,Item2 : pointer) : integer;
function  TotalBytesInMap(Map : tList) : integer;
function  IsOneBlock(      StartAddr,EndAddr : word;
                     var   MemArea           : tMemArea;
                     const MemMap            : tList) : boolean;
function  MakeIndMemMapOK(var   Data         : tProjDataRecord;
                          const MemMap       : tList;
                          var   ErrMsg       : string) : boolean;

implementation

uses Main, MiscUnit, CFGUnit, Comms1, LookupUnit;

{$R *.DFM}

procedure TMapForm.FormShow(Sender: TObject);
begin
  KOTChkBox.Checked := FormStyle = fsStayOnTop;
  Caption := 'Memory Map - ' + ProjData.MCUType;
  DisplayMap(MemMap,true);
end; // procedure TMapForm.FormShow

{****************************************************************************}

procedure DisplayMap(const MemMap : tList; Up : boolean);
var
  i   : integer;
  Buf : string;
begin
  MapForm.Memo1.Clear;
  if Up
  then begin
    for i := (MemMap.Count - 1) downto 0 do
    begin
      with tMemRecord(MemMap[i]^) do
      begin
        Buf := MemoryVNArray[Area];
        SpaceTo(Buf,18);
        Buf := Buf + IntToHex(SA,4) + '  ' + IntToHex(EA,4);
        if length(Desc) > 0 then Buf := Buf + '  (' + Desc + ')';
        MapForm.Memo1.Lines.Add(Buf);
      end;
    end;
  end
  else begin
    for i := 0 to (MemMap.Count - 1) do
    begin
      with tMemRecord(MemMap[i]^) do
      begin
        Buf := MemoryVNArray[Area];
        SpaceTo(Buf,18);
        Buf := Buf + IntToHex(SA,4) + '  ' + IntToHex(EA,4);
        if length(Desc) > 0 then Buf := Buf + '  (' + Desc + ')';
        MapForm.Memo1.Lines.Add(Buf);
      end;
    end;
  end;
end; // procedure DisplayMap

{Function used in sorting the MemMap TList.
 Help for how TList.Sort works from Hayden-R:
 http://www.haydenr.com/delphi/codingtips/codingtip18.htm}
function StartAddressesAscending(Item1,Item2 : pointer) : integer;
begin
  Result := tMemRecordPtr(Item1)^.SA - tMemRecordPtr(Item2)^.SA;
end; // function StartAddressesAscending

{Function to sort memory areas when writing MCU memory according to a map.
 So areas which are external flash memory are written first, then areas
 which are EPROM, then EEPROM and finally RAM and control registers.
 Within each memory type, the list items are sorted in ascending address
 order.}
function SortByAreas1(Item1,Item2 : pointer) : integer;
  function Rank(MemArea : tMemArea) : integer;
  begin
    case MemArea of
      maPageWrite     : Result := 1;
      maByteWrite     : Result := 2;
      maEprom         : Result := 3;
      maEeprom        : Result := 4;
      maExtRam        : Result := 5;
      maRam           : Result := 6;
      maCtrlRegs      : Result := 7;
      maRom           : Result := 8;
      maUndefined     : Result := 9
    else Rank := 0;
    end;
  end; // function Rank
begin
  Result := Rank(tMemRecordPtr(Item1)^.Area) - Rank(tMemRecordPtr(Item2)^.Area);
  if Result = 0
  then Result := tMemRecordPtr(Item1)^.SA - tMemRecordPtr(Item2)^.SA
end; // function SortByAreas1

procedure ClearMemMap(const MemMap : tList);
var
  i : integer;
  TPtr : tMemRecordPtr;
begin
  for i := 0 to (MemMap.Count - 1) do
  begin
    TPtr := MemMap[i];
    if TPtr <> nil then Dispose(TPtr);
  end;
  MemMap.Clear;
end; // procedure ClearMemMap

function  CondenseMapOK(const MemMap : tList) : boolean;
var
  i         : integer;
  Lptr,Tptr : tMemRecordPtr;
begin
  Result := false;
  MemMap.Sort(StartAddressesAscending);
  i := 1;
  while (i < MemMap.Count) do
  begin
    Lptr := MemMap[i-1];
    Tptr := MemMap[i];
    if Tptr^.SA <= Lptr^.EA then exit;  // overlapping addresses cause an error
    if (Tptr^.SA = Lptr^.EA + 1) and (Tptr^.Area = Lptr^.Area)
    then begin
      Lptr^.EA := Tptr^.EA;
      dispose(Tptr);
      MemMap.Delete(i);
    end
    else i := i + 1;
  end;
  Result := true;
end; // function CondenseMapOK

function SAandEAfromLine(    Line : string;
                         var StartAddr,EndAddr : word) : boolean;
var
  i,DotsPos : integer;
  Buf       : string;
begin
  i := 1;
  Result := false;
  while i <= length(Line) do
    if Line[i] = ' ' then delete(Line,i,1) else i := i + 1;
  DotsPos := pos('..',Line);
  if DotsPos = 0 then exit;
  try
    Buf := copy(Line,1,DotsPos-1);
    if length(Buf) = 0 then exit;
    if Buf[1] <> '$' then Buf := '$' + Buf;
    StartAddr := StrToInt(Buf);
    Buf := copy(Line,DotsPos+2,length(Line)-DotsPos-1);
    if length(Buf) = 0 then exit;
    if Buf[1] <> '$' then Buf := '$' + Buf;
    EndAddr := StrToInt(Buf);
  except on EConvertError do exit;
  end;
  Result := true;
end; // function SAandEAfromLine

{****************************************************************************}

function MakeMemMapOK(var   Data         : tProjDataRecord;
                      const CtrlRegArray : tCtrlRegArray;
                      const MemMap       : tList;
                      var   MemTypeArray : tMemTypeArray;
                      var   ErrMsg       : string) : boolean;
var
  AreaList : tStringList;
  Msg      : string;
  Tptr     : tMemRecordPtr;
  MemArea  : tMemArea;
  i        : integer;
  StartAddr,EndAddr : word;

function FillMemArray(Visible : boolean;
                      InBuf   : string;
                      MemType : tMemArea) : boolean;
var
  j : integer;
begin
  if Visible
  then begin
    Result := SAandEAfromLine(InBuf,StartAddr,EndAddr);
    if Result
    then begin
      for j := StartAddr to EndAddr do MemTypeArray[j] := MemType;
      Result := EndAddr >= StartAddr;
    end
    else exit;
  end
  else Result := true;
end; // function FillMemArray

begin
  Result := false;
  fillchar(MemTypeArray,sizeof(MemTypeArray),ord(maUndefined));
  AreaList := tStringList.Create;
  ClearMemMap(MemMap);
  MemMap.Capacity := 15;
  try
    with Data do
    begin
      AreaList.Text := ByteWriteMem;
      for i := 0 to AreaList.Count - 1 do
        if not FillMemArray(ByteWriteVisible,AreaList[i],maByteWrite)
        then begin Msg := 'Format error in byte-written memory'; exit end;

      AreaList.Text := PageWriteMem;
      for i := 0 to AreaList.Count - 1 do
        if not FillMemArray(PageWriteVisible,AreaList[i],maPageWrite)
        then begin Msg := 'Format error in page-written memory'; exit end;

      AreaList.Text := ExtRamMem;
      for i := 0 to AreaList.Count - 1 do
        if not FillMemArray(ExtRamVisible,AreaList[i],maExtRam)
        then begin Msg := 'Format error in External RAM'; exit end;

      if not FillMemArray(EepromVisible,EepromMem,maEeprom)
      then begin Msg := 'Format error in EEPROM'; exit end;

      AreaList.Text := EpromMem;
      for i := 0 to AreaList.Count - 1 do
        if not FillMemArray(EpromVisible,AreaList[i],maEprom)
        then begin Msg := 'Format error in EPROM/OTP ROM'; exit end;

      if not FillMemArray(MaskRomVisible,MaskRomMem,maRom)
      then begin Msg := 'Format error in Mask ROM'; exit end;

      if not FillMemArray(TRUE,RamMem,maRam)
      then begin Msg := 'Format error in On-chip RAM'; exit end;

      if not FillMemArray(TRUE,CtrlRegsMem,maCtrlRegs)
      then begin Msg := 'CFormat error in ontrol Registers'; exit end;
    end; // with Data do

    MemArea := maUndefined;
    Tptr    := nil;
    for i := 0 to $FFFF do
    begin
      if MemTypeArray[i] = MemArea
      then continue
      else begin
        MemArea := MemTypeArray[i];
        if Tptr <> nil
        then begin
          Tptr^.EA := word(i) - 1;
          MemMap.Add(Tptr);
        end;
        new(Tptr);
        Tptr^.Area := MemArea;
        Tptr^.SA   := i;
      end;
    end;
    if Tptr <> nil
    then begin
      Tptr^.EA := $FFFF;
      MemMap.Add(Tptr);
    end;

    MemMap.Pack;
    MemMap.Capacity := MemMap.Count;

{Check that RAM and Control Regs at least are defined}
    if not GetMemAreaAddresses(maRam,MemMap,StartAddr,EndAddr)
    then begin Msg := 'No RAM specified'; exit; end;
    if not GetMemAreaAddresses(maCtrlRegs,MemMap,StartAddr,EndAddr)
    then begin Msg := 'No Control Registers specified'; exit; end;

{Check that the range specified for Control Regs is the same as the range
 in the Control Regs file}
    i := length(CtrlRegArray) - 1;
    if (i < 1) or
       (EndAddr - StartAddr <> CtrlRegArray[i].Addr - CtrlRegArray[0].Addr)
    then begin Msg := 'Control Register range differs from file info'; exit; end;

    Result := true;
  finally
    AreaList.Free;
    if not Result
    then ErrMsg := ErrMsg + CRLF + 'Memory Map:' + CRLF +
                   '    ' + Msg;
  end; // try.. finally..     *)
end; // function MakeMemMapOK

procedure AddMapToOW(const Map : tList; Title : string);
var
  i    : integer;
  Tptr : tMemRecordPtr;
begin
  if Map.Count > 0 then MainForm.OWEdit.Lines.Add(Title);
  for i := 0 to Map.Count - 1 do
  begin
    Tptr := Map[i];
    with Tptr^ do
     MainForm.OWEdit.Lines.Add('  ' + IntToHex(SA,4) + '..' + IntToHex(EA,4));
  end;
end; // procedure AddMapToOW

function  IsOneBlock(      StartAddr,EndAddr : word;
                     var   MemArea           : tMemArea;
                     const MemMap            : tList) : boolean;
var
  i    : integer;
  Mptr : tMemRecordPtr;
begin
  i    := 0;
  Mptr := nil;
  while i < MemMap.Count do
  begin
    Mptr := MemMap[i];
    if (Mptr^.SA <= StartAddr) and (Mptr^.EA >= EndAddr) then break;
    i := i + 1;
  end;
  Result := (i < MemMap.Count);
  if Result then MemArea := Mptr^.Area else MemArea := maUndefined;
end; // function  IsOneBlock

procedure TMapForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(IDH_MEMORY_MAP_DISPLAY);
end;

function  TotalBytesInMap(Map : tList) : integer;
var
  i : integer;
begin
  Result := 0;
  for i := 0 to Map.Count - 1 do
  begin
    Result := Result + tMemRecordPtr(Map[i]).EA - tMemRecordPtr(Map[i]).SA + 1;
  end;  
end; // function  TotalBytesInMap

procedure TMapForm.FNBitBtnClick(Sender: TObject);
begin
  with Sender as tBitBtn do
  begin
    if Tag = 0
    then begin
      Tag := 1;
      Glyph.Assign(Image1.Picture);
      DisplayMap(MemMap,false);
    end
    else begin
      Tag := 0;
      Glyph.Assign(Image2.Picture);
      DisplayMap(MemMap,true);
    end;
  end; // with Sender as tBitBtn do
end;

procedure TMapForm.CloseBtnClick(Sender: TObject);
begin
  MapForm.Close;
end;

procedure TMapForm.KOTChkBoxClick(Sender: TObject);
begin
  if KOTChkBox.Checked then MapForm.FormStyle := fsStayOnTop
                       else MapForm.FormStyle := fsNormal;
end; // procedure TMapForm.KOTChkBoxClick

function  MakeIndMemMapOK(var   Data         : tProjDataRecord;
                          const MemMap       : tList;
                          var   ErrMsg       : string) : boolean;
var
  AreaList          : tStringList;
  Msg               : string;
  Tptr              : tMemRecordPtr;
  i                 : integer;
  StartAddr,EndAddr : word;

begin
  Result := false;
  AreaList := tStringList.Create;
  ClearMemMap(MemMap);
  MemMap.Capacity := 2;
  try
    AreaList.Text := Data.IndMemory;
    for i := 0 to AreaList.Count - 1 do
    begin
      if not Data.IMVisible then continue;
      if not SAandEAfromLine(AreaList[i],StartAddr,EndAddr)
      then begin
        Msg := 'Format error in indirect memory specification';
        exit;
      end;
      new(Tptr);
      with Tptr^ do
      begin
        Area    := maIndMem;
        SA      := StartAddr;
        EA      := EndAddr;
        Desc    := '';
      end;
      MemMap.Add(Tptr);
    end;

    MemMap.Pack;
    MemMap.Capacity := MemMap.Count;
    Result := true;
  finally
    AreaList.Free;
    if not Result
    then begin
      ErrMsg := ErrMsg + CRLF + 'Memory Map:' + CRLF +
                   '    ' + Msg;
      ClearMemMap(MemMap);
    end;
  end; // try.. finally..
end; // function MakeIndMemMapOK

end.

