unit MiscUnit;

interface

uses Classes, Forms, Windows, Buttons, TypeDefs, SysUtils, Menus, Stdctrls,
     Graphics;

procedure DelLeadSpaceTab( var S : string);
procedure DelTrailSpaceTab(var S : string);
procedure DelAllSpaceTab(var S : string);

procedure DelComment(var S : string; Comment : tDelimiterSet);
procedure StripComments(const SList   : tStringList;
                              Comment : tDelimiterSet);

function  ExpandTabs(S : string; Spacing : integer) : string;
procedure SpaceTo(var S : string; Col : integer);
// procedure TabTo(var S : string; Col,TabSpacing : integer);
function  BoolToStr(F : boolean) : string;

function  Return2Bar(S : string) : string;
function  Bar2Return(S : string) : string;

procedure FixCSV(const SList : tStringList);

procedure Delay(ms : dword);
function  BinStrToWord(S : string)     : word;
function  WordToBinStr(Value : word; Digits : integer)   : string;
function  FrameToString : string;
procedure ClearFrameDisplay;

procedure FreeSubMenu(const MenuItem : tMenuItem);
procedure FreeMainMenu(const Menu : tMenu);

implementation

uses Main;

procedure DelLeadSpaceTab(var S : string);
begin
{Delete leading spaces and tabs}
  while (length(S) > 0) and (S[1] in [^I,' '])
  do Delete(S,1,1);
end; // procedure DelLeadSpaceTab

procedure DelTrailSpaceTab(var S : string);
begin
{Delete trailing spaces and tabs}
  while (length(S) > 0) and (S[length(S)] in [^I,' '])
  do Delete(S,length(S),1);
end; // procedure DelTrailSpaceTab

procedure DelAllSpaceTab(var S : string);
var
  i : integer;
begin
{Delete all spaces and tabs}
  i := 1;
  while i <= length(S) do
    if S[i] in [^I,' '] then delete(S,i,1) else i := i + 1;
end; // procedure DelAllSpaceTab

procedure DelComment(var S : string; Comment : tDelimiterSet);
var
  i : integer;
begin
{Strip comments following an asterisk at the start of a line}
  if (cdAsterisk in Comment) and (copy(S,1,1) = '*') then S := '';
{Strip comments following a double slash from the ends of lines}
  if cdDoubleSlash in Comment
  then begin
    i := pos('//',S);
    if i > 0 then Delete(S,i,length(S)-i+1);
  end;
{Strip comments following a semicolon from the ends of lines}
  if cdSemiColon in Comment
  then begin
    i := pos(';',S);
    if i > 0 then Delete(S,i,length(S)-i+1);
  end;
{Strip comments following a curly brace from the ends of lines}
  if cdCurlyBrace in Comment
  then begin
    i := pos(#123,S);
    if i > 0 then Delete(S,i,length(S)-i+1);
  end;
  DelTrailSpaceTab(S);
end;

{Procedure which takes any TStringList variable and removes leading spaces and
 tabs, trailing spaces, zero-length lines, and comments which are defined as:
 1. All characters after a double forward slash
 2. All characters after the first occurence of a semi-colon
 3. Lines beginning with an asterisk in the first character position
 'Comment' is a set type variable which controls which type of comment is
 deleted}
procedure StripComments(const SList   : tStringList;
                              Comment : tDelimiterSet);
var
  i       : integer;
  Buf     : string;
  DelFlag : boolean;
begin
  i := 0;
  while i < SList.Count do
  begin
    Buf     := SList[i];
    DelFlag := false;
{Strip comments}
    DelComment(Buf,Comment);
{Delete leading and trailing spaces and tabs}
    DelLeadSpaceTab(Buf);
    DelTrailSpaceTab(Buf);
{Delete zero-length lines, and those beginning with an asterisk}
    if (length(Buf) = 0) or
       ((cdAsterisk in Comment) and (Buf[1] = '*'))
    then DelFlag := true;

    SList[i] := Buf;
    if DelFlag then SList.Delete(i) else inc(i);
  end; // while i < SList.Count do
end; // procedure StripComments

function ExpandTabs(S : string; Spacing : integer) : string;
var
  i : integer;
begin
  Result := '';
  for i := 1 to length(S) do
  begin
    if S[i] = ^I
    then begin
      if (length(Result) mod Spacing) = 0 then Result := Result + ' ';
      while (length(Result) mod Spacing) <> 0 do Result := Result + ' '
    end
    else Result := Result + S[i];
  end;
end; // function ExpandTabs

procedure SpaceTo(var S : string; Col : integer);
var
  i : integer;
begin
  for i := length(S) to Col - 1 do S := S + ' ';
end; // procedure SpaceTo

function  BoolToStr(F : boolean) : string;
begin
  case F of
    true  : Result := 'True ';
    false : Result := 'False';
  end;
end; // function  Bool2Str

{Procedure to add tabs to the end of a string to bring the displayed length
 up to a particular column. Col must be equal to 8n+1, intermediate values
 will be rounded down to the nearest 8n+1}
procedure TabTo(var S : string; Col,TabSpacing : integer);
var
  i,LastCharPos,TabsToAdd,SpaceToAdd : integer;
begin
{Adjust 'Col' if necessary}
  Col := (((Col - 1) div TabSpacing) * TabSpacing) + 1;
{Get effective length of string}
  LastCharPos := 0;
  for i := 1 to length(S) do
  begin
    if (copy(S,1,1)) <> ^I
    then LastCharPos := LastCharPos + 1
    else LastCharPos := ((LastCharPos div TabSpacing) + 1) * TabSpacing;
  end;
{Is it up to 'Col'?}
  SpaceToAdd := Col - 1 - LastCharPos;
  if SpaceToAdd > 0
  then begin
    TabsToAdd := (SpaceToAdd div TabSpacing) + 1;
    for i := 1 to TabsToAdd do S := S + ^I;
  end;
end; // procedure TabTo

{The following two functions simplify the writing to the registry of strings
 containing carriage return/line feed pairs.  These are written differently
 to registry export files by Win 95 and Win 98.  So I replace such pairs
 by the vertical bar symbol }
function  Return2Bar(S : string) : string;
var i : integer;
begin
  Result := '';
  for i := 1 to length(S) do
    case S[i] of
      ^M : Result := Result + '|';
      ^J : begin end
    else Result := Result + S[i];
    end;
end;

function  Bar2Return(S : string) : string;
var i : integer;
begin
  Result := '';
  for i := 1 to length(S) do
    if S[i] = '|' then Result := Result + CRLF
    else Result := Result + S[i];
end;

procedure FixCSV(const SList : tStringList);
var
  InBuf,OutBuf                : string;
  i,k,LastCommaPos,Quote2Pos  : integer;
  HasSpace,HasQuote           : boolean;
  Ch                          : char;
begin
  for k := 0 to SList.Count - 1 do
  begin
    InBuf := SList[k];
    LastCommaPos := 1;
    i := 0;
    OutBuf := '';
    HasSpace := false;
    HasQuote := false;
    while i < length(InBuf) do
    begin
      i := i + 1;
      Ch := InBuf[i];
      case Ch of
        ' ' : HasSpace := true;
        ',' : begin
                if HasQuote
                then begin end
                else begin
                  if HasSpace
                  then begin
                    HasSpace     := false;
                    Insert('"',OutBuf,LastCommaPos+1);
                    OutBuf       := OutBuf + '"';
                  end;
                  LastCommaPos := length(OutBuf)+1;
                end;
              end;
        '"' : if  HasQuote then HasQuote := false
                           else HasQuote := true;
      end; // case
      OutBuf := OutBuf + Ch;
      if i = length(InBuf)
      then begin
        if HasSpace
        then begin
          Insert('"',OutBuf,LastCommaPos+1);
          OutBuf := OutBuf + '"';
        end;
      end;
    end; // while i < length(Buf) do
    repeat
      Quote2Pos := pos('""',OutBuf);
      if Quote2Pos > 0 then delete(OutBuf,Quote2Pos,1);
    until Quote2Pos = 0;
    SList[k] := OutBuf;
  end; // for k := 0 to SList.Count - 1 do
end; // procedure FixCSV

procedure Delay(ms : DWord);
var
  TheTime : DWord;
begin
  if ms <= 0 then exit;
  TheTime := GetTickCount + ms;
  while GetTickCount < TheTime do Application.ProcessMessages;
end; // procedure Delay

function  BinStrToWord(S : string) : word;
var
  Flag : boolean;
  i,k  : integer;
begin
  Flag   := false;
  Result := 0;
  try
    if (length(S) < 2) or (S[1] <> '%') or (length(S) > 17) then exit;
    delete(S,1,1);
    for i := 1 to length(S) do if not (S[i] in ['0','1']) then exit;
    k      := 0;
    Result := 0;
    for i := length(S) downto 1 do
    begin
      Result := Result + ((ord(S[i])-ord('0')) shl k);
      inc(k);
    end;
  Flag := true;
  finally
    if not Flag
    then raise EConvertError.CreateFmt('%s is not a binary string',[S]);
  end;
end; //

function  WordToBinStr(Value : word; Digits : integer)   : string;
var
  i : integer;
begin
  Result := '';
  for i := 1 to Digits do Result := Result + '0';
  for i := 0 to Digits-1 do
  begin
    if ((1 shl (Digits-1-i)) and Value) > 0
    then Result[i+1] := '1';
  end;
end; // function WordToBinStr

function  FrameToString : string;
var
  i   : integer;
  Buf : string;
begin
  Buf := '';
  with MainForm do
  begin
    Result := 'PC='   + PCLbl.Caption +
              ' SP='  + SPEdit.Text   +
              ' A='   + AEdit.Text    +
              ' B='   + BEdit.Text    +
              ' IX='  + IXEdit.Text   +
              ' IY='  + IYEdit.Text;
    for i := 7 downto 0 do
    begin
      if (CCRPanel.Controls[i] as tLabel).font.style = [fsBold]
      then Buf := Buf + (CCRPanel.Controls[i] as tLabel).Caption
      else Buf := Buf + '-';
    end;
    Result := Result + ' CCR=' + Buf;
  end; // with MainForm do
end; // procedure FrameToString

procedure ClearFrameDisplay;
begin
  with MainForm do
  begin
    PCLbl.Caption := '';
    SPEdit.Clear;
    AEdit.Clear;
    BEdit.Clear;
    IXEdit.Clear;
    IYEdit.Clear;
    Prog.CCR := 0;
  end; // with MainForm do
  InitialReadRegs := false;
end; // procedure ClearFrameDisplay

procedure FreeSubMenu(const MenuItem : tMenuItem);
var
  MItem : tMenuItem;
begin
  while MenuItem.Count > 0 do
  begin
    MItem := MenuItem[0];
    MenuItem.Remove(MItem);
    MItem.Free;
  end;
end; // procedure FreeSubMenu

procedure FreeMainMenu(const Menu : tMenu);
var
  MItem : tMenuItem;
begin
  while Menu.Items.Count > 0 do
  begin
    MItem := Menu.Items[0];
    Menu.Items.Remove(MItem);
    MItem.Free;
  end;
end; // procedure FreeSubMenu

end.

