unit RegRW;

interface

uses Windows, Forms, Dialogs, SysUtils, Controls, CPort, Classes, TypeDefs;

procedure ReadMCUReg(var StackFrame : tStackFrame);
procedure WriteMCUReg(StackFrame : tStackFrame);
procedure ServiceSWI(var StackFrame : tStackFrame; var BPAddr : word);

//procedure EmulateSWI(SWIAddr : word; LocalFrame : tStackFrame);

implementation

uses Main, ListUnit, RegUnit, CmdUnit, LookupUnit, Comms1, LoadUnit,
     MemUnit, MiscUnit, MemRW;

const
  RdSPCmdErrMsg   : string = 'Comms error reading Stack Pointer';
  SrvSWICmdErrMsg : string = 'Comms error servicing SWI';

procedure ReadMCUReg(var StackFrame : tStackFrame);
var
  i             : integer;
  RxStr,EMsg    : String;
  TxByte,RxByte,SPH,SPL,IXH,IXL,IYH,IYL,PCH,PCL : byte;
  CmdOK         : boolean;
begin
  CmdOK := true;
  try
    with CommsForm.ComPort,StackFrame do
    begin
      fillchar(StackFrame,sizeof(StackFrame),0);
      ClearBuffer(true,true);
      TxByte := $81;
      Write(TxByte,1);
      Read(RxByte,1);
      CmdOK := RxByte = $7E;
      if not CmdOK then exit;

      Read(RxByte,1);
      SPH := RxByte;         // MCU returns high byte of stack pointer first
      Read(RxByte,1);
      SPL := RxByte;
      SP := (SPH shl 8) + SPL; // then the low byte
{This program's internal record of the stack pointer is of the value that it
 had just before the read instruction is processed.  The interrupt
 processing decrements the SP by 9, and the talker TSX instruction increments
 it by 1, hence the necessary adjustment to the returned value is -1+9}
      SP := SP-1+9;

      Read(RxByte,1);
      for i := 1 to 9 do Write(RxByte,1);
      ReadStr(RxStr,8);
      CCR := RxByte;
      if length(RxStr) <> 8 then begin CmdOK := false; exit; end;
      B   := ord(RxStr[1]);
      A   := ord(RxStr[2]);
      IXH := ord(RxStr[3]);
      IXL := ord(RxStr[4]);
      IYH := ord(RxStr[5]);
      IYL := ord(RxStr[6]);
      PCH := ord(RxStr[7]);
      PCL := ord(RxStr[8]);

      IX := (IXH shl 8) + IXL;
      IY := (IYH shl 8) + IYL;
      PC := (PCH shl 8) + PCL;
    end; // with CommsForm.ComPort,StackFrame do
  finally
    if not CmdOK
    then begin
      EMsg := 'Comms error reading MCU registers';
      UpdateDiagnostics(0,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
  end; // try.. finally..
end; // procedure ReadMCUReg

procedure WriteMCUReg(StackFrame : tStackFrame);
var
  EMsg          : String;
  TxByte,RxByte : byte;
  Addr          : word;
  CmdOK,EchoOK  : boolean;
  i             : integer;
  TxArray       : array[1..9] of byte;
begin
  CmdOK  := true;
  EchoOK := false;
  Addr   := 0;
  try
    with CommsForm.ComPort,StackFrame do
    begin
      ClearBuffer(true,true);
      TxByte := $C1;
      Write(TxByte,1);
      Read(RxByte,1);
      CmdOK := RxByte = $3E;
      if not CmdOK then exit;

{Send the stack pointer.  Note that this program stores the value that the
 stack pointer had immediately before the command to read the MCU registers
 is performed, and because the TSX instruction adds 1 to the SP the
 necessary adjustment is -9+1.  See also the note above.
 The Talker does not echo back these two bytes}
      Addr := SP - 8;
      TxByte := hi(SP-8);
      Write(TxByte,1);    // send high byte of stack pointer
      TxByte := lo(SP-8);
      Write(TxByte,1);    // send low byte

      TxArray[1] := CCR;
      TxArray[2] := B;
      TxArray[3] := A;
      TxArray[4] := hi(IX);
      TxArray[5] := lo(IX);
      TxArray[6] := hi(IY);
      TxArray[7] := lo(IY);
      TxArray[8] := hi(PC);
      TxArray[9] := lo(PC);

{Send the 9 bytes for the revised stack.  These are echoed back}
      for i := 1 to 9 do
      begin
        TxByte := TxArray[i];
        Write(TxByte,1);
        Read(RxByte,1);
{If WriteMCUReg is used to set a program running on the MCU, and this program
 alters the communication parameters, then the last byte may be echoed wrongly,
 giving the impression that there is a fault. So I don't bother to check
 echoing of the last byte. }
        EchoOK := (RxByte = TxByte) or (i = 9);
        if not EchoOK then exit;
      end;
    end; // with CommsForm.ComPort,StackFrame do
  finally
    if not CmdOK
    then begin
      EMsg := 'Comms error writing MCU registers';
      UpdateDiagnostics(0,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
    if not EchoOK
    then begin
      EMsg := 'Echo error writing MCU registers';
      UpdateDiagnostics(Addr,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
  end; // try.. finally..
end; // function WriteMCUReg

procedure ServiceSWI(var StackFrame : tStackFrame; var BPAddr : word);
var
  RxStr,EMsg : String;
  i          : integer;
  CmdOK      : boolean;
  TxByte,RxByte,BPH,BPL,SPH,SPL,IXH,IXL,IYH,IYL,PCH,PCL : byte;
begin
  CmdOK := true;
  try
    with CommsForm.ComPort,StackFrame do
    begin
      fillchar(StackFrame,sizeof(StackFrame),0);

      ClearBuffer(true,true);
      TxByte := $B5;
      Write(TxByte,1);
      Read(RxByte,1);
      CmdOK := RxByte = $4A;
      if not CmdOK then exit;

      Read(BPH,1);          // MCU sends high byte of breakpoint address..
      Read(BPL,1);          // then low byte
      BPAddr := (BPH shl 8) + BPL;
      BPAddr := BPAddr - 1; // adjust for increment at SWI instruction

      Read(SPH,1);          // MCU returns high byte of stack pointer first
      Read(SPL,1);          // then the low byte
      SP := (SPH shl 8) + SPL;

      SP := SP-1+9;         // adjust for way talker works - the effective SP
                            // is the one prevailing when the SWI was signalled

      Read(RxByte,1);
      for i := 1 to 9 do Write(RxByte,1);
      ReadStr(RxStr,8);
      CCR := RxByte;
      B   := ord(RxStr[1]);
      A   := ord(RxStr[2]);
      IXH := ord(RxStr[3]);
      IXL := ord(RxStr[4]);
      IYH := ord(RxStr[5]);
      IYL := ord(RxStr[6]);
      PCH := ord(RxStr[7]);
      PCL := ord(RxStr[8]);

      IX := (IXH shl 8) + IXL;
      IY := (IYH shl 8) + IYL;
      PC := (PCH shl 8) + PCL;
    end; // with CommsForm.ComPort,StackFrame do
  finally
    if not CmdOK
    then begin
      EMsg := 'Comms error servicing SWI';
      UpdateDiagnostics(0,TxByte,RxByte,EMsg);
      Raise EMCUComErr.Create(EMsg);
    end;
  end; // try.. finally..
end;

{If an SWI is encountered which has been placed by the user in MCU code,
 then it is necessary to run an emulation of the SWI operation, otherwise
 the OnRxChar breakpoint handler will endlessly interrupt processing}
procedure EmulateSWI(SWIAddr : word; LocalFrame : tStackFrame);
//var
//  RxByte     : byte;
begin
end;

end.

