unit RegUnit;

interface

uses Classes, SysUtils, Graphics, TypeDefs;


function  RegCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
function  IndirectRegCheck(const CList   : tStringList;
                           var   Comment : tCmdLineComment;
                           var   Command : tCommand) : boolean;
procedure RegInherentDisplay;
procedure RegInherentChange;
procedure RegSingleDisplay(MemArea : tMemArea);
procedure RegSingleChange(MemArea : tMemArea);
procedure RegDoubleDisplay;
procedure RegDoubleChange;

implementation

uses Main, LookupUnit, CheckUnit, MemRW, RegRW, MiscUnit, BPUnit, RunTraceUnit,
     IndRW;

var
  RegAddr,NewVal : word;
  InhReg         : tInherentRegister;

function  RegCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
var
  EqualPos,i          : integer;
  RegName,ValStr,Buf  : string;
  AreasUsed           : tMemAreaSet;
  ArgType             : tArgType;
  Val                 : longword;
begin
  Result  := false;
  Comment := clcArgErr;
  NewVal  := 0;
  RegAddr := 0;
  if not NumArgsOK(CList,0,1,Comment) then exit;
  if not StateIsAllowed([psStopped,psRun,psRunStop,psTraceStop],Comment) then exit;

  if CList.Count = 1
  then begin
    Command := cmdRegInherentDisplay;
    Comment := clcOK;
    Result  := true;
    exit;
  end;

  Buf      := uppercase(CList[1]);
  EqualPos := pos('=',Buf);
  if EqualPos = length(Buf) then exit;   // always an error to end in =

  if EqualPos = 0                        // generate the register name
  then RegName := Buf
  else begin
    RegName  := copy(Buf,1,EqualPos-1);
    ValStr   := copy(Buf,EqualPos+1,length(Buf)-EqualPos);
  end;

  if length(ValStr) > 0
  then begin
    CList.Add(ValStr);
    if FoundArgOK(CList,2,[atByte,atWord],ArgType,Val)
    then NewVal := Val else exit;
  end;

  if EqualPos = 0
  then begin
    if LookupCtrlRegAddr(RegName,RegAddr,i)
    then begin
      Result    := true;
      Comment   := clcOK;
      Command   := cmdRegSingleDisplay;
    end
    else begin
      if LookupDblRegAddr(RegName,RegAddr,i)
      then begin
        Result    := true;
        Comment   := clcOK;
        Command   := cmdRegDoubleDisplay;
      end
      else begin
        Comment := clcRegNotFound;
        exit;
      end;
    end;
  end // if EqualPos = 0 then..
  else begin
    if LookupInherentReg(RegName,InhReg)
    then begin
      if InhReg = ihPC
      then begin
        Comment := clcNoChangePC;
        exit;
      end;
      if ((InhReg in [ihA,ihB,ihCCR]) and (NewVal <= $FF)) or
         (InhReg in [ihIX,ihIY,ihSP])
      then begin
        Result    := true;
        Comment   := clcOK;
        Command   := cmdRegInherentChange;
      end;
    end // if LookupInherentReg(RegName,InhReg) then..
    else begin
      if LookupCtrlRegAddr(RegName,RegAddr,i) and
         CheckAreas(RegAddr,RegAddr,[maCtrlRegs],MemMap,AreasUsed,Comment) and
         (NewVal <= $FF)
      then begin
        Result    := true;
        Comment   := clcOK;
        Command   := cmdRegSingleChange;
      end
      else begin
        if LookupDblRegAddr(RegName,RegAddr,i) and
           CheckAreas(RegAddr,RegAddr+1,[maCtrlRegs],MemMap,AreasUsed,Comment)
        then begin
          Result    := true;
          Comment   := clcOK;
          Command   := cmdRegDoubleChange;
        end
        else begin
          Comment := clcRegNotFound;
          exit;
        end;
      end;
    end; // if LookupInherentReg(RegName,InhReg) then.. else..
  end; // if EqualPos = 0 then.. else..
end; // function  RegCheck

function  IndirectRegCheck(const CList   : tStringList;
                           var   Comment : tCmdLineComment;
                           var   Command : tCommand) : boolean;
var
  EqualPos,i          : integer;
  RegName,ValStr,Buf  : string;
  ArgType             : tArgType;
  Val                 : longword;
begin
  Result  := false;
  Comment := clcArgErr;
  NewVal  := 0;
  RegAddr := 0;
  if not NumArgsOK(CList,1,1,Comment) then exit;
  if not StateIsAllowed([psStopped],Comment) then exit;

  Buf      := uppercase(CList[1]);
  EqualPos := pos('=',Buf);
  if EqualPos = length(Buf) then exit;   // always an error to end in =

  if EqualPos = 0                        // generate the register name
  then RegName := Buf
  else begin
    RegName  := copy(Buf,1,EqualPos-1);
    ValStr   := copy(Buf,EqualPos+1,length(Buf)-EqualPos);
  end;

  if length(ValStr) > 0
  then begin
    CList.Add(ValStr);
    if FoundArgOK(CList,2,[atByte,atWord],ArgType,Val)
    then NewVal := Val else exit;
  end;

  if not LookupIndirectRegAddr(RegName,RegAddr,i)
  then begin
    Comment := clcRegNotFound;
    exit;
  end;

{Check that the necessary overlay is available if a RAM-resident talker is
 being used. It is assumed that overlays are not necessary with a talker
 in external memory or Eeprom}
  if not OverlaysAvailableOK([maIndRegs],Comment) then exit;

  if EqualPos = 0
  then Command := cmdRegIndirectDisplay
  else Command := cmdRegIndirectChange;
  Result    := true;
  Comment   := clcOK;
end; // function  IndirectRegCheck

procedure RegInherentDisplay;
var
  Frame : tStackFrame;
begin
  ReadMCUReg(Frame);
  Prog.StackFrame := Frame;
  MainForm.OWEdit.Lines.Add(FrameToString);
end; // procedure RegInherentDisplay

procedure RegInherentChange;
var
  LocalFrame : tStackFrame;
  BPptr      : tBPRecordPtr;
begin
  if not InitialReadRegs
  then ReadMCUReg(LocalFrame)
  else LocalFrame := Prog.StackFrame;
  case InhReg of
    ihA   : LocalFrame.A   := NewVal;
    ihB   : LocalFrame.B   := NewVal;
    ihIX  : LocalFrame.IX  := NewVal;
    ihIY  : LocalFrame.IY  := NewVal;
    ihSP  : LocalFrame.SP  := NewVal;
    ihCCR : LocalFrame.CCR := NewVal;
  end;
  Prog.StackFrame := LocalFrame;
  if RetrieveBP(Prog.BRAddr,[bpTransient,bpFixed,bpPass],BPptr)
  then begin
    DiscoverNext(BPptr^,Prog.StackFrame,false);
    UpdateBPWindow;
  end;
end; // procedure RegInherentChange

procedure RegSingleDisplay(MemArea : tMemArea);
var
  Buf                   : string;
  i,Val                 : byte;
  CtrlRegIndex,SlashPos : integer;
begin
  with MainForm.OWEdit do
  begin
    case MemArea of
      maCtrlRegs : LookupCtrlRegName(RegAddr,Buf,CtrlRegIndex);
      maIndRegs  : LookupIndirectRegName(RegAddr,Buf,CtrlRegIndex);
    end;

    Buf := format('%-6.4x',[RegAddr]) + Buf + ' ';

    case MemArea of
      maCtrlRegs : Val := ReadByte(RegAddr);
      maIndRegs  : begin
                     IndMemory[RegAddr] := 0;
                     RWIndRegister(RegAddr,imReadRegs,false);
                     Val := IndMemory[RegAddr];
                   end
    else Val := 0;
    end;

    Buf := Buf + '[$' + format('%2.2x',[Val]) + ',#' + format('%d',[Val]) + ']';
    SelText := Buf;
    SelStart := SelStart + length(Buf);

    for i := 7 downto 0 do
    begin
      case MemArea of
        maCtrlRegs : Buf := CtrlRegs[CtrlRegIndex].Bits[i];
        maIndRegs  : Buf := IndRegs[CtrlRegIndex].Bits[i];
      end;
      SlashPos := pos('/',Buf);
      if SlashPos > 0 then Buf := copy(Buf,1,SlashPos-1);
      if (Val and (1 shl i) > 0) and (Buf <> 'U')
      then begin
        SelAttributes.Style := [fsBold];
        SelAttributes.Color := BoldColor;
        if Buf = 'X' then Buf := '1';
      end
      else begin
        SelAttributes.Style := [];
        SelAttributes.Color := clBlack;
        if Buf = 'X' then Buf := '0';
      end;
      SelText := ' ' + Buf;
      SelStart := SelStart + length(Buf) + 1;
    end; // for i := 7 downto 0 do
    SelAttributes.Style := [];
    SelAttributes.Color := clBlack;
    Lines.Add('');                     // new line
  end; // with MainForm.OWEdit do
end; // procedure RegSingleDisplay

procedure RegDoubleDisplay;
var
  Buf           : string;
  Val           : word;
  CtrlRegIndex  : integer;
begin
  LookupCtrlRegName(RegAddr,Buf,CtrlRegIndex);
  if copy(Buf,length(Buf),1) = 'H' then delete(Buf,length(Buf),1);
  Buf := format('%-6.4x',[RegAddr]) + Buf + ' ';
  Val := ReadWord(RegAddr);
  Buf := Buf + '[$' + format('%4.4x',[Val]) + ', #' + format('%d',[Val]) + ']';
  MainForm.OWEdit.Lines.Add(Buf);
end; // procedure RegDoubleDisplay

procedure RegSingleChange(MemArea : tMemArea);
begin
  case MemArea of
    maCtrlRegs : WriteByte(RegAddr,NewVal,false);
    maIndRegs  : begin
                   IndMemory[RegAddr] := NewVal;
                   RWIndRegister(RegAddr,imWriteRegs,true);
                 end;
  end;
  RegSingleDisplay(MemArea);
end; // procedure RegSingleChange

procedure RegDoubleChange;
begin
  WriteWord(RegAddr,NewVal,false);
  RegDoubleDisplay;
end; // procedure RegDoubleChange

end.


