unit SaveUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, TypeDefs;

type
  TSaveAddrForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    FromAddrEdit: TEdit;
    ToAddrEdit: TEdit;
    BitBtn1: TBitBtn;
    HelpBtn: TBitBtn;
    BitBtn3: TBitBtn;
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure FromAddrEditChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

function  SaveCheck(const CList   : tStringList;
                    var   Comment : tCmdLineComment;
                    var   Command : tCommand) : boolean;
function  IndirectSaveCheck(const CList   : tStringList;
                            var   Comment : tCmdLineComment;
                            var   Command : tCommand) : boolean;
procedure SaveLocalToS19File;
procedure SaveMCUToS19File;
procedure SaveIndirectToS19File;
procedure SaveMemToS19ListingUsingMap(const S19Map     : tList;
                                            MemType    : tMemType;
                                      const S19Listing : tStringList);
procedure SaveLocalToBinFile(MemType : tMemType);
procedure SaveMCUToBinFile;
procedure SaveIndirectToBinFile;

var
  SaveAddrForm: TSaveAddrForm;

implementation

{$R *.DFM}

uses Main, CmdUnit, MemRW, IndRW, CheckUnit, FileUnit, MemUnit, LookupUnit,
     VerifyUnit ;

var
  StartAddr,EndAddr : word;
  FileName          : string;

function  SaveCheck(const CList   : tStringList;
                    var   Comment : tCmdLineComment;
                    var   Command : tCommand) : boolean;
var
  Ext          : string;
  LocalVer     : boolean;
  A1,A2,Val    : longword;
  ArgType      : tArgType;
begin
  Result   := false;
  Comment  := clcArgErr;
  LocalVer := uppercase(CList[0]) = 'SVL';
  A1 := 0; A2 := 0;

  if not NumArgsOK(CList,3,3,Comment) then exit;

  if LocalVer
  then if not StateIsAllowed([psDisconnected..psTraceStop],Comment) then exit
  else if not StateIsAllowed([psStopped],Comment) then exit;
  
  if FoundArgOK(CList,1,[atByte,atWord],ArgType,Val)
  then begin
    case ArgType of
      atByte,atWord : A1 := Val;
    end;
  end else exit;

  if FoundArgOK(CList,2,[atByte,atWord,atLength],ArgType,Val)
  then begin
    case ArgType of
      atByte,atWord : A2 := Val;
      atLength      : A2 := A1 + Val;
    end;
  end else exit;

  FileName := CList[3];
  if FileName[1] = chr(149) then delete(FileName,1,1);
  FileName := DetokenizeFileName(FileName);
  Ext      := uppercase(ExtractFileExt(FileName));

{Check that: 1. A2 is larger than A1
             2. A2 <= $FFFF
             3. (No address range exclusions apply to the list command) }
  if (A2 < A1) or (A2 > $FFFF)
  then begin
    Comment := clcAddrRangeErr;     // Address range error
    exit;
  end;

  if (Ext = '.S19') or (Ext = '.REC')
  then begin
    if LocalVer
    then Command := cmdSaveLocalToS19 else Command := cmdSaveMCUToS19;
  end
  else begin
    if (Ext = '.BIN') or (Ext = '.OBJ')
    then begin
      if LocalVer
      then Command := cmdSaveLocalToBin else Command := cmdSaveMCUToBin;
    end
    else begin
      Comment := clcUnrecognizedFileType;
      exit;
    end;
  end; //

  StartAddr := A1;
  EndAddr   := A2;
  Result    := true;
  Comment   := clcOK;
end; // function  SaveCheck

function  IndirectSaveCheck(const CList   : tStringList;
                            var   Comment : tCmdLineComment;
                            var   Command : tCommand) : boolean;
var
  Ext          : string;
  A1,A2,Val    : longword;
  ArgType      : tArgType;
begin
  Result   := false;
  Comment  := clcArgErr;
  A1 := 0; A2 := 0;

  if not NumArgsOK(CList,3,3,Comment) then exit;

  if not StateIsAllowed([psStopped],Comment) then exit;
  
  if FoundArgOK(CList,1,[atByte,atWord],ArgType,Val)
  then begin
    case ArgType of
      atByte,atWord : A1 := Val;
    end;
  end else exit;

  if FoundArgOK(CList,2,[atByte,atWord,atLength],ArgType,Val)
  then begin
    case ArgType of
      atByte,atWord : A2 := Val;
      atLength      : A2 := A1 + Val;
    end;
  end else exit;

  FileName := CList[3];
  if FileName[1] = chr(149) then delete(FileName,1,1);
  FileName := DetokenizeFileName(FileName);
  Ext      := uppercase(ExtractFileExt(FileName));

{Check that the necessary overlay is available if a RAM-resident talker is
 being used. It is assumed that the overlays are not necessary with a talker
 in external memory or Eeprom}
  if not OverlaysAvailableOK([maIndMem],Comment) then exit;

{Check that: 1. A2 is larger than A1
             2. A2 <= $FFFF
             3. (No address range exclusions apply to the list command) }
  if (A2 < A1) or (A2 > $FFFF)
  then begin
    Comment := clcAddrRangeErr;     // Address range error
    exit;
  end;

  if (Ext = '.S19') or (Ext = '.REC')
  then Command := cmdSaveIndirectToS19
  else begin
    if (Ext = '.BIN') or (Ext = '.OBJ')
    then Command := cmdSaveIndirectToBin
    else begin
      Comment := clcUnrecognizedFileType;
      exit;
    end;
  end; //

  StartAddr := A1;
  EndAddr   := A2;
  Result    := true;
  Comment   := clcOK;
end; // function  IndirectSaveCheck

{Procedure to convert bytes in memory into the string list form conforming to
 the S19 format}
procedure AddToS19Listing(      StartAddr,EndAddr : word;
                                MemType           : tMemType;
                          const S19Listing        : tStringList);
var
  i,j,BytesInLine : integer;
  Check           : word;
  Buf             : string;
begin
  j     := 0;
  Check := 0;
  Buf   := '';
  for i := StartAddr to EndAddr do
  begin
    if length(Buf) = 0
    then begin
      if (i + 15) < EndAddr
      then BytesInLine := 16
      else BytesInLine := EndAddr - i + 1;
      Buf   := 'S1' + IntToHex(BytesInLine + 3,2);
      Check := BytesInLine + 3;
      Buf   := Buf + IntToHex(i,4);
      Check := Check + hi(i) + lo(i);
    end; // if length(Buf) = 0 then..
    j     := j + 1;
    case MemType of
      mtIntrinsic : begin
                      Buf := Buf + IntToHex(Memory[i],2);
                      Check := Check + Memory[i];
                    end;
      mtIndirect  : begin
                      Buf := Buf + IntToHex(IndMemory[i],2);
                      Check := Check + IndMemory[i];
                    end;
    end; // case MemType of
    if ((j mod 16) = 0) or (i = EndAddr)
    then begin
      Check := $FF - lo(Check);
      Buf := Buf + IntToHex(Check,2);
      S19Listing.Add(Buf);
      Buf := '';
    end; // if ((j mod 16) = 0) or (i = EndAddr) then..
  end; // for i := StartAddr to EndAddr do
end; // procedure AddToS19Listing

{Saves local memory to an S19 formatted string list}
procedure SaveMemToS19ListingUsingMap(const S19Map     : tList;
                                            MemType    : tMemType;
                                      const S19Listing : tStringList);
var
  i : integer;
begin
  S19Listing.Clear;
  for i := 0 to S19Map.Count - 1 do
    with tMemRecordPtr(S19Map[i])^ do
    begin
      AddToS19Listing(SA,EA,MemType,S19Listing);
    end;
end; // procedure SaveMemToS19UsingMap

{Saves local memory to an S19 file}
procedure SaveLocalToS19File;
var
  S19Map     : tList;
  Tptr       : TMemRecordPtr;
  S19Listing : tStringList;
begin
  S19Map     := tList.Create;
  S19Listing := tStringList.Create;
  try
    S19Listing.Clear;
    new(Tptr);
    with Tptr^ do begin Area := maUndefined; SA := StartAddr; EA := EndAddr; end;
    S19Map.Add(Tptr);

    SaveMemToS19ListingUsingMap(S19Map,mtIntrinsic,S19Listing);
    S19Listing.SaveToFile(FileName);
  finally
    ClearMemMap(S19Map);
    S19Map.Free;
    S19Listing.Free;
  end;
  MainForm.OWEdit.Lines.Add('Saving complete');
end; // procedure SaveLocalToS19

{Saves contents of MCU memory to an S19 file}
procedure SaveMCUToS19File;
var
  S19Map     : tList;
  Tptr       : TMemRecordPtr;
  S19Listing : tStringList;
begin
  S19Map     := tList.Create;
  S19Listing := tStringList.Create;
  try
    S19Listing.Clear;
    new(Tptr);
    S19Map.Add(Tptr);
    with Tptr^ do begin Area := maUndefined; SA := StartAddr; EA := EndAddr; end;

    ReadMCUMap(S19Map,true);
    SaveMemToS19ListingUsingMap(S19Map,mtIntrinsic,S19Listing);
    S19Listing.SaveToFile(FileName);
  finally
    ClearMemMap(S19Map);
    S19Map.Free;
    S19Listing.Free;
  end;
  MainForm.OWEdit.Lines.Add('Saving complete');
end; // procedure SaveMCUToS19

procedure SaveLocalToBinFile(MemType : tMemType);
var
  BinFile : tFileStream;
  i       : integer;
begin
  BinFile := tFileStream.Create(FileName,fmCreate or fmShareDenyWrite);
  try
    case MemType of
      mtIntrinsic : for i := StartAddr to EndAddr do BinFile.Write(Memory[i],1);
      mtIndirect  : for i := StartAddr to EndAddr do BinFile.Write(IndMemory[i],1);
    end;
  finally
    BinFile.Free;
  end;
end; // procedure SaveLocalToBinFile

procedure SaveMCUToBinFile;
begin
  ReadMCU64K(StartAddr,EndAddr,true);
  SaveLocalToBinFile(mtIntrinsic);
end; // procedure SaveMCUToBinFile

{Saves contents of MCU memory to an S19 file}
procedure SaveIndirectToS19File;
var
  S19Map     : tList;
  Tptr       : TMemRecordPtr;
  S19Listing : tStringList;
begin
  S19Map     := tList.Create;
  S19Listing := tStringList.Create;
  try
    S19Listing.Clear;
    new(Tptr);
    with Tptr^ do
    begin Area := maUndefined; SA := StartAddr; EA := EndAddr; end;
    S19Map.Add(Tptr);

    ReadIndirectMap(S19Map);
    SaveMemToS19ListingUsingMap(S19Map,mtIndirect,S19Listing);
    S19Listing.SaveToFile(FileName);
  finally
    ClearMemMap(S19Map);
    S19Map.Free;
    S19Listing.Free;
  end;
  MainForm.OWEdit.Lines.Add('Saving complete');
end; // procedure SaveMCUToS19

procedure SaveIndirectToBinFile;
begin
  try
    with MainForm.ProgressBar1 do
    begin
      Max      := EndAddr - StartAddr + 1;
      Position := 0;
    end; // with MainForm.ProgressBar1 do
    RWIndMemory(StartAddr,EndAddr,imReadMem,true,true);
    SaveLocalToBinFile(mtIndirect);
  finally
    MainForm.ProgressBar1.Position := 0;
  end;
end; // procedure SaveMCUToBinFile

{******************************************************************************}

procedure TSaveAddrForm.FormShow(Sender: TObject);
begin
  FromAddrEdit.SetFocus;
end; // procedure TSaveAddrForm.FormShow

procedure TSaveAddrForm.EditKeyPress(Sender: TObject; var Key: Char);
begin
{Always allow the backspace, return and tab keys}
  if (Key = ^H) or (Key = ^M) or (Key = ^I) then exit;
  if not (Key in HexChars) then Key := chr(0);
end; // procedure TSaveAddrForm.EditKeyPress

procedure TSaveAddrForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(IDH_SAVE_MEMORY);
end; // procedure TSaveAddrForm.HelpBtnClick

procedure TSaveAddrForm.FromAddrEditChange(Sender: TObject);
begin
  if length(FromAddrEdit.Text) = 4 then ToAddrEdit.SetFocus;
end;

end.


