unit SymbolUnit;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, TypeDefs, ActnList, Menus;

type
  TSymbolForm = class(TForm)
    ListBox1: TListBox;
    BitBtn1: TBitBtn;
    HelpBtn: TBitBtn;
    AddToWatchBtn: TButton;
    Label1: TLabel;
    AddrRadBtn: TRadioButton;
    NameRadBtn: TRadioButton;
    KOTChkBox: TCheckBox;
    Panel1: TPanel;
    SymbolsBtn: TSpeedButton;
    RegistersBtn: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure AddToWatchBtnClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure GetValueClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SortRadBtnClick(Sender: TObject);
    procedure KOTChkBoxClick(Sender: TObject);
    procedure SymbolsBtnClick(Sender: TObject);
    procedure RegistersBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

procedure ShowSymbols;
procedure ShowRegisters;

type
  tSymbolRec = record
                 Addr : word;
                 Desc : string;
               end;
  tSymbolRecPtr = ^tSymbolRec;

var
  SymbolForm: TSymbolForm;
  SymbolList : tList;

implementation

uses Main, CmdUnit, LookupUnit;

{$R *.DFM}

procedure ClearSymbolList;
var
  i    : integer;
  TPtr : tSymbolRecPtr;
begin
  for i := 0 to (SymbolList.Count - 1) do
  begin
    TPtr := SymbolList[i];
    if TPtr <> nil then Dispose(TPtr);
  end;
  SymbolList.Clear;
end; // procedure ClearSymbolList

function ByAddr(Item1,Item2 : pointer) : integer;
begin
  Result := tSymbolRecPtr(Item1)^.Addr - tSymbolRecPtr(Item2)^.Addr;
  if Result = 0
  then begin
    if tSymbolRecPtr(Item1)^.Desc = tSymbolRecPtr(Item2)^.Desc
    then Result := 0
    else if tSymbolRecPtr(Item1)^.Desc < tSymbolRecPtr(Item2)^.Desc
      then Result := -1 else Result := 1;
  end;
end; // function ByAddr

function ByDesc(Item1,Item2 : pointer) : integer;
begin
  if tSymbolRecPtr(Item1)^.Desc = tSymbolRecPtr(Item2)^.Desc
  then Result := 0
  else if tSymbolRecPtr(Item1)^.Desc < tSymbolRecPtr(Item2)^.Desc
    then Result := -1 else Result := 1;
  if Result = 0
  then Result := tSymbolRecPtr(Item1)^.Addr - tSymbolRecPtr(Item2)^.Addr;
end; // function ByDesc

procedure DisplaySymbolList;
var i : integer;
begin
  SymbolForm.ListBox1.Clear;
  if SymbolForm.AddrRadBtn.Checked
    then SymbolList.Sort(ByAddr)
    else SymbolList.Sort(ByDesc);
  for i := 0 to SymbolList.Count - 1 do
  begin
    with tSymbolRecPtr(SymbolList[i])^,SymbolForm.ListBox1.Items do
      Add(IntToHex(Addr,4) + ' ' + Desc);
  end;
end; // procedure DisplaySymbolList

procedure ShowSymbols;
var
  i : integer;
  Tptr : tSymbolRecPtr;
begin
  SymbolForm.Caption := 'Symbol Table';
  SymbolForm.SymbolsBtn.Down := true;
  ClearSymbolList;
  SymbolList.Capacity := length(SymbolTable);
  for i := 0 to length(SymbolTable) - 1 do
  begin
    new(Tptr);
    with Tptr^ do
    begin
      Addr := SymbolTable[i].Val;
      Desc := SymbolTable[i].Name;
    end;
    SymbolList.Add(Tptr);
  end;
  if not SymbolForm.Visible then SymbolForm.Show;
  DisplaySymbolList;
end; // procedure ShowSymbols

procedure ShowRegisters;
var
  i : integer;
  FirstAddr,LastAddr : word;
  Tptr : tSymbolRecPtr;
begin
  SymbolForm.Caption := 'MCU Registers';
  SymbolForm.RegistersBtn.Down := true;
  ClearSymbolList;
  SymbolList.Capacity := length(SymbolTable);
  GetMemAreaAddresses(maCtrlRegs,MemMap,FirstAddr,LastAddr);

  for i := 0 to length(CtrlRegs) - 1 do
  begin
    new(Tptr);
    with Tptr^ do
    begin
      Addr := CtrlRegs[i].Addr + FirstAddr;
      Desc := CtrlRegs[i].Name;
      if length(Desc) = 0 then Desc := char($A0);
    end;
    SymbolList.Add(Tptr);
  end;

  if not SymbolForm.Visible then SymbolForm.Show;
  DisplaySymbolList;
end; // procedure ShowRegisters

procedure TSymbolForm.FormCreate(Sender: TObject);
begin
  SymbolList := tList.Create;
//  HelpContext := IDH_SELECT_SYMBOL_REGISTER;
end; // procedure TSymbolForm.FormCreate

procedure TSymbolForm.FormShow(Sender: TObject);
begin
  KOTChkBox.Checked  := FormStyle = fsStayOnTop;
  ListBox1.Font.Size := OWFontSize;
end; // procedure TSymbolForm.FormShow

procedure TSymbolForm.FormDestroy(Sender: TObject);
begin
  ClearSymbolList;
  SymbolList.Free;
end; // procedure TSymbolForm.FormDestroy

procedure TSymbolForm.AddToWatchBtnClick(Sender: TObject);
var
  i : integer;
begin
  with MainForm do
  begin
    for i := 0 to ListBox1.Items.Count - 1 do
      if ListBox1.Selected[i]
      then begin
        WatchListBox.Items.Add(ListBox1.Items[i]);
        Prog.ProjectHasChanged := true;
      end;
    ListBox1.MultiSelect := false;  // removes the highlighting from the list
    ListBox1.MultiSelect := true;
    CmdEdit.SetFocus;
  end; // with MainForm do
end; //

procedure TSymbolForm.BitBtn1Click(Sender: TObject);
begin
  SymbolForm.Close;
end;

procedure TSymbolForm.GetValueClick(Sender: TObject);
var
  i   : integer;
  Buf : string;
begin
  i := 0;
  if SymbolForm.SymbolsBtn.Down
  then begin
    while (i < ListBox1.Items.Count) and not ListBox1.Selected[i] do inc(i);
    MainForm.CmdEdit.Text := 'L ' + copy(ListBox1.Items[i],1,4) + ' +0';
    ProcessCR;
  end;
  if SymbolForm.RegistersBtn.Down
  then begin
    while (i < ListBox1.Items.Count) and not ListBox1.Selected[i] do inc(i);
    Buf := ListBox1.Items[i];
    Delete(Buf,1,5);
    MainForm.CmdEdit.Text := 'R ' + Buf;
    ProcessCR;
  end;
end; // procedure TSymbolForm.Readcurrentvalue1Click

procedure TSymbolForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(IDH_SYMBOL_TABLE_REGISTER_DISPLAY);
end; // TSymbolForm.HelpBtnClick

procedure TSymbolForm.SortRadBtnClick(Sender: TObject);
begin
  DisplaySymbolList;
end;

procedure TSymbolForm.KOTChkBoxClick(Sender: TObject);
begin
  if KOTChkBox.Checked then SymbolForm.FormStyle := fsStayOnTop
                       else SymbolForm.FormStyle := fsNormal;
end; // procedure TSymbolForm.KOTChkBoxClick

procedure TSymbolForm.SymbolsBtnClick(Sender: TObject);
begin
  ShowSymbols;
end; // procedure TSymbolForm.SymbolsBtnClick

procedure TSymbolForm.RegistersBtnClick(Sender: TObject);
begin
  ShowRegisters;
end; // procedure TSymbolForm.RegistersBtnClick

end.


