type
  tSymbolTableToUse = (stNone,stAsS19,stFromFile);
  tTalkerType       = (ttNone,ttRam,ttEeprom,ttExternal);
  tRunBootScript    = (rbsNever,rbsConnect,rbsBoot);

  tTalkerTypeNameArray = array[tTalkerType] of string;

const
  TTNameArray : tTalkerTypeNameArray =
   ('',
    'RAM Talker',
    'EEPROM Talker',
    'External Talker');

{Data stored in the project file *********************************************}
type
  tProjDataRecord = record

{General Tab}
    MCUType               : string;
    BytesToList           : integer;
    LinesToUnassemble     : integer;
    AutoConnectAtStartUp  : boolean;
    AutoResetAfterConnect : boolean;
    WarnOnBprot           : boolean;
    WarnOnVprog           : boolean;
    NoBreakHandshake      : boolean;
    IgnoreEchoErrors      : boolean;
    FN_CtrlRegs           : string;
    FN_Opcodes            : string;

{COM Port Tab}
    PortToUse             : string;
    CrystalFrequency      : string;
    UploadBaudRate        : integer;
    CommsBaudRate         : integer;
    RemoteResetAvailable  : boolean;
    ResetUsesDTR          : boolean;
    ResetPulseGoesPos     : boolean;
    RemoteSwitchAvailable : boolean;
    IsSelectedWhenPos     : boolean;

{Macros Tab}
    RunBootScript         : tRunBootScript;
    BootScript            : string;
    HasAssociatedMacro    : boolean;
    FN_AssocMacro         : string;
    NoAutoStart           : boolean;
    AbortOnFormatError    : boolean;
    AbortOnCmdFail        : boolean;

{Debug Tab}
    AddBreakSpaces        : boolean;
    NoVectorSubstitution  : boolean;
    NotInStepOver         : boolean;
    ShowStartInst         : boolean;
    ShowStartRegs         : boolean;
    ShowOnceOnly          : boolean;
    ShowBreakInst         : boolean;
    ShowBreakRegs         : boolean;
    SymbolTableToUse      : tSymbolTableToUse;
    FN_SymbolFixed        : string;

{Talkers Tab}
    TalkerType            : tTalkerType;
    WaitForEcho           : boolean;
    FN_RamTalker,
    FN_RamMap,
    FN_EEMap,
    FN_ExtMap             : string;
    ExtTalkerRequires     : tTalkerType;

{Overlays Tab}
    FN_EEOverlay,
    FN_EOverlay,
    FN_ByteWriteOverlay,
    FN_PageWriteOverlay,

{Memory Map Tab}
    RamMem                : string;
    CtrlRegsMem           : string;
    MaskRomMem            : string;
    EepromMem             : string;
    EpromMem              : string;
    ExtRamMem             : string;
    ByteWriteMem          : string;
    PageWriteMem          : string;
    MaskRomVisible        : boolean;
    EepromVisible         : boolean;
    EpromVisible          : boolean;
    ExtRamVisible         : boolean;
    ByteWriteVisible      : boolean;
    PageWriteVisible      : boolean;
    DataPageSize          : integer;
    WriteWholePage        : boolean;

{Notes Tab}
    Notes                 : string;

{Indirect Memory Tab}
    FN_IndRegs            : string;
    IndMemory             : string;
    IMVisible             : boolean;
    IMPageSize            : integer;
    FN_IndMemOvly         : string;
    FN_IndRegsOvly        : string;

{File Names}
    FN_LoadS19            : string;
//    FN_LastLoadedS19      : string;
//    FN_SymbolAsS19        : string;
    FN_LoadBinary         : string;
    FN_Macro              : string;

{Terminal variables}
    InterCharDelay        : integer;
    ShowNonDispChars      : boolean;
    SendOnEnter           : boolean;
    LocalEchoOn           : boolean;
    OnEnterSend           : string;
    NewlineOnReceive      : string;
    CloseOnReceive        : string;
    LoggingActive         : boolean;
    FN_Log                : string;
    FN_ImageFile          : string;
  end;

var
  ProjData   : tProjDataRecord;

{Initialization of project data}
const
  DefaultProjData : tProjDataRecord = (
{General Tab}
    MCUType               : '';
    BytesToList           : 16;
    LinesToUnassemble     : 5;
    AutoConnectAtStartUp  : false;
    AutoResetAfterConnect : false;
    WarnOnBprot           : true;
    WarnOnVprog           : true;
    NoBreakHandshake      : false;
    IgnoreEchoErrors      : false;
    FN_CtrlRegs           : '[JBug11]\Registers\';
    FN_Opcodes            : '[JBug11]\Opcodes\HC11_Opcodes.csv';
{COM Port Tab}
    PortToUse             : '';
    CrystalFrequency      : '';
    UploadBaudRate        : 1200;
    CommsBaudRate         : 9600;
    RemoteResetAvailable  : false;
    ResetUsesDTR          : true;
    ResetPulseGoesPos     : true;
    RemoteSwitchAvailable : false;
    IsSelectedWhenPos     : true;
{Macros Tab}
    RunBootScript         : rbsBoot;
    BootScript            : '';
    HasAssociatedMacro    : false;
    NoAutoStart           : false;
    AbortOnFormatError    : true;
    AbortOnCmdFail        : true;
{Debug Tab}
    AddBreakSpaces        : false;
    NoVectorSubstitution  : false;
    NotInStepOver         : false;
    ShowStartInst         : true;
    ShowStartRegs         : true;
    ShowOnceOnly          : false;
    ShowBreakInst         : true;
    ShowBreakRegs         : true;
    SymbolTableToUse      : stNone;
    FN_SymbolFixed        : '[MyDocs]\';
{Talkers Tab}
    TalkerType            : ttRam;
    WaitForEcho           : false;
    FN_RamTalker          : '[JBug11]\Talkers\';
    FN_RamMap             : '[JBug11]\Talkers\';
    FN_EEMap              : '[JBug11]\Talkers\';
    FN_ExtMap             : '[JBug11]\Talkers\';
    ExtTalkerRequires     : ttNone;
{Overlays Tab}
    FN_EEOverlay          : '[JBug11]\Overlays\';
    FN_EOverlay           : '[JBug11]\Overlays\';
    FN_ByteWriteOverlay   : '[JBug11]\Overlays\';
    FN_PageWriteOverlay   : '[JBug11]\Overlays\';
{Memory Map Tab}
    RamMem                : '';
    CtrlRegsMem           : '';
    MaskRomMem            : '';
    EepromMem             : '';
    EpromMem              : '';
    ExtRamMem             : '';
    ByteWriteMem          : '';
    PageWriteMem          : '';
    MaskRomVisible        : false;
    EepromVisible         : false;
    EpromVisible          : false;
    ExtRamVisible         : false;
    ByteWriteVisible      : false;
    PageWriteVisible      : false;
    DataPageSize          : 64;
    WriteWholePage        : true;
{Notes Tab}
    Notes                 : '';
{Indirect Memory Tab}
    FN_IndRegs            : '[JBug11]\Registers\';
    IndMemory             : '';
    IMVisible             : false;
    IMPageSize            : 16;
    FN_IndMemOvly         : '[JBug11]\Overlays\';
    FN_IndRegsOvly        : '[JBug11]\Overlays\';

{File Names}
    FN_LoadS19            : '[MyDocs]\';
//    FN_LastLoadedS19      : '';
    FN_LoadBinary         : '[MyDocs]\';
    FN_Macro              : '[MyDocs]\';
    
{Terminal variables}
    InterCharDelay        : 0;
    ShowNonDispChars      : true;
    SendOnEnter           : true;
    LocalEchoOn           : true;
    OnEnterSend           : '';
    NewlineOnReceive      : '';
    CloseOnReceive        : '';
    LoggingActive         : false;
    FN_Log                : '[MyDocs]\';
    FN_ImageFile          : '[MyDocs]\';
    );

{Value names used in storing and retrieving project data ********************}

const
{General Tab}
  VN_MCUType               = 'MCU type';
  VN_BytesToList           = 'Bytes to List';
  VN_LinesToUnassemble     = 'Lines to Unassemble';
  VN_AutoConnectAtStartUp  = 'Auto Connect at start up';
  VN_AutoResetAfterConnect = 'Auto Reset after Connect';
  VN_WarnOnBprot           = 'Warn on BPROT not set';
  VN_WarnOnVprog           = 'Warn on Vprog not set';
  VN_NoBreakHandshake      = 'Boot talker without break handshake';
  VN_IgnoreEchoErrors      = 'Ignore echo errors on write';
  VN_FN_RegInfo            = '68HC11 Control Registers';
  VN_FN_OpcodeInfo         = '68HC11 Opcodes';

{COM Port Tab}
  VN_PortToUse             = 'COM port to use';
  VN_CrystalFrequency      = 'MCU Crystal Frequency';
  VN_UploadBaudRate        = 'Upload Baud rate';
  VN_CommsBaudRate         = 'Communication Baud rate';
  VN_RemoteResetAvailable  = 'Remote reset available';
  VN_ResetUsesDTR          = 'Reset uses DTR';
  VN_ResetPulseGoesPos     = 'Reset pulse goes pos';
  VN_RemoteSwitchAvailable = 'Remote switch available';
  VN_IsSelectedWhenPos     = 'Switch is ON when positive';

{Macros Tab}
  VN_RunBootScript         = 'Run boot script';
  VN_RunScript             = 'Run Script';
  VN_BootScript            = 'Boot macro script';
  VN_HasAssociatedMacro    = 'Has associated macro';
  VN_NoAutoStart           = 'Disable AUTOSTART macros';
  VN_AbortOnFormatError    = 'Abort on format error';
  VN_AbortOnCmdFail        = 'Abort on command fail';

{Debug Tab}
  VN_AddBreakSpaces        = 'Add spaces at code breaks';
  VN_NoVectorSubstitution  = 'Disable vector substitution';
  VN_NotInStepOver         = 'No stopping in step-over routines';
  VN_ShowStartInst         = 'Show instruction at start';
  VN_ShowStartRegs         = 'Show registers at start';
  VN_ShowOnceOnly          = 'Show start info only once';
  VN_ShowBreakInst         = 'Show instruction at break';
  VN_ShowBreakRegs         = 'Show registers at break';
  VN_SymbolTableToUse      = 'Symbol table to use';
  VN_FN_SymbolFixed        = 'Fixed symbol table';

{Talkers Tab}
  VN_TalkerType            = 'Talker type';
  VN_WaitForEcho           = 'Wait for echo';
  VN_FN_RamTalker          = 'RAM-resident talker';
  VN_FN_RamMap             = 'RAM talker map file';
  VN_FN_EEMap              = 'EEPROM talker map file';
  VN_FN_ExtMap             = 'External talker map file';
  VN_ExtTalkerRequires     = 'Booting for external talker';

{Overlays Tab}
  VN_FN_EEOverlay          = 'EEPROM programming overlay';
  VN_FN_EOverlay           = 'EPROM programming overlay';
  VN_FN_ByteWriteOverlay   = 'External byte write overlay';
  VN_FN_PageWriteOverlay   = 'External page write overlay';
  VN_FN_SPIOverlay         = 'Indirect memory SPI overlay';
  VN_SPIPageSize           = 'Indirect memory page size';
  VN_FN_SPIRegs            = 'Serial memory control registers';

{Memory Map Tab}
  VN_RamMem                = 'RAM memory';
  VN_CtrlRegsMem           = 'Control Register memory';
  VN_MaskRomMem            = 'Mask ROM memory';
  VN_EepromMem             = 'EEPROM memory';
  VN_EpromMem              = 'EPROM memory';
  VN_ExtRamMem             = 'External RAM memory';
  VN_ByteWriteMem          = 'External byte-written memory';
  VN_PageWriteMem          = 'External page-written memory';
  VN_MaskRomVisible        = 'Mask ROM visible';
  VN_EepromVisible         = 'EEPROM visible';
  VN_EpromVisible          = 'EPROM visible';
  VN_ExtRamVisible         = 'External RAM visible';
  VN_ByteWriteVisible      = 'Byte-written memory visible';
  VN_PageWriteVisible      = 'Page-written memory visible';
  VN_DataPageSize          = 'FLASH data page size';
  VN_WriteWholePage        = 'Writes are always whole page';

{Indirect Memory Tab}
  VN_FN_IndRegs            = 'Indirect memory registers';
  VN_IndMemory             = 'Indirect memory allocation';
  VN_IMVisible             = 'Indirect memory visible';
  VN_IMPageSize            = 'Indirect memory page size';
  VN_FN_IndMemOvly         = 'Indirect memory access overlay';
  VN_FN_IndRegsOvly        = 'Indirect register access overlay';

{File Names}
  VN_FN_LoadS19            = 'Last loaded S19';
  VN_FN_LoadBinary         = 'Last loaded binary';
  VN_FN_Project            = 'Project file name';
  VN_FN_Macro              = 'Last opened macro';
  VN_FN_Log                = 'Terminal log file';
  VN_FN_ImageFile          = 'Terminal send image file';

{Outside of 'Settings'}
  VN_WatchStr              = 'Watch window addresses';

{Terminal constants}
  VN_InterCharDelay        = 'Inter-character delay time';
  VN_ShowNonDispChars      = 'Show non-displayable chars';
  VN_SendOnEnter           = 'Send bytes on <Enter>';
  VN_LocalEchoOn           = 'Local echo on';
  VN_OnEnterSend           = 'Bytes to send on <Enter>';
  VN_NewlineOnReceive      = 'Bytes to trigger newline';
  VN_CloseOnReceive        = 'Bytes to trigger TW closure';
  VN_LoggingActive         = 'Logging active';

{Saved in Registry}
var
  FN_ErrorReport  : string = '[MyDocs]\';
  SymFromList     : boolean = false;
const
  VN_FN_ProjectFile        = 'Project file name';
  VN_FN_ErrorReport        = 'Error report file name';
  VN_PFList                = 'Previous project files';
  VN_BPSortOrder           = 'BP window sort order';
{This is a private variable, only used by me. Requires this value to be
 manually added to the Registry:}  
  VN_SymFromList           = 'Make symbol table from listing';

{Other}
  FN_McuData      : string = '[JBug11]\MCU\McuData.cfg';

{Map data *******************************************************************}

type
  tMapRecord = Record
                 talker_start : word;
                 talker_idle  : word;
                 swi_srv      : word;
                 swi_jmp      : word;
                 illop_jmp    : word;
               end;

var
  MapData : tMapRecord;

const
  DefaultMap : tMapRecord = (
    talker_start : $0000;
    talker_idle  : $0000;
    swi_srv      : $0000;
    swi_jmp      : $0000;
    illop_jmp    : $0000;
  );

const
  VN_talker_start  = 'talker_start';
  VN_talker_idle   = 'talker_idle';
  VN_swi_srv       = 'swi_srv';
  VN_swi_jmp       = 'swi_jmp';
  VN_illop_jmp     = 'illop_jmp';

const
  BaudArrayTop = 6;
type
  tBaudArrayRecord = record
                       Freq,Communication,AltUpload,TalkerUpload : Str7;
                     end;
  tDefaultBaudArray = array[0..BaudArrayTop] of tBaudArrayRecord;
const
  BaudDefaults : tDefaultBaudArray = (
    (Freq:'4.0000'; Communication: '4800';AltUpload: '3840';TalkerUpload: '600'),
    (Freq:'4.1943'; Communication: '5008';AltUpload: '4096';TalkerUpload: '629'),
    (Freq:'7.3728'; Communication: '8861';AltUpload: '7200';TalkerUpload:'1107'),
    (Freq:'8.0000'; Communication: '9600';AltUpload: '7680';TalkerUpload:'1200'),
    (Freq:'9.8304'; Communication:'11520';AltUpload: '9600';TalkerUpload:'1476'),
    (Freq:'12.0000';Communication:'14400';AltUpload:'11520';TalkerUpload:'1800'),
    (Freq:'16.0000';Communication:'19200';AltUpload:''     ;TalkerUpload:'2400')
    );
