unit TermSettingsUnit;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, TypeDefs, Dialogs, ECombo {see note below};

type
  TTermSettingsForm = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    InterCharDelayEdit: TEdit;
    Label2: TLabel;
    OnEnterSendComboBox: TEComboBox;       // TEComboBox in ECombo - note below
    NewlineOnReceiveComboBox: TEComboBox;
    CloseOnReceiveComboBox: TEComboBox;
    LogFileEdit: TEdit;
    Label5: TLabel;
    LogFileBtn: TButton;
    ShowNonDispCharsChkBox: TCheckBox;
    LocalEchoOnChkBox: TCheckBox;
    SendOnEnterChkBox: TCheckBox;
    procedure ComboBoxKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
    procedure InterCharDelayEditKeyPress(Sender: TObject; var Key: Char);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ComboBoxDropUp(Sender: TObject);
    procedure LogFileBtnClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure SendOnEnterChkBoxClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  TermSettingsForm: TTermSettingsForm;

implementation

{$R *.DFM}

uses FileUnit, TermUnit;

var
  SetData : tProjDataRecord;

function ProjDataSame(var Data1,Data2 : tProjDataRecord) : boolean;
begin
  Result :=
{Terminal}
  (Data1.ShowNonDispChars  = Data2.ShowNonDispChars ) and
  (Data1.LocalEchoOn       = Data2.LocalEchoOn      ) and
  (Data1.SendOnEnter       = Data2.SendOnEnter      ) and
  (Data1.OnEnterSend       = Data2.OnEnterSend      ) and
  (Data1.NewlineOnReceive  = Data2.NewlineOnReceive ) and
  (Data1.CloseOnReceive    = Data2.CloseOnReceive   ) and
  (Data1.InterCharDelay    = Data2.InterCharDelay   ) and
  (uppercase(Data1.FN_Log) = uppercase(Data2.FN_Log));
end; // function ProjDataSame

procedure TTermSettingsForm.FormShow(Sender: TObject);
begin
{Copy the settings variables from the global to the local}
  SetData := ProjData;
{Write controls accordingly}
  with SetData do
  begin
    ShowNonDispCharsChkBox.Checked := ShowNonDispChars;

    SendOnEnterChkBox.Checked := SendOnEnter;
    if SendOnEnterChkBox.Checked
    then LocalEchoOnChkBox.Checked := true
    else LocalEchoOnChkBox.Checked := LocalEchoOn;
    LocalEchoOnChkBox.Enabled := not SendOnEnterChkBox.Checked;

    OnEnterSendComboBox.Text       := OnEnterSend;
    NewlineOnReceiveComboBox.Text  := NewlineOnReceive;
    CloseOnReceiveComboBox.Text    := CloseOnReceive;
    InterCharDelayEdit.Text        := IntToStr(InterCharDelay);
    LogFileEdit.Text               := TokenizeFileName(FN_Log);
  end;
end; // procedure TTermSettingsForm.FormShow

function ComboTextOK(S : string) : boolean;
var i : integer;
begin
  Result := false;
  if (length(S) mod 2) <> 0 then exit;
  for i := 1 to length(S) do
  begin
    if not (S[i] in HexChars) then exit;
  end;
  Result := true;
end; // function ComboTextOK

procedure TTermSettingsForm.FormCloseQuery( Sender: TObject;
                                            var CanClose: Boolean);
var
//  F : file;
  CBox : tEComboBox;
begin
  case ModalResult of
    mrCancel :
      begin CanClose := true; exit; end;
    mrOK :
      begin
{Check controls}
        CanClose := false;
        CBox := OnEnterSendComboBox;
        if not ComboTextOK(CBox.Text) then begin CBox.SetFocus; exit; end;
        CBox := NewlineOnReceiveComboBox;
        if not ComboTextOK(CBox.Text) then begin CBox.SetFocus; exit; end;
        CBox := CloseOnReceiveComboBox;
        if not ComboTextOK(CBox.Text) then begin CBox.SetFocus; exit; end;
        CanClose := true;
{Read controls}
        with SetData do
        begin
          ShowNonDispChars  := ShowNonDispCharsChkBox.Checked;
          LocalEchoOn       := LocalEchoOnChkBox.Checked;
          SendOnEnter       := SendOnEnterChkBox.Checked;
          OnEnterSend       := OnEnterSendComboBox.Text;
          NewlineOnReceive  := NewlineOnReceiveComboBox.Text;
          CloseOnReceive    := CloseOnReceiveComboBox.Text;
          InterCharDelay    := StrToInt(InterCharDelayEdit.Text);
          FN_Log            := DeTokenizeFileName(LogFileEdit.Text);
          if (length(ExtractFileName(FN_Log)) > 0) and
             (length(ExtractFileExt(FN_Log)) = 0)
          then FN_Log := FN_Log + '.' + tdDefExt[ftLog];
        end;
        Prog.ProjectHasChanged := not ProjDataSame(ProjData,SetData);
        ProjData := SetData;
        TermUnit.UpdateStatusBar;
      end;
  end;
end; // procedure TTermSettingsForm.FormCloseQuery

procedure TTermSettingsForm.ComboBoxKeyPress(Sender: TObject; var Key: Char);
begin
{Always allow the backspace, return and tab keys}
  if (Key = ^H) or (Key = ^M) or (Key = ^I) then exit;
  if not (Key in HexChars) then Key := chr(0);
end; // procedure TTermSettingsForm.ComboBoxKeyPress

procedure TTermSettingsForm.InterCharDelayEditKeyPress(Sender: TObject;
  var Key: Char);
begin
{Always allow the backspace, return and tab keys}
  if (Key = ^H) or (Key = ^M) or (Key = ^I) then exit;
  if not (Key in DecChars) then Key := chr(0);
end; // procedure TTermSettingsForm.InterCharDelayEditKeyPress

{The TEComboBox type is an improved combo box written by 'an Demian'. Available as
 Enhanced ComboBox v.1.05 from Torry's Delphi site (search for 'OnDropUp')
 <http://www.torry.net/pages.php?s=76>}
procedure TTermSettingsForm.ComboBoxDropUp(Sender: TObject);
var
  Buf,OutBuf : string;
  i,SLen     : integer;
begin
  Buf  := (Sender as tEComboBox).Text;
  SLen := length(Buf);
  If (SLen >= 3) and (copy(Buf,1,1) = '"') and (copy(Buf,SLen,1) = '"')
  then begin
    OutBuf := '';
    for i := 2 to SLen - 1 do OutBuf := OutBuf + IntToHex(ord(Buf[i]),2);
    (Sender as tEComboBox).Text := OutBuf;
  end;
  if (SLen = 6) and (copy(Buf,1,5) = 'Ctrl-') and (Buf[6] > '@')
  then (Sender as tEComboBox).Text := IntToHex(ord(Buf[6])-$40,2);
  if Buf = '<NUL>'  then (Sender as tEComboBox).Text := '00';
  if Buf = '<ESC>'  then (Sender as tEComboBox).Text := '1B';
  if Buf = '<none>' then (Sender as tEComboBox).Text := '';
end;

procedure TTermSettingsForm.LogFileBtnClick(Sender: TObject);
var
  Buf  : string;
begin
  Buf := DeTokenizeFileName(LogFileEdit.Text);
  if OpenDialogOK(Buf,ftLog)
  then LogFileEdit.Text := TokenizeFileName(Buf);
end; // procedure TTermSettingsForm.LogFileBtnClick

procedure TTermSettingsForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(IDH_TERMINAL_WINDOW_SETTINGS);
end; // procedure TTermSettingsForm.HelpBtnClick

procedure TTermSettingsForm.SendOnEnterChkBoxClick(Sender: TObject);
begin
  if SendOnEnterChkBox.Checked then LocalEchoOnChkBox.Checked := true;
  LocalEchoOnChkBox.Enabled := not SendOnEnterChkBox.Checked;
end; //TTermSettingsForm.SendOnEnterChkBoxClick

end.

