unit TermUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Menus, ActnList, TypeDefs, CPort;

type
  TTermForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    AsciiPanel: TPanel;
    HexPanel: TPanel;
    AsciiEdit: TRichEdit;
    HexEdit: TRichEdit;
    Label1: TLabel;
    Label2: TLabel;
    View1: TMenuItem;
    Settings2: TMenuItem;
    Help1: TMenuItem;
    N2: TMenuItem;
    Quit1: TMenuItem;
    HelpIndex1: TMenuItem;
    DefaultSize1: TMenuItem;
    KeeponTop1: TMenuItem;
    StatusBar1: TStatusBar;
    Splitter1: TSplitter;
    Edit1: TMenuItem;
    ClearWindows1: TMenuItem;
    Logging1: TMenuItem;
    FontSize1: TMenuItem;
    N8pt1: TMenuItem;
    N10pt1: TMenuItem;
    N12pt1: TMenuItem;
    Settings1: TMenuItem;
    N1: TMenuItem;
    SendImageFile1: TMenuItem;
    WordWrap1: TMenuItem;
    Contents: TMenuItem;
    HelponTerminal1: TMenuItem;
    On1: TMenuItem;
    Off1: TMenuItem;
    TermPopup: TPopupMenu;
    ClearWindows2: TMenuItem;
    procedure Quit1Click(Sender: TObject);
    procedure HexEditKeyPress(Sender: TObject; var Key: Char);
    procedure AsciiEditKeyPress(Sender: TObject; var Key: Char);
    procedure AsciiPanelEnter(Sender: TObject);
    procedure AsciiPanelExit(Sender: TObject);
    procedure HexPanelEnter(Sender: TObject);
    procedure HexPanelExit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure KeepOnTop1Click(Sender: TObject);
    procedure DefaultSize1Click(Sender: TObject);
    procedure View1Click(Sender: TObject);
    procedure HexEditKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure HexPanelClick(Sender: TObject);
    procedure AsciiPanelClick(Sender: TObject);
    procedure ClearWindows1Click(Sender: TObject);
    procedure FontSizeClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SendImageFile1Click(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure WordWrap1Click(Sender: TObject);
    procedure HelpIndex1Click(Sender: TObject);
    procedure HelponTerminal1Click(Sender: TObject);
    procedure ContentsClick(Sender: TObject);
    procedure Settings1Click(Sender: TObject);
    procedure File1Click(Sender: TObject);
    procedure OnOffClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AsciiEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure HexEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState); // Terminal
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  TermForm  : TTermForm;

function TermCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
procedure OpenTerminal;
function  NewLineCharsRcd(B : byte) : boolean;
function  CloseCharsRcd(B : byte) : boolean;
procedure HandleRxByte(RxByte : byte);
procedure UpdateStatusBar;

implementation

{$R *.DFM}

uses Main, InitUnit, FileUnit, TermSettingsUnit, Comms1, MiscUnit, CmdUnit,
     CheckUnit;

var
  Log                                         : tStringList;
  NewlineBuffer,CloseBuffer,SendBuffer,HexStr : string;
  Backspace : boolean;

const
  EscapePressed : boolean = false;

function TermCheck(const CList   : tStringList;
                   var   Comment : tCmdLineComment;
                   var   Command : tCommand) : boolean;
begin
  Result  := false;
  Comment := clcArgErr;
  if not NumArgsOK(CList,0,0,Comment) then exit;
  if not StateIsAllowed([psConnected,psStopped,psRun],Comment) then exit;
  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  Command   := cmdOpenTerminal;
end; // function TermCheck

procedure OpenTerminal;
begin
  TermForm.Show;
end; // procedure OpenTerminal

{Function to convert a string of bytes written as pairs of characters to  a
 string whose characters are the equivalent byte.  For example:
 '414243' becomes 'ABC'}
function HexToChar(InBuf : string) : string;
var
  i : integer;
begin
  i      := 1;
  Result := '';
  while i < length(InBuf) do
  begin
    Result := Result + chr(StrToInt('$' + copy(InBuf,i,2)));
    i := i + 2;
  end;
end; // function HexToChar

procedure UpdateStatusBar;
begin
  with TermForm.StatusBar1,ProjData do
  begin
    case SendOnEnter of
      true  : SimpleText :=
               'Send characters on <Enter>    ' +
               '[Byte(s) on send: ' + OnEnterSend + ']';
      false : SimpleText :=
               'Send characters immediately'
    end; // case
  end;
end; // procedure UpdateStatusBar

procedure AddLinesToLog;
var
  Buf : string;
  i   : integer;
begin
  if not ProjData.LoggingActive then exit;
  Buf := '';
  with TermForm.AsciiEdit do
  begin
    if Lines.Count > 0 then Buf := Lines[Lines.Count - 1];
  end;
  for i := length(Buf) to 24 do Buf := Buf + ' ';
  Buf := Buf + ' | ';
  with TermForm.HexEdit do
  begin
    if Lines.Count > 0 then Buf := Buf + Lines[Lines.Count - 1];
  end;
  Log.Add(Buf);
end; // procedure AddLinesToLog

procedure TTermForm.FormCreate(Sender: TObject);
begin
  Log := tStringList.Create;
end; // procedure TTermForm.FormCreate

procedure TTermForm.FormShow(Sender: TObject);
begin
  UpdateStatusBar;
  if not FileExists(ProjData.FN_Log) then ProjData.LoggingActive := false;
end; // procedure TTermForm.FormShow

procedure TTermForm.FormActivate(Sender: TObject);
begin
  SetLength(NewLineBuffer,length(ProjData.NewLineOnReceive));
  SetLength(CloseBuffer,  length(ProjData.CloseOnReceive));
  with CommsForm do
  begin
    ComPort.OnRxChar := ComPortRxChar2;
    ComPort.OnBreak  := nil;
  end;
end; // procedure TTermForm.FormActivate

procedure TTermForm.FormDeactivate(Sender: TObject);
var
  F : tFileStream;
  i : integer;
begin
  CommsForm.ComPort.OnRxChar := CommsForm.ComPortRxCharTest;
  CommsForm.ComPort.OnBreak  := CommsForm.ComPortBreak;
  if (Log.Count > 0) and (length(ExtractFileName(ProjData.FN_Log)) > 0)
  then begin
    if FileExists(ProjData.FN_Log)
    then F := tFileStream.Create(ProjData.FN_Log,fmOpenReadWrite or fmShareDenyWrite)
    else F := tFileStream.Create(ProjData.FN_Log,fmCreate or fmShareDenyWrite);
    try
      F.Seek(0,soFromEnd);
      for i := 1 to length(Log.Text) do F.Write(Log.Text[i],1);
      Log.Clear;
    finally
      F.Free;
    end;
  end;
end; // procedure TTermForm.FormDeactivate

procedure TTermForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
//  CommsForm.ComPort.OnRxChar := RxEvent;
end; // procedure TTermForm.FormClose

procedure TTermForm.FormDestroy(Sender: TObject);
begin
  Log.Free;
end; // procedure TTermForm.FormDestroy

procedure TTermForm.AsciiPanelEnter(Sender: TObject);
begin
  AsciiPanel.Color := clAqua;
end; // procedure TTermForm.Panel1Enter

procedure TTermForm.AsciiPanelExit(Sender: TObject);
begin
  AsciiPanel.Color := clBtnFace;
end; // procedure TTermForm.Panel1Exit

procedure TTermForm.HexPanelEnter(Sender: TObject);
begin
  HexPanel.Color := clAqua;
end; // procedure TTermForm.Panel2Enter

procedure TTermForm.HexPanelExit(Sender: TObject);
begin
  HexPanel.Color := clBtnFace;
end; // procedure TTermForm.Panel2Exit

procedure TTermForm.AsciiPanelClick(Sender: TObject);
begin
  AsciiEdit.SetFocus;
end; // procedure TTermForm.Panel1Click

procedure TTermForm.HexPanelClick(Sender: TObject);
begin
  HexEdit.SetFocus;
end; // procedure TTermForm.Panel2Click

procedure TTermForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = chr($1B)
  then begin
    EscapePressed := true;
    Key           := chr(0);
  end;
end; // procedure TTermForm.FormKeyPress

{Menu on-click routines -----------------------------------------------------}

procedure TTermForm.File1Click(Sender: TObject);
begin
  with ProjData do
  begin
    Logging1.Enabled := length(ExtractFileName(FN_Log)) > 0;
    On1.Checked      := LoggingActive;
    Off1.Checked     := not LoggingActive;
  end;
end; // procedure TTermForm.File1Click

procedure TTermForm.View1Click(Sender: TObject);
begin
  with ProjData do
  begin
    KeepOnTop1.Checked    := (TermForm.FormStyle = fsStayOnTop);
    WordWrap1.Checked     := AsciiEdit.WordWrap;
    N8pt1.Checked    := (AsciiEdit.Font.Size = 8);
    N10pt1.Checked   := (AsciiEdit.Font.Size = 10);
    N12pt1.Checked   := (AsciiEdit.Font.Size = 12);
  end;
end; // procedure TTermForm.View1Click

procedure TTermForm.Settings1Click(Sender: TObject);
begin
  TermSettingsForm.ShowModal;
end; // procedure TTermForm.Settings1Click

procedure TTermForm.OnOffClick(Sender: TObject);
var Flag : boolean;
begin
  Flag := (Sender = On1);
  if ProjData.LoggingActive <> Flag then Prog.ProjectHasChanged := true;
  ProjData.LoggingActive := Flag;
  On1.Checked            := Flag;
  Off1.Checked           := not Flag;
end; // procedure TTermForm.On1Click

procedure TTermForm.SendImageFile1Click(Sender: TObject);
var
  ImageFile        : tFileStream;
  ByteBuf,LastByte : byte;
  Buf              : string;
  i                : integer;
begin
  LastByte := 0;
  Buf := ProjData.FN_ImageFile;
  if OpenDialogOK(Buf,ftImage)
  then begin
    if uppercase(ProjData.FN_ImageFile) <> uppercase(Buf)
    then Prog.ProjectHasChanged := true;
    ProjData.FN_ImageFile := Buf;

    TermForm.KeyPreview := true;
    ImageFile := tFileStream.Create(ProjData.FN_ImageFile,
                                    fmOpenRead or fmShareExclusive);
    i := 1;
    try
      if ImageFile.Size > $FFFF then exit;
      try
        while i <= ImageFile.Size do
        begin
          if EscapePressed then exit;
          ImageFile.Read(ByteBuf,1);
          i := i + 1;
          if ProjData.LocalEchoOn
          then begin
            with HexEdit do
            begin
              SelStart := length(Text);
              SelAttributes.Color := clBlack;
              SelText  := IntToHex(ByteBuf,2);
              SelStart := length(Text);
            end;
            with AsciiEdit do
            begin
              SelStart := length(Text);
              SelAttributes.Color := clBlack;
              if (ByteBuf >= $20) and (ByteBuf < $7F)
              then SelText := chr(ByteBuf)
              else if ProjData.ShowNonDispChars
                   then SelText := chr($7F);
              SelStart := length(Text);
            end;
            if (ByteBuf = $0A) and (LastByte = $0D)
            then begin
              AsciiEdit.Lines.Add('');
              HexEdit.Lines.Add('')
            end;
          end;
          CommsForm.ComPort.Write(ByteBuf,1);
          LastByte := ByteBuf;
          if ProjData.InterCharDelay > 0 then Delay(ProjData.InterCharDelay);
        end; // while i <= ImageFile.Size do
      except on EComPort do
        begin
          MessageDlg('Cannot write to COM port. Port may not be open.' + CRLF +
                     'Ensure port is connected before using the terminal',
                     mtError,[mbOK,mbHelp],IDH_CONNECT___DISCONNECT);
        end;
      end; // try.. except on EComPort
    finally
      ImageFile.Free;
      TermForm.KeyPreview := false;
      EscapePressed       := false;
    end; // try.. finally..
  end;
end; // procedure TTermForm.SendImageFile1Click

procedure TTermForm.Quit1Click(Sender: TObject);
begin
  TermForm.Close;
end; // procedure TTermForm.Quit1Click

procedure TTermForm.ClearWindows1Click(Sender: TObject);
begin
  AsciiEdit.Clear;
  AsciiEdit.DefAttributes.Color := clBlack;
  HexEdit.Clear;
  HexEdit.DefAttributes.Color := clBlack;
end; // procedure TTermForm.ClearWindows1Click

procedure TTermForm.DefaultSize1Click(Sender: TObject);
begin
  with TermForm do
  begin
    Left             := 23;
    Top              := 23;
    Width            := 540;
    Height           := 400;
    AsciiPanel.Width := 175;
  end;
end; // procedure TTermForm.DefaultSize1Click

procedure TTermForm.KeepOnTop1Click(Sender: TObject);
begin
  KeepOnTop1.Checked := not KeepOnTop1.Checked;
  case KeepOnTop1.Checked of
    true  : TermForm.FormStyle := fsStayOnTop;
    false : TermForm.FormStyle := fsNormal;
  end;
end; // procedure TTermForm.KeepOnTop1Click

procedure TTermForm.FontSizeClick(Sender: TObject);
var
  TWFontSize : integer;
begin
  TWFontSize := 10;                              // saves getting a hint
  if Sender = N8pt1  then TWFontSize := 8;
  if Sender = N10pt1 then TWFontSize := 10;
  if Sender = N12pt1 then TWFontSize := 12;
  AsciiEdit.Font.Size := TWFontSize;
  HexEdit.Font.Size   := TWFontSize;
end;

procedure TTermForm.WordWrap1Click(Sender: TObject);
begin
  AsciiEdit.WordWrap := not AsciiEdit.WordWrap;
  HexEdit.WordWrap   := AsciiEdit.WordWrap;

  if AsciiEdit.WordWrap
  then begin
    AsciiEdit.ScrollBars := ssVertical;
    HexEdit.ScrollBars   := ssVertical;
  end
  else begin
    AsciiEdit.ScrollBars := ssBoth;
    HexEdit.ScrollBars   := ssBoth;
  end;
end; // procedure TTermForm.WordWrap1Click

procedure TTermForm.HelpIndex1Click(Sender: TObject);
const
  SearchMacro : pchar = 'Search()';
begin
  Application.HelpCommand(HELP_TAB, -2);        // Open with 'Find' tab
end; // procedure TMainForm.Index1Click

procedure TTermForm.ContentsClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_TAB,  0);        // Open with 'Contents' tab
end; // procedure TTermForm.ContentsClick

procedure TTermForm.HelponTerminal1Click(Sender: TObject);
begin
  Application.HelpContext(IDH_TERMINAL_WINDOW);
end; // procedure TTermForm.HelponTerminal1Click

{Following event handler has to be included to suppress the backspace key
 when necessary because merely setting Key to #00 in the OnKeyDown handler
 (the following one) does not stop the backspace event from reaching the
 control}
procedure TTermForm.AsciiEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Backspace := (Key = VK_BACK);
  if Backspace
  then begin
    if ProjData.SendOnEnter and (length(SendBuffer) > 0) then exit;
    Key := 0;
  end;
end; // TTermForm.AsciiEditKeyDown

procedure TTermForm.AsciiEditKeyPress(Sender: TObject; var Key: Char);
var
  TxByte  : byte;
  Buf     : string;
  i       : integer;
  CharBuf : char;
begin
  TxByte := ord(Key);
  try
    if Key = ^M
    then begin
      if ProjData.LoggingActive then AddLinesToLog;
      AsciiEdit.Lines.Add('');   // Adds a new line to the ASCII and
      HexEdit.Lines.Add('');     // HEX panes before sending
      Key := chr(0);             // and suppresses the key stroke
      if ProjData.SendOnEnter
      then begin
        Buf := SendBuffer + HexToChar(ProjData.OnEnterSend);
        if ProjData.InterCharDelay = 0
        then CommsForm.ComPort.WriteStr(Buf)
        else begin
          for i := 1 to length(Buf) do
          begin
            CharBuf := Buf[i];
            CommsForm.ComPort.Write(CharBuf,1);
            Delay(ProjData.InterCharDelay);
          end;
        end;
        SendBuffer := '';
      end; // if SendOnEnter then..
    end

    else begin   // Key <> ^M.
      AsciiEdit.SelStart := length(AsciiEdit.Text);
      AsciiEdit.SelAttributes.Color := clBlack;

      if ProjData.SendOnEnter   // Sending only on press of <Enter>
      then begin
        if Backspace
        then begin
          if (length(SendBuffer) > 0)
          then begin                            // Sending bytes on <Enter>
            delete(SendBuffer,length(SendBuffer),1);      // remove last char
            HexEdit.SelStart  := length(HexEdit.Text) - 2;
            HexEdit.SelLength := 2;           // remove 2 chars from HEX pane
            HexEdit.ClearSelection;
          end;
        end
        else begin
          SendBuffer := SendBuffer + Key;
          HexEdit.SelStart := length(HexEdit.Text); // action in the HEX pane
          HexEdit.SelText  := IntToHex(ord(Key),2);
        end;
      end

      else begin               // sending immediately
        if ProjData.LocalEchoOn
        then begin
          if Backspace
          then begin
            Key := #0;
            if ProjData.ShowNonDispChars then AsciiEdit.SelText := #127;  // action in the ASCII pane
            HexEdit.SelStart := length(HexEdit.Text);                     // action in the HEX pane
            HexEdit.SelText  := '08';
          end
          else begin
            HexEdit.SelStart := length(HexEdit.Text); // action in the HEX pane
            HexEdit.SelText  := IntToHex(ord(Key),2);
          end;
        end
        else begin       // local echo is off
          Key := #0;
        end;
        CommsForm.ComPort.Write(TxByte,1);
      end; // if ProjData.SendOnEnter then.. else..

    end; // if Key = ^M then.. else..

  except on EComPort do
    begin
      MessageDlg('Cannot write to COM port. Port may not be open.' + CRLF +
                 'Ensure port is connected before using the terminal',
                 mtError,[mbOK,mbHelp],IDH_CONNECT___DISCONNECT);
    end;
  end; // try.. except on EComPort
end; // procedure TTermForm.AsciiEditKeyPress

procedure TTermForm.HexEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Backspace := (Key = VK_BACK);
  if Backspace
  then begin
//    if ProjData.SendOnEnter and (length(SendBuffer) > 0) then exit;
    Key := 0;
  end;
end; // procedure TTermForm.HexEditKeyDown

procedure TTermForm.HexEditKeyPress(Sender: TObject; var Key: Char);
var
  TxByte    : byte;
  AsciiChar : char;
  Buf       : string;
  i         : integer;
  CharBuf   : char;
begin
  try
    if Key = ^M
    then begin
      Key := chr(0);
      if length(HexStr) = 1 then exit; // no good typing <enter> after a single
                                       // character
      if ProjData.LoggingActive
      then AddLinesToLog;              // must come before the newline is added
                                       //  to the other pane.
      AsciiEdit.Lines.Add('');
      HexEdit.Lines.Add('');

      if ProjData.SendOnEnter
      then begin
        Buf := SendBuffer + HexToChar(ProjData.OnEnterSend);
        if ProjData.InterCharDelay = 0
        then CommsForm.ComPort.WriteStr(Buf)
        else begin
          for i := 1 to length(Buf) do
          begin
            CharBuf := Buf[i];
            CommsForm.ComPort.Write(CharBuf,1);
            Delay(ProjData.InterCharDelay);
          end;
        end;
        SendBuffer := '';
      end;
    end // if (Key = ^M) then..
    else begin
      if Key in HexChars
      then begin
        HexStr := HexStr + Key;
        if length(HexStr) = 2
        then begin
          TxByte    := StrToInt('$' + HexStr);
          AsciiChar := chr(TxByte);
          HexStr    := '';
          if ProjData.SendOnEnter
          then SendBuffer := SendBuffer + AsciiChar
          else CommsForm.ComPort.Write(TxByte,1);

          if ProjData.LocalEchoOn
          then begin
            AsciiEdit.SelStart := length(AsciiEdit.Text);
            if ((AsciiChar >= chr($7F)) or (AsciiChar <= chr($1F)))
            then begin
              if ProjData.ShowNonDispChars then AsciiEdit.SelText := chr(127);
            end
            else AsciiEdit.SelText := AsciiChar;
          end; // if LocalEchoOn then..

        end; // if length(HexStr) = 2 then..
      end // if Key in HexChars then..

      else begin                       // Key not in HexChars
        Key := #0;
        if Backspace                   // allow backspace to delete last char
        then begin
          if ProjData.SendOnEnter      // sending on <enter>
          then begin
            if length(SendBuffer) > 0
            then begin
              if length(HexStr) = 0
              then begin
                delete(SendBuffer,length(SendBuffer),1);
                with HexEdit do
                begin
                  SelStart := length(Text) - 2;
                  SelLength := 2;
                  ClearSelection;
                end;
              end
              else begin
                with HexEdit do
                begin
                  SelStart := length(Text) - 1;
                  SelLength := 1;
                  ClearSelection;
                  HexStr := '';
                end;
              end;
            end; // if length(SendBuffer) > 0
          end
          else begin                   // sending immediately
            if length(HexStr) = 1
            then begin
              with HexEdit do
              begin
                SelStart := length(Text) - 1;
                SelLength := 1;
                ClearSelection;
                HexStr := '';
              end;
            end
          end; // if ProjData.SendOnEnter then.. else..
        end; // if Backspace then..
      end; // if Key in HexChars then.. else..
    end; // if (Key = ^M) then.. else..
  except on EComPort do
    begin
      MessageDlg('Cannot write to COM port. Port may not be open.' + CRLF +
                 'Ensure port is connected before using the terminal',
                 mtError,[mbOK,mbHelp],IDH_CONNECT___DISCONNECT);
    end;
  end; // try.. except on EComPort
end; // procedure TTermForm.HexEditKeyPress

procedure TTermForm.HexEditKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  with HexEdit do
  begin
    if (length(HexStr) = 2) then HexStr   := '';
    SelStart            := length(Text);
    SelAttributes.Color := clBlack;
  end;
end; // procedure TTermForm.HexEditKeyUp

function NewLineCharsRcd(B : byte) : boolean;
var MaxBufLen,NewlineBufLen : integer;
begin
  with ProjData do
  begin
    MaxBufLen := length(NewlineOnReceive);
    if MaxBufLen = 0
    then begin
      Result := false;
      exit;
    end;
    NewlineBuffer := NewlineBuffer + IntToHex(B,2);
    NewlineBufLen := length(NewlineBuffer);
    if NewlineBufLen > MaxBufLen
    then delete(NewlineBuffer,1,NewlineBufLen-MaxBufLen);
    Result := NewlineBuffer = NewlineOnReceive;
  end; // with ProjData do
end; // function NewLineCharsRcd

function CloseCharsRcd(B : byte) : boolean;
var MaxBufLen,CloseBufLen : integer;
begin
  with ProjData do
  begin
    MaxBufLen := length(CloseOnReceive);
    if MaxBufLen = 0
    then begin
      Result := false;
      exit;
    end;
    CloseBuffer := CloseBuffer + IntToHex(B,2);
    CloseBufLen := length(CloseBuffer);
    if CloseBufLen > MaxBufLen
    then delete(CloseBuffer,1,CloseBufLen-MaxBufLen);
    Result := CloseBuffer = CloseOnReceive;
  end; // with ProjData do
end; // function CloseCharsRcd

procedure HandleRxByte(RxByte : byte);
var
  RxAsciiBuf  : string[1];
  RxHexBuf    : string[2];
begin
  if (RxByte < $7F) and (RxByte > $1F)
  then RxAsciiBuf := chr(RxByte)
  else if ProjData.ShowNonDispChars then RxAsciiBuf := chr(127)
                                    else RxAsciiBuf := '';
  RxHexBuf := IntToHex(RxByte,2);
  with TermForm.AsciiEdit do
  begin
    SelAttributes.Color := clRed;
    SelText             := RxAsciiBuf;
    SelAttributes.Color := clBlack;
    SelStart            := length(Text);
  end;
  with TermForm.HexEdit do
  begin
    SelAttributes.Color := clRed;
    SelText             := RxHexBuf;
    SelAttributes.Color := clBlack;
    SelStart            := length(Text);
    ClearUndo;
  end;

  if NewLineCharsRcd(RxByte)
  then begin
    TermForm.AsciiEdit.Lines.Add('');
    TermForm.HexEdit.Lines.Add('');
  end;
  if CloseCharsRcd(RxByte)
  then MainForm.CmdEdit.SetFocus;
end; // procedure HandleRxByte

initialization

begin
  NewlineBuffer := '';
  CloseBuffer   := '';
end;

end.


