unit TypeDefs;

interface

uses SysUtils, Controls, StdCtrls, Classes, Dialogs, Graphics, Windows;

{Preliminary definitions ---------------------------------------------------- }
type
  Str2 = string[2];
  Str3 = string[3];
  Str4 = string[4];
  Str5 = string[5];
  Str6 = string[6];
  Str7 = string[7];
  Str8 = string[8];

const
  CRLF = ^M^J;
  ProjExt      = '.jbp';
  ProjFileType = 'JBug11.ProjFile';
  ProjFileDesc = 'JBug11 Project File';

{$Include Help\JBug11.dph}        // Help file context ID's

{Include files ---------------------------------------------------------------}
{$Include TD_Data.pas}            // types etc. used in 'Settings'
{$Include TD_Mem.pas}
{$Include TD_Cmd.pas}
{$Include TD_Opcode.pas}          // types etc. used in processing opcodes
{$Include TD_File.pas}
{$Include TD_Layout.pas}

{General global variables ----------------------------------------------------}
type
  EMCUComErr     = class(Exception);
  tTalkArray     = array[0..512] of byte;

const
  HELP_TAB = 15;

{Variables holding address ranges}
var
  Ports           : tStringList;        // list built on start-up
  CHEditStartRow,CHEditEndRow,CHEditStartChar,CHEditEndChar : integer;
  TalkArray       : tTalkArray;         // Array to store talker code
  TalkerLen       : integer;            // number of bytes in the talker
  SupErrMsg       : string;             // suplementary error message
  SupErrMsgSet    : boolean;

type
{used by dialogs querying whether an existing file should be saved before it
 is over-written}
  tSaveType = (stOpen,stClose,stNew,stNewProject);

{Loading and saving variables ----------------------------------------------- }
var
  S19Listing    : tStringList;
  EEOvlyListing,EOvlyListing,ByteOvlyListing,PageOvlyListing,
  IndRegsOvlyListing,IndMemOvlyListing : tStringList;
  TalkerOverlainListing : tStringList;

type
  tOverlayType = (oyNone,oyEEprom,oyEprom,oyByteWrite,oyPageWrite,oyIndRegs,oyIndMem);
const
  OvlyTypeArray : array[tOverlayType] of string = (
    'None',
    'on-chip EEPROM',
    'on-chip EPROM',
    'external byte-written',
    'external page-written',
    'accessing indirect register',
    'accessing indirect memory');

{Variables to store MCU Register contents -----------------------------------}
type
  tStackFrame = record
                  IX,IY,SP,PC : word;
                  CCR,A,B     : byte;
                end;
  tInherentRegister = (ihA,ihB,ihCCR,ihIX,ihIY,ihSP,ihPC);

{Register look-up information ------------------------------------------------ }
type
  tCtrlRegType     = (ioNone,ioSingle,ioHigh,ioLow);
  tCtrlRegDatumPtr = ^tCtrlRegDatum;
  tCtrlRegDatum    = record
                       Addr    : word;
                       Name    : Str8;
                       Bits    : array[0..7] of Str8;
                       Desc    : string;
                       RegType : tCtrlRegType;
                     end;
  tCtrlRegArray    = array of tCtrlRegDatum;
var
  CtrlRegs,IndRegs : tCtrlRegArray;

//  InherentRegister : tInherentRegister;

type
  tArgType      = (atUnknown,atNotExist,atByte,atWord,atWordX,atInteger,
                   atLabel,atAsterisk,atLength);
  tSetOfArgType = set of tArgType;

{Sets of validation characters}
const
  HexChars : Set of char = ['0'..'9','A'..'F','a'..'f'];
  DecChars : Set of char = ['0'..'9'];
  BinChars : Set of char = ['0','1'];

var
{Miscellaneous variables}
  VersionStr        : string;
  DesktopCentre     : tPoint;
  PauseEndSignalled : boolean;
  UserSWIVector     : word = $FFFF;
  UserIllopVector   : word = $FFFF;
  BRAddrStr         : string;
  InitialReadRegs   : boolean = false;
  BprotOK           : boolean = false;
  SuppressOnChange  : boolean = false;

{Variables associated with the display}
var
  LastCmdIndex : integer;

type
  tCommentDelimiter = (cdNone,cdAsterisk,cdDoubleSlash,cdSemiColon,cdCurlyBrace);
  tDelimiterSet     = set of tCommentDelimiter;

{Variable used as basis of Watch listing -----------------------------------}

type
  tSelectListing = (slNone,slSymbols,slRegisters);

var
//  SelectListing : tSelectListing;
  UndeleteList : tStringList;

{Macros ----------------------------------------------------------------------}

const
  Notification : string = ' (startup)';
var
  PlayMacroName     : string = '';
  MacroNameList     : tStringList;
  NameListNeedsRebuilding : boolean = false;
  MacroNestingCount : integer = 0;
  
{Symbol table information ----------------------------------------------------}
type
  tSymbolRecord = record
                    Name : string;
                    Val  : word;
                  end;
  tSymbolTable  = array of tSymbolRecord;

var
  SymbolTable : tSymbolTable;

{Variables associated with diagnostic routines -------------------------------}

var
  DBuffer1,DBuffer2 : string;

procedure UpdateDiagnostics(Addr  : word;
                            Tx,Rx : byte;
                            Msg   : string);

{Program state information ---------------------------------------------------}

{The program state variable is made a property so that it may have a write
 method which will automatically update a display of the current state on
 the status line.  Similarly, there
 is an advantage in making TGAddr and LUAddr properties so that their
 write methods can update their read-only edit boxes}
type
  tProgState = (psDuffData,psDisconnected,psConnected,
                psStopped,psRun,psTrace,psRunStop,psTraceStop);
  tSetOfProgStates = set of tProgState;
  tMacroState = (msStopped,msPlaying,msRecording);
  tSetOfMacroStates = set of tMacroState;

const
  StatusArray : array[tProgState] of string =
   ('Invalid project data (check ''Settings'')',
    'Disconnected',
    'Connected',
    'Stopped',
    'Running',
    'Tracing',
    'Running - stopped at breakpoint',
    'Tracing - stopped at breakpoint');

type
  Array32 = array[0..31] of byte;
var
  ByteArray : Array32;
  StateOnOpen : tProgState;

type
  tProg = class(tPersistent)
            Private
              FState                    : tProgState;
              FMacroState               : tMacroState;
              FLUAddr,FTGAddr,FBRAddr   : word;
              F_FN_Project              : string;
              FProjectHasChanged        : boolean;
              FMacroHasChanged          : boolean;
              procedure SetState(Value : tProgState);
              procedure SetMacroState(Value : tMacroState);
              procedure SetTGAddr(Value : word);
              procedure SetLUAddr(Value : word);
              procedure SetBRAddr(Value : word);
              procedure SetFN_Project(Value : string);
              function  GetCCR : byte;
              procedure SetCCR(Value : byte);
              function  GetStackFrame : tStackFrame;
              procedure SetStackFrame(Value : tStackFrame);
              procedure SetProjectHasChanged(Value : boolean);
              procedure SetMacroHasChanged(Value : boolean);
              procedure RewriteMainFormCaption;
              procedure RewriteMEFormCaption;
            Public
              Property State : tProgState  read FState     write SetState;
              Property MacroState : tMacroState
                                        read FMacroState write SetMacroState;
              Property TGAddr    : word    read FTGAddr    write SetTGAddr;
              Property LUAddr    : word    read FLUAddr    write SetLUAddr;
              Property BRAddr    : word    read FBRAddr    write SetBRAddr;
              Property FN_Project: string
                read F_FN_Project    write SetFN_Project;
              Property CCR : byte read GetCCR write SetCCR;
              Property StackFrame: tStackFrame
                read GetStackFrame   write SetStackFrame;
              Property ProjectHasChanged : boolean
                read FProjectHasChanged write SetProjectHasChanged;
              Property MacroHasChanged : boolean
                read FMacroHasChanged write SetMacroHasChanged;
          end;
var
  Prog       : tProg;

procedure HandleMCUCommError(Msg : string);

implementation

uses Main, Comms1, MacroUnit;

{  tProgState = (psNone,psDisconnected,psConnected,psMonitor,
                psRun,psTrace,psStopBP,psUser);                }

procedure tProg.SetState(Value : tProgState);
begin
  with MainForm do
  begin
    Connect.Enabled    := Value > psDuffData;
    Reset.Enabled      := (Value > psDisconnected) and
                          ProjData.RemoteResetAvailable;
    SwitchBtn.Enabled  := ProjData.RemoteSwitchAvailable and
                          (Value > psDisconnected);
    LoadS19.Enabled    := Value = psStopped;
    SaveS19.Enabled    := Value = psStopped;
    VerifyS19.Enabled  := Value = psStopped;
    CRCS19.Enabled     := Value = psStopped;
    case Value of
      psDuffData,psDisconnected :
        begin
          SwitchBtn.Down  := false;
          CommsForm.ComPort.Close;
          MainForm.StatusBar1.Panels[1].Text := '';
          InitialReadRegs := false;
        end;
      psConnected :
        begin
          CommsForm.ComPort.Open;
          MainForm.StatusBar1.Panels[1].Text := '';
          InitialReadRegs := false;
        end
    else MainForm.StatusBar1.Panels[1].Text := TTNameArray[ProjData.TalkerType];
    end; // case
    FState := Value;
    MainForm.StatusBar1.Panels[0].Text := StatusArray[FState];
  end; // with MainForm do
end; // procedure tProg.SetState

procedure tProg.SetMacroState(Value : tMacroState);
begin
  FMacroState := Value;
  with MainForm do
  begin
{Speed buttons}
    RecordBtn.Down    := Value = msRecording;
    PlayBtn.Down      := Value in [msPlaying];
    RecordBtn.Enabled := Value in [msStopped,msRecording];
    PlayBtn.Enabled   := Value in [msStopped,msPlaying];
{Action items}
    RecordAdd.Enabled := Value in [msStopped,msRecording];
    PlayMacro.Enabled := Value in [msStopped,msPlaying];
  end;
end; // procedure tProg.SetMacroState

procedure tProg.SetTGAddr(Value : word);
begin
  FTGAddr := Value;
  if FTGAddr < $FFFF
  then MainForm.TGLbl.Caption := format('%4.4x',[FTGAddr])
  else MainForm.TGLbl.Caption := '';
end; // procedure tProg.SetTGAddr

procedure tProg.SetLUAddr(Value : word);
begin
  FLUAddr := Value;
  if FLUAddr < $FFFF
  then MainForm.LULbl.Caption := format('%4.4x',[FLUAddr])
  else MainForm.LULbl.Caption := '';
end; // procedure tProg.SetLUAddr

procedure tProg.SetBRAddr(Value : word);
begin
  FBRAddr := Value;
//  if FBRAddr < $FFFF
//  then MainForm.BRLabel.Caption := format('%4.4x',[FBRAddr])
//  else MainForm.BRLabel.Caption := '';
end; // procedure tProg.SetLUAddr

procedure tProg.SetFN_Project(Value : string);
var
  i   : integer;
begin
  F_FN_Project := Value;
{Update the 'previous files' list}
  i := 0;
  while i < PFList.Count do
  begin
    if uppercase(PFList[i]) = uppercase(F_FN_Project)
    then PFList.Delete(i)
    else i := i + 1;
  end;
  if length(ExtractFileName(F_FN_Project)) > 0
  then PFList.Insert(0,F_FN_Project);
  while PFList.Count > 11 do PFList.Delete(11);
end; // procedure tProg.SetFN_Project

procedure tProg.SetProjectHasChanged(Value : boolean);
begin
  FProjectHasChanged := value;
  RewriteMainFormCaption;
end; // procedure tProg.SetDataHasChanged

procedure tProg.RewriteMainFormCaption;
var
  Buf : string;
begin
{Update the main form caption}
  Buf := 'JBug11 - ';

  if length(ExtractFileName(FN_Project)) = 0
  then Buf := Buf + '(unnamed)'
  else Buf := Buf + ExtractFileName(FN_Project);

  if FProjectHasChanged then Buf := Buf + '*';

  Buf := Buf + ' -> ';

  if ProjData.PortToUse = ''
  then Buf := Buf + 'COM?'
  else Buf := Buf + ProjData.PortToUse;

  MainForm.Caption := Buf;
end; // procedure tProg.RewriteMainFormCaption

procedure tProg.RewriteMEFormCaption;
var
  Buf : string;
begin
  Buf := ExtractFileName(ProjData.FN_Macro);
  if length(Buf) = 0 then Buf := '(unnamed)';
  Buf := 'Edit Macro - ' + Buf;
  if FMacroHasChanged then Buf := Buf + '*';
  MEForm.Caption := Buf;
end; // procedure tProg.RewriteMEFormCaption

procedure tProg.SetMacroHasChanged(Value : boolean);
begin
  FMacroHasChanged := Value;
  RewriteMEFormCaption;
end; // tProg.SetMacroHasChanged

procedure HandleMCUCommError(Msg : string);
var
  ContextNum : longint;
  Buf        : string;
  Btns       : tMsgDlgButtons;
  Flag       : boolean;
begin
  Btns := [mbOK,mbHelp];
  Buf  := Msg + CRLF + CRLF;
  with MainForm.OWEdit do
  begin
    Lines.Add(Msg);
    Lines.Add(DBuffer1);
    Lines.Add(DBuffer2);
  end;

  Flag := uppercase(copy(Msg,1,3)) = 'COM';
  if Flag
  then begin
    ContextNum := IDH_COMMS_ERROR;
    Buf        := Buf + 'Reset MCU/reboot the talker';
    if ProjData.RemoteResetAvailable
    then begin
      Buf        := Buf + '?';
      Btns       := [mbYes,mbNo,mbHelp];
    end
  end
  else ContextNum := IDH_ECHO_ERROR;

  case MessageDlg(Buf,mtError,Btns,ContextNum) of
    mrYes :
      begin
        MainForm.OWEdit.Lines.Add('  Reset MCU = Yes');
        MainForm.Reset.Execute;
      end;
    mrNo : MainForm.OWEdit.Lines.Add('  Reset MCU = No');
  end;
  
  if Flag then Prog.MacroState := msStopped;
end; // procedure HandleMCUCommError

procedure UpdateDiagnostics(Addr  : word;
                            Tx,Rx : byte;
                            Msg   : string);
begin
{ Addr:       Sent:    Recvd:
  State:                      }
  DBuffer1 := '  Addr '  + format('%-6.4x',[Addr]) +
                'Sent '  + format('%-5.2x',[Tx])   +
                'Rcvd '  + format('%-5.2x',[Rx]);
  DBuffer2 := '  State:' + StatusArray[Prog.State];
end; // procedure UpdateDiagnostics

function  tProg.GetCCR : byte;
var i : integer;
begin
  Result := 0;
  for i := 0 to 7 do
  begin
    with (MainForm.CCRPanel.Controls[i] as tLabel) do
    begin
      if Tag > 0 then Result := Result + (1 shl i);
    end;
  end;
end; // function tProg.GetCCR

procedure tProg.SetCCR(Value : byte);
var i : integer;
begin
  SuppressOnChange := true;
  for i := 0 to 7 do
  begin
    with (MainForm.CCRPanel.Controls[i] as tLabel) do
    begin
      if (Value shr i) and 1 > 0
      then begin Tag := 1; Font.Color := clBlack;  Font.Style := [fsBold]; end
      else begin Tag := 0; Font.Color := clSilver; Font.Style := []; end;
    end;
  end;
  SuppressOnChange := false;
end; // procedure tProg.SetCCR

function tProg.GetStackFrame : tStackFrame;
begin
  with Result,MainForm do
  begin
    try
      SP  := StrToInt('$0' + SPEdit.Text);
      PC  := StrToInt('$0' + PCLbl.Caption);
      A   := StrToInt('$0' + AEdit.Text);
      B   := StrToInt('$0' + BEdit.Text);
      IX  := StrToInt('$0' + IXEdit.Text);
      IY  := StrToInt('$0' + IYEdit.Text);
    except on EConvertError do begin end; //fillchar(Result,sizeof(Result),0);
    end;
    CCR := GetCCR;
  end;
end; // function GetStackFrame

procedure tProg.SetStackFrame(value : tStackFrame);
begin
  SuppressOnChange := true;
  with MainForm,value do
  begin
    SPEdit.Text    := IntToHex(SP,4);
    PCLbl.Caption  := IntToHex(PC,4);
    AEdit.Text     := IntToHex(A,2);
    BEdit.Text     := IntToHex(B,2);
    IXEdit.Text    := IntToHex(IX,4);
    IYEdit.Text    := IntToHex(IY,4);
    SetCCR(CCR);
  end;
  SuppressOnChange := false;
  InitialReadRegs  := true;
end; // procedure tProg.SetStackFrame

end.

