unit VerifyUnit;

interface

uses Classes, SysUtils, Graphics, TypeDefs, Dialogs, Forms;

function VerifyCheck(const CList   : tStringList;
                     var   Comment : tCmdLineComment;
                     var   Command : tCommand) : boolean;
function VerifyEraseCheck(const CList   : tStringList;
                          var   Comment : tCmdLineComment;
                          var   Command : tCommand) : boolean;
procedure VerifyS19Local;
procedure VerifyS19MCU;
procedure VerifyErase;

implementation

uses Main, CmdUnit, FileUnit, MemRW, LoadUnit, LookupUnit, MiscUnit,
     CheckUnit, MemUnit;

var
  StartAddr,EndAddr : word;

function VerifyCheck(const CList   : tStringList;
                     var   Comment : tCmdLineComment;
                     var   Command : tCommand) : boolean;
var
  FileName,Ext : string;
  IsLocal      : boolean;
begin
  Result  := false;
  Comment := clcArgErr;
  IsLocal := uppercase(CList[0]) = 'VL';

  if not NumArgsOK(CList,1,1,Comment) then exit;

  if IsLocal
  then if not StateIsAllowed([psDisconnected..psTraceStop],Comment) then exit
  else if not StateIsAllowed([psStopped],Comment) then exit;
  
  FileName := CList[1];
  if FileName[1] = chr(149) then delete(FileName,1,1);
  FileName := DetokenizeFileName(FileName);
  if not FileExists(FileName) then begin Comment := clcFileNotFound; exit; end;
  Ext := uppercase(ExtractFileExt(FileName));
  if (Ext <> '.S19') and (Ext <> '.REC')
  then begin Comment := clcUnrecognizedFileType; exit; end;

  if not OpenS19FileOK(FileName,S19Listing,SupErrMsg)
  then begin
    Comment      := clcBadS19Format;
    SupErrMsgSet := true;
    exit;
  end;

  Result    := true;                // Arguments are OK
  Comment   := clcOK;
  if IsLocal
  then Command := cmdVerifyS19Local
  else Command := cmdVerifyS19MCU;
end; // function  VerifyCheck

function VerifyEraseCheck(const CList   : tStringList;
                          var   Comment : tCmdLineComment;
                          var   Command : tCommand) : boolean;
var
  A1,A2,Val : longword;
  ArgType   : tArgType;
begin
  Result  := false;
  Comment := clcArgErr;
  A2      := 0;

  if not NumArgsOK(CList,1,2,Comment) then exit;

  if not StateIsAllowed([psStopped],Comment) then exit;

  if not FoundArgOK(CList,1,[atWord],ArgType,Val) then exit;
  A1 := Val;

  if FoundArgOK(CList,2,[atNotExist,atWord,atLength],
                ArgType,Val)
  then begin
    case ArgType of
      atNotExist : A2 := A1;
      atWord     : A2 := Val;
      atLength   : A2 := A1 + Val;
    end;
  end else exit;

  if (A1 > $FFFF) or (A2 > $FFFF) or (A2 < A1)
  then begin
    Comment := clcAddrRangeErr;
    exit;
  end;

  StartAddr := A1;
  EndAddr   := A2;
  Result    := true;
  Comment   := clcOK;
  Command   := cmdVerifyErase;
end; // function VerifyEraseCheck

procedure VerifyS19Local;
var
  i,j,k  : integer;
  MemLoc : word;
  Buf    : string;
  Flag   : boolean;
begin
  Flag := false;
  j := 0; k := 0; MemLoc := 0;
  try
    for i := 0 to S19Listing.Count - 1 do
    begin
      Buf := S19Listing[i];
{Convert characters to code}
      if copy(Buf,1,2) = 'S1'
      then begin
        MemLoc := StrToInt('$' + copy(Buf,5,4));
        j := 9;
        k := 0;
        while j < length(Buf) - 2 do
        begin
          if Memory[MemLoc + k] <> StrToInt('$' + copy(Buf,j,2)) then exit;
          j := j + 2;
          k := k + 1;
        end;
      end; // if copy(Buf,1,2) = 'S1'
    end; // for i := 0 to S19Listing.Count - 1 do
    Flag := true;
  finally
    if Flag
    then begin
      MainForm.OWEdit.Lines.Add('Memory verifies OK');
    end
    else begin
      with MainForm.OWEdit do
      begin
        SelAttributes.Color := clRed;
        Lines.Add('Verify fails at ' + IntToHex(MemLoc + k,4) +
                  '.    Memory = ' +   IntToHex(Memory[MemLoc + k],2) +
                  '    S19 = ' + copy(Buf,j,2));
        SelAttributes.Color := clBlack;
        if ProjData.AbortOnCmdFail and (Prog.MacroState = msPlaying)
        then begin
          Prog.MacroState := msStopped;
          Lines.Add('Macro aborted on verification failure');
        end;
      end; // with MainForm.OWEdit do
    end; // if Flag then.. else..
    MainForm.ProgressBar1.Position := 0;
  end; // try.. finally..
end; // procedure VerifyS19Local

procedure VerifyS19MCU;
var
  S19Map : tList;
begin
  S19Map := tList.Create;
  try
    BuildS19MapOK(S19Listing,S19Map,MemMap);
    ReadMCUMap(S19Map,true);
    VerifyS19Local;
  finally
    ClearMemMap(S19Map);
    S19Map.Free;
  end;
end; // procedure VerifyS19MCU

procedure VerifyErase;
var
  i : integer;
begin
  with MainForm.OWEdit do
  begin
    ReadMCU64K(StartAddr,EndAddr,true);
    for i := StartAddr to EndAddr do
    begin
      if Memory[i] <> $FF
      then begin
        SelAttributes.Color := clRed;
        Lines.Add('Memory NOT fully erased.  At ' + format('%4.4x',[i]) +
                  ' Memory = ' + format('%2.2x',[Memory[i]]));
        SelAttributes.Color := clBlack;
        exit;
        if ProjData.AbortOnCmdFail and (Prog.MacroState = msPlaying)
        then begin
          Prog.MacroState := msStopped;
          Lines.Add('Macro aborted on verification failure');
        end;
      end; // if MemArray[i] <> $FF
    end; // for i := StartAddr to EndAddr do
    Lines.Add('Memory erased over range ' +
               format('%4.4x',[StartAddr]) + ' to ' +
               format('%4.4x',[EndAddr]));
  end; // with MainForm.OWEdit do
end; // procedure VerifyErase

end.

